/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {
  EditorExerciseExerciseForBackend, EditorExerciseFromBackend, TestAssetForBackend, TestAssetFromBackendWithData
} from "../types/exerciseEditor";

const callingName = 'exerciseEditorLayer'
const controllerPrefix = 'exercise/editor'


export async function getEditorExercise(exerciseId: number, loadTestAssetContents: boolean): Promise<EditorExerciseFromBackend> {
  return genericLayer.get<EditorExerciseFromBackend>(callingName,
    `${controllerPrefix}/${exerciseId}/${loadTestAssetContents}`)
}

/**
 * creates a the given exercise
 * @param exercise the exercise to create
 * @returns {Promise<number>} the created exercise id
 * instead navigate to load exercise and load the given number (exercise id)
 */
export async function createEditorExercise(exercise: EditorExerciseExerciseForBackend): Promise<number> {
  return genericLayer.create<EditorExerciseExerciseForBackend, number>(callingName,
    `${controllerPrefix}/create`, exercise)
}

/**
 * updates the given exercise
 * @param exercise the exercise to create
 */
export async function updateEditorExercise(exercise: EditorExerciseExerciseForBackend): Promise<void> {
  return genericLayer.update<EditorExerciseExerciseForBackend, void>(callingName,
    `${controllerPrefix}/update`, exercise
  )
}


/**
 * returns the full asset
 *
 * NOT LONGER USED because GetFullExercise can now include assets...
 * @param exerciseId
 * @param testId
 * @param asset
 * @returns {Promise<TestAssetForBackend>} the asset with content from backend
 */
export async function getTestAsset(exerciseId: number, testId: number, asset: TestAssetFromBackendWithData): Promise<TestAssetForBackend> {
  return genericLayer.get<TestAssetForBackend>(callingName,
    `${controllerPrefix}/test/asset/${exerciseId}/${testId}/${asset.id}`
  )
}
