/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {
  EditableExercisePreviewFromBackend, ExercisePreviewFromBackend,
  FilterableEditableExercisePreviewFromBackend, FilterableGroupPreviewExerciseTupleFromBackend,
  GroupPreviewExerciseTupleFromBackend
} from "../types/exercisePreview";

const callingName = 'exercisePreviewLayer'
const controllerPrefix = 'exercise/overview'

//TODO maybe rename to accessible exercises preview??
// better rename accessible exercises to exercises preview everywhere!!

// export async function getAccessibleExercisePreviews(): Promise<ReadonlyArray<ExercisePreviewFromBackend>> {
//   return genericLayer.get<ReadonlyArray<ExercisePreviewFromBackend>>(
//     callingName, `${controllerPrefix}/all`
//   )
// }

/**
 * TODO make separate api path!!
 * @returns {Promise<ReadonlyArray<ExercisePreviewFromBackend>>}
 */
export async function getOpenExercisePreviews(): Promise<ReadonlyArray<ExercisePreviewFromBackend>> {

  const list = await genericLayer.get<ReadonlyArray<ExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/all`
  )

  return list.filter(p => p.isReleased === true)
}

/**
 * TODO make separate api path!!
 * only the releases where the user entered a code (or not isVisibleToAllAfterRelease)
 * @returns {Promise<ReadonlyArray<ExercisePreviewFromBackend>>}
 */
export async function getOpenExercisePreviewsViaCode(): Promise<ReadonlyArray<ExercisePreviewFromBackend>> {

  const list = await genericLayer.get<ReadonlyArray<ExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/all`
  )

  return list.filter(p => p.isReleased === true && p.isVisibleToAllAfterRelease === false)
}

/**
 * TODO make separate api path!!
 * @returns {Promise<ReadonlyArray<ExercisePreviewFromBackend>>}
 */
export async function getOpenExercisePreviewsViaVisibility(): Promise<ReadonlyArray<ExercisePreviewFromBackend>> {

  const list = await genericLayer.get<ReadonlyArray<ExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/all`
  )

  return list.filter(p => p.isReleased === true && p.isVisibleToAllAfterRelease === true)
}

/**
 * TODO make separate api path!!
 * @returns {Promise<ReadonlyArray<ExercisePreviewFromBackend>>}
 */
export async function getClosedExercisePreviews(): Promise<ReadonlyArray<ExercisePreviewFromBackend>> {

  const list = await genericLayer.get<ReadonlyArray<ExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/all`
  )

  return list.filter(p => p.isReleased === false)
}

/**
 * TODO make separate api path!!
 * @returns {Promise<ReadonlyArray<ExercisePreviewFromBackend>>}
 */
export async function getClosedExercisePreviewsViaCode(): Promise<ReadonlyArray<ExercisePreviewFromBackend>> {

  const list = await genericLayer.get<ReadonlyArray<ExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/all`
  )

  return list.filter(p => p.isReleased === false && p.isVisibleToAllAfterRelease === false)
}

/**
 * TODO make separate api path!!
 * @returns {Promise<ReadonlyArray<ExercisePreviewFromBackend>>}
 */
export async function getClosedExercisePreviewsViaVisibility(): Promise<ReadonlyArray<ExercisePreviewFromBackend>> {

  const list = await genericLayer.get<ReadonlyArray<ExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/all`
  )

  return list.filter(p => p.isReleased === false && p.isVisibleToAllAfterRelease === true)
}


export async function getOwnExercisePreviews(): Promise<ReadonlyArray<EditableExercisePreviewFromBackend>> {
  return genericLayer.get<ReadonlyArray<EditableExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/own`
  )
}


export async function getAccessibleExercisePreviewsByGroup(): Promise<ReadonlyArray<GroupPreviewExerciseTupleFromBackend>> {
  return genericLayer.get<ReadonlyArray<GroupPreviewExerciseTupleFromBackend>>(
    callingName, `${controllerPrefix}/groups`)
}


/**
 * deletes the given exercise by id
 * @param exerciseId the exercise id
 * @returns {Promise<void>}
 */
export async function deleteExercise(exerciseId:number):Promise<void> {
  return genericLayer.del<void>(callingName,
    `${controllerPrefix}/delete/${exerciseId}`)
}

export function makeFilterableGroupExercises(
  exercises: ReadonlyArray<GroupPreviewExerciseTupleFromBackend>
): ReadonlyArray<FilterableGroupPreviewExerciseTupleFromBackend> {

  const filterable: ReadonlyArray<FilterableGroupPreviewExerciseTupleFromBackend> = exercises.map((exercise) => {
    return {
      ...exercise,
      isDisplayed: true
    }
  })

  return filterable

}


export function makeFilterable(
  exercises: ReadonlyArray<EditableExercisePreviewFromBackend>
): ReadonlyArray<FilterableEditableExercisePreviewFromBackend> {

  const filterable: ReadonlyArray<FilterableEditableExercisePreviewFromBackend> = exercises.map(p => {
    return {
      ...p,
      isDisplayed: true
    }

  })

  return filterable

}