/**
 * Created by janis dähne.
 */


import * as genericLayer from './genericLayer'
import {
  RunCompileSingleFileCommandForBackend, RunAfterTestCommandForBackend, RunTestCommandForBackend,
  TestAnswerFromBackend, RunCompileSingleFileCommandCustomProjectForBackend, RunTestCommandCustomProjectForBackend,
  JustRunProgramCommandForBackend, JustRunProgramTestCustomProjectCommandForBackend
} from "../types/testsAndTestProtocol";

export const callingName = 'testingLayer'
const controllerPrefix = 'testing'


/**
 * runs a test on the test server (through the backend)
 */
export async function runTest(runNormalTestCommand: RunTestCommandForBackend): Promise<ReadonlyArray<TestAnswerFromBackend>> {

  const result: ReadonlyArray<TestAnswerFromBackend>
    = await genericLayer.update<RunTestCommandForBackend, ReadonlyArray<TestAnswerFromBackend>>(
    callingName,
    `${controllerPrefix}/run`, runNormalTestCommand)

  return result
}

export async function runCustomTestCustomProject(runNormalTestCommand: RunTestCommandCustomProjectForBackend): Promise<ReadonlyArray<TestAnswerFromBackend>> {

  const result: ReadonlyArray<TestAnswerFromBackend>
    = await genericLayer.update<RunTestCommandCustomProjectForBackend, ReadonlyArray<TestAnswerFromBackend>>(
    callingName,
    `${controllerPrefix}/run/custom/project/custom`, runNormalTestCommand)

  return result
}

/**
 * runs a sinngle compile file command on the backend and returns the test result
 * @param runCompileSingleFileCommand the command
 * @returns {Promise<TestAnswerFromBackend>}
 */
export async function compileSingleFile(runCompileSingleFileCommand: RunCompileSingleFileCommandForBackend): Promise<TestAnswerFromBackend> {

  const result: TestAnswerFromBackend =
    await genericLayer.update<RunCompileSingleFileCommandForBackend, TestAnswerFromBackend>(callingName,
      `${controllerPrefix}/compile/do/exercise`, runCompileSingleFileCommand)

  return result
}

export async function compileSingleFileCustomProject(runCompileSingleFileCommand: RunCompileSingleFileCommandCustomProjectForBackend): Promise<TestAnswerFromBackend> {

  const result: TestAnswerFromBackend =
    await genericLayer.update<RunCompileSingleFileCommandCustomProjectForBackend, TestAnswerFromBackend>(callingName,
      `${controllerPrefix}/compile/custom/project`, runCompileSingleFileCommand)

  return result
}


/**
 * runs a test as a tutor (with a temp frontend only solution)
 */
export function runTestAsTutor(runTestAsTutorCommand: RunAfterTestCommandForBackend): Promise<ReadonlyArray<TestAnswerFromBackend>> {
  return genericLayer.update<RunAfterTestCommandForBackend,ReadonlyArray<TestAnswerFromBackend>>(callingName,
    `${controllerPrefix}/run/as/tutor`, runTestAsTutorCommand)
}

export function runAfterTest(runAfterTestCommand: RunAfterTestCommandForBackend): Promise<ReadonlyArray<TestAnswerFromBackend>> {
  return genericLayer.update<RunAfterTestCommandForBackend,ReadonlyArray<TestAnswerFromBackend>>(callingName,
    `${controllerPrefix}/run/after`, runAfterTestCommand)
}



export async function justRunProgramTest(justRunProgramCommand: JustRunProgramCommandForBackend): Promise<TestAnswerFromBackend> {

  const result: TestAnswerFromBackend =
    await genericLayer.update<JustRunProgramCommandForBackend, TestAnswerFromBackend>(callingName,
      `${controllerPrefix}/justrunprogram/normal`, justRunProgramCommand)

  return result
}


export async function justRunProgramTestCustomProject(justRunProgramCommand: JustRunProgramTestCustomProjectCommandForBackend): Promise<TestAnswerFromBackend> {

  const result: TestAnswerFromBackend =
    await genericLayer.update<JustRunProgramTestCustomProjectCommandForBackend, TestAnswerFromBackend>(callingName,
      `${controllerPrefix}/justrunprogram/custom/project`, justRunProgramCommand)

  return result
}
