/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {
  AssessedSubmitTestFromBackend, AssessmentForUserWithTestResults,
  StoredTestResultTupleFromBackend
} from "../types/testResults";
import {SolutionVersionTupleFromBackend} from "../types/exerciseSolution";
import {DoExerciseFromBackend} from "../types/doExercise";
import {AssessmentFullBase} from "../types/submissions";


const callingName = 'tutorViewAssessmentLayer'
const controllerPrefix = 'tutor/view/assessment'


export function getExerciseAsTutor(releaseId: number, userId: number, pLangId: number): Promise<DoExerciseFromBackend> {
  return genericLayer.get<DoExerciseFromBackend>(callingName,
    `${controllerPrefix}/exercise/${releaseId}/${userId}/${pLangId}`)
}

export function getUserSolutionAsTutor(releaseId: number, userId: number, pLangId: number): Promise<SolutionVersionTupleFromBackend> {
  return genericLayer.get<SolutionVersionTupleFromBackend>(callingName,
    `${controllerPrefix}/solution/${releaseId}/${userId}/${pLangId}`)
}

export function getAllTestResultsAsTutor(releaseId: number,
                                         userId: number,
                                         pLangId: number,
                                         solutionVersionId: number): Promise<StoredTestResultTupleFromBackend> {
  return genericLayer.get<StoredTestResultTupleFromBackend>(callingName,
    `${controllerPrefix}/testresults/${releaseId}/${userId}/${pLangId}/${solutionVersionId}`)
}

export function getSubmissionTestResultsAsTutor(releaseId: number, userId: number, pLangId: number): Promise<AssessmentForUserWithTestResults> {
  return genericLayer.get<AssessmentForUserWithTestResults>(callingName,
    `${controllerPrefix}/testresults/submit/${releaseId}/${userId}/${pLangId}`)
}


export async function getSingleAssessment(releaseId: number, userId: number, pLangId: number): Promise<AssessmentFullBase> {
  return genericLayer.get<AssessmentFullBase>(
    callingName, `${controllerPrefix}/assessment/${releaseId}/${userId}/${pLangId}`
  )
}


export async function updateSingleAssessment(assessment: AssessmentFullBase): Promise<void> {
  return genericLayer.update<AssessmentFullBase, void>(
    callingName, `${controllerPrefix}/assessment/update`, assessment
  )
}