/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";

import MonacoEditor from "react-monaco-editor";
import IModelContentChangedEvent = monaco.editor.IModelContentChangedEvent;
import Logger from "../../../helpers/logger";

import ReactResizeDetector from "react-resize-detector";
import IEditorOptions = monaco.editor.IEditorOptions;
import {hasEditorSettingsChanged} from "../../../helpers/editorHelper";
import {availableEditorThemeNames, getEditorInternalThemeName} from "../../../helpers/themes";

import {initial as initialCodeEditorSettings} from "../../../state/reducers/editorSettingsDialog/editorSettingsReducer"
import {CodeEditorSettingsFullBase} from "../../../types/userData";
import IModel = monaco.editor.IModel;
import ITextModelUpdateOptions = monaco.editor.ITextModelUpdateOptions;
import IEditorModel = monaco.editor.IEditorModel;
import {SimpleCodeEditorProps} from "../simpleCodeEditorWrapper";
import AceEditor from "react-ace";
import throttle from 'lodash-es/throttle'

//const css = require('./styles.styl');

import {Editor, EditSession, IEditSession, UndoManager, createEditSession} from 'brace';

import 'brace/mode/java';
import 'brace/mode/markdown';
import 'brace/mode/json';
//import 'brace/theme/monokai';
import '../../../../externalLibsForBundling/brace/themes/monokai_changed';
import 'brace/theme/sqlserver';
import {EditorTab} from "../multiTabCodeEditorProps";
import {applySavedSessionFromJSON, convertSession} from "./multiFileAceEditor";

const mapStateToProps = (rootState: RootState, props: SimpleCodeEditorProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    editorSettings: (rootState.userDataSettingsSate.userData.settings
    && rootState.userDataSettingsSate.userData.settings.codeEditorSettings
      ? rootState.userDataSettingsSate.userData.settings.codeEditorSettings
      : initialCodeEditorSettings) as CodeEditorSettingsFullBase
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

interface AceWrapper {
  readonly editor: Editor
}


//TODO when using e.g. json ace craetes a new web worker to validate the text... but the worker is never cleared??
//or just a chrome display bug? dev tools -> sources -> left panel -> service workers #1, #2 ...

//basically we need to do the same thing as in multi tab editor (maybe the comments work...)
//but no need to support sessions yet ;)
class SimpleAceEditor extends React.Component<Props, any> {

  aceEditorWrapper: AceWrapper

  initialSession: IEditSession = createEditSession('', `ace/mode/text` as any)
  editSession: IEditSession = null //this.initialSession
  forceRerenderThrottled: () => void = null


  componentWillMount(): void {
    // noinspection TsLint
    this.forceRerenderThrottled = throttle(this.forceRerender, 200)
  }

  rebuildSessionStorage(editorTabs: EditorTab): void {

    const tab: EditorTab = editorTabs
    this.editSession = createEditSession(tab.content, `ace/mode/${tab.language}` as any)
  }

  applySavedSessionStates(editorTab: EditorTab): void {

    // noinspection TsLint
    const tab: EditorTab = editorTab
    applySavedSessionFromJSON(this.editSession, tab.editSession)
    if (this.props.onEditorSessionChanged) {
      this.props.onEditorSessionChanged(
        {
          ...this.props.editorTab,
          editSession: convertSession(this.editSession)
        }
      )
    }

  }

  componentDidMount(): void {

    this.rebuildSessionStorage(this.props.editorTab)

    //this.applySavedSessionStates(this.props.editorTab)

    //ensure no old undo data is there
     if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {
    //   const tab: EditorTab = this.props.editorTab
    //
       const session = this.aceEditorWrapper.editor.getSession()
    //   session.setMode(`ace/mode/${tab.language}`)
    //   session.setValue(tab.content)
    //
       const undoM = session.getUndoManager()
       undoM.reset()
       session.setUndoManager(undoM)
     }
  }

  componentWillUnmount(): void {
    // if (this.props.onEditorSessionChanged) {
    //   this.props.onEditorSessionChanged(
    //     {
    //       ...this.props.editorTab,
    //       editSession: convertSession(this.editSession)
    //     }
    //   )
    // }
  }



  /**
   * updates the editor ref and tries to set the active session
   * @param {AceWrapper} wrapper
   */
  updateEditorRef(wrapper: AceWrapper): void {
    this.aceEditorWrapper = wrapper

    if (wrapper && wrapper.editor) {
     // wrapper.editor.setSession(this.editSession)

    }
  }

  forceRerender(): void {
    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {
      //console.log("resized")
      this.aceEditorWrapper.editor.resize(false)
    }
  }

  onChange = (newValue: string, event: any) => {

    if (this.onChange) {
      const session = this.aceEditorWrapper.editor.getSession()
      this.props.onChange(
        {
          ...this.props.editorTab,
          content: newValue,
          editSession: convertSession(session),
        }
      )
    }
  }

  render(): JSX.Element {

    return (
      <div className="fh">

        {
          //resize for div is not implemented in js ... just for window
        }
        <ReactResizeDetector handleHeight handleWidth onResize={(w,h) => {
          if (this.forceRerenderThrottled) {
            this.forceRerenderThrottled()
          }
        }} />


          <AceEditor
          width="100%"
          height={this.props.height ? this.props.height : '100%'}
          name={this.props.id}
          mode={this.props.editorTab.language}
          theme={getEditorInternalThemeName(this.props.editorSettings.theme)}
          focus={true}
          ref={(editor: any) => {
            if (editor) {
              this.updateEditorRef(editor)
            }
          }}
          value={this.props.editorTab.content}

          onChange={this.onChange}
          readOnly={this.props.editorTab.isReadonly}
          fontSize={this.props.editorSettings.fontSize}
          wrapEnabled={this.props.editorSettings.useWrapping}
          tabSize={this.props.editorSettings.tabSize}
          showPrintMargin={false}
          highlightActiveLine={true}
          editorProps={{$blockScrolling: true}}
        />
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(SimpleAceEditor)