/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../state/reducers/index";
import SaveButton from "./buttons/saveButton";
import {Divider} from "semantic-ui-react";
import Spinner from "./helpers/Spinner";
import CancelButton from "./buttons/cancelButton";
import SinglePanelSiteWrapper from './singlePanelSiteWrapper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly isBusy: boolean
  /**
   * this is normally just a h2 element using the isCreating property of the dialog (create or update caption)
   */
  readonly caption: JSX.Element
  readonly onSave: () => void | null
  readonly onCancel: () => void
  readonly isValid: boolean

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class FullSiteDialog extends React.Component<Props, any> {
  render(): JSX.Element {

    return (
      <SinglePanelSiteWrapper isFullSite={true}
      >
        {
          this.props.isBusy &&
          <Spinner/>
        }

        <div className="view-padding fh flexed-h">
          <div className="mar-bot">
            <div className="view-caption">
              {
                this.props.caption
              }
            </div>
          </div>

          <div className="flexed-h fh scrollable">
            <div className="flex-stretch pad-right">
              {
                this.props.children
              }
            </div>
          </div>

          <Divider/>

          <div className="flexed">
            {
              this.props.onSave &&
              <SaveButton className="mar-right" disabled={!this.props.isValid}
                          onClick={() => this.props.onSave()}
              />
            }

            <CancelButton onClick={() => this.props.onCancel()}/>
          </div>

        </div>
      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(FullSiteDialog)