/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers/index";
import {setTextToEscape} from "../../state/actions/jsonEscapeEditorActions";
import SimpleCodeEditorWrapper from '../codeEditors/simpleCodeEditorWrapper'
import {EditorTab} from "../codeEditors/multiTabCodeEditorProps";
import {aceTextLanguage} from "../../helpers/editorHelper";
import {SimpleDivider} from "../helpers/simpleDivider";

import MaterialTextArea from '../material/materialTextArea'
import {Icon} from "semantic-ui-react";
import {copyToClipboard} from "../../helpers/clipboard";
import {translate} from "../../../i18n/translationHelper";
import {TranslateIds} from "../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly editorResizeAnchorId: string
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    textToEscape: rootState.jsonEscapeEditorState.text,
    escapedText: rootState.jsonEscapeEditorState.escapedText,

    editorResizeAnchorId: props.editorResizeAnchorId
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setTextToEscape
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class jsonEscapeEditorView extends React.Component<Props, any> {
  render(): JSX.Element {

    const tempEditorTab: EditorTab = {
      id: -1,
      content: this.props.textToEscape,
      language: aceTextLanguage,
      editSession: null,
      isReadonly: false,
      fileNameWithExtensions: '',
      displayIndex: 1,
      isMainFile: true,
    }

    return (
      <div className="fh flexed-h">

        {
          //monaco editor
        }
        <div>
          {translate(TranslateIds.escapeEditor)}
        </div>
        <div style={{height: '50%'}}>
          <SimpleCodeEditorWrapper
            id="jsonEscapeEditor"
            resizeAnchorId={this.props.editorResizeAnchorId}
            editorTab={tempEditorTab}
            onChange={(editorTab: EditorTab) => {
              this.props.setTextToEscape(editorTab.content)
            }}
            onEditorSessionChanged={null}
          />
        </div>

        <SimpleDivider/>

        <div>
          <span className="mar-right-half">{translate(TranslateIds.escapeEditorResult)}</span>
          <Icon name="copy" className="clickable" onClick={() => {
            copyToClipboard(this.props.escapedText)
          }}/>
        </div>
        <MaterialTextArea
          value={this.props.escapedText}
        />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(jsonEscapeEditorView)