/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers/index";

import LoginForm from './loginForm'
import {withRouter} from "react-router";
import RegisterFrom from './registerForm'
import {getUserData} from "../../communicationLayer/loginLayer";
import {UserDataFullBase} from "../../types/userData";
import {setUserData} from "../../state/actions/settingsSite/userSettingsSiteActions";
import {setIsCheckingAlreadyLoggedIn} from "../../state/actions/loginActions";
import {delay} from "../../../debug";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    userId: rootState.userDataSettingsSate.userData.id,
    isCheckingAlreadyLoggedIn: rootState.loginDataState.isCheckingAlreadyLoggedIn,
    isRegisterFormDisplayed: rootState.loginDataState.isRegisterFormDisplayed,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setUserData,
  setIsCheckingAlreadyLoggedIn,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class LoginOverlay extends React.Component<Props, any> {


  async componentWillMount(): Promise<void> {

    //try to get the user data...
    //if success the user is still logged in

    // noinspection TsLint
    let userData: UserDataFullBase = null

    this.props.setIsCheckingAlreadyLoggedIn(true)
    try {
      userData = await delay(getUserData())
    } catch (err) {

    }
    finally {
      this.props.setIsCheckingAlreadyLoggedIn(false)
    }

    if (userData === null) {
      //not logged in ...
    } else {

      //set user data --> login will disappear
      this.props.setUserData(userData)
    }

  }

  render(): JSX.Element {
    return (
      <div className="fh">
        {
          //
          //<RegisterFrom/>
          (this.props.userId <= 0) &&
          <div className="fh login-screen-animated">
            {
              !this.props.isRegisterFormDisplayed &&
              <LoginForm/>
            }

            {
              this.props.isRegisterFormDisplayed &&
              <RegisterFrom/>
            }
          </div>
        }

        {
          //real content after login
          this.props.userId > 0 &&
          this.props.children
        }

      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(LoginOverlay))