/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers/index";
import {List, Icon, Divider} from "semantic-ui-react";
import {SimpleVDivider} from "../helpers/simpleVDivider";
import {Link} from "react-router-dom";
import * as constants from "../../constants";
import {setIsSiteMenuOpen} from '../../state/actions/siteHeaderBarActions'
import {translate} from "../../../i18n/translationHelper";
import {TranslateIds} from "../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    //systemRoleDisplayName: tryGet(() => rootState.userDataSettingsSite.systemRole.displayName, ''),
    systemRole: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole
      : null,
    tempCanCreateExercise: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData
      ? rootState.userDataSettingsSate.userData.tempCanCreateExercise
      : false
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setIsOpen: setIsSiteMenuOpen

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class SiteMenu extends React.Component<Props, any> {


  /**
   * called when a link was clicked so hide the menu
   */
  linkClicked(): void {
    this.props.setIsOpen(false)
  }

  render(): JSX.Element {

    return (
      <div className="header-menu-list flexed">

        <div>
          <Link to={constants.homeLinkPath} onClick={() => this.linkClicked()}>
            <Icon size="huge" name="home"/>
            <span>
              {
                translate(TranslateIds.home)
              }
              </span>
          </Link>
        </div>


        <SimpleVDivider/>

        <div>
          <Link to={constants.openExercisesLinkPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='block layout'/>
              <Icon corner inverted name='unlock'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.allOpenExercises)
              }
              </span>
          </Link>
        </div>

        <SimpleVDivider/>

        <div>
          <Link to={constants.closedExercisesLinkPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='block layout'/>
              <Icon corner inverted name='lock'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.allClosedExercises)
              }
              </span>
          </Link>
        </div>

        <SimpleVDivider/>

        <div>
          <Link to={constants.openExercisesViaCodeLinkPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='block layout'/>
              <Icon corner inverted name='unlock'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.openExercisesViaCode)
              }
              </span>
          </Link>
        </div>

        <SimpleVDivider/>

        <div>
          <Link to={constants.openExercisesViaVisibilityLinkPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='block layout'/>
              <Icon corner inverted name='unlock'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.openExercisesViaVisibility)
              }
              </span>
          </Link>
        </div>

        <SimpleVDivider/>

        <div>
          <Link to={constants.closedExercisesViaCodeLinkPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='block layout'/>
              <Icon corner inverted name='lock'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.closedExercisesViaCode)
              }
              </span>
          </Link>
        </div>

        <SimpleVDivider/>

        <div>
          <Link to={constants.closedExercisesViaVisibilityLinkPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='block layout'/>
              <Icon corner inverted name='lock'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.closedExercisesViaVisibility)
              }
              </span>
          </Link>
        </div>

        <SimpleVDivider/>

        <div>
          <Link to={constants.assessmentStatisticsSiteToLinkPath + "/"} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='area chart'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.assessmentStatisticsForUser)
              }
              </span>
          </Link>
        </div>

        <SimpleVDivider/>

        <div>
          <Link to={constants.manageCustomProjectsSiteLinkMatchPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='archive'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.manageCustomProjects)
              }
              </span>
          </Link>
        </div>


        <SimpleVDivider/>


        <div>
          <Link to={constants.ownExercisesLinkPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='user'/>
              <Icon corner inverted name='block layout'/>
            </Icon.Group>
            <span>
                {
                  translate(TranslateIds.ownExercises)
                }
              </span>
          </Link>
        </div>


        <SimpleVDivider/>


        <div>
          <Link to={constants.groupExercisesLinkPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='users'/>
              <Icon corner inverted name='block layout'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.groupExercises)
              }
              </span>
          </Link>
        </div>


        <SimpleVDivider/>


        {
          this.props.tempCanCreateExercise &&
          <div>
            <Link to={constants.exerciseEditorCreateNewLinkPath} onClick={() => this.linkClicked()}>
              <Icon.Group size='huge'>
                <Icon name='file'/>
                <Icon corner inverted name='plus'/>
              </Icon.Group>
              <span>
              {
                translate(TranslateIds.newExercises)
              }
              </span>
            </Link>
          </div>
        }


        {
          this.props.tempCanCreateExercise &&
          <SimpleVDivider/>
        }


        <div>
          <Link to={constants.groupsLinkPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='users'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.groups)
              }
              </span>
          </Link>
        </div>

        <SimpleVDivider/>

        {
          (
            this.props.systemRole &&
            (this.props.systemRole.canCreateRoles
              || this.props.systemRole.canChangeRoles
              || this.props.systemRole.canDeleteRoles)
          ) &&
          <div>
            <Link to={constants.groupRolesLinkPath} onClick={() => this.linkClicked()}>
              <Icon.Group size='huge'>
                <Icon name='users'/>
                <Icon corner inverted name='law'/>
              </Icon.Group>
              <span>
              {
                translate(TranslateIds.groupRoles)
              }
              </span>
            </Link>
          </div>
        }


        {
          (
            this.props.systemRole &&
            (this.props.systemRole.canCreateGroups
              || this.props.systemRole.canChangeGroups
              || this.props.systemRole.canDeleteGroups)
          ) &&
          <SimpleVDivider/>
        }


        {
          (this.props.systemRole && this.props.systemRole.canManageNewUsers) &&
          <div>
            <Link to={constants.manageNewUsersLinkPath} onClick={() => this.linkClicked()}>
              <Icon.Group size='huge'>
                <Icon name='child'/>
                <Icon corner inverted name='warning circle'/>
              </Icon.Group>
              <span>
              {
                translate(TranslateIds.manageNewUsers)
              }
              </span>
            </Link>
          </div>
        }
        {
          (this.props.systemRole && this.props.systemRole.canManageNewUsers) &&
          <SimpleVDivider/>
        }

        {
          (this.props.systemRole && (this.props.systemRole.canChangeUserData
              || this.props.systemRole.canDeleteActivatedUsers)
          ) &&
          <div>
            <Link to={constants.manageActivatedUsersLinkPath} onClick={() => this.linkClicked()}>
              <Icon.Group size='huge'>
                <Icon name='child'/>
              </Icon.Group>
              <span>
              {
                translate(TranslateIds.manageActivatedUsers)
              }
              </span>
            </Link>
          </div>
        }

        {
          (this.props.systemRole && (this.props.systemRole.canChangeUserData
              || this.props.systemRole.canDeleteActivatedUsers)
          ) &&
          <SimpleVDivider/>
        }

        {
          (this.props.systemRole &&
            (this.props.systemRole.canCreateRoles
              || this.props.systemRole.canChangeRoles
              || this.props.systemRole.canDeleteRoles)
          ) &&
          <div>
            <Link to={constants.systemRolesLinkPath} onClick={() => this.linkClicked()}>
              <Icon.Group size='huge'>
                <Icon name='law'/>
              </Icon.Group>
              <span>
              {
                translate(TranslateIds.systemRoles)
              }
              </span>
            </Link>
          </div>
        }
        {
          (this.props.systemRole &&
            (this.props.systemRole.canCreateRoles
              || this.props.systemRole.canChangeRoles
              || this.props.systemRole.canDeleteRoles)
          ) &&
          <SimpleVDivider/>
        }


        <div>
          <Link to={constants.ownSettingsLinkPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='settings'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.ownSettingsSite)
              }
              </span>
          </Link>
        </div>

        <SimpleVDivider/>

        {
          (this.props.systemRole && this.props.systemRole.canManageTags) &&
          <div>
            <Link to={constants.manageTagsLinkPath} onClick={() => this.linkClicked()}>
              <Icon.Group size='huge'>
                <Icon name='tags'/>
              </Icon.Group>
              <span>
              {
                translate(TranslateIds.manageTags)
              }
              </span>
            </Link>
          </div>
        }
        {
          (this.props.systemRole && this.props.systemRole.canManageTags) &&
          <SimpleVDivider/>
        }

        {
          (this.props.systemRole && this.props.systemRole.canViewDashboard) &&
          <div>
            <Link to={constants.dashboardLinkPath} onClick={() => this.linkClicked()}>
              <Icon.Group size='huge'>
                <Icon name='dashboard'/>
              </Icon.Group>
              <span>
              {
                translate(TranslateIds.dashboard)
              }
              </span>
            </Link>
          </div>
        }
        {
          (this.props.systemRole && this.props.systemRole.canViewDashboard) &&
          <SimpleVDivider/>
        }

        <div>
          <Link to={constants.aboutLinkPath} onClick={() => this.linkClicked()}>
            <Icon.Group size='huge'>
              <Icon name='info'/>
            </Icon.Group>
            <span>
              {
                translate(TranslateIds.about)
              }
              </span>
          </Link>
        </div>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(SiteMenu)