/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {setIsSiteMenuOpen} from "../../state/actions/siteHeaderBarActions";
import * as constants from "../../constants";
import {TranslateIds} from "../../../i18n/ids";
import {translate} from "../../../i18n/translationHelper";
import {Link} from "react-router-dom";
import {Icon} from "semantic-ui-react";
import {SimpleDividerWithSpace} from "../helpers/simpleDividerWithSpace";
import {SimpleVDivider} from "../helpers/simpleVDivider";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    systemRole: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole
      : null,
    tempCanCreateExercise: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData
      ? rootState.userDataSettingsSate.userData.tempCanCreateExercise
      : false,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setIsSiteMenuOpen
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class menu extends React.Component<Props, any> {

  componentDidMount(): void {

    //from https://stackoverflow.com/questions/4320553/handling-clicks-outside-an-element-without-jquery
    const menuContainer = document.getElementById('siteSideMenu')
    menuContainer.onclick = (e) => {
      if (e) {

        //a bit dirty...
        //else the click would not be propagated to the link...
        if (e.target) {
          const srcElement = e.target as Element
          if (srcElement.parentElement.nodeName.toLowerCase() === 'a') {

            return
          }
        }

        e.stopPropagation()
        return
      }
      window.event.cancelBubble = true
    }

    document.onclick = () => this.props.setIsSiteMenuOpen(false)
  }

  componentWillUnmount(): void {
    const menuContainer = document.getElementById('siteSideMenu')
    menuContainer.onclick = null
    document.onclick = null
  }


  /**
   * called when a link was clicked so hide the menu
   */
  linkClicked(): void {
    this.props.setIsSiteMenuOpen(false)
  }

  render(): JSX.Element {
    return (
      <div id="siteSideMenu" className="site-side-menu2 flexed"
      >

        {
          //accessible exercises
        }
        <div className="site-side-menu2-row">

          <div className="header first">
            <Icon name='block layout'/>
            <span>{translate(TranslateIds.exercisesGroup)}</span>
          </div>

          <SimpleDividerWithSpace/>

          <div className="body">

            <div>
              <Link to={constants.openExercisesLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='unlock'/>
                <span>
              {
                translate(TranslateIds.allOpenExercises)
              }
              </span>
              </Link>
            </div>

            <div>
              <Link to={constants.closedExercisesLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='lock'/>
                <span>
              {
                translate(TranslateIds.allClosedExercises)
              }
              </span>
              </Link>
            </div>

            <SimpleVDivider/>

            <div>
              <Link to={constants.openExercisesViaCodeLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='unlock'/>
                <span>
              {
                translate(TranslateIds.openExercisesViaCode)
              }
              </span>
              </Link>
            </div>


            <div>
              <Link to={constants.openExercisesViaVisibilityLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='unlock'/>
                <span>
              {
                translate(TranslateIds.openExercisesViaVisibility)
              }
              </span>
              </Link>
            </div>


            <div>
              <Link to={constants.closedExercisesViaCodeLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='lock'/>
                <span>
              {
                translate(TranslateIds.closedExercisesViaCode)
              }
              </span>
              </Link>
            </div>

            <div>
              <Link to={constants.closedExercisesViaVisibilityLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='lock'/>
                <span>
              {
                translate(TranslateIds.closedExercisesViaVisibility)
              }
              </span>
              </Link>
            </div>
          </div>

        </div>


        {
          //manage exercises
        }
        <div className="site-side-menu2-row">
          <div className="header">
            <Icon name='cubes'/>
            <span>{translate(TranslateIds.manageExercisesGroup)}</span>
          </div>

          <SimpleDividerWithSpace/>

          <div className="body">

            <div>
              <Link to={constants.ownExercisesLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='user'/>
                <span>
                {
                  translate(TranslateIds.ownExercises)
                }
              </span>
              </Link>
            </div>


            <div>
              <Link to={constants.groupExercisesLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='users'/>
                <span>
              {
                translate(TranslateIds.groupExercises)
              }
              </span>
              </Link>
            </div>

            {
              (this.props.systemRole && this.props.systemRole.canManageTags) &&
              <div>
                <div>
                  <Link to={constants.manageTagsLinkPath} onClick={() => this.linkClicked()}>
                    <Icon size="huge" name='tags'/>
                    <span>
              {
                translate(TranslateIds.manageTags)
              }
              </span>
                  </Link>
                </div>
              </div>
            }

            <SimpleVDivider/>

            {
              this.props.tempCanCreateExercise &&
              <div>
                <Link to={constants.exerciseEditorCreateNewLinkPath} onClick={() => this.linkClicked()}>
                  <Icon size="huge" name='plus'/>
                  <span>
              {
                translate(TranslateIds.newExercises)
              }
              </span>
                </Link>
              </div>
            }

          </div>
        </div>


        {
          //manage users
        }
        {
          (this.props.systemRole && (this.props.systemRole.canManageNewUsers || this.props.systemRole.canChangeUserData
            || this.props.systemRole.canDeleteActivatedUsers)) &&

          <div className="site-side-menu2-row">

            <div className="header">
              <Icon name='child'/>
              <span>{translate(TranslateIds.manageUsersGroup)}</span>
            </div>

            <SimpleDividerWithSpace/>

            <div className="body">

              {
                (this.props.systemRole && this.props.systemRole.canManageNewUsers) &&
                <div>
                  <Link to={constants.manageNewUsersLinkPath} onClick={() => this.linkClicked()}>
                    <Icon size="huge" name='asterisk'/>
                    <span>
                      {
                        translate(TranslateIds.manageNewUsers)
                      }
                      </span>
                  </Link>
                </div>
              }

              {
                (this.props.systemRole && (this.props.systemRole.canChangeUserData
                    || this.props.systemRole.canDeleteActivatedUsers)
                ) &&
                <div>
                  <Link to={constants.manageActivatedUsersLinkPath} onClick={() => this.linkClicked()}>
                    <Icon size="huge" name='child'/>
                    <span>
              {
                translate(TranslateIds.manageActivatedUsers)
              }
              </span>
                  </Link>
                </div>
              }

            </div>
          </div>
        }


        {
          //manage roles & groups
        }
        <div className="site-side-menu2-row">
          <div className="header">
            <Icon name='users'/>
            <span>{translate(TranslateIds.manageGroupsAndRolesGroup)}</span>
          </div>

          <SimpleDividerWithSpace/>

          <div className="body">
            <div>
              <Link to={constants.groupsLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='users'/>
                <span>
              {
                translate(TranslateIds.groups)
              }
              </span>
              </Link>
            </div>

            <SimpleVDivider/>

            {
              (
                this.props.systemRole &&
                (this.props.systemRole.canCreateRoles
                  || this.props.systemRole.canChangeRoles
                  || this.props.systemRole.canDeleteRoles)
              ) &&
              <div>
                <Link to={constants.groupRolesLinkPath} onClick={() => this.linkClicked()}>
                  <Icon size="huge" name='law'/>
                  <span>
              {
                translate(TranslateIds.groupRoles)
              }
              </span>
                </Link>
              </div>
            }

            {
              (this.props.systemRole &&
                (this.props.systemRole.canCreateRoles
                  || this.props.systemRole.canChangeRoles
                  || this.props.systemRole.canDeleteRoles)
              ) &&
              <div>
                <Link to={constants.systemRolesLinkPath} onClick={() => this.linkClicked()}>
                  <Icon size="huge" name='law'/>
                  <span>
              {
                translate(TranslateIds.systemRoles)
              }
              </span>
                </Link>
              </div>
            }
          </div>
        </div>


        {
          //others
        }
        <div className="site-side-menu2-row">
          <div className="header">
            <Icon name='ellipsis vertical'/>
            <span>{translate(TranslateIds.other)}</span>
          </div>

          <SimpleDividerWithSpace/>

          <div className="body">

            <div>
              <Link to={constants.manageCustomProjectsSiteLinkMatchPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='archive'/>
                <span>
              {
                translate(TranslateIds.manageCustomProjects)
              }
              </span>
              </Link>
            </div>

            <div>
              <Link to={constants.assessmentStatisticsSiteToLinkPath + "/"} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='area chart'/>
                <span>
              {
                translate(TranslateIds.myAssessmentStatisticsForUser)
              }
              </span>
              </Link>
            </div>

            <div>
              <Link to={constants.ownSettingsLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='settings'/>
                <span>
              {
                translate(TranslateIds.ownSettingsSite)
              }
              </span>
              </Link>
            </div>


            {
              (this.props.systemRole && this.props.systemRole.canViewDashboard) &&
              <div className="site-side-menu-heading solo">
                <div>
                  <Link to={constants.dashboardLinkPath} onClick={() => this.linkClicked()}>
                    <Icon size="huge" name="dashboard"/>
                    <span>
              {
                translate(TranslateIds.dashboard)
              }
              </span>
                  </Link>
                </div>
              </div>
            }

            <SimpleVDivider/>

            <div>
              <Link to={constants.homeLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name="home"/>
                <span>
                  {
                    translate(TranslateIds.home)
                  }
              </span>
              </Link>
            </div>


            <div>
              <Link to={constants.aboutLinkPath} onClick={() => this.linkClicked()}>
                <Icon size="huge" name='info'/>
                <span>
                  {
                    translate(TranslateIds.about)
                  }
              </span>
              </Link>
            </div>

          </div>
        </div>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(menu)