/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";

import ListView from './listView'
import CardsView from './cardsView'

import SinglePanelSiteWrapper from '../../../singlePanelSiteWrapper'
import {SiteViewState} from "../../../../types/siteView";
import {ExerciseState} from "../../../../types/exercisePreview";
import MaterialSearchInput from '../../../material/materialSearchInput'

import EnterReleaseCodeDialog from '../dialogs/enterReleaseCodeDialog'

import {
  getOpenExercisePreviewsAsync, loadOpenExercisePreviewsAsync, setOpenExercises, setSortByKey
} from "../../../../state/actions/allAccessibleExercises/openExercisesSite/openExercisesSiteActions";
import Spinner from "../../../helpers/Spinner";
import {Moment} from "moment";
import {getServerTime} from "../../../../communicationLayer/timeLayer";
import moment = require("moment");
import {setClosedExercises} from "../../../../state/actions/allAccessibleExercises/closedExercisesSite/closedExercisesSiteActions";
import {updateExercisePreviews} from "../../../../helpers/editorHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import history from "../../../../routerHistory";
import * as constants from '../../../../constants'
import {sortAndFilterExercisePreviews} from "../../../../helpers/tablesHelper";
import TagFilterPanel from '../../../tagComponents/tagFilterPanel'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  /**
   * true: load data,
   * false: not
   * this is ued in the all closed exercises site because not both (via code, via visibility) should load data
   * and they use the same state...
   */
  readonly shouldLoadData: boolean

  /**
   * if we display both via code & via visibility we would get 2 dialogs...
   */
  readonly canDisplayEnterReleaseCodeDialog: boolean

  /**
   * true: can display, false not (probably because parent does)
   */
  readonly canDisplayTagFilterPanel: boolean
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    viewState: rootState.openExercisesState.viewState,
    openExercisePreviews: rootState.openExercisesState.openExercisePreviews,
    openExercisePreviewsViaCode: rootState.openExercisesState.openExercisePreviewsViaCode,
    isLoading: rootState.openExercisesState.isLoading
    || rootState.manageTagsState.isLoading,
    isEnterReleaseCodeDialogDisplayed: rootState.openExercisesState.isEnterReleaseCodeDialogDisplayed,
    shouldLoadData: props.shouldLoadData,
    sortByKey: rootState.openExercisesState.sortByKey,
    sortDirection: rootState.openExercisesState.sortDirection,
    canDisplayEnterReleaseCodeDialog: props.canDisplayEnterReleaseCodeDialog,

    selectedFilterTagsIds: rootState.tagsFilterState.selectedFilterTagsIds,
    selectedNegativeFilterTagsIds: rootState.tagsFilterState.selectedNegativeFilterTagsIds,

    canDisplayTagFilterPanel: props.canDisplayTagFilterPanel,
    isTagsFilterDisplayed: rootState.openExercisesState.isTagsFilterDisplayed,
    tags: rootState.manageTagsState.tags,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  loadOpenExercisePreviewsAsync,
  getOpenExercisePreviewsAsync,
  setSortByKey,

  setOpenExercises,
  setClosedExercises,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


// noinspection TsLint
let serverTime: Moment = null

// noinspection TsLint
let updateLeftTimeIntervals: number = -1

class OpenExercisesViaCodeSite extends React.Component<Props, any> {


  async componentWillMount(): Promise<void> {

    if (this.props.shouldLoadData) {
      await this.props.loadOpenExercisePreviewsAsync()
    }

    await this.setupTimeIsUpIntervals()
  }

  componentWillUnmount(): void {
    clearInterval(updateLeftTimeIntervals)
  }

  async setupTimeIsUpIntervals(): Promise<void> {

    // noinspection TsLint
    let serverTime: Moment = moment()

    try {
      serverTime = await getServerTime();
    } catch (err) {
      //TODO better error
      history.push(constants.homeLinkPath)
    }

    const clientServerTimeDiffInS = moment().diff(serverTime, 'seconds')

    updateLeftTimeIntervals = setInterval(() => {

      const copy = updateExercisePreviews(this.props.openExercisePreviews, clientServerTimeDiffInS)

      this.props.setOpenExercises(copy)

    }, 1000)
  }

  render(): JSX.Element {

    const previewsList = sortAndFilterExercisePreviews(this.props.openExercisePreviewsViaCode, this.props.sortDirection,
      this.props.sortByKey, this.props.selectedFilterTagsIds, this.props.selectedNegativeFilterTagsIds)

    return (

      <div className="multiple-single-panels">

        {
          (this.props.canDisplayTagFilterPanel && !this.props.isEnterReleaseCodeDialogDisplayed && this.props.isTagsFilterDisplayed) &&
          <TagFilterPanel
            isLoading={this.props.isLoading}
            allTags={this.props.tags}
          />
        }
        <SinglePanelSiteWrapper>

          {
            this.props.isLoading &&
            <Spinner/>
          }

          {
            !this.props.isEnterReleaseCodeDialogDisplayed &&
            <div className="view-padding">
              <div className="view-head-line">
                <div className="view-caption">
                  <h1>
                    {
                      translate(TranslateIds.openExercisesViaCode)
                    }
                  </h1>
                </div>
                <div className="view-options">
                  <MaterialSearchInput/>
                </div>
              </div>

              <div className="view-content">
                {
                  this.props.viewState === SiteViewState.List &&
                  <ListView exercisePreviews={previewsList} displayAsViaCode={true}
                            sortByKey={this.props.sortByKey}
                            sortDirection={this.props.sortDirection}
                            setSortByKey={this.props.setSortByKey}
                  />
                }
                {
                  this.props.viewState === SiteViewState.Cards &&
                  <CardsView exercisePreviews={previewsList} displayAsViaCode={true}/>
                }
              </div>
            </div>
          }

          {
            (!this.props.isLoading && this.props.isEnterReleaseCodeDialogDisplayed && this.props.canDisplayEnterReleaseCodeDialog) &&
            <EnterReleaseCodeDialog/>
          }


        </SinglePanelSiteWrapper>

      </div>

    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(OpenExercisesViaCodeSite)