/**
 * Created by janis dähne.
 */

import * as React from "react";
import {TooltipProps} from "recharts";
import {AssessmentDiagramDataPoint, getAssessmentChartDataId} from "./chartView";
import {SingleAssessmentStatistic} from "../../../../types/assessmentStatistics";
import {displayDateAndTime} from "../../../../helpers/convertersAndTransformers";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {Icon} from "semantic-ui-react";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly allRawData: ReadonlyArray<SingleAssessmentStatistic>
  readonly displaySubmitTestLine: boolean
  readonly displayNormalTestLine: boolean
}

interface PayloadObj {
  readonly payload: AssessmentDiagramDataPoint
  readonly color: string
}

export const AssessmentTooltip: React.SFC<any> = (props: MyProps & TooltipProps) => {

  const tooltipOriginalDataList = ((props.payload as ReadonlyArray<any>) as ReadonlyArray<PayloadObj>)
  // noinspection TsLint
  let tooltipTitle = ''
  // noinspection TsLint
  let submitTestData: SingleAssessmentStatistic = null
  // noinspection TsLint
  let submitTestPercentage = '0'
  // noinspection TsLint
  let normalTestData: SingleAssessmentStatistic = null
  // noinspection TsLint
  let normalTestPercentage = '0'

  // noinspection TsLint
  let submitPointsColor = 'black'
  // noinspection TsLint
  let normalPointsColor = 'black'

  // noinspection TsLint
  let exerciseDisplayName = ''

  // noinspection TsLint
  let lineIndex = 0

  // noinspection TsLint
  let shouldNotCount = false

  if (tooltipOriginalDataList.length > 0) {

    shouldNotCount = tooltipOriginalDataList[0].payload.shouldNotCount

    const firstRawData: SingleAssessmentStatistic = props.allRawData.find(
      p => getAssessmentChartDataId(p) === tooltipOriginalDataList[0].payload.id)

    tooltipTitle = firstRawData ? displayDateAndTime(firstRawData.lastUpdatedAt, -1) : props.label as any;

    //because we first draw normal lines the colors in tooltipOriginalDataList (for tooltipOriginalDataList[lineIndex])
    //are also in this order
    if (props.displayNormalTestLine) {
      normalTestData = props.allRawData.find(
        p => getAssessmentChartDataId(p) === tooltipOriginalDataList[lineIndex].payload.id)

      exerciseDisplayName = normalTestData.exerciseDisplayName

      // noinspection TsLint
      if (isNaN(tooltipOriginalDataList[lineIndex].payload.normalPointsPercentageValue)) {
        normalTestPercentage = ` (${translate(TranslateIds.notAssessedPoints)})`
      } else {

        // noinspection TsLint
        if (shouldNotCount) {
          normalTestPercentage = ` (${tooltipOriginalDataList[lineIndex].payload.originalNormalPointsPercentageValue}%)`
        } else {
          normalTestPercentage = ` (${tooltipOriginalDataList[lineIndex].payload.normalPointsPercentageValue}%)`
        }
      }

      normalPointsColor = tooltipOriginalDataList[lineIndex].color
      lineIndex++
    }

    if (props.displaySubmitTestLine) {
      //normalTestData & submitTestData are the same objects
      submitTestData = props.allRawData.find(
        p => getAssessmentChartDataId(p) === tooltipOriginalDataList[lineIndex].payload.id)

      exerciseDisplayName = submitTestData.exerciseDisplayName

      // noinspection TsLint
      if (isNaN(tooltipOriginalDataList[lineIndex].payload.submitPointsPercentageValue)) {
        submitTestPercentage = ` (${translate(TranslateIds.notAssessedPoints)})`
      } else {

        // noinspection TsLint
        if (shouldNotCount) {
          submitTestPercentage = ` (${tooltipOriginalDataList[lineIndex].payload.originalSubmitPointsPercentageValue}%)`
        } else {
          submitTestPercentage = ` (${tooltipOriginalDataList[lineIndex].payload.submitPointsPercentageValue}%)`
        }

      }

      submitPointsColor = tooltipOriginalDataList[lineIndex].color
    }


  }

  return (
    <div className="assessment-tooltip">
      <p className="recharts-tooltip-label">
        {
          tooltipTitle
        }
      </p>
      <ul className="recharts-tooltip-item-list assessment-tooltip-ul">

        {
          exerciseDisplayName &&
          <div className="assessment-tooltip-exercise-name">
            {
              exerciseDisplayName
            }
          </div>
        }
        {
          shouldNotCount &&
          <li className="recharts-tooltip-item assessment-tooltip-inline-item"
          >
            <div style={{fontWeight: 'bold'}}>
              <Icon className="warning-colored" name="exclamation triangle"/>
              <span>
              {
                translate(TranslateIds.shouldNotCount)
              }
            </span>
            </div>
          </li>
        }


        {
          props.displaySubmitTestLine &&
          <li className="recharts-tooltip-item assessment-tooltip-inline-item"
              style={{color: submitPointsColor}}
          >
              <span className="recharts-tooltip-item-name">
                {
                  translate(TranslateIds.submitTestPoints)
                }
              </span>
            <span className="recharts-tooltip-item-separator assessment-tooltip-inline-item-separator">
            :
          </span>
            <span className="recharts-tooltip-item-value">
            {
              //dummy assessments only have submitTestData.submitTestPoints == null
              submitTestData &&
              submitTestData.submitTestPoints + '/' +
              (submitTestData.submitTestPoints === null
                ? 'null'
                : submitTestData.maxSubmitTestPoints) +
              submitTestPercentage
            }
          </span>
          </li>
        }

        {
          props.displayNormalTestLine &&
          <li className="recharts-tooltip-item assessment-tooltip-inline-item"
              style={{color: normalPointsColor}}
          >
              <span className="recharts-tooltip-item-name">
                {
                  translate(TranslateIds.normalTests)
                }
              </span>
            <span className="recharts-tooltip-item-separator assessment-tooltip-inline-item-separator">
            :
          </span>
            <span className="recharts-tooltip-item-value">
            {
              //dummy assessments only have submitTestData.normalTestPoints == null
              normalTestData &&
              normalTestData.normalTestPoints + '/' +
              (normalTestData.normalTestPoints === null
                ? 'null'
                : normalTestData.maxNormalTestPoints) +
              normalTestPercentage

            }
          </span>
          </li>
        }

      </ul>
    </div>
  )
}
