/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {isCurrentSortingColumn} from "../../../helpers/tablesHelper";
import {Table} from "semantic-ui-react";
import {SingleAssessmentStatistic} from "../../../types/assessmentStatistics";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {HelpPopup} from "../../helpers/helpPopup";
import {SortDirection} from "../../../types/sortDirection";
import {setSortByKey} from "../../../state/actions/assessmentStatisticsSite/assessmentStatisticsSiteActions";
import ListViewItem from './listViewItem'
import Spinner from "../../helpers/Spinner";
import MaterialSearchInput from '../../material/materialSearchInput'
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import orderBy from 'lodash-es/orderBy'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    assessmentStatistics: rootState.assessmentStatisticsSiteState.assessmentStatistics,
    assessmentStatisticsList: rootState.assessmentStatisticsSiteState.assessmentStatistics ?
      rootState.assessmentStatisticsSiteState.assessmentStatistics.assessmentStatistics : [],

    sortByKey: rootState.assessmentStatisticsSiteState.sortByKey,
    sortDirection: rootState.assessmentStatisticsSiteState.sortDirection,

    isLoading: rootState.assessmentStatisticsSiteState.isLoading
    || rootState.pLangsState.isLoading,

    currentUserId: rootState.userDataSettingsSate.userData
      ? rootState.userDataSettingsSate.userData.id
      : -1,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setSortByKey
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class listView extends React.Component<Props, any> {
  render(): JSX.Element {

    const list: ReadonlyArray<SingleAssessmentStatistic> =
      this.props.sortDirection === SortDirection.none
        ? this.props.assessmentStatisticsList
        : orderBy(this.props.assessmentStatisticsList,
        assessmentStatistic => assessmentStatistic[this.props.sortByKey],
        this.props.sortDirection === SortDirection.ascending ? 'asc' : 'desc')

    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        <div className="view-padding">
          <div className="view-head-line">
            <div className="view-caption">
              <h1>
                <span className="mar-right-half">
                  {
                    translate(TranslateIds.assessmentStatisticsForUser)
                  }
                </span>
                <span>
                  ({list.length})
                </span>
              </h1>
            </div>
            <div className="view-options">
              <MaterialSearchInput/>
            </div>
          </div>

          <div className="view-content">

            <Table sortable padded striped singleLine className="material-table">
              <Table.Header>
                <Table.Row>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumn<SingleAssessmentStatistic>(this.props.sortByKey, 'exerciseDisplayName',
                        this.props.sortDirection)
                    }
                  >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('exerciseDisplayName', this.props.sortDirection + 1)
                  }}
                >
                  {
                    translate(TranslateIds.exercise)
                  }
                </span>
                  </Table.HeaderCell>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumn<SingleAssessmentStatistic>(this.props.sortByKey,
                        'exerciseShortDescription',
                        this.props.sortDirection)
                    }
                  >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('exerciseShortDescription', this.props.sortDirection + 1)
                  }}
                >
                  {
                    translate(TranslateIds.exerciseDescription)
                  }
                </span>
                  </Table.HeaderCell>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumn<SingleAssessmentStatistic>(this.props.sortByKey, 'pLangId',
                        this.props.sortDirection)
                    }
                  >
                    <span
                      className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('pLangId', this.props.sortDirection + 1)
                      }}
                    >
                      <HelpPopup icon="puzzle" defaultText={translate(TranslateIds.pLang)}/>
                    </span>
                  </Table.HeaderCell>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumn<SingleAssessmentStatistic>(this.props.sortByKey, 'exerciseReleaseIsReleased',
                        this.props.sortDirection)
                    }
                  >
                    <span
                      className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('exerciseReleaseIsReleased', this.props.sortDirection + 1)
                      }}
                    >
                     {
                       translate(TranslateIds.isOpen)
                     }
                    </span>
                  </Table.HeaderCell>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumn<SingleAssessmentStatistic>(this.props.sortByKey, 'shouldNotCount',
                        this.props.sortDirection)
                    }
                  >
                      <span
                        className="clickable"
                        onClick={() => {
                          this.props.setSortByKey('shouldNotCount', this.props.sortDirection + 1)
                        }}
                      >
                        <HelpPopup icon="flag outline"
                                   defaultText={translate(TranslateIds.shouldNotCount)}/>
                      </span>
                  </Table.HeaderCell>
                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumn<SingleAssessmentStatistic>(this.props.sortByKey, 'participationCount',
                        this.props.sortDirection)
                    }
                  >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('participationCount', this.props.sortDirection + 1)
                  }}
                >
                  <HelpPopup icon="repeat" wide
                             defaultText={translate(TranslateIds.participationCount_helpText)}/>
                </span>
                  </Table.HeaderCell>
                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumn<SingleAssessmentStatistic>(this.props.sortByKey, 'normalTestPoints',
                        this.props.sortDirection)
                    }
                  >

                <span
                  className="clickable mar-right-half"
                  onClick={() => {
                    this.props.setSortByKey('normalTestPoints', this.props.sortDirection + 1)
                  }}
                >
                  <HelpPopup wide icon="flag"
                             defaultText={translate(TranslateIds.normalTestPoints)}/>
                </span>
                  </Table.HeaderCell>
                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumn<SingleAssessmentStatistic>(this.props.sortByKey, 'submitTestPoints',
                        this.props.sortDirection)
                    }
                  >
                <span className="clickable mar-right-half"
                      onClick={() => {
                        this.props.setSortByKey('submitTestPoints', this.props.sortDirection + 1)
                      }}
                >
                  <HelpPopup wide icon="flag checkered"
                             defaultText={translate(TranslateIds.submitTestPoints)}/>
                </span>
                  </Table.HeaderCell>
                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumn<SingleAssessmentStatistic>(this.props.sortByKey, 'manualPoints',
                        this.props.sortDirection)
                    }
                  >
                <span className="clickable mar-right-half"
                      onClick={() => {
                        this.props.setSortByKey('manualPoints', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.manual)
                  }
                </span>
                    <HelpPopup wide
                               defaultText={translate(TranslateIds.manualPoints_helpText)}/>
                  </Table.HeaderCell>

                  <Table.HeaderCell>
                    <HelpPopup icon="talk" defaultText={translate(TranslateIds.feedbackFormTutor_helpText)}/>
                  </Table.HeaderCell>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumn<SingleAssessmentStatistic>(this.props.sortByKey, 'lastUpdatedAt',
                        this.props.sortDirection)
                    }
                  >
                    <span className="clickable mar-right-half"
                          onClick={() => {
                            this.props.setSortByKey('lastUpdatedAt', this.props.sortDirection + 1)
                          }}
                    >
                      {
                        translate(TranslateIds.lastUpdatedAt)
                      }
                    </span>
                    <HelpPopup defaultText={translate(TranslateIds.lastUpdatedAtSolutionVersion_helpText)}
                               wide
                    />
                  </Table.HeaderCell>

                  {
                    //a user cannot go to his/her submission but another can (if he/she sees it,
                    //he/she has the permission)
                    (this.props.assessmentStatistics && this.props.currentUserId !== this.props.assessmentStatistics.userId) &&
                    <Table.HeaderCell>
                        <span className="mar-right-half">
                          {
                            translate(TranslateIds.openTutorView)
                          }
                        </span>
                      <HelpPopup
                        wide
                        defaultText={translate(TranslateIds.openTutorView_helpText)}/>
                    </Table.HeaderCell>
                  }

                  {
                    //if the target user is the current show a link to edit the exercise
                    //else he/she need so search through open/closed exercises for the link...
                    (this.props.assessmentStatistics && this.props.currentUserId === this.props.assessmentStatistics.userId) &&
                    <Table.HeaderCell>
                        <span className="mar-right-half">
                          {
                            translate(TranslateIds.open)
                          }
                        </span>
                    </Table.HeaderCell>
                  }

                </Table.Row>
              </Table.Header>

              <Table.Body>
                {
                  list.map((singleAssessmentStatistic: SingleAssessmentStatistic, index) => {
                    return (
                      <ListViewItem
                        key={`${singleAssessmentStatistic.exerciseReleaseId}-${singleAssessmentStatistic.pLangId}`}
                        singleAssessmentStatistic={singleAssessmentStatistic}/>
                    )
                  })
                }
              </Table.Body>
            </Table>

          </div>
        </div>

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(listView)