/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Icon} from "semantic-ui-react";
import {HelpPopup} from "../../../helpers/helpPopup";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {
  resetAfterEditor,
  resetDoExerciseSolutionToTemplate, setAfterSolutionFiles,
  setIsDoCodeEditorSettingsDialogDisplayed
} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {openEditEditorSettingsDialog} from "../../../../state/actions/editorSettingsDialog/editorSettingsDialogActions";
import {askDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    codeEditorSettings: rootState.userDataSettingsSate.userData.settings ?
      rootState.userDataSettingsSate.userData.settings.codeEditorSettings
      : null,

    pLang: rootState.doExerciseSiteState.solution.pLangId,

    solution: rootState.doExerciseSiteState.solution,

  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setIsDoCodeEditorSettingsDialogDisplayed,
  openEditEditorSettingsDialog,
  setAfterSolutionFiles,
  resetAfterEditor
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelHeaderBar extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed">

        <div className="clickable"
             onClick={() => {
               this.props.openEditEditorSettingsDialog(this.props.codeEditorSettings,
                 this.props.setIsDoCodeEditorSettingsDialogDisplayed)
             }}
        >
          <Icon name="settings"/>
          <span>
            {
              translate(TranslateIds.editorSettings)
            }
          </span>
        </div>

        <SimpleVDivider/>

        <div className="mar-right-half clickable"
             onClick={async () => {


               const shouldReset = await askDialog(translate(TranslateIds.resetCode_title),
                 translate(TranslateIds.resetCodeToSolution_User_content))

               if (shouldReset) {
                 this.props.resetAfterEditor(this.props.solution)
               }

             }}
        >
          <Icon name="undo"/>
          <span>
            {
              translate(TranslateIds.resetCode)
            }
          </span>
        </div>
        <HelpPopup
          defaultText={translate(TranslateIds.resetSubsequentEditor_helpText)}
          wide
        />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelHeaderBar)