/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import MultiTabCodeEditorWrapper from '../../../codeEditors/multiTabCodeEditorWrapper'
import {
  closeTabChangeAfterEditor,
  openTabChangeAfterEditor,
  setAfterEditor_hasFileChangedMap,
  setAfterEditor_hasUnsavedChanges, setAfterEditor_isFileExplorerDisplayedDoEditor,
  setAfterEditor_isRenamingFileDoEditor, setAfterEditor_newFileNameDoEditor,
  setAfterEditor_renamingFileIdDoEditor, setAfterEditor_renamingFileNameDoEditor,
  setAfterEditor_selectedTabIdDoEditor
} from "../../../../state/actions/doExerciseSite/editorActions/afterEditorActions";
import {SolutionFileDoExerciseFullBase} from "../../../../types/exerciseSolution";
import Logger from "../../../../helpers/logger";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {setAfterSolutionFiles} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {EditorTab} from "../../../codeEditors/multiTabCodeEditorProps";
import {setSolutionFiles_AndRefreshEditor} from "../../../../helpers/editorHelper";
import {setForceEditorUpdateSessionDummyAfterEditor} from "../../../../state/actions/editorHelperState/editorHelperStateActions";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    doExercise: rootState.doExerciseSiteState.exercise,
    afterSolution: rootState.doExerciseSiteState.afterSolution,

    pLangs: rootState.pLangsState.pLangs,

    selectedTabIdDoEditor: rootState.doExerciseSiteState.afterEditorState.selectedTabIdDoEditor,
    isFileExplorerDisplayedDoEditor: rootState.doExerciseSiteState.afterEditorState.isFileExplorerDisplayedDoEditor,
    newFileNameDoEditor: rootState.doExerciseSiteState.afterEditorState.newFileNameDoEditor,
    renamingFileIdDoEditor: rootState.doExerciseSiteState.afterEditorState.renamingFileIdDoEditor,
    isRenamingFileDoEditor: rootState.doExerciseSiteState.afterEditorState.isRenamingFileDoEditor,
    renamingFileNameDoEditor: rootState.doExerciseSiteState.afterEditorState.renamingFileNameDoEditor,
    hasUnsavedChanges: rootState.doExerciseSiteState.afterEditorState.hasUnsavedChanges,
    hasFileChangedMap: rootState.doExerciseSiteState.afterEditorState.hasFileChangedMap,

    forceEditorUpdateSessionDummy: rootState.editorHelperState.forceEditorUpdateSessionDummyAfterEditor,

    //release should be not active and assessment finished then the after tests can be run
    releaseFinishedAndAutomaticAssessmentHasFinished: !rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased &&
    rootState.doExerciseSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setAfterEditor_hasUnsavedChanges,
  setAfterEditor_hasFileChangedMap,
  openTabChangeAfterEditor,
  closeTabChangeAfterEditor,
  setAfterEditor_selectedTabIdDoEditor,
  setAfterEditor_isFileExplorerDisplayedDoEditor,
  setAfterEditor_newFileNameDoEditor,
  setAfterEditor_renamingFileIdDoEditor,
  setAfterEditor_isRenamingFileDoEditor,
  setAfterEditor_renamingFileNameDoEditor,
  setAfterSolutionFiles,

  setSolutionFiles_AndRefreshEditor,
  setForceEditorUpdateSessionDummyAfterEditor,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class afterEditorPanelView extends React.Component<Props, any> {
  render(): JSX.Element {

    const pLang = this.props.pLangs.find(p => p.id === this.props.afterSolution.pLangId)

    if (!pLang) {
      //TODO only error if really not found
      //on first render we have no plang but thats no and error!!
      return null
    }

    const editorTabs = this.props.afterSolution.solutionFiles.map<EditorTab>(p => {
      return {
        id: p.id,
        fileNameWithExtensions: p.fileNameWithExtension,
        content: p.content,
        isMainFile: p.id === this.props.afterSolution.mainFileId,
        isReadonly: !p.isEditableByUser,
        displayIndex: p.displayIndex,
        language: pLang.internalName,
        isDisplayed: p.uiIsDisplayed,
        editSession: p.uiEditSession
      }
    })

    return (
      <div className="fh hidden-overflow-i">
        <MultiTabCodeEditorWrapper
          id="afterEditorPanelView"
          forceEditorUpdateSessionDummy={this.props.forceEditorUpdateSessionDummy}
          height="100%"
          selectedTabId={this.props.selectedTabIdDoEditor}
          onSelectedTabIdChanged={(tabId: number) => {
            this.props.setAfterEditor_selectedTabIdDoEditor(tabId)
          }}

          isFileExplorerDisplayed={this.props.isFileExplorerDisplayedDoEditor}
          onChangeIsFileExplorerDisplayed={(isDisplayed: boolean) => {
            this.props.setAfterEditor_isFileExplorerDisplayedDoEditor(isDisplayed)
          }}

          newFileName={this.props.newFileNameDoEditor}
          onNewFileNameChanged={(name: string) => {
            this.props.setAfterEditor_newFileNameDoEditor(name)
          }}

          renamingFileId={this.props.renamingFileIdDoEditor}
          onRenamingFileIdChanged={(fileId: number) => {
            this.props.setAfterEditor_renamingFileIdDoEditor(fileId)
          }}

          isRenamingFile={this.props.isRenamingFileDoEditor}
          onIsRenamingFileChanged={(isRenamingFileDoEditor: boolean) => {
            this.props.setAfterEditor_isRenamingFileDoEditor(isRenamingFileDoEditor)
          }}

          renamingFileName={this.props.renamingFileNameDoEditor}
          onRenamingFileNameChanged={(newName: string) => {
            this.props.setAfterEditor_renamingFileNameDoEditor(newName)
          }}

          hasUnsavedChanges={this.props.hasUnsavedChanges}
          onHasUnsavedChangesChanged={(hasChanged: boolean) => {
            if (hasChanged === this.props.hasUnsavedChanges) return
            this.props.setAfterEditor_hasUnsavedChanges(true)
          }}

          userCanCreateNewFiles={this.props.doExercise.canUserCreateFiles}

          language={pLang.internalName}

          onCreateFileClicked={(editorTab: EditorTab) => {

            const newFile: SolutionFileDoExerciseFullBase = {
              id: editorTab.id,
              content: editorTab.content,
              fileNameWithExtension: editorTab.fileNameWithExtensions,
              uiIsDisplayed: editorTab.displayIndex >= 0,
              displayIndex: editorTab.displayIndex,
              isEditableByUser: true,
              templateFileId: null,
              uiEditSession: editorTab.editSession
            }

            // this.props.setAfterSolutionFiles(
            //   [...this.props.afterSolution.solutionFiles, newFile]
            // )

            this.props.setSolutionFiles_AndRefreshEditor(
              [...this.props.afterSolution.solutionFiles, newFile],
              this.props.setAfterSolutionFiles,
              this.props.setForceEditorUpdateSessionDummyAfterEditor
            )

          }}

          onOpenTabClicked={(editorTab: EditorTab) => {
            const file = this.props.afterSolution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (open tab clicked)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            const mappedFiles = this.props.afterSolution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                ? {
                  ...file,
                  uiIsDisplayed: true,
                }
                : p
            })

            this.props.openTabChangeAfterEditor(file, mappedFiles, this.props.selectedTabIdDoEditor)
          }}

          onCloseTabClicked={(editorTab: EditorTab) => {

            const file = this.props.afterSolution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (closed tab clicked)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            const mappedFiles = this.props.afterSolution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                ? {
                  ...file,
                  uiIsDisplayed: false,
                }
                : p
            })

            this.props.closeTabChangeAfterEditor(file, mappedFiles,
              this.props.selectedTabIdDoEditor)
          }}

          onFileRename={(editorTab: EditorTab, newName: string) => {

            const currentFile = this.props.afterSolution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {
              Logger.error("file not found (onFileRename)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            this.props.setAfterSolutionFiles(
              this.props.afterSolution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                return file.id === currentFile.id
                  ? {
                    id: file.id,
                    displayIndex: file.displayIndex,
                    fileNameWithExtension: newName,
                    isEditableByUser: file.isEditableByUser,
                    content: file.content,
                    templateFileId: file.templateFileId,
                    uiIsDisplayed: file.uiIsDisplayed,
                    uiEditSession: editorTab.editSession
                  }
                  : file
              })
            )
          }}

          onRemoveFile={(editorTab: EditorTab) => {

            // this.props.setAfterSolutionFiles(
            //   this.props.afterSolution.solutionFiles.filter(p => p.id !== editorTab.id)
            // )

            this.props.setSolutionFiles_AndRefreshEditor(
              this.props.afterSolution.solutionFiles.filter(p => p.id !== editorTab.id),
              this.props.setAfterSolutionFiles,
              this.props.setForceEditorUpdateSessionDummyAfterEditor
            )

          }}

          editorTabs={editorTabs}
          onChange={(editorTab: EditorTab, newValue: string) => {

            const currentFile = this.props.afterSolution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {
              Logger.error("file not found (onChange)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            this.props.setAfterSolutionFiles(
              this.props.afterSolution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                return file.id === currentFile.id
                  ? {
                    id: file.id,
                    displayIndex: file.displayIndex,
                    fileNameWithExtension: file.fileNameWithExtension,
                    isEditableByUser: file.isEditableByUser,
                    content: newValue,
                    templateFileId: file.templateFileId,
                    uiIsDisplayed: file.uiIsDisplayed,
                    uiEditSession: editorTab.editSession
                  }
                  : file
              })
            )
          }}

          onSomeEditorSessionChanged={(editorTabs: ReadonlyArray<EditorTab>) => {
            this.props.setAfterSolutionFiles(
              this.props.afterSolution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                const changedTab = editorTabs.find(p => p.id === file.id)
                return changedTab
                  ? {
                    ...file,
                    uiEditSession: changedTab.editSession
                  }
                  : file
              })
            )
          }}

          hasFileChangedMap={this.props.hasFileChangedMap}
          onUpdateHasFileChangedMap={(changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>) => {
            //performance is tooooo bad...
            //this.props.setHasFileChangedMapDoEditor(updateHasFileChangedMap(changedFile, allTabs, this.props.hasFileChangedMap))
          }}

        />
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(afterEditorPanelView)