/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Icon, Checkbox} from "semantic-ui-react";
import {TestTypeFullBase} from "../../../../types/testTypes";
import {RunnableCustomTestFrontendExclusive} from "../../../../types/customTests";
import {RunnableExerciseTestFrontendExclusive} from "../../../../types/exerciseEditor";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {AssessedSubmitTestFromBackend, TestResultCodes} from "../../../../types/testResults";
import {CheckboxOnChangeEventData} from "../../../../types/reactEvents";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {HelpPopup} from "../../../helpers/helpPopup";
import {OkCircularIcon} from "../../../helpers/okCircularIcon";
import {ErrorCircularIcon} from "../../../helpers/errorCircularIcon";


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFromBackend | null
  readonly testTypes: ReadonlyArray<TestTypeFullBase>

  readonly useDiffIfOutputMismatched: boolean
  readonly setUseDiffIfOutputMismatched: (useDiff: boolean) => void

  readonly useReplaceWhiteSpacesWithBullets: boolean
  readonly setUseReplaceWhiteSpacesWithBullets: (useReplace: boolean) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    test: props.test,
    testTypes: props.testTypes,
    useDiffIfOutputMismatched: props.useDiffIfOutputMismatched,
    setUseDiffIfOutputMismatched: props.setUseDiffIfOutputMismatched,
    useReplaceWhiteSpacesWithBullets: props.useReplaceWhiteSpacesWithBullets,
    setUseReplaceWhiteSpacesWithBullets: props.setUseReplaceWhiteSpacesWithBullets,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class consolePanelHeaderBar extends React.Component<Props, any> {
  render(): JSX.Element {

    return (
      <div className="flexed">
        {
          (this.props.test && this.props.test.result) &&
          <div className="flexed-no-wrap">

            <div className="flexed-no-wrap not-text-wrapping">
              <div className="mar-right-half v-centered">
                {translate(TranslateIds.state)}:
              </div>

              <div>
                {
                  //this is somehow a hack because this is from the original (without react) semantic ui
                  this.props.test.result.passed &&
                  <OkCircularIcon/>
                }
                {
                  !this.props.test.result.passed &&
                  <ErrorCircularIcon/>
                }
              </div>

              <div className="v-centered">
                {
                  this.props.test.result.testResultCode === TestResultCodes.CompileError &&
                  <div>
                    {
                      translate(TranslateIds.compileError)
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.OutputMismatch &&
                  <div>
                    {
                      translate(TranslateIds.outputMismatched)
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.UnknownTestMethod &&
                  <div>
                    {
                      translate(TranslateIds.unknownTestType)
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.TimeoutHit &&
                  <div>
                    {
                      translate(TranslateIds.timeout)
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.DiskLimitHit &&
                  <div>
                    {
                      translate(TranslateIds.diskSpaceLimit)
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.MemoryLimitHit &&
                  <div>
                    {
                      translate(TranslateIds.ramLimit)
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.SomeServerError &&
                  <div>
                    {
                      translate(TranslateIds.unknownTestServerError)
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.UserProgramError &&
                  <div>
                    {
                      translate(TranslateIds.userProgramError)
                    }
                  </div>
                }
              </div>

              <SimpleVDivider/>

              <div className="v-centered not-text-wrapping">
                <div className="flexed-no-wrap not-text-wrapping">
                  <Checkbox className="mar-right-half"
                            label={translate(TranslateIds.useDiffIfOutputMismatched)}
                            checked={this.props.useDiffIfOutputMismatched}
                            onChange={(e, data: CheckboxOnChangeEventData) => {
                              this.props.setUseDiffIfOutputMismatched(data.checked)
                            }}
                  />
                  <HelpPopup defaultText={translate(TranslateIds.useDiffIfOutputMismatched_helpText)}
                             wide
                  />
                </div>
              </div>

              <SimpleVDivider/>

              <div className="v-centered not-text-wrapping">
                <div className="flexed-no-wrap not-text-wrapping">
                  <Checkbox className="mar-right-half"
                            label={translate(TranslateIds.useReplaceWhitespacesWithBullets)}
                            checked={this.props.useReplaceWhiteSpacesWithBullets}
                            onChange={(e, data: CheckboxOnChangeEventData) => {
                              this.props.setUseReplaceWhiteSpacesWithBullets(data.checked)
                            }}
                  />
                  <HelpPopup defaultText={translate(TranslateIds.useReplaceWhitespacesWithBullets_helpText)}/>
                </div>
              </div>

            </div>

            <SimpleVDivider/>

            <div className="v-centered flexed-no-wrap too-much-text-hidden">
              <div className="mar-right-half">
                {translate(TranslateIds.test)}:
              </div>
              <div>
                {this.props.test.displayName}
              </div>
            </div>
          </div>
        }
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(consolePanelHeaderBar)