/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../../state/reducers/index";
import FullSiteDialog from '../../../../../fullSiteDialog'
import ChangeCustomTestView from './changeCustomTestView'
import {
  cancelCustomTestDialog,
  saveCustomTestDialog
} from "../../../../../../state/actions/doExerciseSite/dialogs/changeCustomTestDialog/changeCustomTestActions";
import {Link} from "react-router-dom";
import {Icon} from "semantic-ui-react";
import * as constants from '../../../../../../constants'
import {translate} from "../../../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../../../i18n/ids";
import Logger from "../../../../../../helpers/logger";
import {errorDialog} from "../../../../../../helpers/dialogHelper";
import {notifySuccess} from "../../../../../../helpers/notificationHelper";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isBusy: rootState.changeCustomTestDialogState.isBusy || rootState.testTypesState.isLoading,
    isCreating: rootState.changeCustomTestDialogState.isCreating,
    isValid: rootState.changeCustomTestDialogState.isValid,
    customTest: rootState.changeCustomTestDialogState.customTest,
    releaseCode: rootState.doExerciseSiteState.exercise.exerciseRelease.releaseCode,
    allRunnableCustomTests: rootState.doExerciseSiteState.exercise.customTests

  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  cancelCustomTestDialog,
  saveCustomTestDialog,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ChangeCustomTestDialog extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <FullSiteDialog
        isBusy={this.props.isBusy}
        isValid={this.props.isValid}
        caption={
          <div className="flexed">
            <h2 className="mar-right">
              {
                this.props.isCreating ? translate(TranslateIds.createCustomTest) : translate(TranslateIds.changeCustomTest)
              }
            </h2>

            <Link target="_blank" to={constants.markdownSiteTestSyntaxGuide}
                  style={{marginTop: '0.5em'}}
            >
              <Icon className="mar-right-half" name="external share"/>
              <span>
                {
                  translate(TranslateIds.openHelpInNewTab)
                }
              </span>
            </Link>

          </div>
        }
        onSave={async () => {

          try {
            await this.props.saveCustomTestDialog(this.props.isCreating, this.props.releaseCode, this.props.customTest,
              this.props.allRunnableCustomTests)
          } catch(err) {

            if (this.props.isCreating) {
              errorDialog(translate(TranslateIds.error), translate(TranslateIds.couldNotCreateCustomTest))
            } else {
              errorDialog(translate(TranslateIds.error), translate(TranslateIds.couldNotUpdateCustomTest))
            }
            Logger.error(`could not ${this.props.isCreating ? 'create' : 'update'} custom test, err: ${err.message}`)
            return
          }

          //notify because the user needs to know that he/she not need to save something e.g. solution or exercise
          //this was already done by the save button click
          if (this.props.isCreating) {
            notifySuccess(translate(TranslateIds.createdCustomTest))
          } else {
            notifySuccess(translate(TranslateIds.updatedCustomTest))
          }

        }}
        onCancel={() => {
          this.props.cancelCustomTestDialog()
        }}
      >
        <ChangeCustomTestView/>
      </FullSiteDialog>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeCustomTestDialog)