/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch, Unsubscribe} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Icon, Popup} from "semantic-ui-react";
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import CommonHeaderBarContentWrapper from '../../commonHeaderBarContentWrapper'
import {
  justRunProgramDoExerciseAsync,
  runCompileSingleFileDoExercise,
  saveDoExerciseSolution, setShouldNotCountAsync
} from "../../../state/actions/doExerciseSite/doExerciseCrudActions";
import {
  JustRunProgramCommandForBackend, RunCompileSingleFileCommandForBackend,
  SolutionFileForTestServer
} from "../../../types/testsAndTestProtocol";
import {HelpPopup} from "../../helpers/helpPopup";
import {ReleaseDurationType} from "../../../types/ReleaseDurationType";
import {
  setIsJustRunProgramArgsInputDisplayedDoEditor,
  setIsLeftTimeStringDisplayed,
  setJustRunProgramArgsDoEditor,
  setSelectedConsoleViewTest
} from "../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {addCloseWindowEvent, clearCloseWindowEvent} from "../../../helpers/eventsHelper";
import history from "../../../routerHistory";
import {Prompt} from "react-router";
import * as mousetrap from 'mousetrap'
import {notifyError, notifySuccess} from "../../../helpers/notificationHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {leftIconPopupOffset} from "../../../constants";
import {askDialog, errorDialog} from "../../../helpers/dialogHelper";
import Logger from "../../../helpers/logger";
import {SimpleDivider} from "../../helpers/simpleDivider";
import {ProgramArgsInputView} from "../../helpers/programArgsInputView";
import {justRunProgramArgsMaxLength} from "../../../state/reducers/doExerciseSite/doExerciseSiteReducer";
import {copyToClipboard} from "../../../helpers/clipboard";

const fileSaver = require('file-saver');

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    releaseCode: rootState.doExerciseSiteState.exercise.exerciseRelease.releaseCode,
    doExercise: rootState.doExerciseSiteState.exercise,
    solution: rootState.doExerciseSiteState.solution,
    selectedTabIdDoEditor: rootState.doExerciseSiteState.doEditorState.selectedTabIdDoEditor,
    testTypes: rootState.testTypesState.testTypes,

    exercisePreview: rootState.doExerciseSiteState.exercisePreviewFrontendTemp,
    isLeftTimeStringDisplayed: rootState.doExerciseSiteState.isLeftTimeStringDisplayed,
    editorHasUnsavedChanges: rootState.doExerciseSiteState.doEditorState.hasUnsavedChanges,

    isJustRunProgramArgsInputDisplayed: rootState.doExerciseSiteState.isJustRunProgramArgsInputDisplayed,
    justRunProgramArgs: rootState.doExerciseSiteState.justRunProgramArgs,


    isLoading: rootState.doExerciseSiteState.isLoading
    || rootState.testTypesState.isLoading
    || rootState.pLangsState.isLoading
    || rootState.systemSettingsState.isLoading,

    //can be simplified
    canUserEditExercise: (rootState.doExerciseSiteState.exercisePreviewFrontendTemp
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen)
    || (rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
      && rootState.doExerciseSiteState.exercise.exerciseRelease.hasLimitedWorkingTime === false
      && rootState.doExerciseSiteState.exercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely),

    isSiteHeaderBarDisplayed: rootState.siteHeaderBarState.isSiteHeaderBarDisplayedAndEnabled,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

  saveDoExerciseSolution,
  runCompileSingleFileDoExercise,
  setIsLeftTimeStringDisplayed,
  setSelectedConsoleViewTest,
  setShouldNotCountAsync,
  justRunProgramDoExerciseAsync,
  setIsJustRunProgramArgsInputDisplayedDoEditor,
  setJustRunProgramArgsDoEditor,

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


/**
 * if the left time is set and below or equals this the
 * icon will be different colored
 * @type {number}
 */
export const criticalLeftTimeInS = 60

class HeaderBar extends React.Component<Props, any> {

  componentDidMount(): void {
    addCloseWindowEvent()

    mousetrap.bindGlobal("ctrl+s", this.onSaveKeyPress)

  }

  componentWillUnmount(): void {
    clearCloseWindowEvent()

    mousetrap.unbind("ctrl+s")
  }

  //TODO maybe disable during some dialog is displayed...
  //TODO maybe disable header bar if some dialog is displayed?
  onSaveKeyPress = async (e: ExtendedKeyboardEvent | null) => {
    //else the browser would ask to save the site.. (on windows)

    if (!this.props.canUserEditExercise) {
      return
    }

    if (e) {
      e.preventDefault()
    }

    const isOk = await this.props.saveDoExerciseSolution(this.props.releaseCode, this.props.solution)

    if (isOk) {
      notifySuccess(translate(TranslateIds.savedFiles))
    } else {
      //error displayed in saveDoExerciseSolution
    }

  }

  render(): JSX.Element {

    const allFiles = this.props.solution.solutionFiles ? this.props.solution.solutionFiles : []

    //this is not the real main file (saved in the solution) but the CURRENT FILE
    const mainFile = allFiles.find(p => p.id === this.props.selectedTabIdDoEditor)

    const leftArea = (
      <div>

        <Prompt when={this.props.editorHasUnsavedChanges}
                message={translate(TranslateIds.youHaveUnsavedChangedYouWantToExit)}/>

        {
          //save all files
          !this.props.isLoading && this.props.canUserEditExercise &&
          <div className="flexed">
            <div className="clickable mar-right-half"
                 onClick={() => this.onSaveKeyPress(null)}
            >
              <Icon name="copy"/>
              <span>
            {
              translate(TranslateIds.saveAllFiles)
            }
          </span>
            </div>
            <HelpPopup defaultText={translate(TranslateIds.saveAllFiles_helpText)}/>
            <SimpleVDivider/>
          </div>
        }

        {
          //compile single file
          !this.props.isLoading && this.props.canUserEditExercise &&
          <div className="flexed">
            <div className={
              (allFiles.length > 0 && mainFile) ? 'clickable mar-right-half' : 'div-disabled mar-right-half'
            }
                 onClick={async () => {

                   if (allFiles.length === 0) {
                     return
                   }

                   if (!mainFile) {
                     //...maybe the exercise has no default files...
                     //or no file is selected...
                     return
                   }

                   const command: RunCompileSingleFileCommandForBackend = {
                     generatedCode: this.props.releaseCode,
                     plangId: this.props.solution.pLangId,
                     mainFileName: mainFile.fileNameWithExtension,
                     files: allFiles.map<SolutionFileForTestServer>(p => {
                       return {
                         fileName: p.fileNameWithExtension,
                         fileContent: p.content
                       }
                     })
                   }

                   if (this.props.editorHasUnsavedChanges) {
                     await this.onSaveKeyPress(null)
                   }

                   //else the old result could be displayed...
                   //actually new test should be displayed (because in runCompileSingleFileDoExercise we forge a temp test)
                   //but this don't happen somehow so clear
                   this.props.setSelectedConsoleViewTest(null)

                   this.props.runCompileSingleFileDoExercise(command, this.props.testTypes)

                 }}
            >
              <Icon name="bug"/>
              <span>
            {
              translate(TranslateIds.compileSingleFile)
            }
          </span>
            </div>
            <HelpPopup
              defaultText={translate(TranslateIds.compileSingleFile_helpText)}
              wide
            />
            <SimpleVDivider/>
          </div>
        }

        {
          //just run program
          //behind compile single file in case we want to make cmd args accessible inline
          !this.props.isLoading && this.props.canUserEditExercise &&
          <div className="flexed">
            <div className={
              (allFiles.length > 0) ? 'clickable mar-right-half' : 'div-disabled mar-right-half'
            }
                 onClick={async () => {

                   if (allFiles.length === 0) {
                     return
                   }

                   const command: JustRunProgramCommandForBackend = {
                     CmdArgs: this.props.justRunProgramArgs ? this.props.justRunProgramArgs : '',
                     releaseCode: this.props.releaseCode,
                     plangId: this.props.solution.pLangId,
                   }

                   if (this.props.editorHasUnsavedChanges) {
                     await this.onSaveKeyPress(null)
                   }

                   //clear old result
                   this.props.setSelectedConsoleViewTest(null)

                   this.props.justRunProgramDoExerciseAsync(command, this.props.testTypes)

                 }}
            >
              <Icon name="rocket"/>
              <span>
                {
                  translate(TranslateIds.justRunProgram)
                }
              </span>
            </div>

            {
              /*
              looks better but worse for testing ... (we want to change args fast...)
              <div style={{display: 'inline-block'}} className="mar-right-half">
              <Popup
                wide="very"
                trigger={
                  <Icon className={this.props.justRunProgramArgs ? 'clickable toggled-colored' : 'clickable'}
                        name="exclamation circle"/>
                }
                content={
                  <ProgramArgsInputView programArgs={this.props.justRunProgramArgs}
                                        setProgramArgs={this.props.setJustRunProgramArgsDoEditor}/>
                }
                offset={leftIconPopupOffset}
                on="click"
                open={this.props.isJustRunProgramArgsInputDisplayed}
                onOpen={() => this.props.setIsJustRunProgramArgsInputDisplayedDoEditor(true)}
                onClose={() => this.props.setIsJustRunProgramArgsInputDisplayedDoEditor(false)}
              />
            </div>
               */
            }

            <HelpPopup className="mar-right-half"
                       defaultText={translate(TranslateIds.justRunProgram_helpText) + justRunProgramArgsMaxLength}
                       wide
            />

            <div style={{display: 'inline-block'}} className="clickable mar-right-half" onClick={() => {
              this.props.setIsJustRunProgramArgsInputDisplayedDoEditor(!this.props.isJustRunProgramArgsInputDisplayed)
            }}>
              <Icon className={this.props.justRunProgramArgs ? 'toggled-colored' : ''}
                    name="font"/>
            </div>

            {
              this.props.isJustRunProgramArgsInputDisplayed &&
              <div>
                <ProgramArgsInputView programArgs={this.props.justRunProgramArgs}
                                      setProgramArgs={this.props.setJustRunProgramArgsDoEditor}/>
              </div>
            }


          </div>
        }


      </div>
    )

    const isTimeLimited = this.props.doExercise.exerciseRelease.hasLimitedWorkingTime
      || (this.props.doExercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Limited && !!this.props.doExercise.exerciseRelease.automaticEndAt)

    const isReleased = this.props.doExercise.exerciseRelease.isReleased

    const rightArea = (
      <div className="flexed">
        {
          //not yet checked
          this.props.isSiteHeaderBarDisplayed &&
          ((this.props.solution.shouldNotCount === false && this.props.exercisePreview && this.props.exercisePreview.isStillOpen)) &&
          <div className="flexed mar-left">

            <div className="clickable mar-right-half" onClick={async () => {

              const shouldNotCount = await askDialog(translate(TranslateIds.doExerciseShouldNotCount_title),
                translate(TranslateIds.doExerciseShouldNotCount_content))

              if (!shouldNotCount) return

              try {
                await this.props.setShouldNotCountAsync(this.props.releaseCode)
              } catch (err) {
                Logger.error('could not set should not count')

                errorDialog(translate(TranslateIds.serverError),
                  translate(TranslateIds.couldNotSetShouldNotCount_doExercise))
              }

            }}>
              <Icon className="mar-right-half" name="flag outline"/>
              <span>
                  {
                    translate(TranslateIds.doExerciseShouldNotCount)
                  }
                </span>
            </div>

            <HelpPopup defaultText={translate(TranslateIds.doExerciseShouldNotCount_helpText)}/>
          </div>
        }

        {
          this.props.isSiteHeaderBarDisplayed &&
          ((this.props.exercisePreview && isTimeLimited
            && this.props.exercisePreview.isStillOpen && this.props.solution.shouldNotCount === false)) &&
          <SimpleVDivider/>
        }
        {
          (this.props.exercisePreview && isTimeLimited) &&
          <div className="flexed">
            <div>
              {
                this.props.exercisePreview.isStillOpen &&
                (this.props.exercisePreview.leftTimeInS === null
                  || (this.props.exercisePreview.leftTimeInS !== null && this.props.exercisePreview.leftTimeInS > criticalLeftTimeInS)) &&
                <Popup
                  trigger={
                    <div className="clickable"
                         onClick={() => {
                           this.props.setIsLeftTimeStringDisplayed(!this.props.isLeftTimeStringDisplayed)
                         }}
                    >
                      <Icon name="wait"/>
                    </div>
                  }
                  content={`${translate(TranslateIds.doEditorRemainingTime_helpText)}${criticalLeftTimeInS}${translate(
                    TranslateIds.doEditorRemainingTime_helpText_part2)}`}
                  offset={leftIconPopupOffset}
                />
              }
              {
                this.props.exercisePreview.isStillOpen &&
                this.props.exercisePreview.leftTimeInS !== null &&
                this.props.exercisePreview.leftTimeInS <= criticalLeftTimeInS &&
                <Popup
                  trigger={
                    <div className="clickable"
                         onClick={() => {
                           this.props.setIsLeftTimeStringDisplayed(!this.props.isLeftTimeStringDisplayed)
                         }}
                    >
                      <Icon name="wait" className="orange-colored"/>
                    </div>
                  }
                  content={`${translate(TranslateIds.doEditorRemainingTime_helpText)}${criticalLeftTimeInS}${translate(
                    TranslateIds.doEditorRemainingTime_helpText_part2)}`}
                  offset={leftIconPopupOffset}
                />
              }
              {
                !this.props.exercisePreview.isStillOpen &&
                <Icon name="wait" className="error-colored"/>
              }
            </div>

            <div>
              {
                this.props.isLeftTimeStringDisplayed &&
                this.props.exercisePreview.leftTimeString
              }
            </div>
          </div>
        }

      </div>
    )
    const rightAreaClosedRelease = (
      <div>
        {
          this.props.exercisePreview &&
          <div className="flexed">
            <div>
              {
                translate(TranslateIds.releaseHasFinished)
              }
            </div>
          </div>
        }
      </div>
    )


    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea}
                                     rightArea={
                                       isReleased ?
                                         rightArea
                                         : rightAreaClosedRelease
                                     }/>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(HeaderBar)