/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {HelpPopup} from "../../../helpers/helpPopup";
import {AssessedSubmitTestFromBackend, AssessedSubmitTestFrontendExclusive} from "../../../../types/testResults";
import {percentageNeededToPassedAutomaticAssessment} from "../../../../constants";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly submitTestResults: ReadonlyArray<AssessedSubmitTestFrontendExclusive>
}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    submitTestResults: props.submitTestResults
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class submitTestsPanelHeaderBar extends React.Component<Props, any> {
  // noinspection TsLint
  render(): JSX.Element {

    const maxSummedPoints = this.props.submitTestResults
      .map(p => p.weight)
      .reduce(((previousValue, currentValue) => previousValue + currentValue), 0)

    const reachedSummedPoints = this.props.submitTestResults
      .map(p => p.result && p.result.passed ? p.weight : 0)
      .reduce(((previousValue, currentValue) => previousValue + currentValue), 0)

    // noinspection TsLint
    const reachedPercentage = maxSummedPoints > 0
      ? reachedSummedPoints * 100 / maxSummedPoints
      : 100

    // noinspection TsLint
    const hasPassedSummed = reachedPercentage >= percentageNeededToPassedAutomaticAssessment

    return (
      <div className="flexed">

        <div className={`flexed mar-right-half ${hasPassedSummed ? 'ok-colored' : 'error-colored'}`}>
          <div>
            {
              reachedSummedPoints
            }
          </div>
          /
          <div className="mar-right-half">
            {
              maxSummedPoints
            }
          </div>
          (
          <div>
            {
              reachedPercentage.toFixed(2)
            }
            %
          </div>
          )
        </div>

        <HelpPopup defaultText={translate(TranslateIds.reachedSubmitTestPoints_helpText)}
                   wide
        />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(submitTestsPanelHeaderBar)