/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../state/reducers/index";
import FullSiteDialog from '../../../../fullSiteDialog'
import ViewTestView from './viewTestView'
import {cancelViewTestDialog} from "../../../../../state/actions/doExerciseSite/dialogs/viewTestDialog/viewTestDialogActions";
import {Link} from "react-router-dom";
import * as constants from '../../../../../constants'
import {Icon} from "semantic-ui-react";
import {translate} from "../../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly cancelViewTestDialog: () => void

  /**
   * returns the download link for a given file
   */
  readonly getDownloadTestAssetLink: (testId: number, fileId: number, isSubmitTest: boolean) => string
}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    isBusy: rootState.viewTestDialogState.isBusy,

    runnableTest: rootState.viewTestDialogState.runnableTest,
    cancelViewTestDialog: props.cancelViewTestDialog,
    getDownloadTestAssetLink: props.getDownloadTestAssetLink
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class viewTestDialog extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <FullSiteDialog
        isBusy={this.props.isBusy}
        isValid={true}
        caption={
          <div className="flexed">
            <h2 className="mar-right">
              {
                translate(TranslateIds.testContent)
              }
            </h2>

            <Link target="_blank" to={constants.markdownSiteTestSyntaxGuide}
                  style={{marginTop: '0.5em'}}
            >
              <Icon className="mar-right-half" name="external share"/>
              <span>
                {
                  translate(TranslateIds.openHelpInNewTab)
                }
              </span>
            </Link>

          </div>
        }
        onSave={null}
        onCancel={() => {
          this.props.cancelViewTestDialog()
        }}
      >
        <ViewTestView getDownloadTestAssetLink={this.props.getDownloadTestAssetLink}/>
      </FullSiteDialog>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(viewTestDialog)