/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../state/reducers/index";
import {Form, Tab, List, Icon} from "semantic-ui-react";
import MaterialInput from '../../../../material/materialInput'
import SimpleCodeEditorWrapper from '../../../../codeEditors/simpleCodeEditorWrapper'
import ConsolePanelView from '../../consolePanel/consolePanelView'
import {TestProtocolItem} from "../../../../../types/testResults";
import {
  convertSizeToHumanReadableSize,
  convertTestProtocolFromString
} from "../../../../../helpers/convertersAndTransformers";
import {TestAssetFromBackendWithData} from "../../../../../types/exerciseEditor";
import {debugOriginUrl} from "../../../../../../debug";
import {translate} from "../../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../../i18n/ids";
import {aceTextLanguage} from "../../../../../helpers/editorHelper";
import {EditorTab} from "../../../../codeEditors/multiTabCodeEditorProps";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly getDownloadTestAssetLink: (testId: number, fileId: number, isSubmitTest: boolean) => string
}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    test: rootState.viewTestDialogState.runnableTest,
    testTypes: rootState.testTypesState.testTypes,
    getDownloadTestAssetLink: props.getDownloadTestAssetLink,
    useDiffIfOutputMismatched: false, //not important when viewing, creating a test
    //why not... TODO but maybe create an own reducer for frontend settings
    useReplaceWhiteSpacesWithBullets: true
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


//TODO maybe merge this with src/components/exerciseEditorSite/editorTestsPanel/dialogs/changeTestView.tsx
//and or src/components/doExerciseSite/customTestsDialog/changeCustomTestView.tsx
//they display basically the same data...
class viewTestView extends React.Component<Props, any> {

  render(): JSX.Element {

    const testType = this.props.testTypes.find(p => p.id === this.props.test.testTypeId)

    //we fake a test result to reuse the console view
    const visualTest: ReadonlyArray<TestProtocolItem> | null = convertTestProtocolFromString(this.props.test.content,
      testType)

    const tempEditorTab: EditorTab = {
      id: this.props.test.id,
      content: this.props.test.content,
      language: aceTextLanguage,
      editSession: null,
      isReadonly: true,
      fileNameWithExtensions: this.props.test.displayName,
      displayIndex: this.props.test.displayIndex,
      isMainFile: true,
    }

    return (
      <div className="fh">
        <div className="ui form divided-checkboxes fh" style={{height: 'auto' /*firefox fix*/}}>

          <Form.Group widths='equal'>
            <Form.Field>
              <label>
                {
                  translate(TranslateIds.testType)
                }
              </label>
              <MaterialInput
                value={testType ? testType.displayName : translate(TranslateIds.unknownTestType)}
                disabled
              />
            </Form.Field>

            <Form.Field>
              <label>
                {
                  translate(TranslateIds.points)
                }
              </label>
              <MaterialInput
                value={this.props.test.weight.toString()}
                disabled
              />
            </Form.Field>

            <Form.Field>
              <label>
                {
                  translate(TranslateIds.displayName)
                }
              </label>
              <MaterialInput
                value={this.props.test.displayName}
                disabled
              />
            </Form.Field>
          </Form.Group>

        </div>

        <Tab menu={{secondary: true, pointing: true}} panes={
          [
            {
              menuItem: translate(TranslateIds.testContentVisually),
              render: () => {
                return (
                  <div className="mar-bot">
                    <ConsolePanelView test={null} protocolItems={visualTest} testTypes={this.props.testTypes}
                                      useDiffIfOutputMismatched={this.props.useDiffIfOutputMismatched}
                                      useReplaceWhiteSpacesWithBullets={this.props.useReplaceWhiteSpacesWithBullets}
                    />
                  </div>
                )
              }
            },
            {
              menuItem: translate(TranslateIds.testContent),
              render: () => {
                return (
                  <div id="ChangeTestView"
                       style={{height: '300px'}} className="mar-bot">
                    {
                      //do not embed monaco into form else some text are will get the min, max text area size...
                    }
                    <SimpleCodeEditorWrapper
                      id="viewTestView"
                      editorTab={tempEditorTab}
                      resizeAnchorId="ChangeTestView"
                      onChange={null}
                      onEditorSessionChanged={null}
                    />
                  </div>
                )
              }
            },
            {
              menuItem: translate(TranslateIds.testSettingsAndFiles),
              render: () => {
                return (
                  <div className="ui form divided-checkboxes fh">
                    <Form.Group widths='equal'>
                      <Form.Field>
                        <label>
                          {
                            translate(TranslateIds.timeoutInMs)
                          }
                        </label>
                        <MaterialInput
                          type="number"
                          value={this.props.test.testSettings.timeoutInMs.toString(10)}
                          disabled
                        />
                      </Form.Field>

                      <Form.Field>
                        <label>
                          {
                            translate(TranslateIds.maxRamInKb)
                          }
                        </label>
                        <MaterialInput
                          type="number"
                          value={this.props.test.testSettings.memoryLimitInKb.toString(10)}
                          disabled
                        />
                      </Form.Field>

                      <Form.Field>
                        <label>
                          {
                            translate(TranslateIds.maxDiskSpaceInKb)
                          }
                        </label>
                        <MaterialInput
                          type="number"
                          value={this.props.test.testSettings.maxDiskSpaceInKb.toString(10)}
                          disabled
                        />
                      </Form.Field>
                    </Form.Group>

                    <h4 className="ui dividing header">
                      {
                        translate(TranslateIds.files)
                      }
                    </h4>

                    <div>

                      <List>
                        {
                          this.props.test.files.map((file: TestAssetFromBackendWithData) => {
                            return (
                              <List.Item key={file.id}>
                                <List.Icon size='large' verticalAlign='middle'>
                                  <a target="_blank" download
                                     href={this.props.getDownloadTestAssetLink(this.props.test.id, file.id, this.props.test.isSubmitTest)}>
                                    <Icon name='download'/>
                                  </a>
                                </List.Icon>
                                <List.Content>
                                  <List.Header>

                                    <div className="flexed">
                                      <div className="mar-right-half">
                                        {
                                          file.displayName
                                        }
                                      </div>

                                      <div className="not-important-colored">
                                        (
                                        {convertSizeToHumanReadableSize(file.size)}
                                        )
                                      </div>
                                    </div>

                                  </List.Header>
                                  <List.Description>
                                    <div className="not-important-colored">
                                      {
                                        file.mimeType
                                      }
                                    </div>
                                  </List.Description>
                                </List.Content>
                              </List.Item>
                            )
                          })
                        }
                      </List>

                    </div>


                  </div>
                )
              }
            },
          ]

        }/>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(viewTestView)