/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Icon, Popup} from "semantic-ui-react";
import {HelpPopup} from "../../../helpers/helpPopup";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {openEditEditorSettingsDialog} from "../../../../state/actions/editorSettingsDialog/editorSettingsDialogActions";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {displayDateAndTime} from "../../../../helpers/convertersAndTransformers";
import {leftIconPopupOffset} from "../../../../constants";
import {
  setCustomProjectCodeEditorSettingsDialogDisplayed,
  setMainFileIdCustomProject
} from "../../../../state/actions/editCustomProjectSite/editCustomProjectActions";
import {SyntheticEvent} from "react";
import MaterialSelectWrapper from '../../../material/materialSelectWrapper'
import {CustomProjectSolutionFileFullBase} from "../../../../types/customProjects";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    codeEditorSettings: rootState.userDataSettingsSate.userData.settings ?
      rootState.userDataSettingsSate.userData.settings.codeEditorSettings
      : null,

    solutionCreatedAt: rootState.editCustomProjectSiteState.customProject.solution.createdAt,
    solution: rootState.editCustomProjectSiteState.customProject.solution,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setCustomProjectCodeEditorSettingsDialogDisplayed,
  openEditEditorSettingsDialog,
  setMainFileIdCustomProject

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelHeaderBar extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed">

        <div className="clickable"
             onClick={() => {
               this.props.openEditEditorSettingsDialog(this.props.codeEditorSettings,
                 this.props.setCustomProjectCodeEditorSettingsDialogDisplayed)
             }}
        >
          <Icon name="settings"/>
          <span>
            {
              translate(TranslateIds.editorSettings)
            }
          </span>
        </div>

        {
          this.props.solution.solutionFiles.length > 0 &&
          <SimpleVDivider/>
        }

        {
          this.props.solution.solutionFiles.length > 0 &&
          <div className="flexed">
            <span className="mar-right-half">{translate(TranslateIds.mainFile)}:</span>
            <MaterialSelectWrapper
              value={this.props.solution.mainFileId}
            >
              <select value={this.props.solution.mainFileId}
                      onChange={(e: SyntheticEvent<HTMLSelectElement>) => {
                        const id = parseInt(e.currentTarget.value, 10)
                        this.props.setMainFileIdCustomProject(id)
                      }}
                      style={{margin: '0', minWidth: '100px'}}
              >
                {
                  this.props.solution.solutionFiles.map((file: CustomProjectSolutionFileFullBase) => {
                    return (
                      <option key={file.id} value={file.id}>{file.fileNameWithExtension}</option>
                    )
                  })
                }
              </select>
            </MaterialSelectWrapper>
          </div>
        }


        {
          this.props.solutionCreatedAt &&
          <SimpleVDivider/>
        }

        {
          this.props.solutionCreatedAt &&
          <Popup
            trigger={
              <div className="hoverable">

                <Icon className="mar-right-half" name="send"/>
                <span className="mar-right-half">
              {
                translate(TranslateIds.solutionServerVersionCreatedAt)
              }
              </span>
                <span>
              {
                displayDateAndTime(this.props.solutionCreatedAt, -1)
              }
              </span>
              </div>
            }
            content={translate(TranslateIds.solutionServerVersionCreatedAt_helpText)}
            offset={leftIconPopupOffset}
          />
        }


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelHeaderBar)