/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import MultiTabCodeEditorWrapper from '../../../codeEditors/multiTabCodeEditorWrapper'
import {
  closeTabChangeCustomProjectSolutionEditor,
  openTabChangeCustomProjectSolutionEditor, setHasFileChangedMapCustomProjectSolutionEditor, setHasUnsavedChanges,
  setIsFileExplorerDisplayedCustomProjectSolutionEditor, setIsRenamingFileCustomProjectSolutionEditor, setNewFileNameCustomProjectSolutionEditor,
  setRenamingFileIdCustomProjectSolutionEditor,
  setRenamingFileNameCustomProjectSolutionEditor,
  setSelectedTabIdCustomProjectSolutionEditor
} from "../../../../state/actions/editCustomProjectSite/editorActions/customProjectSolutionEditorActions";
import {SolutionFileDoExerciseFullBase} from "../../../../types/exerciseSolution";
import Logger from "../../../../helpers/logger";
import {
  setSolutionFiles_AndRefreshEditor,
} from "../../../../helpers/editorHelper";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {EditorTab, MultiTabEditorState} from "../../../codeEditors/multiTabCodeEditorProps";
import {
  setForceEditorUpdateSessionDummyCustomProjectSolutionEditorAction,
} from "../../../../state/actions/editorHelperState/editorHelperStateActions";
import {CustomProjectSolutionFileFullBase} from "../../../../types/customProjects";
import {
  onCreateCustomProjectSolutionFile,
  setSolutionFilesCustomProject
} from "../../../../state/actions/editCustomProjectSite/editCustomProjectActions";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    solution: rootState.editCustomProjectSiteState.customProject.solution,

    pLangs: rootState.pLangsState.pLangs,

    selectedTabIdCustomProjectSolutionEditor: rootState.editCustomProjectSiteState.solutionEditorState.selectedTabIdDoEditor,
    isFileExplorerDisplayedCustomProjectSolutionEditor: rootState.editCustomProjectSiteState.solutionEditorState.isFileExplorerDisplayedDoEditor,
    newFileNameCustomProjectSolutionEditor: rootState.editCustomProjectSiteState.solutionEditorState.newFileNameDoEditor,
    renamingFileIdCustomProjectSolutionEditor: rootState.editCustomProjectSiteState.solutionEditorState.renamingFileIdDoEditor,
    isRenamingFileCustomProjectSolutionEditor: rootState.editCustomProjectSiteState.solutionEditorState.isRenamingFileDoEditor,
    renamingFileNameCustomProjectSolutionEditor: rootState.editCustomProjectSiteState.solutionEditorState.renamingFileNameDoEditor,
    hasUnsavedChanges: rootState.editCustomProjectSiteState.solutionEditorState.hasUnsavedChanges,
    hasFileChangedMap: rootState.editCustomProjectSiteState.solutionEditorState.hasFileChangedMap,

    forceEditorUpdateSessionDummy: rootState.editorHelperState.forceEditorUpdateSessionDummyCustomProjectSolutionEditor,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setSelectedTabIdCustomProjectSolutionEditor,
  setIsFileExplorerDisplayedCustomProjectSolutionEditor,
  setNewFileNameCustomProjectSolutionEditor,
  setRenamingFileIdCustomProjectSolutionEditor,
  setIsRenamingFileCustomProjectSolutionEditor,
  setRenamingFileNameCustomProjectSolutionEditor,

  openTabChangeCustomProjectSolutionEditor,
  closeTabChangeCustomProjectSolutionEditor,
  setHasUnsavedChanges,

  setSolutionFilesCustomProject,
  setHasFileChangedMapCustomProjectSolutionEditor,

  setSolutionFiles_AndRefreshEditor,
  setForceEditorUpdateSessionDummyCustomProjectSolutionEditorAction,
  onCreateCustomProjectSolutionFile,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelView extends React.Component<Props, any> {

  render(): JSX.Element {

    const pLang = this.props.pLangs.find(p => p.id === this.props.solution.pLangId)

    if (!pLang) {
      //TODO only error if really not found
      //on first render we have no plang but thats no and error!!
      return null
    }

    const editorTabs = this.props.solution.solutionFiles.map<EditorTab>(p => {
      return {
        id: p.id,
        fileNameWithExtensions: p.fileNameWithExtension,
        content: p.content,
        isMainFile: p.id === this.props.solution.mainFileId,
        isReadonly: false,
        displayIndex: p.displayIndex,
        language: pLang.internalName,
        editSession: p.uiEditSession
      }
    })

    return (
      <div className="fh hidden-overflow-i">
        <MultiTabCodeEditorWrapper
          id="customProjectEditorPanelView"
          forceEditorUpdateSessionDummy={this.props.forceEditorUpdateSessionDummy}
          height="100%"
          selectedTabId={this.props.selectedTabIdCustomProjectSolutionEditor}
          onSelectedTabIdChanged={(tabId: number) => {
            this.props.setSelectedTabIdCustomProjectSolutionEditor(tabId)
          }}

          isFileExplorerDisplayed={this.props.isFileExplorerDisplayedCustomProjectSolutionEditor || editorTabs.length === 0}
          onChangeIsFileExplorerDisplayed={(isDisplayed: boolean) => {
            this.props.setIsFileExplorerDisplayedCustomProjectSolutionEditor(isDisplayed)
          }}

          newFileName={this.props.newFileNameCustomProjectSolutionEditor}
          onNewFileNameChanged={(name: string) => {
            this.props.setNewFileNameCustomProjectSolutionEditor(name)
          }}

          renamingFileId={this.props.renamingFileIdCustomProjectSolutionEditor}
          onRenamingFileIdChanged={(fileId: number) => {
            this.props.setRenamingFileIdCustomProjectSolutionEditor(fileId)
          }}

          isRenamingFile={this.props.isRenamingFileCustomProjectSolutionEditor}
          onIsRenamingFileChanged={(isRenamingFileCustomProjectSolutionEditor: boolean) => {
            this.props.setIsRenamingFileCustomProjectSolutionEditor(isRenamingFileCustomProjectSolutionEditor)
          }}

          renamingFileName={this.props.renamingFileNameCustomProjectSolutionEditor}
          onRenamingFileNameChanged={(newName: string) => {
            this.props.setRenamingFileNameCustomProjectSolutionEditor(newName)
          }}

          hasUnsavedChanges={this.props.hasUnsavedChanges}
          onHasUnsavedChangesChanged={(hasChanged: boolean) => {
            if (hasChanged === this.props.hasUnsavedChanges) return
            this.props.setHasUnsavedChanges(true)
          }}

          userCanCreateNewFiles={true}

          language={pLang.internalName}

          onCreateFileClicked={(editorTab: EditorTab) => {

            const newFile: SolutionFileDoExerciseFullBase = {
              id: editorTab.id,
              content: editorTab.content,
              fileNameWithExtension: editorTab.fileNameWithExtensions,
              uiIsDisplayed: editorTab.displayIndex >= 0,
              displayIndex: editorTab.displayIndex,
              isEditableByUser: true,
              templateFileId: null,
              uiEditSession: editorTab.editSession
            }

            //this creates a new file and opens it
            //if it's the first file then it gets the main file
            this.props.onCreateCustomProjectSolutionFile(newFile,
              this.props.solution.solutionFiles,
              this.props.selectedTabIdCustomProjectSolutionEditor
              )

            // this.props.setSolutionFiles_AndRefreshEditor(
            //   [...this.props.solution.solutionFiles, newFile],
            //   this.props.setSolutionFilesCustomProject,
            //   this.props.setForceEditorUpdateSessionDummyCustomProjectSolutionEditorAction
            // )

            //this.props.setSolutionFiles_CreateFile([...this.props.solution.solutionFiles, newFile])

          }}

          onOpenTabClicked={(editorTab: EditorTab) => {
            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (open tab clicked)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            //do we still need this???
            const mappedFiles = this.props.solution.solutionFiles.map<CustomProjectSolutionFileFullBase>(p => {
              return p.id === file.id
                ? {
                  ...file,
                  uiIsDisplayed: true,
                }
                : p
            })
            this.props.openTabChangeCustomProjectSolutionEditor(file, mappedFiles, this.props.selectedTabIdCustomProjectSolutionEditor)

            //this.props.openTabChangeCustomProjectSolutionEditor(file, this.props.solution.solutionFiles, this.props.selectedTabIdCustomProjectSolutionEditor)
          }}

          onCloseTabClicked={(editorTab: EditorTab) => {

            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (closed tab clicked)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            //do we still need this???
            const mappedFiles = this.props.solution.solutionFiles.map<CustomProjectSolutionFileFullBase>(p => {
              return p.id === file.id
                ? {
                  ...file,
                  uiIsDisplayed: false,
                }
                : p
            })

            this.props.closeTabChangeCustomProjectSolutionEditor(file, mappedFiles,
              this.props.selectedTabIdCustomProjectSolutionEditor)

            // this.props.closeTabChangeCustomProjectSolutionEditor(file, this.props.solution.solutionFiles,
            //   this.props.selectedTabIdCustomProjectSolutionEditor)
          }}

          onFileRename={(editorTab: EditorTab, newName: string) => {

            const currentFile = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {
              Logger.error("file not found (onFileRename)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            this.props.setSolutionFilesCustomProject(
              this.props.solution.solutionFiles.map<CustomProjectSolutionFileFullBase>(file => {
                return file.id === currentFile.id
                  ? {
                    id: file.id,
                    displayIndex: file.displayIndex,
                    fileNameWithExtension: newName,
                    isEditableByUser: file.isEditableByUser,
                    content: file.content,
                    uiIsDisplayed: file.uiIsDisplayed,
                    uiEditSession: editorTab.editSession
                  }
                  : file
              })
            )
          }}

          onRemoveFile={(editorTab: EditorTab) => {

            this.props.setSolutionFiles_AndRefreshEditor(
              this.props.solution.solutionFiles.filter(p => p.id !== editorTab.id),
              this.props.setSolutionFilesCustomProject,
              this.props.setForceEditorUpdateSessionDummyCustomProjectSolutionEditorAction
            )

            // this.props.setSolutionFiles(
            //   this.props.solution.solutionFiles.filter(p => p.id !== editorTab.id)
            // )

          }}

          editorTabs={editorTabs}
          onChange={(editorTab: EditorTab, newValue: string) => {

            const currentFile = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {
              Logger.error("file not found (onChange)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            this.props.setSolutionFilesCustomProject(
              this.props.solution.solutionFiles.map<CustomProjectSolutionFileFullBase>(file => {
                return file.id === currentFile.id
                  ? {
                    id: file.id,
                    displayIndex: file.displayIndex,
                    fileNameWithExtension: file.fileNameWithExtension,
                    isEditableByUser: file.isEditableByUser,
                    content: newValue,
                    uiIsDisplayed: file.uiIsDisplayed,
                    uiEditSession: editorTab.editSession
                  }
                  : file
              })
            )
          }}
          onSomeEditorSessionChanged={(editorTabs: ReadonlyArray<EditorTab>) => {
            this.props.setSolutionFilesCustomProject(
              this.props.solution.solutionFiles.map<CustomProjectSolutionFileFullBase>(file => {
                const changedTab = editorTabs.find(p => p.id === file.id)
                return changedTab
                  ? {
                    ...file,
                    uiEditSession: changedTab.editSession
                  }
                  : file
              })
            )
          }}

          hasFileChangedMap={this.props.hasFileChangedMap}
          onUpdateHasFileChangedMap={(changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>) => {
            //performance is tooooo bad...
            //this.props.setHasFileChangedMapCustomProjectSolutionEditor(updateHasFileChangedMap(changedFile, allTabs, this.props.hasFileChangedMap))
          }}

        />
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelView)