/**
 * Created by janis dähne.
 */

import * as constants from '../../../constants'

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";

import PanelGrid from '../../panelSystem/panelGrid/panelGrid'
import Panel from '../../panelSystem/panel/panel'
import TabContentHost from '../../panelSystem/panel/tabContentHost'

import {RouteComponentProps, withRouter} from "react-router";
import {parseId} from "../../../helpers/convertersAndTransformers";
import {
  loadExerciseEditor, setBotLeftPanelViewSize, setBotRightPanelViewSize, setBottomLeftSelectedTabIndex,
  setBottomRightSelectedTabIndex,
  setExerciseEditorHDividerPosOffset, setExerciseEditorVDividerPosOffset,
  setIsChangeExerciseEditorSettingsDialogDisplayed, setTopLeftPanelViewSize,
  setTopLeftSelectedTabIndex, setTopRightPanelViewSize,
  setTopRightSelectedTabIndex
} from "../../../state/actions/exerciseEditorSite/exerciseEditorSiteActions";
import Spinner from "../../helpers/Spinner";

//---bottom right panel
import ChangeTestsPanelHeaderBar from './exerciseTestsPanel/editorTestsPanelHeaderBar'
import ChangeTestsPanelView from './exerciseTestsPanel/editorTestsPanelView'


//---bottom left panel
import NotePanelView from './notePanel/notePanelView'

import TagsPanelView from './tagsPanel/tagsPanelView'
import TagsPanelHeaderBar from './tagsPanel/tagsPanelHeaderBar'

import AssetsPanelView from './assetsPanel/assetsPanelView'
import AssetsPanelHeaderBar from './assetsPanel/assetsPanelHeaderBar'

import TemplatesPanelView from './codeTemplatesPanel/codeTemplatesPanelView'
import TemplatesPanelHeaderBar from './codeTemplatesPanel/codeTemplatesPanelHeaderBar'

//---top right panel
import PreviewPanelView from './previewPanel/previewPanelView'
import ExerciseSettingsPanelView from './exerciseSettingsPanel/exerciseSettingsPanelView'
import ExerciseSyntaxHelpPanelView from './exerciseSyntaxHelpPanel/exerciseSyntaxHelpPanelView'
import ExerciseSyntaxHelpPanelHeaderBar from './exerciseSyntaxHelpPanel/exerciseSyntaxHelpPanelHeaderBar'

//--- top left panel
import DescriptionEditorPanelView from './descriptionEditorPanel/descriptionEditorPanelView'
import DescriptionEditorHeaderBar from './descriptionEditorPanel/descriptionEditorHeaderBar'


//--- dialogs
import ChangeTestDialog from './exerciseTestsPanel/dialogs/changeTestDialog'
import ChangeSelectedTagsDialog from './tagsPanel/dialogs/changeSelectedTagsDialog'
import ChangeCodeTemplateDialog from './codeTemplatesPanel/dialogs/changeCodeTemplateDialog'
import EditorSettingsDialog from '../../editorSettingsDialog/editorSettingsDialog'
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {LoginTimeoutHelper} from "../../../helpers/loginTimeoutHelper";
import {
  setDescriptionContent,
  setExerciseEditorAssets
} from "../../../state/actions/exerciseEditorSite/subSets/editorExerciseDescriptionActions";
import {createAndRefreshAssetUrlsAndObjects, freeAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import {setTutorViewDummy} from "../../../state/actions/tutorViewSite/tutorViewSiteActions";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    editorExercise: rootState.exerciseEditorState.editorExercise,

    exerciseDescriptionContent: rootState.exerciseEditorState.editorExercise.exerciseDescription.content,

    isLoading: rootState.exerciseEditorState.isLoading || rootState.testTypesState.isLoading || rootState.pLangsState.isLoading,

    topRightSelectedTabIndex: rootState.exerciseEditorState.topRightSelectedTabIndex,
    topLeftSelectedTabIndex: rootState.exerciseEditorState.topLeftSelectedTabIndex,
    bottomRightSelectedTabIndex: rootState.exerciseEditorState.bottomRightSelectedTabIndex,
    bottomLeftSelectedTabIndex: rootState.exerciseEditorState.bottomLeftSelectedTabIndex,

    topRightPanelViewSize: rootState.exerciseEditorState.topRightPanelViewSize,
    topLeftPanelViewSize: rootState.exerciseEditorState.topLeftPanelViewSize,
    botRightPanelViewSize: rootState.exerciseEditorState.botRightPanelViewSize,
    botLeftPanelViewSize: rootState.exerciseEditorState.botLeftPanelViewSize,

    isChangeTestDialogDisplayed: rootState.exerciseEditorState.isChangeTestDialogDisplayed,
    isChangeSelectedTagsDialogDisplayed: rootState.exerciseEditorState.isChangeSelectedTagsDialogDisplayed,
    isChangeCodeTemplateDialogDisplayed: rootState.exerciseEditorState.isChangeCodeTemplateDialogDisplayed,
    isChangeEditorSettingsDialogDisplayed: rootState.exerciseEditorState.isChangeEditorSettingsDialogDisplayed,

    assets: rootState.exerciseEditorState.editorExercise.exerciseDescription.assets,

    isNoDialogDisplayed: !rootState.exerciseEditorState.isChangeTestDialogDisplayed
    && !rootState.exerciseEditorState.isChangeSelectedTagsDialogDisplayed
    && !rootState.exerciseEditorState.isChangeCodeTemplateDialogDisplayed
    && !rootState.exerciseEditorState.isChangeEditorSettingsDialogDisplayed
    ,

    vDividerPosOffset: rootState.exerciseEditorState.vDividerPosOffset,
    hDividerPosOffset: rootState.exerciseEditorState.hDividerPosOffset,

  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  loadExerciseEditor,

  setIsChangeExerciseEditorSettingsDialogDisplayed,

  setTopLeftSelectedTabIndex,
  setTopRightSelectedTabIndex,
  setBottomRightSelectedTabIndex,
  setBottomLeftSelectedTabIndex,

  setExerciseEditorHDividerPosOffset,
  setExerciseEditorVDividerPosOffset,

  setTopRightPanelViewSize,
  setTopLeftPanelViewSize,
  setBotRightPanelViewSize,
  setBotLeftPanelViewSize,

  setExerciseEditorAssets,
  setDescriptionContent,

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


interface MatchedProps {
  /**
   * the exercise id to load or undefined to create a new one
   */
  readonly exerciseId: string | undefined
}

class exerciseEditorSite extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {


  async componentWillMount(): Promise<void> {

    const exerciseId = parseId(this.props.match.params.exerciseId)

    if (exerciseId !== null) {
      //Load

      if (LoginTimeoutHelper.isReLogin && exerciseId === LoginTimeoutHelper.lastKnownExerciseEditorId) {
        //do not reload or reset site state...
        //user re logged in he/she wants to continue

        createAndRefreshAssetUrlsAndObjects(this.props.editorExercise.exerciseDescription.assets)

        return
      } else {
        LoginTimeoutHelper.lastKnownExerciseEditorId = exerciseId
      }

      //check if we want to create a copy or just edit an exercise

      if (this.props.match.path === constants.createCopyFromOldExerciseLinkMatchPath) {

        this.props.loadExerciseEditor(exerciseId, true, true)
        //then reset all ids to pretend all is new...

      } else {
        this.props.loadExerciseEditor(exerciseId, false, false)
      }

    } else {
      //create

      if (this.props.editorExercise.id <= 0) {
        if (LoginTimeoutHelper.isReLogin && this.props.editorExercise
          && this.props.editorExercise.id === LoginTimeoutHelper.lastKnownExerciseEditorId
        ) {
          //do not reload or reset site state...
          //user re logged in he/she want's to continue
          return
        } else {
          //this would give us an old id because loadExerciseEditor will create a new one...
          //LoginTimeoutHelper.lastKnownExerciseEditorId = this.props.editorExercise.id
        }
      }

      //assume create exercise
      await this.props.loadExerciseEditor(null, false, false)

      //grab the new exercise id
      LoginTimeoutHelper.lastKnownExerciseEditorId = this.props.editorExercise.id
    }
  }

  componentWillUnmount(): void {
    //if we not freeing we would leak memory... if we just exit the site
    freeAssetUrlsAndObjects()
  }

  render(): JSX.Element {

    return (
      <div className="fh">

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          this.props.isChangeTestDialogDisplayed &&
          <ChangeTestDialog/>
        }
        {
          this.props.isChangeSelectedTagsDialogDisplayed &&
          <ChangeSelectedTagsDialog/>
        }
        {
          this.props.isChangeCodeTemplateDialogDisplayed &&
          <ChangeCodeTemplateDialog/>
        }
        {
          this.props.isChangeEditorSettingsDialogDisplayed &&
          <EditorSettingsDialog/>
        }

        {
          this.props.isNoDialogDisplayed &&
          <PanelGrid
            width="100%"
            height="100%"

            topRightPanelViewSize={this.props.topRightPanelViewSize}
            topLeftPanelViewSize={this.props.topLeftPanelViewSize}
            botRightPanelViewSize={this.props.botRightPanelViewSize}
            botLeftPanelViewSize={this.props.botLeftPanelViewSize}

            hDividerPosOffset={this.props.hDividerPosOffset}
            vDividerPosOffset={this.props.vDividerPosOffset}
            onHDividerPosOffsetChanged={(Offset: number) => {
              this.props.setExerciseEditorHDividerPosOffset(Offset)
            }}
            onVDividerPosOffsetChanged={(Offset: number) => {
              this.props.setExerciseEditorVDividerPosOffset(Offset)
            }}

            topLeftPanel={
              <Panel
                applyDefaultPadding={false}
                selectedTabIndex={this.props.topLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopLeftSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setTopLeftPanelViewSize}
                panelViewSize={this.props.topLeftPanelViewSize}
              >
                <TabContentHost
                  icon={'font'}
                  title={translate(TranslateIds.descriptionEditor)}
                  applyDefaultPadding={false}
                  tabHeaderBar={
                    <DescriptionEditorHeaderBar
                      setIsEditorSettingsDialogDisplayed={this.props.setIsChangeExerciseEditorSettingsDialogDisplayed}
                    />
                  }
                >
                  <DescriptionEditorPanelView
                    descriptionContent={this.props.exerciseDescriptionContent}
                    setContent={(content: string) => {
                      this.props.setDescriptionContent(content)
                    }}/>
                </TabContentHost>

              </Panel>
            }

            topRightPanel={
              <Panel
                selectedTabIndex={this.props.topRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setTopRightPanelViewSize}
                panelViewSize={this.props.topRightPanelViewSize}
              >

                <TabContentHost
                  title={translate(TranslateIds.properties)}
                  icon={'options'}
                >
                  <ExerciseSettingsPanelView/>
                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.preview)}
                  icon={'unhide'}
                >
                  <PreviewPanelView descriptionContent={this.props.editorExercise.exerciseDescription.content}
                                    assets={this.props.editorExercise.exerciseDescription.assets}
                                    dummy={false /*not needed because we have tabs that will re render if tab index changed*/}
                  />
                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.guide)}
                  icon={'book'}
                  tabHeaderBar={
                    <ExerciseSyntaxHelpPanelHeaderBar/>
                  }
                >
                  <ExerciseSyntaxHelpPanelView/>
                </TabContentHost>


              </Panel>
            }

            bottomLeftPanel={
              <Panel
                selectedTabIndex={this.props.bottomLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomLeftSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotLeftPanelViewSize}
                panelViewSize={this.props.botLeftPanelViewSize}
              >

                <TabContentHost
                  title={translate(TranslateIds.templates)}
                  icon={'file code outline'}
                  tabHeaderBar={
                    <TemplatesPanelHeaderBar/>
                  }
                >
                  <TemplatesPanelView/>
                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.assets)}
                  icon={'file'}
                  tabHeaderBar={
                    <AssetsPanelHeaderBar assets={this.props.assets}
                                          setAssets={(assets) => {
                                            this.props.setExerciseEditorAssets(assets)
                                          }}
                    />
                  }
                >
                  <AssetsPanelView assets={this.props.assets} setAssets={(assets) => {
                    this.props.setExerciseEditorAssets(assets)
                  }}/>
                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.tags)}
                  icon={'tags'}
                  tabHeaderBar={
                    <TagsPanelHeaderBar/>
                  }
                >
                  <TagsPanelView/>
                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.note)}
                  icon={'sticky note'}
                >
                  <NotePanelView/>
                </TabContentHost>

              </Panel>
            }

            bottomRightPanel={
              <Panel
                selectedTabIndex={this.props.bottomRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotRightPanelViewSize}
                panelViewSize={this.props.botRightPanelViewSize}
              >
                <TabContentHost
                  title={translate(TranslateIds.normalTests)}
                  icon={'bug'}
                  tabHeaderBar={
                    <ChangeTestsPanelHeaderBar isForSubmitTests={false}
                    />
                  }
                >
                  <ChangeTestsPanelView isForSubmitTests={false}
                  />
                </TabContentHost>

                <TabContentHost
                  icon="flag checkered"
                  title={translate(TranslateIds.submitTests)}
                  tabHeaderBar={
                    <ChangeTestsPanelHeaderBar isForSubmitTests={true}
                    />
                  }
                >
                  <ChangeTestsPanelView isForSubmitTests={true}
                  />
                </TabContentHost>

              </Panel>
            }


          />
        }


      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(exerciseEditorSite))