/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {TestAssetForBackend} from "../../../../../types/exerciseEditor";
import {RootState} from "../../../../../state/reducers/index";
import {Icon, List} from "semantic-ui-react";
import {convertSizeToHumanReadableSize} from "../../../../../helpers/convertersAndTransformers";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly files: ReadonlyArray<TestAssetForBackend>
  readonly canDownloadFiles: boolean

  /**
   * to re use this component a generic get download file href func
   */
  readonly getDownloadFileHRef: (file: TestAssetForBackend) => string


  readonly canRemoveFile: boolean
  readonly onRemoveFile: (file: TestAssetForBackend) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    files: props.files,
    canDownloadFiles: props.canDownloadFiles,
    getDownloadFileHRef: props.getDownloadFileHRef,
    canRemoveFile: props.canRemoveFile,
    onRemoveFile: props.onRemoveFile,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class changeTestFilesListView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <List>
          {
            this.props.files.map((file: TestAssetForBackend) => {
              return (
                <List.Item key={file.id}>
                  <List.Icon size='large' verticalAlign='middle'>

                    {
                      this.props.canRemoveFile &&
                      <div className="clickable"
                           onClick={() => {
                             this.props.onRemoveFile(file)
                           }}
                      >
                        <Icon name='remove'/>
                      </div>
                    }

                    {
                      !this.props.canRemoveFile &&
                      <div>
                        <Icon name="file"/>
                      </div>
                    }


                  </List.Icon>
                  <List.Content>
                    <List.Header>

                      <div className="flexed">
                        <div className="mar-right-half">

                          {
                            //file can only be downloaded if it's on the server...
                            //exercise is bulk created (everything at once) so when only creating an exercise
                            //the assets are not uploaded yet...
                            !this.props.canDownloadFiles &&
                              <span>
                                {
                                  file.displayName
                                }
                              </span>
                          }

                          {
                            //after the exercise is saved the user can download the test assets from backend
                            this.props.canDownloadFiles &&
                            <a target="_blank" download
                               href={this.props.getDownloadFileHRef(file)}>
                              <span>
                                {
                                  file.displayName
                                }
                              </span>
                            </a>
                          }
                        </div>

                        <div className="not-important-colored">
                          (
                          {convertSizeToHumanReadableSize(file.size)}
                          )
                        </div>
                      </div>

                    </List.Header>
                    <List.Description>
                      <div className="not-important-colored">
                        {
                          file.mimeType
                        }
                      </div>
                    </List.Description>
                  </List.Content>
                </List.Item>
              )
            })
          }
        </List>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(changeTestFilesListView)