/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Checkbox, Form} from "semantic-ui-react";
import MaterialInput from '../../../material/materialInput'
import {
  setId,
  setDisplayName,
  setCanCreateExercises,
  setCanChangeExercises,
  setCanDeleteExercises,
  setCanReleaseExercises,
  setCanCopyExercises,
  setCanAssessExercises,
  setCanSeeExercisesFromOthersInGroup,
  setCanAddUserToGroup,
  setCanRemoveMemberFromGroup,
  setCanChangeOtherMembersRole,
  setCanSeeOtherUsers,
} from "../../../../state/actions/groupRolesSite/dialog/groupRoleSetAction";
import {validationMessageKeys, validationRules} from "../../../../state/reducers/groupRolesSite/dialog/groupRoleReducer";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    groupRole: rootState.groupRolesChangeDialogState.groupRole
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setId,
  setDisplayName,
  setCanCreateExercises,
  setCanChangeExercises,
  setCanDeleteExercises,
  setCanReleaseExercises,
  setCanCopyExercises,
  setCanAssessExercises,
  setCanSeeExercisesFromOthersInGroup,
  setCanAddUserToGroup,
  setCanRemoveMemberFromGroup,
  setCanChangeOtherMembersRole,
  setCanSeeOtherUsers,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ChangeGroupRoleView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <div className="ui form divided-checkboxes">
          <Form.Group widths='equal'>
            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.displayName)
                }
              </label>
              <MaterialInput
                value={this.props.groupRole.displayName}
                onChange={(e) => this.props.setDisplayName(e.currentTarget.value)}
                validations={validationRules.displayName}
                validationMessageKey={validationMessageKeys.displayName}
              />
            </Form.Field>
          </Form.Group>

          <h4 className="ui dividing header">
            {
              translate(TranslateIds.other)
            }
          </h4>
          <Form.Group widths='equal'>

            <Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canCreateExercises)}
                          checked={this.props.groupRole.canCreateExercises}
                          onChange={(e, data) => {
                            this.props.setCanCreateExercises(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canChangeExercises)}
                          checked={this.props.groupRole.canChangeExercises}
                          onChange={(e, data) => {
                            this.props.setCanChangeExercises(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canDeleteExercises)}
                          checked={this.props.groupRole.canDeleteExercises}
                          onChange={(e, data) => {
                            this.props.setCanDeleteExercises(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
            <Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canReleaseExercises)}
                          checked={this.props.groupRole.canReleaseExercises}
                          onChange={(e, data) => {
                            this.props.setCanReleaseExercises(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canCopyExercises)}
                          checked={this.props.groupRole.canCopyExercises}
                          onChange={(e, data) => {
                            this.props.setCanCopyExercises(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canSeeExercisesFromOthersInGroup)}
                          checked={this.props.groupRole.canSeeExercisesFromOthersInGroup}
                          onChange={(e, data) => {
                            this.props.setCanSeeExercisesFromOthersInGroup(data.checked)
                          }}
                />
              </Form.Field>


              {
                //<Form.Field>
                  //<Checkbox label='Darf andere Benutzer sehen'
                            //checked={this.props.dialogData.canSeeOtherUsers}
                            //onChange={(e, data) => {
//                              this.props.setCanSeeOtherUsers(data.checked)
                            //}}
                  ///>
                //</Form.Field>
              }

            </Form.Field>
          </Form.Group>


          <h4 className="ui dividing header">
            {
              translate(TranslateIds.groups)
            }
          </h4>
          <Form.Group widths='equal'>
            <Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canAddUserToGroup)}
                          checked={this.props.groupRole.canAddUserToGroup}
                          onChange={(e, data) => {
                            this.props.setCanAddUserToGroup(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canRemoveMemberFromGroup)}
                          checked={this.props.groupRole.canRemoveMemberFromGroup}
                          onChange={(e, data) => {
                            this.props.setCanRemoveMemberFromGroup(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canChangeOtherMembersRole)}
                          checked={this.props.groupRole.canChangeOtherMembersRole}
                          onChange={(e, data) => {
                            this.props.setCanChangeOtherMembersRole(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
          </Form.Group>


          <h4 className="ui dividing header">
            {
              translate(TranslateIds.submissions)
            }
          </h4>
          <Form.Group widths='equal'>
            <Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canAssessExercises)}
                          checked={this.props.groupRole.canAssessExercises}
                          onChange={(e, data) => {
                            this.props.setCanAssessExercises(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
          </Form.Group>


        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeGroupRoleView)