/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Table, Icon} from "semantic-ui-react";
import * as constants from '../../../constants'
import {setSelectedGroupId} from "../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {
  openEditGroupDialog, quitGroup,
  setSortByKey
} from "../../../state/actions/groupsSite/groupsSiteActions";
import {Link} from "react-router-dom";
import {UserGroupFullBase} from "../../../types/group";
import {isCurrentSortingColumn} from "../../../helpers/tablesHelper";
import {SortDirection} from "../../../types/sortDirection";
import {copyToClipboard} from "../../../helpers/clipboard";
import {askDialog} from "../../../helpers/dialogHelper";
import {deleteUserGroupAsync} from "../../../state/actions/groupsSite/groupsCrudActions";
import {HelpPopup} from "../../helpers/helpPopup";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {orderBy} from "lodash-es";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    groups: rootState.groupsState.groups,
    sortByKey: rootState.groupsState.sortByKey,
    sortDirection: rootState.groupsState.sortDirection,

    systemRole: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole
      : null,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setSelectedGroupId,
  openEditGroupDialog,
  quitGroup,
  setSortByKey,
  deleteUserGroupAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class GroupsTableView extends React.Component<Props, any> {

  render(): JSX.Element {

    const list: ReadonlyArray<UserGroupFullBase> =
      this.props.sortDirection === SortDirection.none
        ? this.props.groups
        : orderBy(this.props.groups, user => user[this.props.sortByKey],
        this.props.sortDirection === SortDirection.ascending ? 'asc' : 'desc')

    return (
      <div>
        <Table sortable selectable padded striped singleLine className="material-table">

          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <span>
                  {
                    translate(TranslateIds.open)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<UserGroupFullBase>(this.props.sortByKey, 'displayName',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('displayName', this.props.sortDirection + 1)
                      }}
                >

                  {
                    translate(TranslateIds.displayName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<UserGroupFullBase>(this.props.sortByKey, 'email',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('email', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.email)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  //TODO backend send members length
                  isCurrentSortingColumn<UserGroupFullBase>(this.props.sortByKey, 'id',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('id', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.members)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                    {
                      translate(TranslateIds.change)
                    }
                  </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    translate(TranslateIds.leaveGroup)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span className="mar-right-half">
                    {
                      translate(TranslateIds.delete_)
                    }
                  </span>
                <HelpPopup defaultText={"Die Gruppe kann nur gelöscht werden, wenn ... TODO BACKEND"}/>

              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((group, index) => {
                return (
                  <Table.Row key={group.id}>
                    <Table.Cell>
                      <div className="clickable">
                        <Link to={constants.getSingleGroupLinkPath(group.id)}>
                          <Icon name="share"/>
                        </Link>
                      </div>
                    </Table.Cell>

                    <Table.Cell>
                          <span>
                            {group.displayName}
                          </span>
                    </Table.Cell>

                    <Table.Cell>
                      {
                        group.email &&
                        <Icon name="clone" className="clickable"
                              onClick={() => {
                                copyToClipboard(group.email)
                              }}
                        />
                      }
                      <span>
                          {group.email}
                        </span>
                    </Table.Cell>

                    <Table.Cell>
                      <span>
                        ??? Backend???
                      </span>
                    </Table.Cell>

                    <Table.Cell>
                      <div className="clickable"
                           onClick={() => {
                             this.props.openEditGroupDialog(group)
                           }}
                      >
                        <Icon name="write"/>
                      </div>
                    </Table.Cell>

                    <Table.Cell>

                      {
                        (this.props.systemRole && this.props.systemRole.canChangeGroups) &&
                        <div className="clickable" onClick={async () => {

                          const shouldRemove = await askDialog(translate(TranslateIds.leaveGroup_title),
                            `${translate(TranslateIds.leaveGroup_content)} (${group.displayName})`)

                          if (shouldRemove) {
                            this.props.quitGroup(group)
                          }


                        }}>
                          <Icon name="remove" className="clickable"/>
                        </div>
                      }

                    </Table.Cell>

                    <Table.Cell>

                      {
                        (this.props.systemRole && this.props.systemRole.canDeleteGroups) &&
                        <div className="clickable"
                             onClick={async () => {

                               const shouldRemove = await askDialog(translate(TranslateIds.deleteGroup_title),
                                 `${translate(TranslateIds.deleteGroup_content)} (${group.displayName})`)

                               if (shouldRemove) {
                                 this.props.deleteUserGroupAsync(group.id)
                               }


                             }}>
                          <Icon name="trash" className="clickable"/>
                        </div>
                      }


                    </Table.Cell>

                  </Table.Row>
                )
              })
            }
          </Table.Body>
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(GroupsTableView)