/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {Icon, Table, Checkbox, Divider} from "semantic-ui-react";
import MaterialSearchInput from '../../material/materialSearchInput'
import GroupsTableView from './groupsListView'
import GroupsCardsView from './groupsCardsView'
import {SiteViewState} from "../../../types/siteView";
import {getGroupsAsync} from "../../../state/actions/groupsSite/groupsCrudActions";
import Spinner from "../../helpers/Spinner";
import ChangeGroupDataDialog from './dialogs/changeGroupDataDialog'
import {loadGroupsSite, setSortByKey} from "../../../state/actions/groupsSite/groupsSiteActions";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    viewState: rootState.groupsState.viewState,
    isLoading: rootState.groupsState.isLoading,
    isChangeDialogDisplayed: rootState.groupsState.isChangeDialogDisplayed,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  loadGroupsSite
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class GroupsSite extends React.Component<Props, any> {


  componentWillMount(): void {
    this.props.loadGroupsSite()
  }

  render(): JSX.Element {
    return (
      <div>
        <SinglePanelSiteWrapper>

          {
            this.props.isLoading &&
            <Spinner/>
          }

          {
            !this.props.isChangeDialogDisplayed &&
            <div className="view-padding">
              <div className="view-head-line">
                <div className="view-caption">
                  <h1>
                    {
                      translate(TranslateIds.groups)
                    }
                  </h1>
                </div>
                <div className="view-options">
                  <MaterialSearchInput/>
                </div>
              </div>

              <div className="view-content">
                {
                  this.props.viewState === SiteViewState.List &&
                  <GroupsTableView/>
                }

                {
                  this.props.viewState === SiteViewState.Cards &&
                  <GroupsCardsView/>
                }
              </div>
            </div>
          }


          {
            !this.props.isLoading && this.props.isChangeDialogDisplayed &&
            <ChangeGroupDataDialog/>
          }


        </SinglePanelSiteWrapper>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(GroupsSite)