/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import MaterialInput from '../../../material/materialInput'
import MaterialSelectWrapper from '../../../material/materialSelectWrapper'
import {Popup, Form, Icon, Button} from "semantic-ui-react";
import {
  setEmail,
  setLastName,
  setFirstName,
  setPassword, setSystemRole,
} from "../../../../state/actions/manageActivatedUsersSite/dialogs/activatedUserDataSetActions";

import {resetPasswordAsync} from '../../../../state/actions/manageActivatedUsersSite/dialogs/changeActivatedUserDataCrudActions'
import {
  validationRules,
  validationMessageKeys
} from "../../../../state/reducers/manageActivatedUsersSite/dialog/otherUserDataReducer";
import {SystemRoleFullBase} from "../../../../types/userData";
import {SyntheticEvent} from "react";
import {askDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {copyToClipboard} from "../../../../helpers/clipboard";

//const css = require('./styles.styl');

export const noneSystemRole: SystemRoleFullBase = {
  id: -1,
  displayName: 'Keine',
  email: null,
  canCreateGroups: false,
  canChangeGroups: false,
  canDeleteGroups: false,
  canCreateRoles: false,
  canChangeRoles: false,
  canDeleteRoles: false,
  canManageNewUsers: false,
  canChangeOtherUsersSystemRole: false,
  canDeleteActivatedUsers: false,
  canChangeUserData: false,
  canChangeSystemSettings: false,
  canManageTags: false,
  canViewDashboard: false
}

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    activatedUser: rootState.changeActivatedUsersDataDialogState.userData,
    systemRoles: rootState.manageActivatedUsersState.systemRoles,

    currentUserSystemRole: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole
      : null,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setEmail,
  setLastName,
  setFirstName,
  setPassword,
  setSystemRole,
  resetPasswordAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ChangeUserDataView extends React.Component<Props, any> {
  render(): JSX.Element {

    const systemRolesWithDefault: ReadonlyArray<SystemRoleFullBase> = [noneSystemRole, ...this.props.systemRoles]

    return (
      <div>

        <div className="ui form divided-checkboxes">
          <Form.Group widths='equal'>
            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.firstName)
                }
              </label>
              <MaterialInput
                value={this.props.activatedUser.firstName}
                onChange={(e) => this.props.setFirstName(e.currentTarget.value)}
                validations={validationRules.firstName}
                validationMessageKey={validationMessageKeys.firstName}
                disabled={!this.props.currentUserSystemRole || !this.props.currentUserSystemRole.canChangeUserData}
              />
            </Form.Field>
            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.lastName)
                }
              </label>
              <MaterialInput
                value={this.props.activatedUser.lastName}
                onChange={(e) => this.props.setLastName(e.currentTarget.value)}
                validations={validationRules.lastName}
                validationMessageKey={validationMessageKeys.lastName}
                disabled={!this.props.currentUserSystemRole || !this.props.currentUserSystemRole.canChangeUserData}
              />
            </Form.Field>
          </Form.Group>

          <Form.Group widths='equal'>

            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.email)
                }
              </label>
              <MaterialInput
                value={this.props.activatedUser.email}
                onChange={(e) => this.props.setEmail(e.currentTarget.value)}
                validations={validationRules.email}
                validationMessageKey={validationMessageKeys.email}
                disabled={!this.props.currentUserSystemRole || !this.props.currentUserSystemRole.canChangeUserData}
              />
            </Form.Field>
            <Form.Field>
              <label>
                {
                  translate(TranslateIds.systemRole)
                }
              </label>

              <MaterialSelectWrapper
                value={this.props.activatedUser.systemRole.id}
              >
                <select
                  value={this.props.activatedUser.systemRole.id}
                  onChange={(e: SyntheticEvent<HTMLSelectElement>) => {

                    const id = parseInt(e.currentTarget.value, 10)
                    if (isNaN(id)) {
                      //TODO some error
                    } else {
                      const systemRole = systemRolesWithDefault.find(p => p.id === id)
                      this.props.setSystemRole(systemRole)
                    }
                  }}
                  disabled={!this.props.currentUserSystemRole || !this.props.currentUserSystemRole.canChangeOtherUsersSystemRole}
                >
                  {
                    systemRolesWithDefault.map(systemRole => {
                      return (
                        <option key={systemRole.id} value={systemRole.id}>
                          {
                            systemRole.displayName
                          }
                        </option>
                      )
                    })
                  }
                </select>
              </MaterialSelectWrapper>


            </Form.Field>
          </Form.Group>

          <Form.Field>
            {
              //used for save & cancel button
              this.props.children
            }
          </Form.Field>


          {
            (this.props.currentUserSystemRole && this.props.currentUserSystemRole.canChangeUserData) &&
            <h3 className="ui dividing header">
              {
                translate(TranslateIds.resetPassword)
              }
            </h3>
          }
          {
            (this.props.currentUserSystemRole && this.props.currentUserSystemRole.canChangeUserData) &&
            <Form.Group widths='equal'>
              <Form.Field>
                <div className="form-field-label-with-icon">
                  <label className="mar-right-half">
                    {
                      translate(TranslateIds.generatedPassword)
                    }
                  </label>
                  <Popup className="mar-right-half"
                    trigger={<Icon name="question circle" className="hoverable"/>}
                    content={translate(TranslateIds.generatedPassword_helpText)}
                    wide
                  />

                  {
                    this.props.activatedUser.password &&
                    <Icon name="clone" className="clickable mar-right-half"
                          onClick={() => {
                            copyToClipboard(this.props.activatedUser.password)
                          }}
                    />
                  }

                  <div className="two-flexed-stretch-right">
                    <Button circular icon='undo' className="mar-right"
                            onClick={async () => {

                              const shouldReset = await askDialog(translate(TranslateIds.resetPassword_title),
                                translate(TranslateIds.resetPassword_content))

                              if (shouldReset) {
                                this.props.resetPasswordAsync({
                                  password: '', //currently not uses, server resets password to random
                                  targetUserId: this.props.activatedUser.targetUserId
                                })
                              }
                            }}
                    />
                    {
                      //generated by backend --> no check
                      //                                   validations={validationRules.password}
                      //validationMessageKey={validationMessageKeys.password}
                    }
                    <MaterialInput readonly
                                   value={this.props.activatedUser.password}

                    />
                  </div>
                </div>
              </Form.Field>
            </Form.Group>

          }

        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeUserDataView)