/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {SelectableUserFrontendExclusive} from "../../../types/users";
import {Table, Icon, Checkbox} from "semantic-ui-react";
import {CheckboxOnChangeEventData} from "../../../types/reactEvents";
import {deleteActivatedUserAsync} from "../../../state/actions/manageActivatedUsersSite/manageActivatedUsersSiteCrudActions";
import {
  setActivatedUserUiIsSelected,
  setAllActivatedUsersUiIsSelected, setIsChangeUserDataDialogDisplayed
} from "../../../state/actions/manageActivatedUsersSite/manageActivatedUsersSiteActions";
import {setTargetUser} from "../../../state/actions/manageActivatedUsersSite/dialogs/changeActivatedUserDataDialogActions";
import {copyToClipboard} from "../../../helpers/clipboard";
import {displayDateAndTime} from "../../../helpers/convertersAndTransformers";
import {askDialog} from "../../../helpers/dialogHelper";
import {HelpPopup} from "../../helpers/helpPopup";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {getAssessmentStatisticsSiteToLinkPath} from "../../../constants";
import {Link} from "react-router-dom";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly activatedUser: SelectableUserFrontendExclusive
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    activatedUser: props.activatedUser,
    userLangId: rootState.userDataSettingsSate.userData.settings
      ? rootState.userDataSettingsSate.userData.settings.langId
      : -1,

    systemRole: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole
      : null,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setIsChangeUserDataDialogDisplayed,
  setTargetUser,
  setActivatedUserUiIsSelected,
  setAllActivatedUsersUiIsSelected,
  deleteActivatedUserAsync,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

//TODO docs beispiel
class testRow extends React.Component<Props, any> {
  render(): JSX.Element {

    const activatedUser = this.props.activatedUser

    return (
      <Table.Row>
        <Table.Cell>
          <Checkbox checked={activatedUser.uiIsSelected}
                    onChange={(event: any, data: CheckboxOnChangeEventData) => {
                      return this.props.setActivatedUserUiIsSelected(activatedUser.id, data.checked)
                    }}/>
        </Table.Cell>

        <Table.Cell>
                      <span>
                        {activatedUser.lastName}
                      </span>
        </Table.Cell>
        <Table.Cell>
                      <span>
                        {activatedUser.firstName}
                      </span>
        </Table.Cell>
        <Table.Cell>
                      <span>
                        {activatedUser.token}
                      </span>
        </Table.Cell>
        <Table.Cell>
          <Icon name="clone" className="clickable"
                data-clipboard-text={activatedUser.email}
                onClick={() => {
                  copyToClipboard(activatedUser.email)
                }}
          />
          <span>
                        {activatedUser.email}
                      </span>
        </Table.Cell>
        <Table.Cell>
                      <span>
                        {
                          activatedUser.systemRole ? activatedUser.systemRole.displayName : translate(
                            TranslateIds.noSystemRole)
                        }
                      </span>
        </Table.Cell>
        <Table.Cell>
                      <span>
                        {
                          displayDateAndTime(activatedUser.createdAt, this.props.userLangId)
                        }
                      </span>
        </Table.Cell>
        <Table.Cell>
                      <span>
                        {
                          displayDateAndTime(activatedUser.lastLoginAt, this.props.userLangId)
                        }
                      </span>
        </Table.Cell>
        <Table.Cell>
          <div>
            <Link style={{margin: '0'}} to={getAssessmentStatisticsSiteToLinkPath(activatedUser.id)}>
              <Icon name="area graph" />
            </Link>
          </div>
        </Table.Cell>
        <Table.Cell>
          {
            this.props.systemRole &&
            (this.props.systemRole.canChangeOtherUsersSystemRole || this.props.systemRole.canChangeUserData)
            &&
            <div className="clickable" onClick={() => {
              this.props.setTargetUser(activatedUser)
              this.props.setIsChangeUserDataDialogDisplayed(true)
            }}>
              <Icon name="write"></Icon>
            </div>
          }

        </Table.Cell>
        <Table.Cell>
          {
            activatedUser.isCurrentUser &&
            <HelpPopup defaultText={translate(TranslateIds.thisIsYou)}/>
          }
          {
            (!activatedUser.isCurrentUser && (this.props.systemRole && this.props.systemRole.canDeleteActivatedUsers)) &&
            <div className="clickable"
                 onClick={async () => {

                   const shouldRemove = await askDialog(translate(TranslateIds.deleteActivatedUser_title),
                     `${translate(
                       TranslateIds.deleteActivatedUser_content)} (${activatedUser.lastName}, ${activatedUser.firstName})`)

                   if (shouldRemove) {
                     this.props.deleteActivatedUserAsync(activatedUser)
                   }

                 }}
            >
              <Icon name="remove"></Icon>
            </div>
          }


        </Table.Cell>
      </Table.Row>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(testRow)