/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {loadManageCustomProjectSite} from "../../../state/actions/manageCustomProjectsSite/manageCustomProjectsSiteActions";
import {errorDialog} from "../../../helpers/dialogHelper";
import Logger from "../../../helpers/logger";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../material/materialSearchInput'

import ListView from './listView'
import Spinner from "../../helpers/Spinner";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.manageCustomProjectsSiteState.isLoading
    || rootState.systemSettingsState.isLoading
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  loadManageCustomProjectSite
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ManageCustomProjectsSite extends React.Component<Props, any> {

  async componentWillMount(): Promise<void> {
    try {
      await this.props.loadManageCustomProjectSite()
    } catch (err) {
      errorDialog(translate(TranslateIds.serverError), translate(TranslateIds.couldNotLoadCustomProjectOverviews))
      Logger.error(`error loading custom projects: ${err.message}`)
      return null
    }
  }

  render(): JSX.Element {
    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        <div className="view-padding">
          <div className="view-head-line">
            <div className="view-caption">
              <h1>
                {
                  translate(TranslateIds.manageCustomProjectsSite)
                }
              </h1>
            </div>
            <div className="view-options">
              <MaterialSearchInput/>
            </div>
          </div>

          <div className="view-content">
            <ListView/>
          </div>
        </div>

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ManageCustomProjectsSite)