/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import TableView from '../manageOwnOrGroupExerciseComponents/listView'
import CardsView from '../manageOwnOrGroupExerciseComponents/cardsView'
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {Divider, Segment} from "semantic-ui-react";
import MaterialSearchInput from '../../material/materialSearchInput'
import {SiteViewState} from "../../../types/siteView";
import Spinner from "../../helpers/Spinner";
import {
  deleteGroupExercisesAsync,
  getGroupExercisesAsync, loadManageGroupExercisesSite,
  setSortByKey
} from "../../../state/actions/manageGroupExercisesSite/groupExercisesSiteActions";
import {
  EditableExercisePreviewFromBackend,
  FilterableEditableExercisePreviewFromBackend
} from "../../../types/exercisePreview";
import TagFilterPanel from '../../tagComponents/tagFilterPanel'
import {SortDirection} from "../../../types/sortDirection";
import {sortAndFilterExercisePreviews} from "../../../helpers/tablesHelper";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    groupExercises: rootState.groupExercisesState.groupExercises,
    viewState: rootState.groupExercisesState.viewState,
    isLoading: rootState.groupExercisesState.isLoading
    || rootState.manageTagsState.isLoading,

    sortByKey: rootState.groupExercisesState.sortByKey,
    sortDirection: rootState.groupExercisesState.sortDirection,

    selectedFilterTagsIds: rootState.tagsFilterState.selectedFilterTagsIds,
    selectedNegativeFilterTagsIds: rootState.tagsFilterState.selectedNegativeFilterTagsIds,

    tags: rootState.manageTagsState.tags,

    isTagsFilterDisplayed: rootState.groupExercisesState.isTagsFilterDisplayed,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  loadManageGroupExercisesSite,
  setSortByKey,
  deleteGroupExercisesAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class GroupExercisesSite extends React.Component<Props, any> {

  componentWillMount(): void {

    this.props.loadManageGroupExercisesSite()
  }

  render(): JSX.Element {
    return (

      <div className="multiple-single-panels">

        {
          //we now display TagFilterPanel that can show the loading...
          //BUT is could be hidden...
          this.props.isLoading &&
          <Spinner/>
        }

        {
          this.props.isTagsFilterDisplayed &&
          <TagFilterPanel
            isLoading={this.props.isLoading}
            allTags={this.props.tags}
          />
        }


        {
          //foreach
          this.props.groupExercises.map((groupTuple, index) => {
            return (
              <SinglePanelSiteWrapper key={groupTuple.groupPreview.id}>
                <div className="view-padding">
                  <div className="view-head-line">
                    <div className="view-caption">
                      <h1>
                        {groupTuple.groupPreview.displayName}
                      </h1>
                    </div>
                    <div className="view-options">
                      <MaterialSearchInput/>
                    </div>
                  </div>
                  <div className="view-content">
                    {

                      this.props.viewState === SiteViewState.List &&
                      <TableView exercisePreviews={
                        sortAndFilterExercisePreviews(groupTuple.exercisePreviews, this.props.sortDirection,
                          this.props.sortByKey, this.props.selectedFilterTagsIds, this.props.selectedNegativeFilterTagsIds)
                      }
                                 comesFromOwnExercisesSite={false}
                                 sortByKey={this.props.sortByKey}
                                 sortDirection={this.props.sortDirection}
                                 setSortByKey={this.props.setSortByKey}
                                 onDeleteExercise={(exercisePreview: EditableExercisePreviewFromBackend) => {
                                   this.props.deleteGroupExercisesAsync(exercisePreview.id)
                                 }}
                                 currentUserGroupRole={groupTuple.groupRole}


                      />
                    }

                    {
                      this.props.viewState === SiteViewState.Cards &&
                      <CardsView exercisePreviews={
                        sortAndFilterExercisePreviews(groupTuple.exercisePreviews, this.props.sortDirection,
                          this.props.sortByKey, this.props.selectedFilterTagsIds, this.props.selectedNegativeFilterTagsIds)
                      }
                                 comesFromOwnExercisesSite={false}
                                 onDeleteExercise={(exercisePreview: EditableExercisePreviewFromBackend) => {
                                   this.props.deleteGroupExercisesAsync(exercisePreview.id)
                                 }}
                                 currentUserGroupRole={groupTuple.groupRole}
                      />
                    }
                  </div>
                </div>


              </SinglePanelSiteWrapper>
            )
          })
        }

      </div>

    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(GroupExercisesSite)