/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../../state/reducers/index";
import ZoomedExerciseReleaseView from './zoomedExerciseReleaseView'
import {EditableExercisePreviewFromBackend} from "../../../../../../types/exercisePreview";
import {Icon} from "semantic-ui-react";
import {setIsZoomedExerciseReleaseDisplayed} from "../../../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/exerciseReleasesSiteActions";
import {resetZoomedExerciseReleaseDialog} from "../../../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/dialogs/zoomedRelease/zoomedReleaseDialogActions";
import MaterialSearchInput from '../../../../../material/materialSearchInput'
import {BackArrow} from "../../../../../helpers/backArrow";
import {translate} from "../../../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  readonly exercise: EditableExercisePreviewFromBackend
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    release: rootState.zoomedExerciseReleaseState.release,
    exercise: props.exercise
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setIsZoomedExerciseReleaseDisplayed,
  resetZoomedExerciseReleaseDialog
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class zoomedExerciseReleaseDialog extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <div className="view-padding">
          <div className="view-head-line">
            <div className="view-caption">
              <div className="mar-right back-arrow">
                <div className="clickable"
                     onClick={() => {
                       this.props.setIsZoomedExerciseReleaseDisplayed(false)
                       this.props.resetZoomedExerciseReleaseDialog()
                     }}
                >
                  <BackArrow/>
                </div>
              </div>

              <h2 className="clear-margin">
                {
                  translate(TranslateIds.releasesFor)
                }
                {
                  this.props.exercise &&
                  ` ${this.props.exercise.displayName}`
                }
              </h2>

            </div>
            <div className="view-options">
              {
                //we only use this so that the left arrow aligns with the release view arrow
                //without the search input we get different heights and we center the arrow vertically
              }
              <MaterialSearchInput isHidden={true} disabled />
            </div>
          </div>

          <div className="view-content">
            <ZoomedExerciseReleaseView/>
          </div>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(zoomedExerciseReleaseDialog)