/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Icon} from "semantic-ui-react";
import CommonHeaderBarContentWrapper from '../../../commonHeaderBarContentWrapper'
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {
  loadSubmissionsSite,
  rerunAllSubmissionsAsync, rerunSingleSubmissionAsync, setAutomaticRefreshViewIntervalCounter,
  setIsAutomaticRefreshViewEnabled, setRunningAssessment
} from "../../../../state/actions/ownOrGroupExerciseActions/submissionsSite/submissionsSiteActions";
import {getAllSubmissionDownloadLink, getAssessmentResultsDownloadLink} from "../../../../constants";
import {getSubmissions} from "../../../../state/actions/ownOrGroupExerciseActions/submissionsSite/submissionsCrudActions";
import {askDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {HelpPopup} from "../../../helpers/helpPopup";
import {SortDirection} from "../../../../types/sortDirection";


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    selectedReleaseId: rootState.submissionsSiteState.selectedReleaseId,
    releases: rootState.submissionsSiteState.releases,
    submissions: rootState.submissionsSiteState.submissionTuple
      ? rootState.submissionsSiteState.submissionTuple.submissions
      : [],
    selectedExerciseId: rootState.submissionsSiteState.selectedExerciseId,
    isLoading: rootState.submissionsSiteState.isLoading
    || rootState.pLangsState.isLoading,

    isAutomaticRefreshViewEnabled: rootState.submissionsSiteState.isAutomaticRefreshViewEnabled,
    automaticRefreshViewIntervalCounter: rootState.submissionsSiteState.automaticRefreshViewIntervalCounter,

    sortByKey: rootState.submissionsSiteState.sortByKey,
    sortDirection: rootState.submissionsSiteState.sortDirection,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  loadSubmissionsSite,
  rerunSingleSubmissionAsync,
  setRunningAssessment,
  getSubmissions,

  setIsAutomaticRefreshViewEnabled,
  setAutomaticRefreshViewIntervalCounter,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


//e.g. 5 --> 4,3,2,1,0
const automaticRefreshViewIntervalIns = 5

class HeaderBar extends React.Component<Props, any> {


  automaticRefreshViewIntervalHandle: number | null = null

  componentWillUnmount(): void {
    this.clearAutoRefreshViewInterval()
  }

  clearAutoRefreshViewInterval(): void {
    if (this.automaticRefreshViewIntervalHandle) {
      //clear handle
      clearInterval(this.automaticRefreshViewIntervalHandle)
      this.automaticRefreshViewIntervalHandle = null
    }
  }

  createAutoRefreshViewInterval(): void {
    this.automaticRefreshViewIntervalHandle = setInterval(() => {

      this.props.setAutomaticRefreshViewIntervalCounter(this.props.automaticRefreshViewIntervalCounter - 1)

      if (this.props.automaticRefreshViewIntervalCounter < 0) {
        this.props.setAutomaticRefreshViewIntervalCounter(automaticRefreshViewIntervalIns)
        this.refreshView()
      }

    }, 1000)

  }

  refreshView(): void {
    this.props.loadSubmissionsSite(this.props.selectedExerciseId, this.props.selectedReleaseId, false, false,
      true, false)
  }

  render(): JSX.Element {

    const selectedRelease = this.props.releases.find(p => p.id === this.props.selectedReleaseId)

    if (!selectedRelease) {
      //TODO only error if really not found
      //on first render this might not be set because the list was not yet set
      return null
    }

    const canRerunAssessment =
      selectedRelease.hasAutomaticAssessmentStarted
      && selectedRelease.hasAutomaticAssessmentFinished

    // noinspection RedundantConditionalExpressionJS
    const leftArea = (
      <div>
        <div className={this.props.isAutomaticRefreshViewEnabled ? 'div-disabled' : 'clickable'}
             onClick={() => {
               this.refreshView()
             }}
        >
          <Icon name="refresh"/>
          <span>
            {
              translate(TranslateIds.refreshViw)
            }
          </span>
        </div>

        <SimpleVDivider/>

        <div
        >
          <a
            target="_blank"
            download
            href={getAssessmentResultsDownloadLink(
              selectedRelease.generatedCode,
              this.props.sortByKey,
              this.props.sortDirection === SortDirection.descending ? false : true
            )}
          >
            <Icon name="share"/>
            <span>
            {
              translate(TranslateIds.exportAssessments)
            }
          </span>
          </a>
        </div>

        <SimpleVDivider/>

        <div
        >
          <a target="_blank"
             href={selectedRelease ? getAllSubmissionDownloadLink(selectedRelease.generatedCode) : null}>
            download
            <Icon name="download" flipped="horizontally"/>
            <span>
              {
                translate(TranslateIds.downloadAllSubmissions)
              }
            </span>
          </a>
        </div>

        <SimpleVDivider/>

        <div className={canRerunAssessment ? 'clickable' : ' div-disabled'}
             onClick={async () => {

               const shouldReAssess = await askDialog(translate(TranslateIds.rerunAutomaticAssessment_title),
                 translate(TranslateIds.rerunAutomaticAssessment_content))

               if (!shouldReAssess) return

               //this don't reports the state...
               //this.props.rerunAllSubmissionsAsync(selectedRelease ? selectedRelease.generatedCode : null)

               // noinspection TsLint
               for (let i = 0; i < this.props.submissions.length; i++) {
                 const submission = this.props.submissions[i]

                 this.props.setRunningAssessment(submission)

                 await this.props.rerunSingleSubmissionAsync(
                   selectedRelease ? selectedRelease.generatedCode : null, submission, false)
               }

               this.props.setRunningAssessment(null)

               //then reload results
               this.props.getSubmissions(selectedRelease ? selectedRelease.generatedCode : null)

             }}
        >
          <Icon name="refresh"/>
          <span className="mar-right-half">
            {
              translate(TranslateIds.rerunAutomaticAssessment)
            }
          </span>
          <HelpPopup wide defaultText={translate(TranslateIds.rerunAutomaticAssessment_helpText)}/>
        </div>

        <SimpleVDivider/>

        <div className="clickable"
             onClick={() => {

               //clear handle if any
               this.clearAutoRefreshViewInterval()
               this.props.setAutomaticRefreshViewIntervalCounter(1)

               if (this.props.isAutomaticRefreshViewEnabled === false) { //then enable
                 this.createAutoRefreshViewInterval()
               }

               this.props.setIsAutomaticRefreshViewEnabled(!this.props.isAutomaticRefreshViewEnabled)
             }}
        >
          <Icon name="refresh" className={this.props.isAutomaticRefreshViewEnabled ? 'clickable-toggled' : ''}/>
          {
            this.props.isAutomaticRefreshViewEnabled &&
            <span className="clickable-toggled">
              {
                translate(
                  TranslateIds.automaticRefreshViewIn) +
                ` ${this.props.automaticRefreshViewIntervalCounter}` +
                `${translate(TranslateIds.seconds_abbr)}`
              }
            </span>
          }

          {
            !this.props.isAutomaticRefreshViewEnabled &&
            <span>
            {
              translate(TranslateIds.automaticRefreshView)
            }
          </span>
          }


        </div>

      </div>
    )


    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea} disabled={this.props.isLoading}/>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(HeaderBar)