/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../material/materialSearchInput'
import Spinner from "../../helpers/Spinner";
import ChangeTagDialog from "./dialogs/changeTagDialog";
import {loadManageTagsSite, openEditTagDialog} from "../../../state/actions/manageTagsSite/manageTagsActions";
import {deleteTagAsync, getTagsAsync} from "../../../state/actions/manageTagsSite/manageTagsSiteCrudActions";
import {TagFullBase} from "../../../types/tag";
import Tag from '../../tagComponents/tag'
import {askDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import orderBy from 'lodash-es/orderBy'

const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    tags: rootState.manageTagsState.tags,
    isLoading: rootState.manageTagsState.isLoading,
    isChangeDialogDisplayed: rootState.manageTagsState.isChangeDialogDisplayed
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  openEditTagDialog,
  deleteTagAsync,
  loadManageTagsSite
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ManageTagsSite extends React.Component<Props, any> {

  componentWillMount(): void {
    this.props.loadManageTagsSite()
  }

  render(): JSX.Element {


    const list: ReadonlyArray<TagFullBase> = orderBy(this.props.tags, (tag) => tag.displayName, 'asc')

    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          !this.props.isChangeDialogDisplayed &&
          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <h1>
                  {
                    translate(TranslateIds.tags)
                  }
                </h1>
              </div>
              <div className="view-options">
                <MaterialSearchInput/>
              </div>
            </div>

            <div className="view-content flexed separate-tabs">
              {
                list.map((tag, index2) => {
                  return (
                    <div key={tag.id}>
                      <Tag tag={tag}
                           leftIconName="write"
                           onLeftIconClicked={() => {
                             this.props.openEditTagDialog(tag)
                           }}
                           rightIconName="remove"
                           onRightIconClicked={async () => {

                             const shouldRemove = await askDialog(translate(TranslateIds.deleteTag_title),
                               `${translate(TranslateIds.deleteTag_content)} (${tag.displayName})`)

                             if (shouldRemove) {
                               this.props.deleteTagAsync(tag)
                             }

                           }}
                      />
                    </div>
                  )
                })
              }
            </div>
          </div>
        }

        {
          !this.props.isLoading && this.props.isChangeDialogDisplayed &&
          <ChangeTagDialog/>
        }

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ManageTagsSite)