/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../state/reducers/index";
import {Table, Checkbox, Icon, Button} from "semantic-ui-react"
import {CheckboxOnChangeEventData} from "../../../../../types/reactEvents";
import {
  invertAllSelections,
  setAllUsersSelected, setSortByKey,
  setUserSelected
} from "../../../../../state/actions/singleGroupSite/dialogs/addUsers/addUsersDialogActions";
import {AddableUserFrontendExclusive} from "../../../../../types/users";
import {isCurrentSortingColumn} from "../../../../../helpers/tablesHelper";
import {SortDirection} from "../../../../../types/sortDirection";
import ListViewItem from './listViewItem'
import {translate} from "../../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../../i18n/ids";
import orderBy from 'lodash-es/orderBy'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    //...rootState.users.activatedUsers,
    members: rootState.singleGroupState.members,
    users: rootState.addUsersDialogState.users,
    sortByKey: rootState.addUsersDialogState.sortByKey,
    sortDirection: rootState.addUsersDialogState.sortDirection,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setUserSelected,
  setAllUsersSelected,
  invertAllSelections,
  setSortByKey
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class AddUserView extends React.Component<Props, any> {

  handleUserSelection = (selectedUser: AddableUserFrontendExclusive, isSelected: boolean) =>  {
    this.props.setUserSelected(selectedUser, isSelected)
  }

  render(): JSX.Element {

    const list: ReadonlyArray<AddableUserFrontendExclusive> =
      this.props.sortDirection === SortDirection.none
        ? this.props.users
        : orderBy(this.props.users, user => user[this.props.sortByKey],
        this.props.sortDirection === SortDirection.ascending ? 'asc' : 'desc')

    return (
      <div>

        <div className="two-flexed-stretch-left">

          <div className="flexed">
            <div>
              <Button circular icon='retweet' className="mar-right-half"
                      onClick={() => this.props.invertAllSelections()}
              />
              <span>
                {
                  translate(TranslateIds.invertSelection)
                }
              </span>
            </div>

          </div>
        </div>

        <Table sortable selectable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <div>
                  <Checkbox
                    checked={this.props.users.every(p => p.uiIsSelected) && this.props.users.length > 0}
                    onChange={(e, data: CheckboxOnChangeEventData) => {
                      this.props.setAllUsersSelected(data.checked)
                    }}
                  />
                </div>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<AddableUserFrontendExclusive>(this.props.sortByKey, 'lastName',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastName', this.props.sortDirection + 1)
                      }}
                >

                  {
                    translate(TranslateIds.lastName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumn<AddableUserFrontendExclusive>(this.props.sortByKey, 'firstName',
                  this.props.sortDirection)
              }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('firstName', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.firstName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumn<AddableUserFrontendExclusive>(this.props.sortByKey, 'token',
                  this.props.sortDirection)
              }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('token', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.token)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<AddableUserFrontendExclusive>(this.props.sortByKey, 'isActive',
                    this.props.sortDirection)
                }
              >
                <Icon name="user outline"/>
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('isActive', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.isActive)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumn<AddableUserFrontendExclusive>(this.props.sortByKey, 'email',
                  this.props.sortDirection)
              }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('email', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.email)
                  }
                </span>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((user, index) => {
                //console.log("render")
                //don't use arrow function to handle this because this would
                //return always a new function (??) and thus all items are re rendered...

                //TODO maybe use everytime es next approach (https://daveceddia.com/avoid-bind-when-passing-props/)
                return (
                  <ListViewItem key={user.id} user={user}
                                setUserSelected={this.handleUserSelection}
                  />
                )
              })
            }
          </Table.Body>
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(AddUserView)