/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {
  removeMultipleMemberAndRefreshAsync, setIsAddUsersDialogDisplayed,
  setViewState
} from "../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {Icon} from "semantic-ui-react";
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import ViewStateChanger from '../../helpers/viewStateChanger'
import CommonHeaderBarContentWrapper from '../../commonHeaderBarContentWrapper'
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {askDialog} from "../../../helpers/dialogHelper";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    viewState: rootState.singleGroupState.viewState,
    isLoading: rootState.singleGroupState.isLoading,
    groupId: rootState.singleGroupState.selectedGroupId,
    members: rootState.singleGroupState.members
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setViewState,
  setIsAddUsersDialogDisplayed,
  removeMultipleMemberAndRefreshAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class headerBarContent extends React.Component<Props, any> {
  render(): JSX.Element {

    const isSomeUserSelected = this.props.members.some(p => p.uiIsSelected)

    const currentUser = this.props.members.find(p => p.isCurrentUser)
    // noinspection TsLint
    let currentUserGroupRole: GroupRoleWithPermissionFullBase = null
    if (currentUser && currentUser.groupRole) {
      currentUserGroupRole = currentUser.groupRole
    }

    const leftArea = (
      <div>

        {
          (currentUserGroupRole && currentUserGroupRole.canAddUserToGroup) &&
          <div className="clickable"
               onClick={() => this.props.setIsAddUsersDialogDisplayed(true)}
          >
            <Icon name="plus"/>
            <span>
            {
              translate(TranslateIds.addMember)
            }
          </span>
          </div>
        }

        {
          (currentUserGroupRole && currentUserGroupRole.canAddUserToGroup) &&
          <SimpleVDivider/>
        }

        {
          (currentUserGroupRole && currentUserGroupRole.canRemoveMemberFromGroup) &&
          <div className={isSomeUserSelected ? 'clickable' : 'div-disabled'}
               onClick={async () => {
                 const userIds =
                   this.props.members
                     .filter(p => p.uiIsSelected)
                     .map(p => p.id)

                 // noinspection TsLint
                 let shouldRelease = true

                 if (userIds.length > 0) {

                   shouldRelease = await askDialog(translate(TranslateIds.releaseMultipleMembersFromGroup_title),
                     `${translate(TranslateIds.releaseMultipleMembersFromGroup_content)} (${userIds.length} users)`
                   )
                 }

                 if (shouldRelease) {
                   this.props.removeMultipleMemberAndRefreshAsync(userIds, this.props.groupId)
                 }

               }}
          >
            <Icon name="remove"/>
            <span>
            {
              translate(TranslateIds.releaseSelectedMemberFromGroup)
            }
               </span>
          </div>
        }


      </div>
    )

    const rightArea = (
      <ViewStateChanger viewState={this.props.viewState}
                        setViewStateFunc={(viewState) => this.props.setViewState(viewState)}/>
    )

    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea} rightArea={rightArea} disabled={this.props.isLoading}/>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(headerBarContent)