/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Table, Icon, Checkbox, Popup} from "semantic-ui-react";
import SelectOptionsList, {SelectableOption} from '../../helpers/selectOptionList'
import {
  removeMemberAndRefreshAsync, setAllMemberUiIsSelected,
  setMemberGroupRoleAndRefreshAsync, setMemberUiIsSelected, setSortByKey,
  setUiIsChangeGroupRolePopupDisplayedAction
} from "../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {CheckboxOnChangeEventData} from "../../../types/reactEvents";
import {leftIconPopupOffset} from "../../../constants";
import {SelectableGroupUserFrontend} from "../../../types/group";
import {isCurrentSortingColumn} from "../../../helpers/tablesHelper";
import {SortDirection} from "../../../types/sortDirection";
import ListViewItem from './listViewItem'
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";
import orderBy from 'lodash-es/orderBy'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly member: SelectableGroupUserFrontend
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    members: rootState.singleGroupState.members,
    groupRoles: rootState.singleGroupState.groupRoles,
    groupId: rootState.singleGroupState.selectedGroupId,
    sortByKey: rootState.singleGroupState.sortByKey,
    sortDirection: rootState.singleGroupState.sortDirection,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setMemberGroupRoleAndRefreshAsync,
  setUiIsChangeGroupRolePopupDisplayedAction,
  removeMemberAndRefreshAsync,
  setAllMemberUiIsSelected,
  setMemberUiIsSelected,
  setSortByKey
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class MemberListView extends React.Component<Props, any> {
  render(): JSX.Element {

    const list: ReadonlyArray<SelectableGroupUserFrontend> =
      this.props.sortDirection === SortDirection.none
        ? this.props.members
        : orderBy(this.props.members, user => user[this.props.sortByKey],
        this.props.sortDirection === SortDirection.ascending ? 'asc' : 'desc')

    const currentUser = this.props.members.find(p => p.isCurrentUser)
    // noinspection TsLint
    let currentUserGroupRole: GroupRoleWithPermissionFullBase = null
    if (currentUser && currentUser.groupRole) {
      currentUserGroupRole = currentUser.groupRole
    }


    return (
      <div>
        <Table sortable selectable padded striped singleLine className="material-table">

          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <Checkbox
                  checked={
                    this.props.members.every(p => p.uiIsSelected || p.isCurrentUser) && this.props.members.length > 0
                  }
                  onChange={(e, data: CheckboxOnChangeEventData) => {
                    this.props.setAllMemberUiIsSelected(data.checked)
                  }}
                />
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableGroupUserFrontend>(this.props.sortByKey, 'lastName',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastName', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.lastName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableGroupUserFrontend>(this.props.sortByKey, 'firstName',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('firstName', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.firstName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableGroupUserFrontend>(this.props.sortByKey, 'token',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('token', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.token)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableGroupUserFrontend>(this.props.sortByKey, 'isActivated',
                    this.props.sortDirection)
                }
              >
                <Popup
                  trigger={<Icon name="user outline" className="hoverable"/>}
                  content={translate(TranslateIds.isActivated_HelpText)}
                  wide
                />
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('isActivated', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.isActive)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableGroupUserFrontend>(this.props.sortByKey, 'email',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('email', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.email)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableGroupUserFrontend>(this.props.sortByKey, 'groupRole',
                    this.props.sortDirection)
                }
              >
                <Icon name="law" className="mar-right-half"/>
                <span>
                  {
                    translate(TranslateIds.groupRole)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    translate(TranslateIds.releaseMemberFromGroup)
                  }
                </span>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((member, index) => {
                return (
                  <ListViewItem key={member.id} member={member} currentUserGroupRole={currentUserGroupRole} />
                )
              })
            }
          </Table.Body>
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(MemberListView)