/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Icon, Divider, Breadcrumb} from "semantic-ui-react";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../material/materialSearchInput'
import MemberListView from './memberListView'
import {tryGet} from "../../../helpers/typeHelpers";
import MemberCardsView from './memberCardsView'
import {UserGroupFullBase} from "../../../types/group";
import * as constants from '../../../constants'
import {Link} from "react-router-dom";
import {SiteViewState} from "../../../types/siteView";
import AddUserDialog from './dialogs/addUserDialog/addUserDialog'
import {RouteComponentProps, withRouter} from "react-router";
import {loadSingleGroupSite, setSelectedGroupId} from "../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {parseId} from "../../../helpers/convertersAndTransformers";
import Spinner from "../../helpers/Spinner";
import {getGroupsAsync} from "../../../state/actions/groupsSite/groupsCrudActions";
import {BackArrow} from "../../helpers/backArrow";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    //we cannot do this another way because there is no backend func to give us the group + members + roles...
    groups: rootState.singleGroupState.groups,
    groupId: rootState.singleGroupState.selectedGroupId,
    viewState: rootState.singleGroupState.viewState,
    isLoading: rootState.singleGroupState.isLoading,
    isAddUsersDialogDisplayed: rootState.singleGroupState.isAddUsersDialogDisplayed,
    groupRoles: rootState.singleGroupState.groupRoles,
    currentUserId: rootState.userDataSettingsSate.userData.id
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  loadSingleGroupSite,
  getGroupsAsync,
  setSelectedGroupId
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

interface MatchedProps {
  readonly id: string
}

class SingleGroupSite extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  componentWillMount(): void {

    const groupId = parseId(this.props.match.params.id)

    if (groupId !== null) {

      if (this.props.groups === null || this.props.groups.length === 0) {
        //this is possible if a user opens this page directly in the browser
        this.props.loadSingleGroupSite(groupId, true)
        return
      }

      //always load because loadSingleGroupSite will reset the state...
      this.props.loadSingleGroupSite(groupId, false)
    }

  }

  render(): JSX.Element {

    const group = this.props.groups.find((g) => g.id === this.props.groupId)

    return (
      <div>
        <SinglePanelSiteWrapper>

          {
            this.props.isLoading &&
            <Spinner/>
          }

          {
            !this.props.isAddUsersDialogDisplayed &&
            <div className="view-padding">
              <div className="view-head-line">
                <div className="view-caption">

                  <div className="mar-right back-arrow">
                    <Link to={constants.groupsLinkPath}>
                      <BackArrow/>
                    </Link>
                  </div>

                  <h1 className="clear-margin">
                    {
                      group &&
                      group.displayName
                    }
                  </h1>

                </div>
                <div className="view-options">
                  <MaterialSearchInput/>
                </div>
              </div>

              <div className="view-content">
                {
                  this.props.viewState === SiteViewState.List &&
                  <MemberListView/>
                }
                {
                  this.props.viewState === SiteViewState.Cards &&
                  <MemberCardsView/>
                }
              </div>
            </div>
          }

          {
            this.props.isAddUsersDialogDisplayed &&
            <AddUserDialog/>
          }


        </SinglePanelSiteWrapper>
      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(SingleGroupSite))