/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Card, Icon, Checkbox, Divider} from "semantic-ui-react";
import {openEditSystemRoleDialog} from "../../../state/actions/systemRolesSite/systemRolesSiteActions";
import {deleteSystemRolesAsync} from "../../../state/actions/systemRolesSite/systemRolesCrudActions";
import {copyToClipboard} from "../../../helpers/clipboard";
import {askDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    systemRoles: rootState.systemRolesState.systemRoles,

    userSystemRole: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole
      : null,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  openEditSystemRoleDialog,
  deleteSystemRolesAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class SystemRolesCardsView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <Card.Group>
          {
            this.props.systemRoles.map((systemRole, index) => {
              return (<Card key={systemRole.id}>
                  <Card.Content>
                    <Card.Header>
                      <span>{systemRole.displayName}</span>
                    </Card.Header>

                    <Card.Meta>
                      <Icon name="clone" style={{visibility: (systemRole.email ? null : 'hidden')}}
                            className="clickable mar-right-half"
                            onClick={() => {
                              copyToClipboard(systemRole.email)
                            }}
                      />
                      <span>{systemRole.email}</span>
                    </Card.Meta>

                    <Card.Description>
                      <Checkbox label={translate(TranslateIds.canCreateGroups)}
                                disabled checked={systemRole.canCreateGroups}/>
                      <Checkbox label={translate(TranslateIds.canChangeGroups)}
                                disabled checked={systemRole.canChangeGroups}/>
                      <Checkbox label={translate(TranslateIds.canDeleteGroups)}
                                disabled checked={systemRole.canDeleteGroups}/>

                      <Divider/>

                      <Checkbox label={translate(TranslateIds.canCreateRoles)} disabled
                                checked={systemRole.canCreateRoles}/>
                      <Checkbox label={translate(TranslateIds.canChangeRoles)} disabled
                                checked={systemRole.canChangeRoles}/>
                      <Checkbox label={translate(TranslateIds.canDeleteRoles)} disabled
                                checked={systemRole.canDeleteRoles}/>

                      <Divider/>

                      <Checkbox label={translate(TranslateIds.canManageNewUsers)} disabled
                                checked={systemRole.canManageNewUsers}/>
                      <Checkbox label={translate(TranslateIds.canChangeOtherUsersSystemRole)} disabled
                                checked={systemRole.canChangeOtherUsersSystemRole}/>
                      <Checkbox label={translate(TranslateIds.canDeleteActivatedUsers)} disabled
                                checked={systemRole.canDeleteActivatedUsers}/>
                      <Checkbox label={translate(TranslateIds.canChangeOtherUserData)} disabled
                                checked={systemRole.canChangeUserData}/>
                      <Checkbox label={translate(TranslateIds.canChangeSystemSettings)} disabled
                                checked={systemRole.canChangeSystemSettings}/>
                      <Checkbox label={translate(TranslateIds.canManageTags)} disabled
                                checked={systemRole.canManageTags}/>

                      <Checkbox label={translate(TranslateIds.canViewDashboard)} disabled
                                checked={systemRole.canViewDashboard}/>


                    </Card.Description>

                  </Card.Content>

                  <Card.Content extra>
                    <div className="two-flexed">
                      {
                        (this.props.userSystemRole && this.props.userSystemRole.canChangeRoles) &&
                        <div className="clickable"
                             onClick={() => {
                               this.props.openEditSystemRoleDialog(systemRole)
                             }}
                        >
                          <Icon name="write"/>
                          <span>
                          {
                            translate(TranslateIds.changeSystemRole)
                          }
                        </span>
                        </div>
                      }
                      {
                        (this.props.userSystemRole && this.props.userSystemRole.canDeleteRoles) &&
                        <div className="clickable"
                             onClick={async () => {

                               const shouldRemove = await askDialog(translate(TranslateIds.deleteSystemRole_title),
                                 `${translate(TranslateIds.deleteSystemRole_content)} (${systemRole.displayName})`
                               )

                               if (shouldRemove) {
                                 this.props.deleteSystemRolesAsync(systemRole)
                               }

                             }}
                        >
                          <Icon name="remove"/>
                          <span>
                          {
                            translate(TranslateIds.deleteSystemRole)
                          }
                        </span>
                        </div>
                      }
                    </div>
                  </Card.Content>
                </Card>
              )
            })
          }
        </Card.Group>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(SystemRolesCardsView)