/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Icon} from "semantic-ui-react";
import {HelpPopup} from "../../../helpers/helpPopup";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {
  resetDoExerciseSolutionToTemplate,
  setIsDoCodeEditorSettingsDialogDisplayed
} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {openEditEditorSettingsDialog} from "../../../../state/actions/editorSettingsDialog/editorSettingsDialogActions";
import {
  resetTutorEditorToUserSolution,
  setSolutionFilesTutorEditor,
  setTutorViewIsCodeEditorSettingsDialogDisplayed
} from "../../../../state/actions/tutorViewSite/tutorViewSiteActions";
import {askDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    codeEditorSettings: rootState.userDataSettingsSate.userData.settings ?
      rootState.userDataSettingsSate.userData.settings.codeEditorSettings
      : null,

    userSolution: rootState.tutorViewSiteState.userSolution,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setTutorViewIsCodeEditorSettingsDialogDisplayed,
  openEditEditorSettingsDialog,
  resetTutorEditorToUserSolution
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelHeaderBar extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed">

        <div className="clickable"
             onClick={() => {
               this.props.openEditEditorSettingsDialog(this.props.codeEditorSettings,
                 this.props.setTutorViewIsCodeEditorSettingsDialogDisplayed)
             }}
        >
          <Icon name="settings"/>
          <span>
            {
              translate(TranslateIds.editorSettings)
            }
          </span>
        </div>

        <SimpleVDivider/>

        <div className="clickable"
             onClick={async () => {

               const shouldReset = await askDialog(translate(TranslateIds.resetTutorEditorCode_title),
                 translate(TranslateIds.resetTutorEditorCode_content)
               )

               if (shouldReset) {
                 this.props.resetTutorEditorToUserSolution(this.props.userSolution)
               }
             }}
        >
          <Icon name="undo"/>
          <span>
            {
              translate(TranslateIds.resetTutorEditorCode)
            }
          </span>
        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelHeaderBar)