/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import Spinner from "../../helpers/Spinner";
import {
  loadTutorViewSite,
  setBotLeftPanelViewSize, setBotRightPanelViewSize, setBottomLeftSelectedTabIndex,
  setBottomRightSelectedTabIndex,
  setDoExerciseHDividerPosOffset,
  setDoExerciseVDividerPosOffset, setSelectedConsoleViewTest, setTopLeftPanelViewSize,
  setTopLeftSelectedTabIndex, setTopRightPanelViewSize,
  setTopRightSelectedTabIndex, setTutorViewDummy, setUseDiffIfOutputMismatched, setUseReplaceWhiteSpacesWithBullets
} from "../../../state/actions/tutorViewSite/tutorViewSiteActions";
import PanelGrid from "../../panelSystem/panelGrid/panelGrid";
import TabContentHost from "../../panelSystem/panel/tabContentHost";
import Panel from '../../panelSystem/panel/panel'
import {RouteComponentProps, withRouter} from "react-router";
import {parseId} from "../../../helpers/convertersAndTransformers";

import PreviewPanelView from '../exerciseEditorSite/previewPanel/previewPanelView'

import TestsPanelView from '../doExerciseSite/testsPanel/testsPanelView'

import ConsolePanelView from '../doExerciseSite/consolePanel/consolePanelView'
import ConsolePanelHeaderBar from '../doExerciseSite/consolePanel/consolePanelHeaderBar'
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {
  cancelViewTestDialog,
  openViewTestDialog
} from "../../../state/actions/tutorViewSite/dialogs/viewTestDialog/viewTestDialogActions";
import ViewTestDialog from '../doExerciseSite/testsPanel/viewTestDialog/viewTestDialog'
import {debugOriginUrl} from "../../../../debug";
import SubmitTestsPanelHeaderBar from '../doExerciseSite/submitTestsPanel/submitTestsPanelHeaderBar'
import EditorSettingsDialog from '../../editorSettingsDialog/editorSettingsDialog'

import UserSolutionEditorPanelHeaderBar from './userSolutionEditorPanel/editorPanelHeaderBar'
import UserSolutionEditorPanel from './userSolutionEditorPanel/editorPanelView'

import TutorEditorPanelHeaderBar from './tutorEditorPanel/editorPanelHeaderBar'
import TutorEditorPanel from './tutorEditorPanel/editorPanelView'
import {
  runNormalTestAsTutor,
  runSubmitTestAsTutor
} from "../../../state/actions/tutorViewSite/tutorViewSiteCrudActions";
import {RunAfterTestCommandForBackend} from "../../../types/testsAndTestProtocol";

import NormalTestsForTutorPanelHeaderBar from './normalTestsPanel/normalTestsPanelHeaderBar'
import SubmitTestsForTutorPanelHeaderBar from './submitTestsPanel/submitTestsPanelHeaderBar'

import ManualAssessmentPanelView from './manualAssessmentPanel/manualAssessmentPanelView'
import ManualAssessmentPanelHeaderBar from './manualAssessmentPanel/manualAssessmentPanelHeaderBar'
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {errorDialog} from "../../../helpers/dialogHelper";
import {LoginTimeoutHelper} from "../../../helpers/loginTimeoutHelper";
import {createAndRefreshAssetUrlsAndObjects, freeAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import {setDoExerciseDummy} from "../../../state/actions/doExerciseSite/doExerciseSiteActions";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    exercise: rootState.tutorViewSiteState.exercise,

    normalTestsForTutor: rootState.tutorViewSiteState.normalTestsForTutor,
    submitTestsForTutor: rootState.tutorViewSiteState.submitTestsForTutor,

    submitTestResults: rootState.tutorViewSiteState.submitTestResults,
    userSolution: rootState.tutorViewSiteState.userSolution,
    userSolutionPLangId: rootState.tutorViewSiteState.userSolution.pLangId,

    tutorEditorSolution: rootState.tutorViewSiteState.tutorEditorSolution,

    testTypes: rootState.testTypesState.testTypes,

    useDiffIfOutputMismatched: rootState.tutorViewSiteState.useDiffIfOutputMismatched,
    useReplaceWhiteSpacesWithBullets: rootState.tutorViewSiteState.useReplaceWhiteSpacesWithBullets,

    selectedConsoleViewTest: rootState.tutorViewSiteState.selectedConsoleViewTest,


    isNoDialogDisplayed:
    !rootState.tutorViewSiteState.isViewTestDialogDisplayed
    && !rootState.tutorViewSiteState.isCodeEditorSettingsDialogDisplayed,

    topRightSelectedTabIndex: rootState.tutorViewSiteState.topRightSelectedTabIndex,
    topLeftSelectedTabIndex: rootState.tutorViewSiteState.topLeftSelectedTabIndex,
    bottomRightSelectedTabIndex: rootState.tutorViewSiteState.bottomRightSelectedTabIndex,
    bottomLeftSelectedTabIndex: rootState.tutorViewSiteState.bottomLeftSelectedTabIndex,

    topRightPanelViewSize: rootState.tutorViewSiteState.topRightPanelViewSize,
    topLeftPanelViewSize: rootState.tutorViewSiteState.topLeftPanelViewSize,
    botRightPanelViewSize: rootState.tutorViewSiteState.botRightPanelViewSize,
    botLeftPanelViewSize: rootState.tutorViewSiteState.botLeftPanelViewSize,

    hDividerPosOffset: rootState.tutorViewSiteState.hDividerPosOffset,
    vDividerPosOffset: rootState.tutorViewSiteState.vDividerPosOffset,

    isSomeTestRunning: rootState.tutorViewSiteState.exercise.tests.some(p => p.uiIsRunning)
    || rootState.tutorViewSiteState.exercise.customTests.some(p => p.uiIsRunning)
    || rootState.tutorViewSiteState.normalTestsForTutor.some(p => p.uiIsRunning)
    || rootState.tutorViewSiteState.submitTestsForTutor.some(p => p.uiIsRunning)
    ,


    isViewTestDialogDisplayed: rootState.tutorViewSiteState.isViewTestDialogDisplayed,
    isCodeEditorSettingsDialogDisplayed: rootState.tutorViewSiteState.isCodeEditorSettingsDialogDisplayed,

    isLoading: rootState.tutorViewSiteState.isLoading
    || rootState.testTypesState.isLoading
    || rootState.pLangsState.isLoading
    || rootState.systemSettingsState.isLoading,

    releaseFinishedAndAutomaticAssessmentHasFinished: !rootState.tutorViewSiteState.exercise.exerciseRelease.isReleased &&
    rootState.tutorViewSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,

    dummy: rootState.tutorViewSiteState.dummy,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setDoExerciseHDividerPosOffset,
  setDoExerciseVDividerPosOffset,
  setTopLeftSelectedTabIndex,
  setTopLeftPanelViewSize,
  setTopRightSelectedTabIndex,
  setTopRightPanelViewSize,
  setBottomLeftSelectedTabIndex,
  setBotLeftPanelViewSize,
  setBottomRightSelectedTabIndex,
  setBotRightPanelViewSize,

  loadTutorViewSite,
  setTutorViewDummy,
  setSelectedConsoleViewTest,
  openViewTestDialog,
  cancelViewTestDialog,

  runNormalTestAsTutor,
  runSubmitTestAsTutor,
  setUseDiffIfOutputMismatched,
  setUseReplaceWhiteSpacesWithBullets,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

interface MatchedProps {
  readonly releaseId: string
  readonly userId: string
  readonly pLangId: string
}

class tutorViewSite extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {


  async componentWillMount(): Promise<void> {

    const releaseId = parseId(this.props.match.params.releaseId)
    const userId = parseId(this.props.match.params.userId)
    const pLangId = parseId(this.props.match.params.pLangId)

    if (releaseId === null || userId === null || pLangId === null) {
      errorDialog(
        translate(TranslateIds.error),
        translate(TranslateIds.someUrlParametersNotFound)
      )
      return
    }

    const viewId = `${releaseId}-${userId}-${pLangId}`

    if (LoginTimeoutHelper.isReLogin && viewId === LoginTimeoutHelper.lastKnownTutorAssessmentId) {
      //do not reload or reset site state...
      //user re logged in he/she want's to continue
      //if we would reload then the code would be gone (even though the frontend settings would store a copy... but that's bad style)

      //unmount cleared everything...
      createAndRefreshAssetUrlsAndObjects(this.props.exercise.exerciseDescription.assets)
      setTutorViewDummy(false)

      return
    } else {
      LoginTimeoutHelper.lastKnownTutorAssessmentId = viewId
    }

    this.props.loadTutorViewSite(releaseId, userId, pLangId)

  }

  componentWillUnmount(): void {
    //if we not freeing we would leak memory... if we just exit the site
    freeAssetUrlsAndObjects()
    setTutorViewDummy(true) //enable re rendering
  }

  render(): JSX.Element {
    return (
      <div className="fh">

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          this.props.isViewTestDialogDisplayed &&
          <ViewTestDialog
            cancelViewTestDialog={() => {
              this.props.cancelViewTestDialog()
            }}
            getDownloadTestAssetLink={(testId: number, fileId: number, isSubmitTest: boolean) => {
              return `${debugOriginUrl}/api/assets/${this.props.exercise.exerciseRelease.releaseCode}/${testId}/${fileId}/${isSubmitTest}/true`
            }}
          />
        }


        {
          this.props.isCodeEditorSettingsDialogDisplayed &&
          <EditorSettingsDialog/>
        }

        {
          this.props.isNoDialogDisplayed &&
          <PanelGrid
            width="100%"
            height="100%"

            topRightPanelViewSize={this.props.topRightPanelViewSize}
            topLeftPanelViewSize={this.props.topLeftPanelViewSize}
            botRightPanelViewSize={this.props.botRightPanelViewSize}
            botLeftPanelViewSize={this.props.botLeftPanelViewSize}

            hDividerPosOffset={this.props.hDividerPosOffset}
            vDividerPosOffset={this.props.vDividerPosOffset}
            onHDividerPosOffsetChanged={(Offset) => {
              this.props.setDoExerciseHDividerPosOffset(Offset)
            }}
            onVDividerPosOffsetChanged={(Offset) => {
              this.props.setDoExerciseVDividerPosOffset(Offset)
            }}

            topLeftPanel={
              <Panel
                applyDefaultPadding={false}
                selectedTabIndex={this.props.topLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopLeftSelectedTabIndex(index)
                }}

                onSizeChangedClicked={this.props.setTopLeftPanelViewSize}
                panelViewSize={this.props.topLeftPanelViewSize}
              >
                <TabContentHost
                  title={translate(TranslateIds.tutorEditor)}
                  icon={'font'}
                  applyDefaultPadding={false}
                  defaultHelpText={translate(TranslateIds.tutorEditor_helpText)}
                  tabHeaderBar={
                    <TutorEditorPanelHeaderBar/>
                  }
                >
                  <TutorEditorPanel/>
                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.submissionCodeEditor)}
                  icon={'send outline'}
                  applyDefaultPadding={false}
                  defaultHelpText={translate(TranslateIds.submissionCode_helpText)}
                  tabHeaderBar={
                    <UserSolutionEditorPanelHeaderBar/>
                  }
                >
                  <UserSolutionEditorPanel/>
                </TabContentHost>

              </Panel>
            }

            topRightPanel={
              <Panel
                selectedTabIndex={this.props.topRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setTopRightPanelViewSize}
                panelViewSize={this.props.topRightPanelViewSize}
              >
                <TabContentHost
                  title={translate(TranslateIds.exerciseTaskDescription)}
                  icon={'align left'}
                >

                  <PreviewPanelView descriptionContent={this.props.exercise.exerciseDescription.content}
                                    assets={this.props.exercise.exerciseDescription.assets}
                                    dummy={this.props.dummy}
                  />

                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.manualAssessment)}
                  icon={'flag'}
                  tabHeaderBar={
                    this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <ManualAssessmentPanelHeaderBar/>
                  }
                >
                  {
                    this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <ManualAssessmentPanelView/>
                  }
                  {
                    !this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <div>
                      {
                        translate(TranslateIds.theReleaseIsStillOpenOrAssessmentNotYetFinished)
                      }
                    </div>
                  }


                </TabContentHost>


              </Panel>
            }

            bottomLeftPanel={
              <Panel
                selectedTabIndex={this.props.bottomLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomLeftSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotLeftPanelViewSize}
                panelViewSize={this.props.botLeftPanelViewSize}
              >
                {
                  //readonly test results
                }
                <TabContentHost
                  title={translate(TranslateIds.normalTests)}
                  icon={'bug'}
                  tabHeaderBar={null}
                >
                  <TestsPanelView
                    displayWeightIndicatorsForResults={false}
                    tests={this.props.exercise.tests}
                    selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                    isUserAllowedToRunTests={false}
                    editorHasUnsavedChanges={false}
                    isSomeTestRunning={false}
                    onSaveSolution={null}
                    setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.setSelectedConsoleViewTest(test)
                    }}

                    onRunTest={null}
                    openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.openViewTestDialog(runnableTest)
                    }}
                  />

                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.submitTests)}
                  icon={'flag checkered'}
                  tabHeaderBar={

                    this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <SubmitTestsPanelHeaderBar submitTestResults={this.props.submitTestResults}/>

                  }
                  defaultHelpText={translate(
                    TranslateIds.theSubmitTestResultsAreOnlyDisplayedAfterAutomaticAssessmentHasFinished)}
                >

                  {
                    !this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <div>
                      {
                        translate(TranslateIds.theReleaseIsStillOpenOrAssessmentNotYetFinished)
                      }
                    </div>
                  }
                  {
                    this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <TestsPanelView
                      displayWeightIndicatorsForResults={true}
                      tests={this.props.submitTestResults}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      isUserAllowedToRunTests={false}
                      editorHasUnsavedChanges={false}
                      isSomeTestRunning={false}
                      onSaveSolution={null}
                      setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.setSelectedConsoleViewTest(test)
                      }}

                      onRunTest={null}
                      openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.openViewTestDialog(runnableTest)
                      }}
                    />
                  }

                </TabContentHost>

                {
                  // not readonly tests (so that the tutor don't change the result...
                }

                <TabContentHost
                  title={translate(TranslateIds.tutorNormalTests)}
                  icon={'bug'}
                  tabHeaderBar={
                    <NormalTestsForTutorPanelHeaderBar/>
                  }
                >
                  <TestsPanelView
                    displayWeightIndicatorsForResults={false}
                    tests={this.props.normalTestsForTutor}
                    selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                    isUserAllowedToRunTests={true}
                    editorHasUnsavedChanges={false}
                    isSomeTestRunning={this.props.isSomeTestRunning}
                    onSaveSolution={null}
                    setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.setSelectedConsoleViewTest(test)
                    }}

                    onRunTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive) => {

                      const runTestAsTutorCommand: RunAfterTestCommandForBackend = {
                        releaseCode: this.props.exercise.exerciseRelease.releaseCode,
                        isSubmitTest: false,
                        testIds: [test.id],
                        isNormalTest: true,
                        plangId: this.props.tutorEditorSolution.pLangId,
                        mainFileId: this.props.tutorEditorSolution.mainFileId,
                        solutionFiles: this.props.tutorEditorSolution.solutionFiles
                      }
                      this.props.runNormalTestAsTutor(runTestAsTutorCommand)
                    }}
                    openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.openViewTestDialog(runnableTest)
                    }}
                  />

                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.tutorSubmitTests)}
                  icon={'flag checkered'}
                  tabHeaderBar={
                    this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <SubmitTestsForTutorPanelHeaderBar/>
                  }
                >
                  {
                    !this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <div>
                      {
                        translate(TranslateIds.theReleaseIsStillOpenOrAssessmentNotYetFinished)
                      }
                    </div>
                  }
                  {
                    this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <TestsPanelView
                      displayWeightIndicatorsForResults={true}
                      tests={this.props.submitTestsForTutor}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      isUserAllowedToRunTests={true}
                      editorHasUnsavedChanges={false}
                      isSomeTestRunning={this.props.isSomeTestRunning}
                      onSaveSolution={null}
                      setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.setSelectedConsoleViewTest(test)
                      }}

                      onRunTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive) => {

                        const runTestAsTutorCommand: RunAfterTestCommandForBackend = {
                          releaseCode: this.props.exercise.exerciseRelease.releaseCode,
                          isSubmitTest: true,
                          testIds: [test.id],
                          isNormalTest: false,
                          plangId: this.props.tutorEditorSolution.pLangId,
                          mainFileId: this.props.tutorEditorSolution.mainFileId,
                          solutionFiles: this.props.tutorEditorSolution.solutionFiles
                        }
                        this.props.runSubmitTestAsTutor(runTestAsTutorCommand)
                      }}
                      openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.openViewTestDialog(runnableTest)
                      }}
                    />
                  }
                </TabContentHost>

              </Panel>
            }

            bottomRightPanel={
              <Panel
                selectedTabIndex={this.props.bottomRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotRightPanelViewSize}
                panelViewSize={this.props.botRightPanelViewSize}
              >
                <TabContentHost
                  title={translate(TranslateIds.consoleView)}
                  icon={'terminal'}
                  tabHeaderBar={
                    <ConsolePanelHeaderBar test={this.props.selectedConsoleViewTest} testTypes={this.props.testTypes}
                                           useDiffIfOutputMismatched={this.props.useDiffIfOutputMismatched}
                                           setUseDiffIfOutputMismatched={this.props.setUseDiffIfOutputMismatched}
                                           useReplaceWhiteSpacesWithBullets={this.props.useReplaceWhiteSpacesWithBullets}
                                           setUseReplaceWhiteSpacesWithBullets={this.props.setUseReplaceWhiteSpacesWithBullets}
                    />
                  }
                >
                  <ConsolePanelView test={this.props.selectedConsoleViewTest} protocolItems={null}
                                    testTypes={this.props.testTypes}
                                    useDiffIfOutputMismatched={this.props.useDiffIfOutputMismatched}
                                    useReplaceWhiteSpacesWithBullets={this.props.useReplaceWhiteSpacesWithBullets}
                  />
                </TabContentHost>
              </Panel>
            }
          />
        }

      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(tutorViewSite))