/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import MultiTabCodeEditorWrapper from "../../../codeEditors/multiTabCodeEditorWrapper";
import {
  closeTabChangeUserSolutionEditorTutorView,
  openTabChangeUserSolutionEditorTutorView,
  setIsFileExplorerDisplayedUserSolutionEditorTutorView,
  setSelectedTabIdUserSolutionEditorTutorView
} from "../../../../state/actions/tutorViewSite/editorActions/userSolutionEditorActions";
import {SolutionFileDoExerciseFullBase} from "../../../../types/exerciseSolution";
import Logger from "../../../../helpers/logger";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {EditorTab, MultiTabEditorState} from "../../../codeEditors/multiTabCodeEditorProps";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    pLangs: rootState.pLangsState.pLangs,
    solution: rootState.tutorViewSiteState.userSolution,

    selectedTabIdDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.selectedTabIdDoEditor,
    isFileExplorerDisplayedDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.isFileExplorerDisplayedDoEditor,
    newFileNameDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.newFileNameDoEditor,
    renamingFileIdDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.renamingFileIdDoEditor,
    isRenamingFileDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.isRenamingFileDoEditor,
    renamingFileNameDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.renamingFileNameDoEditor,
    hasUnsavedChanges: rootState.tutorViewSiteState.userSolutionEditorState.hasUnsavedChanges,
    hasFileChangedMap: rootState.tutorViewSiteState.userSolutionEditorState.hasFileChangedMap,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setIsFileExplorerDisplayedUserSolutionEditorTutorView,
  setSelectedTabIdUserSolutionEditorTutorView,

  openTabChangeUserSolutionEditorTutorView,
  closeTabChangeUserSolutionEditorTutorView,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelView extends React.Component<Props, any> {


  componentWillMount(): void {

  }

  render(): JSX.Element {

    const pLang = this.props.pLangs.find(p => p.id === this.props.solution.pLangId)

    if (!pLang) {
      //TODO only error if really not found
      //on first render we have no plang but thats no and error!!
      return null
    }

    const editorTabs = this.props.solution.solutionFiles.map<EditorTab>(p => {
      return {
        id: p.id,
        fileNameWithExtensions: p.fileNameWithExtension,
        content: p.content,
        isMainFile: p.id === this.props.solution.mainFileId,
        isReadonly: true,
        displayIndex: p.displayIndex,
        language: pLang.internalName,
      }
    })

    return (
      <div className="fh hidden-overflow-i">
        <MultiTabCodeEditorWrapper
          id="editorPanelView"
          height="100%"
          selectedTabId={this.props.selectedTabIdDoEditor}
          forceEditorUpdateSessionDummy={MultiTabEditorState.normal}
          onSelectedTabIdChanged={(tabId: number) => {
            this.props.setSelectedTabIdUserSolutionEditorTutorView(tabId)
          }}

          isFileExplorerDisplayed={this.props.isFileExplorerDisplayedDoEditor}
          onChangeIsFileExplorerDisplayed={(isDisplayed: boolean) => {
            this.props.setIsFileExplorerDisplayedUserSolutionEditorTutorView(isDisplayed)
          }}

          newFileName={this.props.newFileNameDoEditor}
          onNewFileNameChanged={null}

          renamingFileId={this.props.renamingFileIdDoEditor}
          onRenamingFileIdChanged={null}

          isRenamingFile={this.props.isRenamingFileDoEditor}
          onIsRenamingFileChanged={null}

          renamingFileName={this.props.renamingFileNameDoEditor}
          onRenamingFileNameChanged={null}

          hasUnsavedChanges={false}
          onHasUnsavedChangesChanged={null}

          userCanCreateNewFiles={false}

          language={pLang.internalName}

          onCreateFileClicked={null}

          onOpenTabClicked={(editorTab: EditorTab) => {
            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {

              Logger.error("file not found (onOpenTabClicked)")

              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
                )

              return
            }

            const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                ? {
                  ...file,
                  uiIsDisplayed: true,
                }
                : p
            })

            this.props.openTabChangeUserSolutionEditorTutorView(file, mappedFiles, this.props.selectedTabIdDoEditor)
          }}

          onCloseTabClicked={(editorTab: EditorTab) => {

            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (onCloseTabClicked)")

              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )

              return
            }

            const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                ? {
                  ...file,
                  uiIsDisplayed: false,
                }
                : p
            })

            this.props.closeTabChangeUserSolutionEditorTutorView(file, mappedFiles,
              this.props.selectedTabIdDoEditor)
          }}

          onFileRename={null}

          onRemoveFile={null}

          editorTabs={editorTabs}
          onChange={(editorTab: EditorTab, newValue: string) => {
          }}
          onSomeEditorSessionChanged={() => {

          }}

          hasFileChangedMap={this.props.hasFileChangedMap}
          onUpdateHasFileChangedMap={(changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>) => {
            //performance is tooooo bad...
            //this.props.setHasFileChangedMapDoEditor(updateHasFileChangedMap(changedFile, allTabs, this.props.hasFileChangedMap))
          }}

        />
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelView)