/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers/index";
import {TagFullBase} from "../../types/tag";
import SinglePanelSiteWrapper from '../singlePanelSiteWrapper'
import Tag from './tag'
import Spinner from "../helpers/Spinner";
import {translate} from "../../../i18n/translationHelper";
import {TranslateIds} from "../../../i18n/ids";
import {
  setTagFilterSelectedNegativeFilterTagsIds,
  setTagFilterSelectedTagIds
} from "../../state/actions/tagsFilterActions";
import {HelpPopup} from "../helpers/helpPopup";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly allTags: ReadonlyArray<TagFullBase>
  readonly isLoading: boolean

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    allTags: props.allTags,
    selectedTags: rootState.tagsFilterState.selectedFilterTagsIds,
    selectedNegativeFilterTagsIds: rootState.tagsFilterState.selectedNegativeFilterTagsIds,
    isLoading: props.isLoading
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setTagFilterSelectedTagIds,
  setTagFilterSelectedNegativeFilterTagsIds
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


//one can use sortAndFilterExercisePreviews to apply sorting & tags filter
//also for all tags the manage tags site is useful
class tagFilterPanel extends React.Component<Props, any> {

  render(): JSX.Element {

    const tagPair = this.props.allTags.map(p => {
      return {
        tag: p,
        isSelected: this.props.selectedTags.some(id => id === p.id),
        isSelectedNegative: this.props.selectedNegativeFilterTagsIds.some(id => id === p.id)
      }
    })

    return (
      <SinglePanelSiteWrapper outerClassName="tag-filter-sticky">


        <div className="flexed">
          <h2 className="mar-right-half">
            {
              translate(TranslateIds.tagsFilter)
            }
          </h2>
          <HelpPopup style={{marginTop: '0.6em'}} defaultText={translate(TranslateIds.tagFilter_helpText)} wide/>
        </div>


        {
          this.props.isLoading &&
          <Spinner/>
        }

        <div className="flexed separate-tabs">
          {
            this.props.allTags.map(tag => {
              return (
                <div key={tag.id}>
                  <Tag tag={tag}
                       className={
                         tagPair.find(p => p.tag.id === tag.id).isSelected
                           ? 'selected-tag-filter clickable'
                           : tagPair.find(p => p.tag.id === tag.id).isSelectedNegative
                           ? 'clickable selected-negative-tag-filter'
                           : 'clickable selectable-tab-filter'
                       }
                       onClick={(tag: TagFullBase) => {

                         if (this.props.selectedTags.some(id => id === tag.id)) {
                           //after select -->  un select positive, negative select

                           this.props.setTagFilterSelectedTagIds(this.props.selectedTags.filter(id => id !== tag.id))
                           this.props.setTagFilterSelectedNegativeFilterTagsIds(
                             [...this.props.selectedNegativeFilterTagsIds, tag.id])

                         } else {

                           //not selected as positive

                           if (this.props.selectedNegativeFilterTagsIds.some(id => id === tag.id)) {

                             //after negative selected --> un select negative
                             this.props.setTagFilterSelectedNegativeFilterTagsIds(
                               this.props.selectedNegativeFilterTagsIds.filter(id => id !== tag.id))

                           } else {
                             //not yet selected at all --> select
                             this.props.setTagFilterSelectedTagIds([...this.props.selectedTags, tag.id])
                           }
                         }
                       }}
                  />
                </div>
              )
            })
          }
        </div>


      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(tagFilterPanel)