/**
 * Created by janis dähne.
 */
import {debugOriginUrl} from "../debug";
import {AssessmentFullBase} from "./types/submissions";

/**
 * the percentage needed to pass the automatic assessment
 * a message will be displayed to indicate if the user has passed
 *
 * the comparison will be made as
 *
 * user reached points >= percentageNeededToPassedAutomaticAssessment ? --> passed
 *
 * @type {number}
 */
export const percentageNeededToPassedAutomaticAssessment = 50

export const homeLinkPath = '/'
export const dashboardLinkPath = '/dashboard'

export const ownExercisesLinkPath = '/exercises/own'
export const groupExercisesLinkPath = '/exercises/group'

//displays both via code and via visibility
export const openExercisesLinkPath = '/exercises/open/all'
//displays both via code and via visibility
export const closedExercisesLinkPath = '/exercises/closed/all'

//only the releases where the user entered a code
export const openExercisesViaCodeLinkPath = '/exercises/open/code'
//via release setting is visible for all
export const openExercisesViaVisibilityLinkPath = '/exercises/open/visible'

//only the releases where the user entered a code
export const closedExercisesViaCodeLinkPath = '/exercises/closed/code'
//via release setting is visible for all
export const closedExercisesViaVisibilityLinkPath = '/exercises/closed/visible'


export const groupsLinkPath = '/groups'


export const markdownSiteTestSyntaxGuide = '/guides/markdown/tests'
export const markdownSiteExerciseSyntaxGuide = '/guides/markdown/exercise'

/**
 * only for the router ... for programmatic access use getExerciseReleaseLinkPath
 * this is used in the route component
 * @type {string}
 */
export const exerciseReleasesLinkPath = '/exercises/:exerciseId/:comesFromOwnExercisesSite/releases'
export const exerciseReleasesToLinkPath = '/exercises'

export function getExerciseReleaseLinkPath(exerciseId: number, comesFromOwnExercisesSite: boolean): string {
  return `${exerciseReleasesToLinkPath}/${exerciseId}/${comesFromOwnExercisesSite}/releases`
}

export const submissionsSiteLinkPath = `${exerciseReleasesLinkPath}/:releaseId/submissions`

export function getSubmissionsSiteToLinkPath(exerciseId: number, comesFromOwnExercisesSite: boolean, exerciseReleaseId: number): string {
  return `${exerciseReleasesToLinkPath}/${exerciseId}/${comesFromOwnExercisesSite}/releases/${exerciseReleaseId}/submissions`
}


export const tutorViewSiteLinkPartPath = '/editor/tutor'
export const tutorViewSiteLinkMatchPath = '/editor/tutor/:releaseId/:userId/:pLangId'

export function getTutorViewForSubmission(releaseId: number, userId: number, pLangId: number): string {
  return `${tutorViewSiteLinkPartPath}/${releaseId}/${userId}/${pLangId}`
}

/**
 * this is only for the router to know the path
 * but cannot be used to programmatically navigate, use getSingleGroupLinkPath()
 * @type {string}
 */
export const singleGroupLinkPath = '/groups/:id'

export function getSingleGroupLinkPath(groupId: number): string {
  return `${groupsLinkPath}/${groupId}`
}

export const groupRolesLinkPath = '/roles/groups'
export const systemRolesLinkPath = '/roles/system'

export const ownSettingsLinkPath = '/settings'

const manageUsersLinkPathPart = '/manage/users'
//which should be one of the following 2
export const manageNewUsersLinkPath = `${manageUsersLinkPathPart}/new`
export const manageActivatedUsersLinkPath = `${manageUsersLinkPathPart}/activated`

export const manageTagsLinkPath = '/manage/tags'

export const aboutLinkPath = '/about'


export const exerciseEditorCreateNewLinkPath = '/editor/exercises/create'
export const exerciseEditorChangeLinkPath = '/editor/exercises/edit/:exerciseId'
export const exerciseEditorChangeToLinkPath = '/editor/exercises/edit'

export function getExerciseEditorEditToLink(exerciseId: number): string {
  return `${exerciseEditorChangeToLinkPath}/${exerciseId}`
}

export const createCopyFromOldExerciseLinkPath = '/editor/exercises/copy/from'
export const createCopyFromOldExerciseLinkMatchPath = '/editor/exercises/copy/from/:exerciseId'

export function getCreateCopyFromOldExerciseToLinkPath(exerciseId: number): string {
  return `${createCopyFromOldExerciseLinkPath}/${exerciseId}`
}

export const doExerciseSiteLinkPath = '/doeditor/exercises/:releaseCode/:mode?'
export const doExerciseSiteToLinkPath = '/doeditor/exercises'

export function getDoExerciseSiteToLink(releaseCode: string): string {
  return `${doExerciseSiteToLinkPath}/${releaseCode}`
}


export const manageCustomProjectsSiteLinkMatchPath = '/manage/projects/own'


export const createCustomProjectSiteToLinkPath = '/manage/projects/own/create'

export const getCustomProjectLinkPath = '/manage/projects/own/edit'
export const getCustomProjectMatchLinkPath = '/manage/projects/own/edit/:customProjectId'

export function getCustomProjectToLinkPath(customProjectId: number): string {
  return `${getCustomProjectLinkPath}/${customProjectId}`
}

//if we have no targetUserId then use current user as targetUserId
export const assessmentStatisticsSiteLinkMatchPath = '/assessment/statistics/:targetUserId?'
export const assessmentStatisticsSiteToLinkPath = '/assessment/statistics'

export function getAssessmentStatisticsSiteToLinkPath(targetUserId: number): string {
  return `${assessmentStatisticsSiteToLinkPath}/${targetUserId}`
}

// ---- other links

export const submissionDownloadLinkPart = '/api/submissions/download/submission'

//TODO use id not code
/**
 * returns the download link for a specific submission
 * @param {string} releaseCode the release code
 * @param {number} userId the user id
 * @param {number} pLangId the p lang id
 * @returns {string} the link url
 */
export function getSubmissionDownloadLink(releaseCode: string, userId: number, pLangId: number): string {
  return `${debugOriginUrl}${submissionDownloadLinkPart}/${releaseCode}/${userId}/${pLangId}`
}

export function getAllSubmissionDownloadLink(releaseCode: string): string {
  return `${debugOriginUrl}${submissionDownloadLinkPart}/all/${releaseCode}`
}


export const assessmentResultsDownloadLinkPart = '/api/submissions/export/csv'

export function getAssessmentResultsDownloadLink(releaseCode: string, sortKey: keyof AssessmentFullBase, ascending: boolean): string {
  return `${debugOriginUrl}${assessmentResultsDownloadLinkPart}/${releaseCode}/${sortKey}/${ascending}`
}

export const leftIconPopupOffset = 12

export const assetPrefix = 'asset://'

export const showMarkdownCodeLineNumbers = true