/**
 * Created by janis dähne.
 */

import swal from 'sweetalert2'
import {SweetAlertOptions, SweetAlertType} from 'sweetalert2'
import Logger from "./logger";
import {translate} from "../../i18n/translationHelper";
import {TranslateIds} from "../../i18n/ids";


export enum DialogType {
  success,
  warning,
  info,
  question,
  error,
}


function translateDialogType(type?: DialogType): SweetAlertType {

  if (type === DialogType.success) return 'success'

  if (type === DialogType.info) return 'info'

  if (type === DialogType.error) return 'error'

  if (type === DialogType.warning) return 'warning'

  if (type === DialogType.question) return 'question'

  return 'info'
}

export function infoDialog(title: string, message: string): Promise<void> {
  const options: SweetAlertOptions = {
    title: title,
    text: message,
    type: translateDialogType(DialogType.info),
    showConfirmButton: true,
    showCloseButton: false,
    showCancelButton: false,
    confirmButtonText: translate(TranslateIds.ok),
    allowOutsideClick: false,
    allowEnterKey: true,
  }

  Logger.debug("dialogHelper", message)

  return new Promise<void>((resolve) => {

    swal(options)
      .then(() => {
        resolve()
      })
  })
}

export function errorDialog(title: string, message: string): Promise<void> {
  const options: SweetAlertOptions = {
    title: title,
    text: message,
    type: translateDialogType(DialogType.error),
    showConfirmButton: true,
    showCloseButton: false,
    showCancelButton: false,
    confirmButtonText: translate(TranslateIds.ok),
    allowOutsideClick: false,
    allowEnterKey: true,
  }

  Logger.error(message)

  return new Promise<void>((resolve) => {

    swal(options)
      .then(() => {
        resolve()
      })
  })
}

export function warningDialog(title: string, message: string): Promise<void> {
  const options: SweetAlertOptions = {
    title: title,
    text: message,
    type: translateDialogType(DialogType.warning),
    showConfirmButton: true,
    showCloseButton: false,
    showCancelButton: false,
    confirmButtonText: translate(TranslateIds.ok),
    allowOutsideClick: false,
    allowEnterKey: true,
  }

  Logger.warn(message)

  return new Promise<void>((resolve) => {

    swal(options)
      .then(() => {
        resolve()
      })
  })
}

export function askDialog(title: string, message: string): Promise<boolean> {

  const options: SweetAlertOptions = {
    title: title,
    text: message,
    type: translateDialogType(DialogType.question),
    showConfirmButton: true,
    showCloseButton: false,
    showCancelButton: true,
    confirmButtonText: translate(TranslateIds.yes),
    cancelButtonText: translate(TranslateIds.no),
    allowOutsideClick: false,
    allowEnterKey: true,
  }

  return new Promise<boolean>((resolve) => {

    Logger.debug("dialogHelper", `showing dialog title ${title}...`)

    swal(options)
      .then(() => {
        Logger.debug("dialogHelper", `...dialog title ${title} youser took yes`)
        resolve(true)
      }, () => {
        Logger.debug("dialogHelper", `...dialog title ${title} youser took no`)
        resolve(false)
      })
  })


}