/**
 * Created by janis dähne
 */




'use strict'
import {
  AnswerTypes, ExerciseQuestion, QuestionParseResult, QuestionType,
  SurveySyntaxParseResult
} from "../types/questions";
import {startsWith} from "../helpers/stringHelper";

///<reference path="../../../../typings/surveys.d.ts"/>


//this parses the syntax but not interpret the question content

class SurveyParser {

  //static iconKey = '[icon]'

  static SplitString = '\n'

  static answerKey = '[answers'

  static questionKey = '[question'

  /**
   * NOT USED YET BECAUSE NO SURVEYS
   * @param text
   * @returns {SurveySyntaxParseResult}
   */
  static parseSurveyText(text:string):SurveySyntaxParseResult { //vue ready obj

    let parseResult:SurveySyntaxParseResult = {
      errors: [],
      result: {
        //icon: null,
        questions: []
      }
    }

    let lines = text.split(this.SplitString)

    //let iconKeyFound = false
    let questionFound = false
    let answerFound = false

    for (let i = 0; i < lines.length; i++) {
      let line = lines[i];

      if (line === '') continue

      //parse icon
      /*
       if (!iconKeyFound && line === SurveyParser.iconKey) {
       iconKeyFound = true
       i++
       parseResult.result.icon = lines[i]
       continue
       }
       */

      //parse question

      //if (questionFound === false && line.startsWith(SurveyParser.questionKey)) {
      if (questionFound === false && startsWith(line, SurveyParser.questionKey)) {

        i++
        questionFound = true

        /*
         if (answerFound) {
         //error answer key terminates question text
         parseResult.errors.push('[answers] muss nach [question] stehen')
         return parseResult

         } else*/
        //{

        let newQuestion = {
          questionType: '',
          question: '',

          answersType: '',
          possibleAnswers: <string[]>[]
        }


        if (line.indexOf(QuestionType. markdown) !== -1) { //markdown?

          //get markdown

          let searchMarkdown = true
          let markdownTextArray = []
          let markdownText = ''

          while (searchMarkdown && i < lines.length) {

            let markdownLine = lines[i]

            //if (markdownLine.startsWith(SurveyParser.answerKey)) {
            if (startsWith(markdownLine, SurveyParser.answerKey)) {

              line = markdownLine
              //remove the last n empty lines
              while (markdownTextArray.length > 0 && markdownTextArray[markdownTextArray.length - 1].trim() === '') {
                markdownTextArray.pop()
              }

              markdownText = markdownTextArray.join('\n')

              break
            } else {

              //check for special string in question
              let invalidSequence = SurveyParser.containsInvalidSequence(markdownLine)
              if (invalidSequence !== null) {
                if (invalidSequence === SurveyParser.questionKey && questionFound) {

                  return SurveyParser.returnWithError(
                    SurveyParser.errorStringNoAnswerTag(parseResult.result.questions.length), parseResult)

                } else {
                  return SurveyParser.returnWithError(
                    SurveyParser.errorStringInvalidSequence(i, invalidSequence), parseResult)
                }
              }

              //markdownText += markdownLine + '\n'
              markdownTextArray.push(markdownLine)
            }
            i++
          }

          newQuestion.questionType = QuestionType. markdown
          newQuestion.question = markdownText

        }
        else if (line.indexOf(QuestionType.pdf) !== -1) {

          //get pdf name
          let pdfNameWithExtension = lines[i]

          //check for special string in question
          let invalidSequence = SurveyParser.containsInvalidSequence(pdfNameWithExtension)
          if (invalidSequence !== null) {
            if (invalidSequence === SurveyParser.questionKey && questionFound) {

              return SurveyParser.returnWithError(
                SurveyParser.errorStringNoAnswerTag(parseResult.result.questions.length), parseResult)

            } else {
              return SurveyParser.returnWithError(
                SurveyParser.errorStringInvalidSequence(i, invalidSequence), parseResult)
            }
          }

          newQuestion.questionType = QuestionType.pdf
          newQuestion.question = pdfNameWithExtension

        }
        else if (line.indexOf(QuestionType.text) !== -1) { //normal text?

          //get normal text
          let searchText = true
          let text = ''

          while (searchText && i < lines.length) {

            let textLine = lines[i]


            //if (textLine.startsWith(SurveyParser.answerKey)) {
            if (startsWith(textLine, SurveyParser.answerKey)) {
              line = textLine
              break
            } else {

              //check for special string in question
              let invalidSequence = SurveyParser.containsInvalidSequence(textLine)
              if (invalidSequence !== null) {
                if (invalidSequence === SurveyParser.questionKey && questionFound) {

                  return SurveyParser.returnWithError(
                    SurveyParser.errorStringNoAnswerTag(parseResult.result.questions.length), parseResult)

                } else {
                  return SurveyParser.returnWithError(
                    SurveyParser.errorStringInvalidSequence(i, invalidSequence), parseResult)
                }
              }

              text += textLine + '\n'
            }
            i++
          }

          newQuestion.questionType = QuestionType.text
          newQuestion.question = text

        }
        else if (line.indexOf(QuestionType.latex) !== -1) { //latex ?

          //get normal text
          let searchText = true
          let text = ''

          while (searchText && i < lines.length) {

            let textLine = lines[i]

            //if (textLine.startsWith(SurveyParser.answerKey)) {
            if (startsWith(textLine, SurveyParser.answerKey)) {
              line = textLine
              break
            } else {

              //check for special string in question
              let invalidSequence = SurveyParser.containsInvalidSequence(textLine)
              if (invalidSequence !== null) {
                if (invalidSequence === SurveyParser.questionKey && questionFound) {

                  return SurveyParser.returnWithError(
                    SurveyParser.errorStringNoAnswerTag(parseResult.result.questions.length), parseResult)

                } else {
                  return SurveyParser.returnWithError(
                    SurveyParser.errorStringInvalidSequence(i, invalidSequence), parseResult)
                }
              }

              text += textLine + '\n'
            }
            i++
          }

          newQuestion.questionType = QuestionType.latex
          newQuestion.question = text
        }
        else {

          return SurveyParser.returnWithError('Kein Fragetyp angegeben, erwartet [question=' +
            QuestionType.markdown + '] oder [question=' +
            QuestionType.pdf + '] oder [question=' +
            QuestionType.text + '] oder [question=' +
            QuestionType.latex + ']', parseResult)
        }

        //now get answer type (line was wrong so we already got the next line, esle we would have not escaped)
        while (i < lines.length && line.indexOf(SurveyParser.answerKey) === -1) {
          i++
          line = lines[i]
        }

        //after the while loop we found the anser key so reset questionFound to be able to parse the next question
        questionFound = false

        if (line.indexOf(AnswerTypes.radioType) !== -1) {
          newQuestion.answersType = AnswerTypes.radioType

        } else if (line.indexOf(AnswerTypes.checkboxType) !== -1) {
          newQuestion.answersType = AnswerTypes.checkboxType

        } else if (line.indexOf(AnswerTypes.textType) !== -1) {
          newQuestion.answersType = AnswerTypes.textType

        } else {
          parseResult.errors.push('Kein Antworttype angegeben, erwartet ' + SurveyParser.answerKey + '=' +
            AnswerTypes.radioType + '] oder ' + SurveyParser.answerKey + '=' +
            AnswerTypes.checkboxType + '] oder ' + SurveyParser.answerKey + '=' +
            AnswerTypes.textType)
          return parseResult
        }


        //now get answers to the question

        i++
        line = lines[i]

        //while (i < lines.length && line.startsWith(SurveyParser.questionKey) === false) {
        while (i < lines.length && startsWith(line, SurveyParser.questionKey) === false) {

          if (line.trim() === '') {
            i++
            line = lines[i]
            continue
          }

          //check for special string in answers
          let invalidSequence = SurveyParser.containsInvalidSequence(line)
          if (invalidSequence !== null) {
            return SurveyParser.returnWithError(
              SurveyParser.errorStringInvalidSequence(i, invalidSequence), parseResult)
          }

          newQuestion.possibleAnswers.push(line)
          i++
          line = lines[i]
        }

        //check if we have at least 1 answer possibility
        if (newQuestion.possibleAnswers.length === 0 && newQuestion.answersType !== AnswerTypes.textType) {

          return SurveyParser.returnWithError('Die ' + (parseResult.result.questions.length + 1) +
            '. Frage hat keine Antwortmöglichkeiten', parseResult)
        }

        //if type is radio we need at least 2 answers else the user would not have a choice
        if (newQuestion.answersType === AnswerTypes.radioType && newQuestion.possibleAnswers.length < 2) {

          return SurveyParser.returnWithError('Die ' + (parseResult.result.questions.length + 1) +
            '. benutzt den Typ "' + AnswerTypes.radioType + '" und nur 1 Antwortmöglichkeit (keine Auswahlt)',
            parseResult)
        }

        //one step back to get the question line
        i--

        //reset answer to parse next answer
        answerFound = false

        parseResult.result.questions.push(newQuestion)
      }
      //}
    }

    //check if we have at least 1 question
    if (parseResult.result.questions.length === 0) {
      parseResult.errors.push('Keine Frage eingegeben')
    }

    return parseResult

  }

  /**
   * parses a text for a exercise task description (question)
   * @param text
   * @returns {QuestionParseResult}
   */
  static parseQuestion(text:string):QuestionParseResult {

    let parseResult:QuestionParseResult = {
      errors: [],
      result: {
        questionType: QuestionType.unknown,
        question: ''
      }
    }

    let lines = text.split(this.SplitString)

    let questionFound = false
    let answerFound = false

    for (let i = 0; i < lines.length; i++) {
      let line = lines[i];

      if (line === '') continue

      //parse question

      if (questionFound === false && startsWith(line, SurveyParser.questionKey)) {

        i++
        questionFound = true

        let newQuestion:ExerciseQuestion = {
          questionType: QuestionType.unknown,
          question: '',
        }


        if (line.indexOf(QuestionType. markdown) !== -1) { //markdown?

          //get markdown

          let searchMarkdown = true
          let markdownTextArray = []
          let markdownText = ''

          //read all
          while (searchMarkdown && i < lines.length) {

            let markdownLine = lines[i]

            //markdownText += markdownLine + '\n'
            markdownTextArray.push(markdownLine)

            i++
          }

          newQuestion.questionType = QuestionType. markdown
          newQuestion.question = markdownTextArray.join('\n')

        }
        else if (line.indexOf(QuestionType.pdf) !== -1) {

          //get pdf name
          let pdfNameWithExtension = lines[i]

          newQuestion.questionType = QuestionType.pdf
          newQuestion.question = pdfNameWithExtension

        }
        else if (line.indexOf(QuestionType.text) !== -1) { //normal text?

          //get normal text
          let searchText = true
          let text = ''

          while (searchText && i < lines.length) {

            let textLine = lines[i]
            text += textLine + '\n'
            i++
          }

          newQuestion.questionType = QuestionType.text
          newQuestion.question = text

        }
        else if (line.indexOf(QuestionType.latex) !== -1) { //latex ?

          //get normal text
          let searchText = true
          let text = ''

          while (searchText && i < lines.length) {

            let textLine = lines[i]

            text += textLine + '\n'
            i++
          }

          newQuestion.questionType = QuestionType.latex
          newQuestion.question = text
        }
        else {

          return SurveyParser.returnQuestionWithError('Kein Aufgabentyp angegeben, erwartet [question=' +
            QuestionType. markdown + '] oder [question=' +
            QuestionType.pdf + '] oder [question=' +
            QuestionType.text + '] oder [question=' +
            QuestionType.latex + ']', parseResult)
        }

        parseResult.result = newQuestion
        //we only allow one question...
        return parseResult
      }
    }

    //check if we have at least 1 question
    if (!parseResult.result || !parseResult.result.question || parseResult.result.question.length === 0) {
      parseResult.errors.push('No task specified')
    }

    return parseResult

  }

  static errorStringNoAnswerTag(questionIndex:number):string {
    return 'Für die ' + (questionIndex + 1) + '. Frage sind keine Antwortmöglichkeiten vorhanden'
  }

  static errorStringInvalidSequence(rowIndex:number, invalidSequence:string):string {
    return 'In Zeile ' + (rowIndex + 1) + ' kommt die ungültige Zeichenkette "' + invalidSequence + '" vor'
  }

  static returnWithError(errorMessage:string, result:SurveySyntaxParseResult):SurveySyntaxParseResult {

    result.errors.push(errorMessage)
    return result
  }

  static returnQuestionWithError(errorMessage:string, result:QuestionParseResult):QuestionParseResult {

    result.errors.push(errorMessage)
    return result
  }

  static containsInvalidSequence(line:string):string {

    if (line.indexOf(SurveyParser.answerKey) !== -1) {
      return SurveyParser.answerKey
    }

    if (line.indexOf(SurveyParser.questionKey) !== -1) {
      return SurveyParser.questionKey
    }

    return null
  }

  static contains<T>(array:T[], item:T): boolean {

    for (let i = 0; i < array.length; i++) {
      let element = array[i];

      if (item === element) {
        return true
      }

    }
    return false

  }

}


export = SurveyParser
