/**
 * Created by janis dähne.
 */
import {delay} from "../../../../debug";
import {ActionType} from "../../reducers/doExerciseSite/doExerciseSiteActionTypes";
import {GET_doExerciseAction} from "../../reducers/doExerciseSite/crud/getDoExerciseReducer";
import {
  getAllTestResults, getDoExercise, getSolutionVersion, getSubmitTestResults,
  saveSolutionVersion, setShouldNotCount
} from "../../../communicationLayer/doExerciseLayer";
import {GET_doExercise_solutionVersionAction} from "../../reducers/doExerciseSite/crud/getSolutionReducer";
import {GET_allTestResultsAction} from "../../reducers/doExerciseSite/crud/getAllTestResultsReducer";
import {UPDATE_doExerciseSolutionAction} from "../../reducers/doExerciseSite/crud/updateDoEditorSolutionReducer";
import {SolutionVersionDoExerciseFullBase} from "../../../types/exerciseSolution";
import {RUN_testAction} from "../../reducers/doExerciseSite/crud/runTestReducer";
import {compileSingleFile, justRunProgramTest, runAfterTest, runTest} from "../../../communicationLayer/testingLayer";
import {
  RunCompileSingleFileCommandForBackend, RunAfterTestCommandForBackend,
  RunTestCommandForBackend, JustRunProgramCommandForBackend
} from "../../../types/testsAndTestProtocol";
import {RUN_customTestAction} from "../../reducers/doExerciseSite/crud/runCustomTestReducer";
import {RUN_compileSingleFileAction} from "../../reducers/doExerciseSite/crud/runCompileSingleFileReducer";
import {AwaitActions, AwaitActionsWithResult, MultiActions} from "../types";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {setTestsAreRunningAndResetResult} from "./sub/testActions";
import {setCustomTestIsRunning} from "./sub/customTestActions";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {resetDoExerciseTestResults, setSelectedConsoleViewTest} from "./doExerciseSiteActions";
import {
  setHasFileChangedMapDoEditor,
  setHasUnsavedChanges
} from "./editorActions/doEditorActions";
import {KnownInternalTestTypes, TestTypeFullBase} from "../../../types/testTypes";
import {GET_submitTestResultsAction} from "../../reducers/doExerciseSite/crud/getSubmitTestResults";
import {errorDialog, warningDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";
import {setAfterSubmitTestsAreRunningAndResetResult} from "./sub/afterSubmitTestActions";
import {RUN_afterNormalTestAction} from "../../reducers/doExerciseSite/crud/after/runAfterNormalTest";
import {setAfterNormalTestsAreRunningAndResetResult} from "./sub/afterNormalTestActions";
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {RUN_afterSubmitTestAction} from "../../reducers/doExerciseSite/crud/after/runAfterSubmitTest";
import {MultiTabEditorState} from "../../../components/codeEditors/multiTabCodeEditorProps";
import {setForceEditorUpdateSessionDummyDoEditor} from "../editorHelperState/editorHelperStateActions";
import {SET_shouldNotCountParticipationAction} from "../../reducers/doExerciseSite/crud/setShouldNotCountParticipationReducer";
import {JUST_RUN_programAction} from "../../reducers/doExerciseSite/crud/justRunProgramReducer";


export function getDoExerciseAsync(releaseCode: string): GET_doExerciseAction {
  return {
    type: ActionType.GET_doExercise,
    payload: delay(getDoExercise(releaseCode))
  }
}


export function getDoExerciseSolutionVersion(releaseCode: string, pLangId: number): GET_doExercise_solutionVersionAction {
  return {
    type: ActionType.GET_doExercise_solutionVersion,
    payload: delay(getSolutionVersion(releaseCode, pLangId, null))
  }
}


export function getDoExerciseAllTestResults(releaseCode: string, pLangId: number, solutionVersionId: number): GET_allTestResultsAction {
  return {
    type: ActionType.GET_allTestResults,
    payload: delay(getAllTestResults(releaseCode, pLangId, solutionVersionId))
  }
}

export function getDoExerciseSubmitTestResults(releaseCode: string, pLangId: number): GET_submitTestResultsAction {
  return {
    type: ActionType.GET_submitTestResults,
    payload: delay(getSubmitTestResults(releaseCode, pLangId))
  }
}




/**
 * this also reset has unsaved changes
 * @param {string} releaseCode
 * @param {SolutionVersionDoExerciseFullBase} solutionVersion
 * @returns {UPDATE_doExerciseSolutionAction}
 */
export function saveDoExerciseSolution(releaseCode: string, solutionVersion: SolutionVersionDoExerciseFullBase): AwaitActionsWithResult<boolean> {
  return async (dispatch) => {


    //we call this because ... see MultiFileAceEditor
    dispatch(setForceEditorUpdateSessionDummyDoEditor(MultiTabEditorState.beforeChangingFiles))

    try {
      await dispatch(_saveDoExerciseSolution(releaseCode, solutionVersion))
    } catch(err) {

      dispatch(setForceEditorUpdateSessionDummyDoEditor(MultiTabEditorState.afterChangingFiles))
      dispatch(setForceEditorUpdateSessionDummyDoEditor(MultiTabEditorState.normal))

        errorDialog(translate(TranslateIds.error),
          translate(TranslateIds.couldNotSaveFiles))
      return false
    }

    dispatch(resetDoExerciseTestResults())

    dispatch(setHasUnsavedChanges(false))
    dispatch(setHasFileChangedMapDoEditor({}))

    dispatch(setForceEditorUpdateSessionDummyDoEditor(MultiTabEditorState.afterChangingFiles))
    dispatch(setForceEditorUpdateSessionDummyDoEditor(MultiTabEditorState.normal))

    return true
  }
}

export function _saveDoExerciseSolution(releaseCode: string, solutionVersion: SolutionVersionDoExerciseFullBase): UPDATE_doExerciseSolutionAction {
  return {
    type: ActionType.UPDATE_doExerciseSolution,
    payload: delay(saveSolutionVersion(releaseCode, solutionVersion))
  }
}


/**
 * run the given tests, sets the test states to running and selects the last or the first failed test
 * @param {RunTestCommandForBackend} runNormalTestCommand
 * @returns {AwaitActionsWithResult<boolean>} true: all tests passed, false: at least 1 did not compile or not passed
 */
export function runDoExerciseTest(runNormalTestCommand: RunTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {

    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))

    const runningTests = runNormalTestCommand.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().doExerciseSiteState.exercise.tests
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(setTestsAreRunningAndResetResult(runningTests, getState().doExerciseSiteState.exercise.tests))

    await dispatch(_runDoExerciseTest(runNormalTestCommand))

    const ranTests = runNormalTestCommand.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        getState().doExerciseSiteState.exercise.tests
          .find(test =>
            test.id === testId
          )
      )

    if (ranTests.length === 0) {
      warningDialog(
        translate(TranslateIds.warning),
        translate(TranslateIds.probablyNoTestRun)
      )
      return
    }

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(ranTests[ranTests.length - 1]))
    }

    return !!firstFailedTest
  }
}

export function _runDoExerciseTest(runNormalTestCommand: RunTestCommandForBackend): RUN_testAction {
  return {
    type: ActionType.RUN_test,
    payload: delay(runTest(runNormalTestCommand))
  }
}

/**
 *
 * @param {RunTestCommandForBackend} runNormalTestCommand
 * @param {ReadonlyArray<RunnableCustomTestFrontendExclusive>} allTests
 * @returns {AwaitActionsWithResult<boolean>} true: all tests passed, false: at least 1 did not compile or not passed
 */
export function runCustomDoExerciseTest(runNormalTestCommand: RunTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {

    //save changes if code changed
    if (getState().doExerciseSiteState.doEditorState.hasUnsavedChanges) {
      //first save changes
      //normally this should be func arguments but would the component re render on every code change??
     const hasSaved = await dispatch(
        saveDoExerciseSolution(getState().doExerciseSiteState.exercisePreviewFrontendTemp.generatedCode, getState().doExerciseSiteState.solution))

      if (!hasSaved) {
       return false
      }
    }

    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))

    const runningTests = runNormalTestCommand.testIds
      .map<RunnableCustomTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().doExerciseSiteState.exercise.customTests
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(setCustomTestIsRunning(runningTests, getState().doExerciseSiteState.exercise.customTests))
    await dispatch(_runCustomDoExerciseTest(runNormalTestCommand))

    const updatedTest = getState().doExerciseSiteState.exercise.customTests.find(
      p => p.id === runNormalTestCommand.testIds[runNormalTestCommand.testIds.length - 1])

    if (!updatedTest) {
      warningDialog(
        translate(TranslateIds.warning),
        translate(TranslateIds.probablyNoTestRun)
      )
      return
    }

    const ranTests = runNormalTestCommand.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        getState().doExerciseSiteState.exercise.tests
          .find(test =>
            test.id === testId
          )
      )

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(updatedTest))
    }

    return !!firstFailedTest
  }
}

export function _runCustomDoExerciseTest(runNormalTestCommand: RunTestCommandForBackend): RUN_customTestAction {
  return {
    type: ActionType.RUN_customTest,
    payload: delay(runTest(runNormalTestCommand))
  }
}

export function runCompileSingleFileDoExercise(runCompileSingleFileCommand: RunCompileSingleFileCommandForBackend, testTypes: ReadonlyArray<TestTypeFullBase>): AwaitActions {
  return async (dispatch, getState) => {

    const compileTestType = testTypes.find(p => p.internalName === KnownInternalTestTypes.compileTest)

    if (!compileTestType) {

      errorDialog(translate(TranslateIds.error),
        translate(TranslateIds.unknownTestType))

      return
    }

    await dispatch(_runCompileSingleFileDoExercise(runCompileSingleFileCommand))

    const compileResult = getState().doExerciseSiteState.lastCompileSingleFileResult

    const tempTest: RunnableExerciseTestFrontendExclusive = {
      id: -1,
      displayName: 'Compile Single File',
      result: compileResult,
      content: '',
      uiIsRunning: false,
      files: [],
      displayIndex: -1,
      weight: 0,
      isSubmitTest: false,
      testSettings: {
        id: -1,
        maxDiskSpaceInKb: -1,
        timeoutInMs: -1,
        memoryLimitInKb: -1
      },
      testTypeId: compileTestType.id
    }

    dispatch(setSelectedConsoleViewTest(tempTest))

  }
}

export function _runCompileSingleFileDoExercise(runCompileSingleFileCommand: RunCompileSingleFileCommandForBackend): RUN_compileSingleFileAction {
  return {
    type: ActionType.RUN_compileSingleFile,
    payload: delay(compileSingleFile(runCompileSingleFileCommand))
  }
}

export function justRunProgramDoExerciseAsync(justRunProgramCommand: JustRunProgramCommandForBackend, testTypes: ReadonlyArray<TestTypeFullBase>): AwaitActions {
  return async (dispatch, getState) => {

    // const justRunTestType = testTypes.find(p => p.internalName === KnownInternalTestTypes.justRunTest)
    //
    // if (!justRunTestType) {
    //
    //   errorDialog(translate(TranslateIds.error),
    //     translate(TranslateIds.unknownTestType))
    //
    //   return
    // }

    await dispatch(_justRunProgramDoExercise(justRunProgramCommand))

    const testResult = getState().doExerciseSiteState.lastJustRunProgramResult

    const tempTest: RunnableExerciseTestFrontendExclusive = {
      id: -1,
      displayName: 'Just Run Program',
      result: testResult,
      content: '',
      uiIsRunning: false,
      files: [],
      displayIndex: -1,
      weight: 0,
      isSubmitTest: false,
      testSettings: {
        id: -1,
        maxDiskSpaceInKb: -1,
        timeoutInMs: -1,
        memoryLimitInKb: -1
      },
      testTypeId: -1
    }

    dispatch(setSelectedConsoleViewTest(tempTest))

  }
}

export function _justRunProgramDoExercise(justRunProgramCommand: JustRunProgramCommandForBackend): JUST_RUN_programAction {
  return {
    type: ActionType.JUST_RUN_program,
    payload: delay(justRunProgramTest(justRunProgramCommand))
  }
}


//--- after normal tests, after submit tests

/**
 * run the given tests, sets the test states to running and selects the last or the first failed test
 * @returns {AwaitActionsWithResult<boolean>} true: all tests passed, false: at least 1 did not compile or not passed
 */
export function runAfterNormalTest(runTestWithSolution: RunAfterTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {
    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))


    const runningTests = runTestWithSolution.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().doExerciseSiteState.afterNormalTests
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(
      setAfterNormalTestsAreRunningAndResetResult(runningTests, getState().doExerciseSiteState.afterNormalTests))

    //run the test
    await dispatch(_runAfterNormalTest(runTestWithSolution))


    const ranTests = runTestWithSolution.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        getState().doExerciseSiteState.afterNormalTests
          .find(test =>
            test.id === testId
          )
      )

    if (ranTests.length === 0) {

      warningDialog(translate(TranslateIds.warning),
        translate(TranslateIds.probablyNoTestRun)
      )
      return
    }

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(ranTests[ranTests.length - 1]))
    }

    return !!firstFailedTest
  }
}

export function _runAfterNormalTest(runTestWithSolution: RunAfterTestCommandForBackend): RUN_afterNormalTestAction {
  return {
    type: ActionType.RUN_afterNormalTest,
    payload: delay(runAfterTest(runTestWithSolution))
  }
}


/**
 * run the given tests, sets the test states to running and selects the last or the first failed test
 * @returns {AwaitActionsWithResult<boolean>} true: all tests passed, false: at least 1 did not compile or not passed
 */
export function runAfterSubmitTest(runTestWithSolution: RunAfterTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {
    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))


    const runningTests = runTestWithSolution.testIds
      .map<AssessedSubmitTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().doExerciseSiteState.afterSubmitTests
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(
      setAfterSubmitTestsAreRunningAndResetResult(runningTests, getState().doExerciseSiteState.afterSubmitTests))

    //run the test
    await dispatch(_runAfterSubmitTest(runTestWithSolution))


    const ranTests = runTestWithSolution.testIds
      .map<AssessedSubmitTestFrontendExclusive>(testId =>
        getState().doExerciseSiteState.afterSubmitTests
          .find(test =>
            test.id === testId
          )
      )

    if (ranTests.length === 0) {

      warningDialog(translate(TranslateIds.warning),
        translate(TranslateIds.probablyNoTestRun)
      )
      return
    }

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(ranTests[ranTests.length - 1]))
    }

    return !!firstFailedTest
  }
}

export function _runAfterSubmitTest(runTestWithSolution: RunAfterTestCommandForBackend): RUN_afterSubmitTestAction {
  return {
    type: ActionType.RUN_afterSubmitTest,
    payload: delay(runAfterTest(runTestWithSolution))
  }
}


export function setShouldNotCountAsync(releaseCode: string): SET_shouldNotCountParticipationAction {
  return {
    type: ActionType.SET_shouldNotCountParticipation,
    payload: delay(setShouldNotCount(releaseCode))
  }
}