/**
 * Created by janis dähne.
 */
import {
  RESET_TestResultsAction, ResetAction, SET_botLeftPanelViewSizeAction, SET_botRightPanelViewSizeAction,
  SET_bottomLeftSelectedTabIndexAction,
  SET_bottomRightSelectedTabIndexAction, SET_dummyAction, SET_exercisePreviewFrontendAction,
  SET_hDividerPosOffsetAction,
  SET_isCodeEditorSettingsDialogDisplayedAction,
  SET_isCustomTestDialogDisplayedAction, SET_isJustRunProgramArgsInputDisplayedAction,
  SET_isLeftTimeStringDisplayedAction, SET_isLoadingAction, SET_isOnGotFocusInterceptionOverlayDisplayedAction,
  SET_isViewTestDialogDisplayedAction, SET_justRunProgramArgsAction,
  SET_selectedConsoleViewTestAction,
  SET_selectedPLangIdAction, SET_solutionAction, SET_topLeftPanelViewSizeAction,
  SET_topLeftSelectedTabIndexAction, SET_topRightPanelViewSizeAction,
  SET_topRightSelectedTabIndexAction, SET_useDiffIfOutputMismatchedAction, SET_useReplaceWhiteSpacesWithBulletsAction,
  SET_vDividerPosOffsetAction
} from "../../reducers/doExerciseSite/doExerciseSiteReducer";
import {ActionType} from "../../reducers/doExerciseSite/doExerciseSiteActionTypes";
import {AwaitActions, MultiActions} from "../types";
import {
  getDoExerciseAllTestResults, getDoExerciseAsync, getDoExerciseSolutionVersion,
  getDoExerciseSubmitTestResults
} from "./doExerciseCrudActions";
import {getPLangsAsync} from "../pLangsActions";
import {getTestTypesAsync} from "../testTypesActions";
import {ExerciseCodeTemplateFullBase, RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {
  SET_mainFileIdAction,
  SET_solutionFilesAction
} from "../../reducers/doExerciseSite/subSets/setDoExerciseSolutionReducer";
import {SolutionFileDoExerciseFullBase, SolutionVersionDoExerciseFullBase} from "../../../types/exerciseSolution";
import {
  openTabChangeDoEditor,
  setSelectedTabIdDoEditor
} from "./editorActions/doEditorActions";
import {getSystemSettingsAsync} from "../systemSettingsActions";
import {getNextFrontendId} from "../../../helpers/frontendIdHelper";
import {ExercisePreviewsFrontendExclusive} from "../../../components/sites/allAccessibleExercisesSite/openExercises/allOpenExercisesSite";
import {createAndRefreshAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import {AssessedSubmitTestFromBackend, AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {PanelViewSize} from "../../../components/panelSystem/panelGrid/panelGrid";
import {
  SET_submitTestsForTutorAction,
  SET_normalTestsForTutorAction
} from "../../reducers/tutorViewSite/tutorViewSiteReducer";
import {errorDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";
import {
  SET_afterMainFileIdAction,
  SET_afterSolutionFilesAction
} from "../../reducers/doExerciseSite/subSets/setAfterSolutionReducer";
import {setAfterEditor_selectedTabIdDoEditor} from "./editorActions/afterEditorActions";
import {
  mergeCustomTestResults, mergeNormalTestResults,
  mergeSubmitTestResults
} from "../../../helpers/convertersAndTransformers";
import {setTests} from "./sub/testActions";
import {setAfterNormalTests} from "./sub/afterNormalTestActions";
import {setAfterSubmitTests} from "./sub/afterSubmitTestActions";
import {setCustomTests} from "./sub/customTestActions";
import {MultiTabEditorState} from "../../../components/codeEditors/multiTabCodeEditorProps";
import {
  setForceEditorUpdateSessionDummyAfterEditor,
  setForceEditorUpdateSessionDummyDoEditor
} from "../editorHelperState/editorHelperStateActions";
import {RunnableCustomProjectTestFrontendExclusive} from "../../../types/customProjects";


export function setIsLeftTimeStringDisplayed(isLeftTimeStringDisplayed: boolean): SET_isLeftTimeStringDisplayedAction {
  return {
    type: ActionType.SET_isLeftTimeStringDisplayed,
    isLeftTimeStringDisplayed
  }
}

/**
 *
 * @param {ExercisePreviewsFrontendExclusive} exercisePreviewFrontendTemp
 * @returns {SET_exercisePreviewFrontendAction}
 */
export function setExercisePreviewFrontendDoExercise(exercisePreviewFrontendTemp: ExercisePreviewsFrontendExclusive | null): SET_exercisePreviewFrontendAction {
  return {
    type: ActionType.SET_exercisePreviewFrontend,
    exercisePreviewFrontendTemp: exercisePreviewFrontendTemp
  }
}

//-- normal solution
export function setSolutionFiles(solutionFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>): SET_solutionFilesAction {
  return {
    type: ActionType.SET_solutionFiles,
    solutionFiles
  }
}

export function setMainFileIdDoExercise(mainFileId: number): SET_mainFileIdAction {
  return {
    type: ActionType.SET_mainFileId,
    mainFileId
  }
}

//--- after solution

export function setAfterSolutionFiles(solutionFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>): SET_afterSolutionFilesAction {
  return {
    type: ActionType.SET_afterSolutionFiles,
    solutionFiles
  }
}

export function setAfterMainFileId(mainFileId: number): SET_afterMainFileIdAction {
  return {
    type: ActionType.SET_afterMainFileId,
    mainFileId
  }
}

export function setSelectedPLangId(selectedPLangId: number): SET_selectedPLangIdAction {
  return {
    type: ActionType.SET_selectedPLangId,
    selectedPLangId
  }
}


export function resetDoExerciseSolutionToTemplate(currentPLangId: number, codeTemplates: ReadonlyArray<ExerciseCodeTemplateFullBase>): MultiActions {
  return (dispatch, getState) => {


    const codeTemplate = codeTemplates.find(p => p.pLangId === currentPLangId)

    if (!codeTemplate) {
      errorDialog(translate(TranslateIds.error),
        translate(TranslateIds.couldNotFindCodeTemplate)
      )
      return
    }

    // noinspection TsLint
    let newMainFileId = 0

    const newSolutionFiles = codeTemplate.templateFiles.map<SolutionFileDoExerciseFullBase>(p => {

      const copy: SolutionFileDoExerciseFullBase = {
        id: getNextFrontendId(),
        fileNameWithExtension: p.fileNameWithExtension,
        isEditableByUser: p.isEditableByUser,
        displayIndex: p.displayIndex,
        content: p.content,
        uiIsDisplayed: p.displayIndex >= 0,
        templateFileId: p.id,
        uiEditSession: null
      }

      if (p.id === codeTemplate.mainFileId) {
        newMainFileId = copy.id
      }

      return copy
    })

    //no idea why this is working without delay...
    //when using pattern e.g. in setSolutionFiles_AndRefreshEditor than this is only working with delay..
    dispatch(setForceEditorUpdateSessionDummyDoEditor(MultiTabEditorState.beforeChangingFiles))

    dispatch(setSolutionFiles(newSolutionFiles))

    // //clear session in order to force a rebuild of sessions obj
    // UndoHistoryHelper.clearAll()

    dispatch(setMainFileIdDoExercise(newMainFileId))

    dispatch(setForceEditorUpdateSessionDummyDoEditor(MultiTabEditorState.afterChangingFiles))
    dispatch(setForceEditorUpdateSessionDummyDoEditor(MultiTabEditorState.normal))

    //also reset test results
    dispatch(resetNormalTestResults())
    dispatch(resetCustomTestResults())

    if (newSolutionFiles.length > 0) {
      //select first tab
      dispatch(openTabChangeDoEditor(newSolutionFiles[0], newSolutionFiles,
        getState().doExerciseSiteState.doEditorState.selectedTabIdDoEditor))
    }


  }
}

export function resetAfterEditor(solution: SolutionVersionDoExerciseFullBase): MultiActions {
  return dispatch => {


    dispatch(setForceEditorUpdateSessionDummyAfterEditor(MultiTabEditorState.beforeChangingFiles))

    dispatch(setAfterSolutionFiles([...solution.solutionFiles]))
    dispatch(setAfterMainFileId(solution.mainFileId))

    dispatch(setForceEditorUpdateSessionDummyAfterEditor(MultiTabEditorState.afterChangingFiles))
    dispatch(setForceEditorUpdateSessionDummyAfterEditor(MultiTabEditorState.normal))

    //also reset after test results
    dispatch(resetAfterNormalTestResults())
    dispatch(resetAfterSubmitTestResults())

  }
}

//resets normal, custom tests

export function resetDoExerciseTestResults(): MultiActions {
  return dispatch => {

    dispatch(_resetDoExerciseTestResults())
    dispatch(setSelectedConsoleViewTest(null))

  }
}

export function _resetDoExerciseTestResults(): RESET_TestResultsAction {
  return {
    type: ActionType.RESET_TestResults,
  }
}

//-- reset test results

export function resetNormalTestResults(): MultiActions {
  return (dispatch, getState) => {
    dispatch(setTests(mergeNormalTestResults(getState().doExerciseSiteState.exercise.tests, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}

export function resetCustomTestResults(): MultiActions {
  return (dispatch, getState) => {
    dispatch(setCustomTests(mergeCustomTestResults(getState().doExerciseSiteState.exercise.customTests, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}

export function resetAfterNormalTestResults(): MultiActions {
  return (dispatch, getState) => {
    dispatch(setAfterNormalTests(mergeNormalTestResults(getState().doExerciseSiteState.afterNormalTests, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}

export function resetAfterSubmitTestResults(): MultiActions {
  return (dispatch, getState) => {
    dispatch(setAfterSubmitTests(mergeSubmitTestResults(getState().doExerciseSiteState.afterSubmitTests, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}


export function setSelectedConsoleViewTest(selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive
  | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null): SET_selectedConsoleViewTestAction {
  return {
    type: ActionType.SET_selectedConsoleViewTest,
    selectedConsoleViewTest
  }
}

//--- grid system, tab indices
export function setTopRightSelectedTabIndex(topRightSelectedTabIndex: number): SET_topRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_topRightSelectedTabIndex,
    topRightSelectedTabIndex
  }
}

export function setTopLeftSelectedTabIndex(topLeftSelectedTabIndex: number): SET_topLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_topLeftSelectedTabIndex,
    topLeftSelectedTabIndex
  }
}

export function setBottomRightSelectedTabIndex(bottomRightSelectedTabIndex: number): SET_bottomRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomRightSelectedTabIndex,
    bottomRightSelectedTabIndex
  }
}

export function setBottomLeftSelectedTabIndex(bottomLeftSelectedTabIndex: number): SET_bottomLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomLeftSelectedTabIndex,
    bottomLeftSelectedTabIndex
  }
}


//--- console view settings

export function setUseDiffIfOutputMismatched(useDiffIfOutputMismatched: boolean): SET_useDiffIfOutputMismatchedAction {
  return {
    type: ActionType.SET_useDiffIfOutputMismatched,
    useDiffIfOutputMismatched
  }
}

export function setUseReplaceWhiteSpacesWithBullets(useReplaceWhiteSpacesWithBullets: boolean): SET_useReplaceWhiteSpacesWithBulletsAction {
  return {
    type: ActionType.SET_useReplaceWhiteSpacesWithBullets,
    useReplaceWhiteSpacesWithBullets
  }
}

//--- gripper
export function setDoExerciseHDividerPosOffset(hDividerPosOffset: number): SET_hDividerPosOffsetAction {
  return {
    type: ActionType.SET_hDividerPosOffset,
    hDividerPosOffset
  }
}

export function setDoExerciseVDividerPosOffset(vDividerPosOffset: number): SET_vDividerPosOffsetAction {
  return {
    type: ActionType.SET_vDividerPosOffset,
    vDividerPosOffset
  }
}

//--- panel view sizes

export function setTopRightPanelViewSize(topRightPanelViewSize: PanelViewSize): SET_topRightPanelViewSizeAction {
  return {
    type: ActionType.SET_topRightPanelViewSize,
    topRightPanelViewSize
  }
}

export function setTopLeftPanelViewSize(topLeftPanelViewSize: PanelViewSize): SET_topLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_topLeftPanelViewSize,
    topLeftPanelViewSize
  }
}

export function setBotRightPanelViewSize(botRightPanelViewSize: PanelViewSize): SET_botRightPanelViewSizeAction {
  return {
    type: ActionType.SET_botRightPanelViewSize,
    botRightPanelViewSize
  }
}

export function setBotLeftPanelViewSize(botLeftPanelViewSize: PanelViewSize): SET_botLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_botLeftPanelViewSize,
    botLeftPanelViewSize
  }
}


//--- dialogs
export function setIsCustomTestDialogDisplayed(isCustomTestDialogDisplayed: boolean): SET_isCustomTestDialogDisplayedAction {
  return {
    type: ActionType.SET_isCustomTestDialogDisplayed,
    isCustomTestDialogDisplayed
  }
}

export function setIsViewTestDialogDisplayed(isViewTestDialogDisplayed: boolean): SET_isViewTestDialogDisplayedAction {
  return {
    type: ActionType.SET_isViewTestDialogDisplayed,
    isViewTestDialogDisplayed
  }
}

export function setIsDoCodeEditorSettingsDialogDisplayed(isDisplayed: boolean): SET_isCodeEditorSettingsDialogDisplayedAction {
  return {
    type: ActionType.SET_isCodeEditorSettingsDialogDisplayed,
    isDisplayed: isDisplayed
  }
}


export function loadDoExerciseSite(releaseCode: string): AwaitActions {
  return async (dispatch, getState) => {

    //resets displayed dialogs & all old states (console output...)
    dispatch(resetDoExerciseSite()) //this also sets the dummy to true


    //clear the old timer
    dispatch(setExercisePreviewFrontendDoExercise(null))

    await dispatch(getSystemSettingsAsync())

    await dispatch(getTestTypesAsync())

    await dispatch(getPLangsAsync())

    await dispatch(getDoExerciseAsync(releaseCode))

    //this needs to be called BEFORE getDoExerciseSolutionVersion
    //because in getDoExerciseSolutionVersion we check LastKnownDoEditorReleaseId
    FrontendSettingsManager.setLastKnownDoEditorReleaseId(
      FrontendSettingsManager.getFrontendSettings().doEditorReleaseId)
    FrontendSettingsManager.setDoEditorReleaseId(getState().doExerciseSiteState.exercise.exerciseRelease.id)

    const lastEditedPLangId = getState().doExerciseSiteState.exercise.lastEditedPLangId
    await dispatch(getDoExerciseSolutionVersion(releaseCode, lastEditedPLangId))


    await dispatch(
      getDoExerciseAllTestResults(releaseCode, lastEditedPLangId, getState().doExerciseSiteState.solution.id))

    //try to select the first displayed tab (if any)
    if (getState().doExerciseSiteState.solution.solutionFiles.length > 0
      && getState().doExerciseSiteState.solution.solutionFiles.filter(p => p.uiIsDisplayed).length > 0) {
      dispatch(setSelectedTabIdDoEditor(
        getState().doExerciseSiteState.solution.solutionFiles.filter(p => p.uiIsDisplayed)[0].id))
    }

    //also for after editor

    if (getState().doExerciseSiteState.afterSolution.solutionFiles.length > 0
      && getState().doExerciseSiteState.afterSolution.solutionFiles.filter(p => p.uiIsDisplayed).length > 0) {
      dispatch(setAfterEditor_selectedTabIdDoEditor(
        getState().doExerciseSiteState.afterSolution.solutionFiles.filter(p => p.uiIsDisplayed)[0].id))
    }


    //if the release has ended AND the results are there... get the submit test results...

    if (!getState().doExerciseSiteState.exercise.exerciseRelease.isReleased
      && getState().doExerciseSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished) {
      await dispatch(getDoExerciseSubmitTestResults(releaseCode, getState().doExerciseSiteState.solution.pLangId))
    }

    //create the assets
    createAndRefreshAssetUrlsAndObjects(getState().doExerciseSiteState.exercise.exerciseDescription.assets)

    //somehow force the preview panel to re render
    //this is also called if the user must re log in
    dispatch(setDoExerciseDummy(false))

  }
}


/**
 * this is not recommended bcause the solution might not match the required file structure (names, ...)
 * so only call this if you are sure it will work (or on your own risk)
 *
 * CURRENTLY ONLY USED FOR RECOVERY CODE FROM FRONTEND SETTINGS
 * but this is needed for crash recovery for the solution files...
 * BUT it's bad...
 * @param {SolutionVersionDoExerciseFullBase} solution
 * @returns {SET_solutionAction}
 */
export function setSolutionVersion(solution: SolutionVersionDoExerciseFullBase): SET_solutionAction {
  return {
    type: ActionType.SET_solution,
    solution
  }
}

export function setDoExerciseDummy(dummy: boolean): SET_dummyAction {
  return {
    type: ActionType.SET_dummy,
    dummy
  }
}


export function setJustRunProgramArgsDoEditor(justRunProgramArgs: string): SET_justRunProgramArgsAction {
  return {
    type: ActionType.SET_justRunProgramArgs,
    justRunProgramArgs
  }
}

export function setIsJustRunProgramArgsInputDisplayedDoEditor(isJustRunProgramArgsInputDisplayed: boolean): SET_isJustRunProgramArgsInputDisplayedAction {
  return {
    type: ActionType.SET_isJustRunProgramArgsInputDisplayed,
    isJustRunProgramArgsInputDisplayed
  }
}

export function resetDoExerciseSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}

export function setIsLoadingDoExerciseSite(isLoading: boolean): SET_isLoadingAction {
  return {
    type: ActionType.SET_isLoading,
    isLoading
  }
}

export function setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite(isOnGotFocusInterceptionOverlayDisplayed: boolean): SET_isOnGotFocusInterceptionOverlayDisplayedAction {
  return {
    type: ActionType.SET_isOnGotFocusInterceptionOverlayDisplayed,
    isOnGotFocusInterceptionOverlayDisplayed
  }
}


