/**
 * Created by janis dähne.
 */

import {AwaitActions, MultiActions} from "../../../types";
import {
  ResetAction,
  SET_isCreatingAction,
  SET_isHelpDisplayedAction,
  SET_selectedTabIndexAction
} from "../../../../reducers/editCustomProjectSite/dialogs/changeCustomProjectTestDialog/changeCustomProjectTestDialogReducer";
import {ActionType} from "../../../../reducers/editCustomProjectSite/dialogs/changeCustomProjectTestDialog/changeCustomProjectTestDialogActionTypes";
import {
  CustomProjectTestFullBase,
  RunnableCustomProjectTestFrontendExclusive
} from "../../../../../types/customProjects";
import {setIsChangeTestDialogDisplayed} from "../../editCustomProjectActions";
import {setChangCustomProjectTest} from "./customProjectTestActions";
import {setCustomProjectTests} from "../../subSet/customProjectTestActions";
import {convertCustomProjectTestFromBackendToRunnableFrontendCustomProjectTest} from "../../../../../helpers/convertersAndTransformers";
import {
  createCustomProjectTest,
  updateCustomProjectTest
} from "../../../../../communicationLayer/editCustomProjectLayer";
import {createProjectTest, updateProjectTest} from "./customProjectTestCrudActions";


export function setIsHelpDisplayedAction(isDisplayed: boolean): SET_isHelpDisplayedAction {
  return {
    type: ActionType.SET_isHelpDisplayed,
    isDisplayed
  }
}

export function setIsCreating(isCreating: boolean): SET_isCreatingAction {
  return {
    type: ActionType.SET_isCreating,
    isCreating
  }
}

export function setSelectedTabIndex(index: number): SET_selectedTabIndexAction {
  return {
    type: ActionType.SET_selectedTabIndex,
    index
  }
}


export function openCreateCustomProjectTestDialog(): MultiActions {
  return dispatch => {

    dispatch(resetChangeCustomProjectTestDialog())
    dispatch(setIsCreating(true))

    dispatch(setIsChangeTestDialogDisplayed(true))
  }
}

export function openEditCustomProjectTestDialog(test: RunnableCustomProjectTestFrontendExclusive): MultiActions {
  return dispatch => {

    dispatch(resetChangeCustomProjectTestDialog())
    dispatch(setIsCreating(false))
    dispatch(setChangCustomProjectTest(test))

    dispatch(setIsChangeTestDialogDisplayed(true))
  }
}

export function saveCustomProjectTestDialog(customTest: CustomProjectTestFullBase,
                                            isCreatingNewCustomTest: boolean,
                                            allRunnableCustomTests: ReadonlyArray<RunnableCustomProjectTestFrontendExclusive>): AwaitActions {
  return async (dispatch, getState) => {


    //first save/create the custom test
    //then add/replace the custom test with the saved test from backend

    // noinspection TsLint
    let updateTests: ReadonlyArray<RunnableCustomProjectTestFrontendExclusive> = []

    const customProjectId = getState().editCustomProjectSiteState.customProject.id

    if (isCreatingNewCustomTest) {

      await dispatch(createProjectTest(customProjectId, customTest))

      const updatedRunnableTest: RunnableCustomProjectTestFrontendExclusive =
        convertCustomProjectTestFromBackendToRunnableFrontendCustomProjectTest(
          getState().changeCustomProjectTestDialogState.test
        )

      updateTests = [...allRunnableCustomTests, updatedRunnableTest]

    } else {

      //update test

      await dispatch(updateProjectTest(customProjectId, customTest))

      const updatedRunnableTest: RunnableCustomProjectTestFrontendExclusive =
        convertCustomProjectTestFromBackendToRunnableFrontendCustomProjectTest(
          getState().changeCustomProjectTestDialogState.test
        )

      updateTests = allRunnableCustomTests.map<RunnableCustomProjectTestFrontendExclusive>(p => {
        return p.id === updatedRunnableTest.id
          ? updatedRunnableTest
          : p
      })
    }

    dispatch(setCustomProjectTests(updateTests))
    dispatch(setIsChangeTestDialogDisplayed(false))

  }
}

export function cancelCustomProjectTestDialog(): MultiActions {
  return dispatch => {

    dispatch(resetChangeCustomProjectTestDialog())
    dispatch(setIsChangeTestDialogDisplayed(false))
  }
}


export function resetChangeCustomProjectTestDialog(): ResetAction {
  return {
    type: ActionType.RESET,
  }
}