/**
 * Created by janis dähne.
 */
import {
  RESET_TestResultsAction,
  ResetAction,
  SET_botLeftPanelViewSizeAction,
  SET_botRightPanelViewSizeAction,
  SET_bottomLeftSelectedTabIndexAction,
  SET_bottomRightSelectedTabIndexAction, SET_displayNameAction, SET_hDividerPosOffsetAction, SET_initialPLangIdAction,
  SET_isChangeTestDialogDisplayedAction,
  SET_isCustomProjectCodeEditorSettingsDialogDisplayedAction, SET_isJustRunProgramArgsInputDisplayedAction,
  SET_justRunProgramArgsAction,
  SET_selectedConsoleViewTestAction,
  SET_topLeftPanelViewSizeAction,
  SET_topLeftSelectedTabIndexAction, SET_topRightPanelViewSizeAction,
  SET_topRightSelectedTabIndexAction, SET_useDiffIfOutputMismatchedAction, SET_useReplaceWhiteSpacesWithBulletsAction,
  SET_vDividerPosOffsetAction
} from "../../reducers/editCustomProjectSite/editCustomProjectSiteReducer";
import {ActionType} from "../../reducers/editCustomProjectSite/editCustomProjectSiteActionTypes";
import {PanelViewSize} from "../../../components/panelSystem/panelGrid/panelGrid";
import {AwaitActions, MultiActions} from "../types";
import {
  getOwnCustomProjectsAsync, loadManageCustomProjectSite,
  resetManageCustomProjectsSite
} from "../manageCustomProjectsSite/manageCustomProjectsSiteActions";
import {getSystemSettingsAsync} from "../systemSettingsActions";
import {getTestTypesAsync} from "../testTypesActions";
import {getPLangsAsync} from "../pLangsActions";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {
  CustomProjectSolutionFileFullBase,
  RunnableCustomProjectTestFrontendExclusive
} from "../../../types/customProjects";
import {
  SET_mainFileIdAction,
  SET_solutionFilesAction
} from "../../reducers/editCustomProjectSite/subSets/setCustomProjectSolutionReducer";
import {setSolutionFiles_AndRefreshEditor} from "../../../helpers/editorHelper";
import {setForceEditorUpdateSessionDummyCustomProjectSolutionEditorAction} from "../editorHelperState/editorHelperStateActions";
import {
  openTabChangeCustomProjectSolutionEditor,
  setSelectedTabIdCustomProjectSolutionEditor
} from "./editorActions/customProjectSolutionEditorActions";
import {getCustomProjectAsync, getCustomProjectTestResultsAsync} from "./editCustomProjectCrudActions";
import {setCustomProjectTests} from "./subSet/customProjectTestActions";
import {mergeCustomTestResultsCustomProject} from "../../../helpers/convertersAndTransformers";


export function setCustomProjectDisplayName(displayName: string): SET_displayNameAction {
  return {
    type: ActionType.SET_displayName,
    displayName
  }
}

export function resetCustomProjectTestResults(): MultiActions {
  return (dispatch, getState) => {
    dispatch(setCustomProjectTests(mergeCustomTestResultsCustomProject(getState().editCustomProjectSiteState.customProject.tests, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}

//-- normal solution
export function setSolutionFilesCustomProject(solutionFiles: ReadonlyArray<CustomProjectSolutionFileFullBase>): SET_solutionFilesAction {
  return {
    type: ActionType.SET_solutionFiles,
    solutionFiles
  }
}


export function onCreateCustomProjectSolutionFile(newFile: CustomProjectSolutionFileFullBase, allFiles: ReadonlyArray<CustomProjectSolutionFileFullBase>, selectedEditorTabId: number): AwaitActions {
  return async (dispatch, getState) => {

    const newFiles: ReadonlyArray<CustomProjectSolutionFileFullBase> = [...allFiles, newFile]

    await dispatch(setSolutionFiles_AndRefreshEditor(
      newFiles,
      setSolutionFilesCustomProject,
      setForceEditorUpdateSessionDummyCustomProjectSolutionEditorAction
      )
    )

    if (allFiles.length === 0) {
      //this is the first file set it as the main file
      dispatch(setMainFileIdCustomProject(newFile.id))
    }

    //open the new file
    dispatch(openTabChangeCustomProjectSolutionEditor(newFile, newFiles, selectedEditorTabId))

  }
}

export function setMainFileIdCustomProject(mainFileId: number): SET_mainFileIdAction {
  return {
    type: ActionType.SET_mainFileId,
    mainFileId
  }
}

export function setCustomProjectCodeEditorSettingsDialogDisplayed(isDisplayed: boolean): SET_isCustomProjectCodeEditorSettingsDialogDisplayedAction {
  return {
    type: ActionType.SET_isCustomProjectCodeEditorSettingsDialogDisplayed,
    isDisplayed: isDisplayed
  }
}

export function setSelectedConsoleViewTest(selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive
  | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | RunnableCustomProjectTestFrontendExclusive | null): SET_selectedConsoleViewTestAction {
  return {
    type: ActionType.SET_selectedConsoleViewTest,
    selectedConsoleViewTest
  }
}

//--- panel indices

export function setTopRightSelectedTabIndex(index: number): SET_topRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_topRightSelectedTabIndex,
    topRightSelectedTabIndex: index
  }
}

export function setTopLeftSelectedTabIndex(index: number): SET_topLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_topLeftSelectedTabIndex,
    topLeftSelectedTabIndex: index
  }
}

export function setBottomRightSelectedTabIndex(index: number): SET_bottomRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomRightSelectedTabIndex,
    bottomRightSelectedTabIndex: index
  }
}

export function setBottomLeftSelectedTabIndex(index: number): SET_bottomLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomLeftSelectedTabIndex,
    bottomLeftSelectedTabIndex: index
  }
}


//--- dialog actions

export function setIsChangeTestDialogDisplayed(isDisplayed: boolean): SET_isChangeTestDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeTestDialogDisplayed,
    isDisplayed
  }
}


//--- panel view sizes

export function setTopRightPanelViewSize(topRightPanelViewSize: PanelViewSize): SET_topRightPanelViewSizeAction {
  return {
    type: ActionType.SET_topRightPanelViewSize,
    topRightPanelViewSize
  }
}

export function setTopLeftPanelViewSize(topLeftPanelViewSize: PanelViewSize): SET_topLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_topLeftPanelViewSize,
    topLeftPanelViewSize
  }
}

export function setBotRightPanelViewSize(botRightPanelViewSize: PanelViewSize): SET_botRightPanelViewSizeAction {
  return {
    type: ActionType.SET_botRightPanelViewSize,
    botRightPanelViewSize
  }
}

export function setBotLeftPanelViewSize(botLeftPanelViewSize: PanelViewSize): SET_botLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_botLeftPanelViewSize,
    botLeftPanelViewSize
  }
}


//--- console view settings

export function setUseDiffIfOutputMismatched(useDiffIfOutputMismatched: boolean): SET_useDiffIfOutputMismatchedAction {
  return {
    type: ActionType.SET_useDiffIfOutputMismatched,
    useDiffIfOutputMismatched
  }
}

export function setUseReplaceWhiteSpacesWithBullets(useReplaceWhiteSpacesWithBullets: boolean): SET_useReplaceWhiteSpacesWithBulletsAction {
  return {
    type: ActionType.SET_useReplaceWhiteSpacesWithBullets,
    useReplaceWhiteSpacesWithBullets
  }
}

//--- panels grippers
export function setCustomProjectHDividerPosOffset(hDividerPosOffset: number): SET_hDividerPosOffsetAction {
  return {
    type: ActionType.SET_hDividerPosOffset,
    hDividerPosOffset
  }
}

export function setCustomProjectVDividerPosOffset(vDividerPosOffset: number): SET_vDividerPosOffsetAction {
  return {
    type: ActionType.SET_vDividerPosOffset,
    vDividerPosOffset
  }
}

export function setInitialPLangId(initialPLangId: number): SET_initialPLangIdAction {
  return {
    type: ActionType.SET_initialPLangId,
    initialPLangId
  }
}


export function resetCustomProjectTestResultsDuringSave(): MultiActions {
  return dispatch => {

    dispatch(_resetCustomProjectTestResultsDuringSave())
    dispatch(setSelectedConsoleViewTest(null))

  }
}

//resets test results
/**
 * call this during saving the solution
 * @returns {RESET_TestResultsAction}
 */
export function _resetCustomProjectTestResultsDuringSave(): RESET_TestResultsAction {
  return {
    type: ActionType.RESET_TestResults,
  }
}


export function loadEditCustomProjectSite(customProjectId: number | null, isCreating: boolean, loadTestAssetContents: boolean): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(resetEditCustomProjectSite())

    await dispatch(loadManageCustomProjectSite())

    await dispatch(getTestTypesAsync())

    await dispatch(getPLangsAsync())

    if (isCreating) {
      //nothing to do here
    } else {

      const lastEditedPLangId = getState().editCustomProjectSiteState.customProject.lastEditorPLangId

      await dispatch(getCustomProjectAsync(customProjectId, loadTestAssetContents))


      await dispatch(getCustomProjectTestResultsAsync(customProjectId, getState().editCustomProjectSiteState.customProject.solution.pLangId))

      // await dispatch(
      //   getDoExerciseAllTestResults(releaseCode, lastEditedPLangId, getState().doExerciseSiteState.solution.id))

      //try to select the first displayed tab (if any)
      if (getState().editCustomProjectSiteState.customProject.solution.solutionFiles.length > 0
        && getState().editCustomProjectSiteState.customProject.solution.solutionFiles.filter(
          p => p.uiIsDisplayed).length > 0) {
        dispatch(setSelectedTabIdCustomProjectSolutionEditor(
          getState().editCustomProjectSiteState.customProject.solution.solutionFiles.filter(
            p => p.uiIsDisplayed)[0].id))
      }


    }


  }
}

export function setJustRunProgramArgsCustomProject(justRunProgramArgs: string): SET_justRunProgramArgsAction {
  return {
    type: ActionType.SET_justRunProgramArgs,
    justRunProgramArgs
  }
}

export function setIsJustRunProgramArgsInputDisplayedCustomProject(isJustRunProgramArgsInputDisplayed: boolean): SET_isJustRunProgramArgsInputDisplayedAction {
  return {
    type: ActionType.SET_isJustRunProgramArgsInputDisplayed,
    isJustRunProgramArgsInputDisplayed
  }
}

export function resetEditCustomProjectSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}