/**
 * Created by janis dähne.
 */
import {SET_testsAction} from "../../../reducers/editCustomProjectSite/editCustomProjectSiteReducer";
import {CustomProjectTestFullBase, RunnableCustomProjectTestFrontendExclusive} from "../../../../types/customProjects";
import {ActionType} from "../../../reducers/editCustomProjectSite/editCustomProjectSiteActionTypes";
import {DELETE_customProjectTestAction} from "../../../reducers/editCustomProjectSite/crud/deleteCustomProjectTestReducer";
import {delay} from "../../../../../debug";
import {deleteCustomProjectTest} from "../../../../communicationLayer/editCustomProjectLayer";
import {AwaitActions, MultiActions} from "../../types";
import {notifyError, notifySuccess} from "../../../../helpers/notificationHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";


export function setCustomProjectTests(tests: ReadonlyArray<RunnableCustomProjectTestFrontendExclusive>): SET_testsAction {
  return {
    type: ActionType.SET_tests,
    tests
  }
}


export function setCustomProjectTestIsRunning(tests: ReadonlyArray<RunnableCustomProjectTestFrontendExclusive>, allTests: ReadonlyArray<RunnableCustomProjectTestFrontendExclusive>): MultiActions {
  return dispatch => {

    const copy = allTests.map<RunnableCustomProjectTestFrontendExclusive>(p => {
      return tests.some(runningTest => p.id === runningTest.id)
        ? {
          ...p,
          uiIsRunning: true
        }
        : p
    })

    dispatch(setCustomProjectTests(copy))
  }
}

export function deleteCustomTestCustomProjectAsync(customProjectId: number, customTest: RunnableCustomProjectTestFrontendExclusive,
                                                   customTests: ReadonlyArray<RunnableCustomProjectTestFrontendExclusive>): AwaitActions {
  return async (dispatch) => {


    try {
      await dispatch(_deleteCustomTestCustomProjectAsync(customProjectId, customTest.id))

      //then remove the test from frontend
      const newCustomTests = customTests.filter(p => p.id !== customTest.id)
      dispatch(setCustomProjectTests(newCustomTests))

    } catch (err) {
      notifyError(translate(TranslateIds.couldNotDeleteCustomTest))
      return
    }

    notifySuccess(translate(TranslateIds.deletedCustomTest))

  }
}

export function _deleteCustomTestCustomProjectAsync(customProjectId: number, customTestId: number): DELETE_customProjectTestAction {
  return {
    type: ActionType.DELETE_customProjectTest,
    payload: delay(deleteCustomProjectTest(customProjectId, customTestId))
  }
}