/**
 * Created by janis dähne.
 */


import {
  SET_selectedExerciseIdAction, ResetAction,
  SET_comesFromOwnExercisesSiteAction, SET_isZoomedExerciseReleaseDisplayedAction, SET_isChangeDialogDisplayedAction,
  SET_sortByKeyAction
} from "../../../reducers/ownOrGroupExerciseReducers/exerciseReleasesSite/releaseSiteReducer";
import {ActionType} from "../../../reducers/ownOrGroupExerciseReducers/exerciseReleasesSite/releasesSiteActionTypes";
import {AwaitActions} from "../../types";
import {deleteExerciseReleaseAsync, getExerciseReleaseAsync} from "./getReleasesSiteCrudAction";
import {getOwnExercisesAsync} from "../../manageOwnExercisesSite/ownExercisesSiteActions";
import {getGroupExercisesAsync} from "../../manageGroupExercisesSite/groupExercisesSiteActions";
import {SortDirection} from "../../../../types/sortDirection";
import {ExerciseReleaseFromBackend} from "../../../../types/exerciseRelease";
import {getPLangsAsync} from "../../pLangsActions";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {getUserPermissionForGroup} from "../../groupStorage/groupStorageCrudActions";


export function setSortByKey(sortByKey: keyof ExerciseReleaseFromBackend, sortDirection: SortDirection): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSelectedExerciseId(exerciseId: number): SET_selectedExerciseIdAction {
  return {
    type: ActionType.SET_selectedExerciseId,
    id: exerciseId
  }
}

export function setComesFromOwnExercisesSite(comesFromOwnExercisesSite: boolean): SET_comesFromOwnExercisesSiteAction {
  return {
    type: ActionType.SET_comesFromOwnExercisesSite,
    comesFromOwnExercisesSite
  }
}


export function deleteExerciseReleaseAsyncAndReload(exerciseReleaseId: number, exerciseId: number, loadExercises: boolean, comesFromOwnExercisesSite: boolean): AwaitActions {
  return async (dispatch) => {

    await dispatch(deleteExerciseReleaseAsync(exerciseReleaseId))

    //TODO only if success

    if (loadExercises) {
      if (comesFromOwnExercisesSite) {
        await dispatch(getOwnExercisesAsync())
      } else {
        await dispatch(getGroupExercisesAsync())
      }
    }

    await dispatch(getExerciseReleaseAsync(exerciseId))

  }
}


export function setIsZoomedExerciseReleaseDisplayed(isDisplayed: boolean): SET_isZoomedExerciseReleaseDisplayedAction {
  return {
    type: ActionType.SET_isZoomedExerciseReleaseDisplayed,
    isDisplayed
  }
}

export function setIsChangeDialogDisplayedAction(isDisplayed: boolean): SET_isChangeDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeDialogDisplayed,
    isDisplayed
  }
}

export function loadExerciseReleasesSite(exerciseId: number, loadExercises: boolean, comesFromOwnExercisesSite: boolean): AwaitActions {
  return async (dispatch, getState) => {

    if (loadExercises) {
      if (comesFromOwnExercisesSite) {
        await dispatch(getOwnExercisesAsync())
      } else {
        await dispatch(getGroupExercisesAsync())
      }
    }

    const exercise = getState().releaseSiteState.exercises.find(p => p.id === exerciseId)

    if (!exercise) {

      errorDialog(translate(TranslateIds.error), translate(TranslateIds.couldNotFindExerciseOrNoPermission))
      return
    }

    dispatch(getUserPermissionForGroup(exercise.managingGroupId))

    //we need the plangs for the dialog to select a plang
    await dispatch(getPLangsAsync())

    dispatch(getExerciseReleaseAsync(exerciseId))

  }
}

export function resetExerciseReleasesSite(resetSiteLocalExercises: boolean): ResetAction {
  return {
    type: ActionType.RESET,
    resetSiteLocalExercises
  }
}