/**
 * Created by janis dähne.
 */

import {ActionType} from "../../../reducers/ownOrGroupExerciseReducers/submissionsSite/submissionsSiteActionTypes";
import {
  ResetAction, SET_automaticRefreshViewIntervalCounterAction, SET_isAutomaticRefreshViewEnabledAction,
  SET_runningAssessmentAction,
  SET_selectedReleaseIdAction, SET_sortByKeyAction
} from "../../../reducers/ownOrGroupExerciseReducers/submissionsSite/submissionsSiteReducer";
import {AwaitActions} from "../../types";
import {getExerciseReleaseAsync} from "../releasesSiteActions/getReleasesSiteCrudAction";
import {
  _rerunAutomaticAssessmentActionAsync,
  _rerunSingleAutomaticAssessmentActionAsync, getSubmissions
} from "./submissionsCrudActions";
import {getGroupExercisesAsync} from "../../manageGroupExercisesSite/groupExercisesSiteActions";
import {getOwnExercisesAsync} from "../../manageOwnExercisesSite/ownExercisesSiteActions";
import {AssessmentFullBase} from "../../../../types/submissions";
import {SortDirection} from "../../../../types/sortDirection";
import {getPLangsAsync} from "../../pLangsActions";
import Logger from "../../../../helpers/logger";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";


export function setSortByKey(sortByKey: keyof AssessmentFullBase, sortDirection: SortDirection): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSelectedReleaseId(exerciseReleaseId: number): SET_selectedReleaseIdAction {
  return {
    type: ActionType.SET_selectedReleaseId,
    id: exerciseReleaseId
  }
}

export function rerunSingleSubmissionAsync(releaseCode: string, runningAssessment: AssessmentFullBase, reloadSubmissions: boolean): AwaitActions {
  return async (dispatch) => {

    await dispatch(
      _rerunSingleAutomaticAssessmentActionAsync(releaseCode, runningAssessment.userId, runningAssessment.pLangId))


    if (reloadSubmissions) {
      //then reload page
      dispatch(getSubmissions(releaseCode))
    }

  }
}

/**
 * this sets the assessment as running and also clears the result (if an assessment is given)
 * @param {AssessmentFullBase} runningAssessment
 * @returns {SET_runningAssessmentAction}
 */
export function setRunningAssessment(runningAssessment: AssessmentFullBase | null): SET_runningAssessmentAction {
  return {
    type: ActionType.SET_runningAssessment,
    runningAssessment
  }
}


/**
 *
 * this is not really used because we don't get feedback from the server...
 * instead run every single assessment sequentially and report state
 * @deprecated
 * @param {string} releaseCode
 * @returns {AwaitActions}
 */
export function rerunAllSubmissionsAsync(releaseCode: string): AwaitActions {
  return async (dispatch) => {

    await dispatch(_rerunAutomaticAssessmentActionAsync(releaseCode))

    //then reload page
    dispatch(getSubmissions(releaseCode))

  }
}

/**
 *
 * @param {number} exerciseId
 * @param {number} releaseId
 * @param loadExercise
 * @param comesFromOwnExercisesSite
 * @param {boolean} loadExerciseReleases true: load the exercise releases because we need to releases from the releases site
 * and when the user starts with this site then the releases site is not loaded yet,
 * false: not
 * @param loadPLangs true: load all plangs (on page load), false: not (e.g. if we only want to refresh the view the plangs won't likely change)
 * @returns {AwaitActions}
 */
export function loadSubmissionsSite(exerciseId: number, releaseId: number,
                                    loadExercise: boolean, comesFromOwnExercisesSite: boolean,
                                    loadExerciseReleases: boolean, loadPLangs: boolean): AwaitActions {
  return async (dispatch, getState) => {

    if (loadExercise) {
      if (comesFromOwnExercisesSite) {
        await dispatch(getOwnExercisesAsync())
      } else {
        await dispatch(getGroupExercisesAsync())
      }
    }

    if (loadExerciseReleases) {
      await dispatch(getExerciseReleaseAsync(exerciseId))
    }

    //workaround until we changed the api to use the release id instead of the code
    const release = getState().releaseSiteState.releases.find(p => p.id === releaseId)

    if (!release) {
      Logger.error("no release found (submissionsSiteActions)")

      errorDialog(translate(TranslateIds.error),
        translate(TranslateIds.couldNotFindRelease)
        )
      return
    }

    if (loadPLangs) {
      await dispatch(getPLangsAsync())
    }

    dispatch(getSubmissions(release.generatedCode))

  }
}


export function setAutomaticRefreshViewIntervalCounter(automaticRefreshViewIntervalCounter: number): SET_automaticRefreshViewIntervalCounterAction {
  return {
    type: ActionType.SET_automaticRefreshViewIntervalCounter,
    automaticRefreshViewIntervalCounter,
  }
}

export function setIsAutomaticRefreshViewEnabled(isAutomaticRefreshViewEnabled: boolean): SET_isAutomaticRefreshViewEnabledAction {
  return {
    type: ActionType.SET_isAutomaticRefreshViewEnabled,
    isAutomaticRefreshViewEnabled,
  }
}

export function resetSubmissionsSite(resetSiteLocalExercises: boolean, resetSiteLocalReleases: boolean): ResetAction {
  return {
    type: ActionType.RESET,
    resetSiteLocalExercises,
    resetSiteLocalReleases
  }
}