/**
 * Created by janis dähne.
 */
import {
  ResetAction,
  SET_viewStateAction, SET_selectedGroupIdAction, SET_isAddUsersDialogDisplayedAction,
  SET_user_uiIsChangeGroupRolePopupDisplayedAction, SET_member_uiIsSelectedAction, SET_all_member_uiIsSelectedAction,
  SET_sortByKeyAction
} from "../../reducers/singleGroupSite/singleGroupSiteReducer";
import {SiteViewState} from "../../../types/siteView";
import {ActionType} from "../../reducers/singleGroupSite/singleGroupSiteActionTypes";
import {AwaitActions} from "../types";
import {
  getMembersAsync, removeMemberAsync,
  setMemberGroupRoleAsync
} from "./singleGroupSiteCrudActions";
import {getGroupsAsync} from "../groupsSite/groupsCrudActions";
import {getGroupRolesAsync} from "../groupRolesSite/groupRolesCrudActions";
import {SelectableGroupUserFrontend} from "../../../types/group";
import {SortDirection} from "../../../types/sortDirection";


export function setSelectedGroupId(groupId: number): SET_selectedGroupIdAction {
  return {
    type: ActionType.SET_selectedGroupId,
    groupId
  }
}


export function setIsAddUsersDialogDisplayed(isDisplayed: boolean): SET_isAddUsersDialogDisplayedAction {
  return {
    type: ActionType.SET_isAddUsersDialogDisplayed,
    isDisplayed
  }
}

export function setUiIsChangeGroupRolePopupDisplayedAction(isDisplayed: boolean, userId: number): SET_user_uiIsChangeGroupRolePopupDisplayedAction {
  return {
    type: ActionType.SET_user_uiIsChangeGroupRolePopupDisplayed,
    isDisplayed,
    userId
  }
}


export function setMemberGroupRoleAndRefreshAsync(userId: number, groupId: number, groupRoleId: number): AwaitActions {
  return async (dispatch) => {

    dispatch(setUiIsChangeGroupRolePopupDisplayedAction(false, userId))
    await dispatch(setMemberGroupRoleAsync(userId, groupId, groupRoleId, true))
    dispatch(loadSingleGroupSite(groupId, false))

  }
}

export function removeMultipleMemberAndRefreshAsync(userIds: ReadonlyArray<number>, groupId: number): AwaitActions {
  return async (dispatch) => {

    for (const userId of userIds) {
      await dispatch(removeMemberAsync(userId, groupId))
    }

    dispatch(loadSingleGroupSite(groupId, false))
  }
}

export function removeMemberAndRefreshAsync(userId: number, groupId: number): AwaitActions {
  return async (dispatch) => {

    await dispatch(removeMemberAsync(userId, groupId))
    dispatch(loadSingleGroupSite(groupId, false))
  }
}


export function setMemberUiIsSelected(memberId: number, isSelected: boolean): SET_member_uiIsSelectedAction {
  return {
    type: ActionType.SET_member_uiIsSelected,
    memberId,
    isSelected
  }
}

export function setAllMemberUiIsSelected(isSelected: boolean): SET_all_member_uiIsSelectedAction {
  return {
    type: ActionType.SET_all_member_uiIsSelected,
    isSelected
  }
}

export function setViewState(viewState: SiteViewState): SET_viewStateAction {
  return {
    type: ActionType.SET_viewState,
    viewState
  }
}


export function setSortByKey(sortByKey: keyof SelectableGroupUserFrontend, sortDirection: SortDirection): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

/**
 * loads the group roles, then the group members
 * @param {number} groupId
 * @param loadGroups true: load groups (because the user started with this site and we don't know any group,
 * false: the user came from the groups site so they are loaded...
 * @returns {AwaitActions}
 */
export function loadSingleGroupSite(groupId: number, loadGroups: boolean): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(resetSingleGroupSite(loadGroups ? true : false))

    if (loadGroups) {
      await dispatch(getGroupsAsync())
    }

    await dispatch(getGroupRolesAsync())

    dispatch(setSelectedGroupId(groupId))

    const groupRoles = getState().singleGroupState.groupRoles
    const currentUserId = getState().userDataSettingsSate.userData.id

    dispatch(getMembersAsync(groupId, currentUserId, groupRoles))

  }
}

export function resetSingleGroupSite(resetGroups: boolean): ResetAction {
  return {
    type: ActionType.RESET,
    resetGroups
  }
}
