/**
 * Created by janis dähne.
 */


import {Action} from "redux";
import {ReduxPromiseBaseAction, RootState} from "../reducers/index";
import {ThunkAction} from "redux-thunk";


/**
 * a type for crud actions (we need to wait for resolve/reject...
 *
 * note: the result is not void but we could not find any docs about the return type...
 *
 *
 * usage:
 * export function loadSettingsSite(): AwaitAction {
 *  return async (dispatch) => {
 *     await dispatch(getLangsAsync())
 *     dispatch(getUserSettingsAndDataAsync())
 *  }
 * }
 */
export type AwaitActions = (dispatch: <T>(action: ReduxPromiseBaseAction<T, any> | Action | AwaitActions | MultiActions | AwaitActionsWithResult<any> | void) => void, getState: () => RootState) => Promise<void>
//export type AwaitAction = (dispatch: <T>(action: ReduxPromiseBaseAction<T> | Action) => Promise<T>, getState: () => RootState) => Promise<void>

export type AwaitActionsWithResult<R> = (dispatch: <T>(action: ReduxPromiseBaseAction<T, any> | Action | AwaitActions | MultiActions | AwaitActionsWithResult<T>) => void, getState: () => RootState) => Promise<R>

/**
 * a better (own) Thunk<void, void, void> type to ensure only actions are passed in
 *
 *
 * try to avoid using getState because this makes the function less transparent...
 *
 */
export type MultiActions = (dispatch: <T>(action: Action | MultiActions | void) => void, getState: () => RootState) => void