/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "./_notExhausiveHelper";

export type State = {
  readonly lastKnownUrl: string
}


const initial: State = {
  lastKnownUrl: window.location.toString()
}


export enum ActionType {
  SET_LAST_KNOWN_URL = 'HistoryReducer_SET_LAST_KNOWN_URL',
  RESET = 'HistoryReducer_RESET',
}

export interface HistoryReducerActionBase extends Action {
  readonly type: ActionType
}

export interface SetLastKnownUrlAction extends HistoryReducerActionBase {
  readonly type: ActionType.SET_LAST_KNOWN_URL
  readonly lastKnownUrl: string
}


export interface ResetAction extends HistoryReducerActionBase {
  readonly type: ActionType.RESET
}

export type AllActions = SetLastKnownUrlAction | ResetAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_LAST_KNOWN_URL:
      return {
        ...state,
        lastKnownUrl: action.lastKnownUrl

      }
    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

