/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "./dashboardSiteActionTypes";
import {
  ClientServerSettings, HardwareUsageStatistic, HardwareUsageStatisticFrontendExclusive,
  LogicalUsageStatistic, TestServerSettings
} from "../../../types/dashboard";

import {
  AllActions as getClientServerSettingsActions,
  reducer as getClientServerSettingsReducer
} from "./crud/getClientServerSettingsReducer"
import {notExhaustive} from "../_notExhausiveHelper";

import {AllActions as getHardwareUsageActions, reducer as getHardwareUsageReducer} from './crud/getHardwareUsageReducer'

import {AllActions as getLogicalUsageActions, reducer as getLogicalUsageReducer} from './crud/getLogicalUsageReducer'

import {
  AllActions as getTestServerSettingsActions,
  reducer as getTestServerSettingsReducer
} from './crud/getTestServerSettingsReducer'

export type State = {
  readonly isLoading: boolean

  readonly hardwareUsageStatistic: ReadonlyArray<HardwareUsageStatisticFrontendExclusive>

  readonly logicalUsageStatistic: LogicalUsageStatistic | null

  readonly clientServerSettings: ClientServerSettings | null

  readonly testServerSettings: TestServerSettings | null

}


export const initial: State = {
  isLoading: false,

  hardwareUsageStatistic: [],

  logicalUsageStatistic: null,
  clientServerSettings: null,

  testServerSettings: null
}


export interface SET_isLoadingAction extends ActionBase {
  readonly type: ActionType.SET_isLoading
  readonly isLoading: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction

  | SET_isLoadingAction
  | getClientServerSettingsActions
  | getHardwareUsageActions
  | getLogicalUsageActions
  | getTestServerSettingsActions


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_testServerSettings_PENDING:
    case ActionType.GET_testServerSettings_FULFILLED:
    case ActionType.GET_testServerSettings_REJECTED:
      return getTestServerSettingsReducer(state, action)

    case ActionType.GET_logicalUsage_PENDING:
    case ActionType.GET_logicalUsage_FULFILLED:
    case ActionType.GET_logicalUsage_REJECTED:
      return getLogicalUsageReducer(state, action)

    case ActionType.GET_hardwareUsage_PENDING:
    case ActionType.GET_hardwareUsage_FULFILLED:
    case ActionType.GET_hardwareUsage_REJECTED:
      return getHardwareUsageReducer(state, action)

    case ActionType.GET_clientServerSettings_PENDING:
    case ActionType.GET_clientServerSettings_FULFILLED:
    case ActionType.GET_clientServerSettings_REJECTED:
      return getClientServerSettingsReducer(state, action)

    case ActionType.SET_isLoading:
      return {
        ...state,
        isLoading: action.isLoading
      }

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

