/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../../doExerciseSiteActionTypes";
import {initial, State} from "../../doExerciseSiteReducer";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import Logger from "../../../../../helpers/logger";
import {notExhaustive} from "../../../_notExhausiveHelper";
import {mergeNormalTestResults} from "../../../../../helpers/convertersAndTransformers";
import {TestAnswerFromBackend} from "../../../../../types/testsAndTestProtocol";


export interface RUN_afterNormalTestAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<TestAnswerFromBackend>, RUN_afterNormalTest_PENDINGAction, RUN_afterNormalTest_FULFILLEDAction, RUN_afterNormalTest_REJECTEDAction> {
  readonly type: ActionType.RUN_afterNormalTest
  readonly payload: Promise<ReadonlyArray<TestAnswerFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface RUN_afterNormalTest_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RUN_afterNormalTest_PENDING
}

//automatically dispatched by redux promise middleware
export interface RUN_afterNormalTest_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<TestAnswerFromBackend>> {
  readonly type: ActionType.RUN_afterNormalTest_FULFILLED
  readonly payload: ReadonlyArray<TestAnswerFromBackend>
}

//automatically dispatched by redux promise middleware
export interface RUN_afterNormalTest_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RUN_afterNormalTest_REJECTED
  readonly payload: any
}


export type AllActions =
  RUN_afterNormalTest_PENDINGAction
  | RUN_afterNormalTest_FULFILLEDAction
  | RUN_afterNormalTest_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RUN_afterNormalTest_PENDING:
      return {
        ...state,
        isLoading: false
      }
    case ActionType.RUN_afterNormalTest_FULFILLED:
      return {
        ...state,
        isLoading: false,
        afterNormalTests: mergeNormalTestResults(state.afterNormalTests, action.payload)
      }
    case ActionType.RUN_afterNormalTest_REJECTED:
      return {
        ...state,
        isLoading: false,
        afterNormalTests: mergeNormalTestResults(state.afterNormalTests, [])
      }

    default:
      notExhaustive(action)
      return state
  }
}

