/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {State, initial} from "../doExerciseSiteReducer";
import Logger from "../../../../helpers/logger";
import {SolutionVersionTupleFromBackend} from "../../../../types/exerciseSolution";
import {getNextFrontendId} from "../../../../helpers/frontendIdHelper";
import orderBy from 'lodash-es/orderBy'

export interface GET_doExercise_solutionVersionAction
  extends ActionBase, ReduxPromiseAction<SolutionVersionTupleFromBackend, GET_doExercise_solutionVersion_PENDINGAction, GET_doExercise_solutionVersion_FULFILLEDAction, GET_doExercise_solutionVersion_REJECTEDAction> {
  readonly type: ActionType.GET_doExercise_solutionVersion
  readonly payload: Promise<SolutionVersionTupleFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_solutionVersion_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_doExercise_solutionVersion_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_solutionVersion_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<SolutionVersionTupleFromBackend> {
  readonly type: ActionType.GET_doExercise_solutionVersion_FULFILLED
  readonly payload: SolutionVersionTupleFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_solutionVersion_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_doExercise_solutionVersion_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_doExercise_solutionVersion_PENDINGAction
  | GET_doExercise_solutionVersion_FULFILLEDAction
  | GET_doExercise_solutionVersion_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_doExercise_solutionVersion_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_doExercise_solutionVersion_FULFILLED:

      const solutionFromBackend = {
        ...action.payload.solutionVersion,
        //ensure same file display order (because display indices are not applied)
        //we cannot use ids because the backend saves files in any order --> ids are seqentially
        solutionFiles: orderBy(action.payload.solutionVersion.solutionFiles, p => p.fileNameWithExtension, 'asc')
      }

      return {
        ...state,
        isLoading: false,
        solution: solutionFromBackend,
        allSolutionVersionPreviews: action.payload.allSolutionVersionPreviews,
        afterSolution: {
          ...solutionFromBackend,
          id: getNextFrontendId() //just in case someone tries to save this --> this should be the same id as the real solution
        }
      }
    case ActionType.GET_doExercise_solutionVersion_REJECTED:
      return {
        ...state,
        isLoading: false,
        solution: null,
        allSolutionVersionPreviews: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

