/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "./changeCustomTestDialogActionTypes";
import {notExhaustive} from "../../../_notExhausiveHelper";
import {CustomTestForBackend} from "../../../../../types/customTests";
import {
  initial as initialCustomTest,
  AllActions as SetCustomTestActions,
  reducer as customTestReducer,
  validationRules
} from './customTestReducer'
import {isValidDialog} from "../../../../../validation/dialogValidations";

import {AllActions as CreateCustomTestActions, reducer as createCustomTestReducer} from './createCustomTestReducer'
import {AllActions as UpdateCustomTestActions, reducer as updateCustomTestReducer} from './updateCustomTestReducer'

export type State = {
  readonly isBusy: boolean
  readonly isCreating: boolean

  readonly isValid: boolean

  readonly customTest: CustomTestForBackend
}


export const initial: () => State = () => {
  return {
    isBusy: false,
    isCreating: false,
    customTest: initialCustomTest(),
    isValid: isValidDialog(initialCustomTest(), validationRules)
  }
}


export interface SET_isCreatingAction extends ActionBase {
  readonly type: ActionType.SET_isCreating
  readonly isCreating: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_isCreatingAction

  | SetCustomTestActions

  | CreateCustomTestActions
  | UpdateCustomTestActions


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_id:
    case ActionType.SET_testTypeId:
    case ActionType.SET_displayName:
    case ActionType.SET_content:
    case ActionType.SET_files:
    case ActionType.SET_displayIndex:

      const temp = customTestReducer(state.customTest, action)

      return {
        ...state,
        customTest: temp,
        isValid: isValidDialog(temp, validationRules)
      }

    case ActionType.CREATE_customTest_PENDING:
    case ActionType.CREATE_customTest_FULFILLED:
    case ActionType.CREATE_customTest_REJECTED:
      return createCustomTestReducer(state, action)

    case ActionType.UPDATE_customTest_PENDING:
    case ActionType.UPDATE_customTest_FULFILLED:
    case ActionType.UPDATE_customTest_REJECTED:
      return updateCustomTestReducer(state, action)


    case ActionType.SET_isCreating:
      return {
        ...state,
        isCreating: action.isCreating
      }


    case ActionType.RESET:
      return initial()

    default:
      notExhaustive(action)
      return state
  }
}


