/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "./changeCustomTestDialogActionTypes";
import {CustomTestFromBackend} from "../../../../../types/customTests";
import {initial, State} from "./changeCustomTestDialogReducer";
import Logger from "../../../../../helpers/logger";
import {convertCustomTestFromBackendToFrontend} from "../../../../../helpers/convertersAndTransformers";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {notExhaustive} from "../../../_notExhausiveHelper";


export interface UPDATE_customTestAction
  extends ActionBase, ReduxPromiseAction<CustomTestFromBackend, UPDATE_customTest_PENDINGAction, UPDATE_customTest_FULFILLEDAction, UPDATE_customTest_REJECTEDAction> {
  readonly type: ActionType.UPDATE_customTest
  readonly payload: Promise<CustomTestFromBackend>
}

//automatically dispatched by redux promise middleware
export interface UPDATE_customTest_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_customTest_PENDING
}

//automatically dispatched by redux promise middleware
export interface UPDATE_customTest_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<CustomTestFromBackend> {
  readonly type: ActionType.UPDATE_customTest_FULFILLED
  readonly payload: CustomTestFromBackend
}

//automatically dispatched by redux promise middleware
export interface UPDATE_customTest_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.UPDATE_customTest_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_customTest_PENDINGAction
  | UPDATE_customTest_FULFILLEDAction
  | UPDATE_customTest_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_customTest_PENDING:
      return {
        ...state,
        isBusy: true
      }
    case ActionType.UPDATE_customTest_FULFILLED:
      return {
        ...state,
        isBusy: false,
        customTest: convertCustomTestFromBackendToFrontend(action.payload)
      }
    case ActionType.UPDATE_customTest_REJECTED:
      return {
        ...state,
        isBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

