/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./editorExerciseSiteActionTypes";
import {State, initial} from "./exerciseEditorSiteReducer";
import Logger from "../../../helpers/logger";
import {EditorExerciseExerciseForBackend} from "../../../types/exerciseEditor";


export interface CREATE_editorExerciseAction
  extends ActionBase, ReduxPromiseAction<number, CREATE_editorExercise_PENDINGAction, CREATE_editorExercise_FULFILLEDAction, CREATE_editorExercise_REJECTEDAction> {
  readonly type: ActionType.CREATE_editorExercise
  readonly payload: Promise<number>
}

//automatically dispatched by redux promise middleware
export interface CREATE_editorExercise_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.CREATE_editorExercise_PENDING
}

//automatically dispatched by redux promise middleware
export interface CREATE_editorExercise_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<number> {
  readonly type: ActionType.CREATE_editorExercise_FULFILLED
  readonly payload: number
}

//automatically dispatched by redux promise middleware
export interface CREATE_editorExercise_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.CREATE_editorExercise_REJECTED
  readonly payload: any
}


export type AllActions =
  CREATE_editorExercise_PENDINGAction
  | CREATE_editorExercise_FULFILLEDAction
  | CREATE_editorExercise_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.CREATE_editorExercise_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.CREATE_editorExercise_FULFILLED:
      return {
        ...state,
        isLoading: false,
        editorExercise: {
          ...state.editorExercise,
          id: action.payload
        },
        savedNewExercise: true
      }
    case ActionType.CREATE_editorExercise_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

