/**
 * Created by janis dähne.
 */

import {ExerciseCodeTemplateFileFullBase, ExerciseCodeTemplateFullBase} from "../../../../../types/exerciseEditor";
import {ActionBase, ActionType} from "./changeCodeTemplateDialogActionTypes";
import {notExhaustive} from "../../../_notExhausiveHelper";
import {
  getValidationCollection, getValidationMessagesCollection,
  isIdString, isNotNan, isValidId
} from "../../../../../validation/validations";
import {getNextFrontendId} from "../../../../../helpers/frontendIdHelper";
import {translate} from "../../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../../i18n/ids";

export type State = ExerciseCodeTemplateFullBase


export const initial: () => State = () => {
  return {
    id: getNextFrontendId(),
    pLangId: NaN,
    mainFileId: NaN,
    templateFiles: []
  }
}

export const validationRules = getValidationCollection<State>({
  id: [],
  pLangId: [isNotNan],
  mainFileId: [isNotNan],
  templateFiles: []
})

export const validationMessageKeys = getValidationMessagesCollection<State>({
  id: '',
  pLangId: translate(TranslateIds.invalidSelectOption),
  mainFileId: translate(TranslateIds.invalidSelectOption),
  templateFiles: '',
})


export interface SET_pLangIdAction extends ActionBase {
  readonly type: ActionType.SET_pLangId
  readonly pLangId: number
}

export interface SET_mainFileIdAction extends ActionBase {
  readonly type: ActionType.SET_mainFileId
  readonly mainFileId: number
}

export interface SET_templateFilesAction extends ActionBase {
  readonly type: ActionType.SET_templateFiles
  readonly templateFiles: ReadonlyArray<ExerciseCodeTemplateFileFullBase>
}


export type AllActions =
  | SET_pLangIdAction
  | SET_mainFileIdAction
  | SET_templateFilesAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_pLangId:
      return {
        ...state,
        pLangId: action.pLangId
      }

    case ActionType.SET_mainFileId:
      return {
        ...state,
        mainFileId: action.mainFileId
      }

    case ActionType.SET_templateFiles:
      return {
        ...state,
        templateFiles: action.templateFiles
      }

    default:
      notExhaustive(action)
      return state
  }
}

