/**
 * Created by janis dähne.
 */

import {notExhaustive} from "../../_notExhausiveHelper";
import {ActionBase, ActionType} from "../editorExerciseSiteActionTypes";
import {EditorExercisePropertiesFullBase} from "../../../../types/exerciseEditor";

export type State = EditorExercisePropertiesFullBase


export const initial: State = {
  displayName: '',
  shortDescription: '',
  isOnlyVisibleToMe: true,
  canUserCreateCustomTests: true,
  canUserCreateFiles: true,
  showSubmitResults: true,
  userGroupId: -1
}

export interface SET_displayNameAction extends ActionBase {
  readonly type: ActionType.SET_displayName
  readonly displayName: string
}

export interface SET_shortDescriptionAction extends ActionBase {
  readonly type: ActionType.SET_shortDescription
  readonly shortDescription: string
}

export interface SET_isOnlyVisibleToMeAction extends ActionBase {
  readonly type: ActionType.SET_isOnlyVisibleToMe
  readonly isOnlyVisibleToMe: boolean
}

export interface SET_canUserCreateCustomTestsAction extends ActionBase {
  readonly type: ActionType.SET_canUserCreateCustomTests
  readonly canUserCreateCustomTests: boolean
}

export interface SET_canUserCreateFilesAction extends ActionBase {
  readonly type: ActionType.SET_canUserCreateFiles
  readonly canUserCreateFiles: boolean
}

export interface SET_showSubmitResultsAction extends ActionBase {
  readonly type: ActionType.SET_showSubmitResults
  readonly showSubmitResults: boolean
}

export interface SET_userGroupIdAction extends ActionBase {
  readonly type: ActionType.SET_userGroupId
  readonly userGroupId: number
}


export type AllActions =
  SET_displayNameAction
  | SET_shortDescriptionAction
  | SET_isOnlyVisibleToMeAction
  | SET_canUserCreateCustomTestsAction
  | SET_canUserCreateFilesAction
  | SET_showSubmitResultsAction
  | SET_userGroupIdAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_displayName:
      return {
        ...state,
        displayName: action.displayName
      }

    case ActionType.SET_shortDescription:
      return {
        ...state,
        shortDescription: action.shortDescription
      }

    case ActionType.SET_isOnlyVisibleToMe:
      return {
        ...state,
        isOnlyVisibleToMe: action.isOnlyVisibleToMe
      }

    case ActionType.SET_canUserCreateCustomTests:
      return {
        ...state,
        canUserCreateCustomTests: action.canUserCreateCustomTests
      }

    case ActionType.SET_canUserCreateFiles:
      return {
        ...state,
        canUserCreateFiles: action.canUserCreateFiles
      }

    case ActionType.SET_showSubmitResults:
      return {
        ...state,
        showSubmitResults: action.showSubmitResults
      }

    case ActionType.SET_userGroupId:
      return {
        ...state,
        userGroupId: action.userGroupId
      }

    default:
      notExhaustive(action)
      return state
  }
}

