/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {initial, State} from "./groupRolesSiteReducer";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionType, ActionBase} from "./groupRolesSiteActionTypes";
import {notExhaustive} from "../_notExhausiveHelper";
import Logger from "../../../helpers/logger";



export interface DELETE_groupRoleAction
  extends ActionBase, ReduxPromiseAction<void, DELETE_groupRole_PENDINGAction, DELETE_groupRole_FULFILLEDAction, DELETE_groupRole_REJECTEDAction> {
  readonly type: ActionType.DELETE_groupRole
  readonly payload: Promise<void>
}

//automatically dispatched by redux promise middleware
export interface DELETE_groupRole_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DELETE_groupRole_PENDING
}

//automatically dispatched by redux promise middleware
export interface DELETE_groupRole_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void> {
  readonly type: ActionType.DELETE_groupRole_FULFILLED
  readonly payload: void
}

//automatically dispatched by redux promise middleware
export interface DELETE_groupRole_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.DELETE_groupRole_REJECTED
  readonly payload: any
}


export type AllActions =
  DELETE_groupRole_PENDINGAction
  | DELETE_groupRole_FULFILLEDAction
  | DELETE_groupRole_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.DELETE_groupRole_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.DELETE_groupRole_FULFILLED:
      return {
        ...state,
        isLoading: false
      }
    case ActionType.DELETE_groupRole_REJECTED:
      Logger.error('rejected delete data, prev state', state)
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}


