/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionType, ActionBase} from "./groupRolesSiteActionTypes";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import Logger from "../../../helpers/logger";
import {initial, State} from "./groupRolesSiteReducer";

export interface GET_groupRolesAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<GroupRoleWithPermissionFullBase>, GET_groupRoles_PENDINGAction, GET_groupRoles_FULFILLEDAction, GET_groupRoles_REJECTEDAction> {
  readonly type: ActionType.GET_groupRoles
  readonly payload: Promise<ReadonlyArray<GroupRoleWithPermissionFullBase>>
}

//automatically dispatched by redux promise middleware
export interface GET_groupRoles_PENDINGAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<GroupRoleWithPermissionFullBase>> {
  readonly type: ActionType.GET_groupRoles_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_groupRoles_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<GroupRoleWithPermissionFullBase>> {
  readonly type: ActionType.GET_groupRoles_FULFILLED
  readonly payload: ReadonlyArray<GroupRoleWithPermissionFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_groupRoles_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<ReadonlyArray<GroupRoleWithPermissionFullBase>> {
  readonly type: ActionType.GET_groupRoles_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_groupRoles_PENDINGAction
  | GET_groupRoles_FULFILLEDAction
  | GET_groupRoles_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_groupRoles_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_groupRoles_FULFILLED:
      return {
        ...state,
        groupRoles: action.payload,
        isLoading: false
      }
    case ActionType.GET_groupRoles_REJECTED:
      return {
        ...state,
        groupRoles: [],
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

