/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "./actionTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {UserGroupFullBase} from "../../../../types/group";
import {initial, State} from "./changeGroupReducer";
import {notExhaustive} from "../../_notExhausiveHelper";


export interface CREATE_groupAction
  extends ActionBase, ReduxPromiseAction<UserGroupFullBase, CREATE_group_PENDINGAction, CREATE_group_FULFILLEDAction, CREATE_group_REJECTEDAction> {
  readonly type: ActionType.CREATE_group
  readonly payload: Promise<UserGroupFullBase>
}

//automatically dispatched by redux promise middleware
export interface CREATE_group_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.CREATE_group_PENDING
}

//automatically dispatched by redux promise middleware
export interface CREATE_group_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<UserGroupFullBase> {
  readonly type: ActionType.CREATE_group_FULFILLED
  readonly payload: UserGroupFullBase
}

//automatically dispatched by redux promise middleware
export interface CREATE_group_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<UserGroupFullBase> {
  readonly type: ActionType.CREATE_group_REJECTED
  readonly payload: any
}


export type AllActions =
  CREATE_group_PENDINGAction
  | CREATE_group_FULFILLEDAction
  | CREATE_group_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.CREATE_group_PENDING:
      return {
        ...state,
        isBusy: true
      }
    case ActionType.CREATE_group_FULFILLED:
      return {
        ...state,
        isBusy: false,
      }
    case ActionType.CREATE_group_REJECTED:
      return {
        ...state,
        isBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}