/**
 * Created by janis dähne.
 */



import {Action} from "redux";
import {notExhaustive} from "./_notExhausiveHelper";
import {escapeJSON} from "../../helpers/jsonTransformer";

export type State = {
  readonly text: string
  readonly escapedText: string
}


export const initial: State = {
  text: '',
  escapedText: ''
}

export interface ActionBase extends Action {
  readonly type: ActionType
}


export enum ActionType {
  SET_text = 'jsonEscapeEditorReducer_SET_text',
  RESET = 'jsonEscapeEditorReducer_RESET',
}


export interface SET_textAction extends ActionBase {
  readonly type: ActionType.SET_text
  readonly text: string
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_textAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_text:
      return {
        ...state,
        text: action.text,
        escapedText: escapeJSON(action.text)
      }

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

