/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "./_notExhausiveHelper";
import {LogItem, SeverityLevel} from '../../helpers/logger'

export type State = {
  readonly isOpen: boolean

  //see logger.ts > logWithLevel why we can't store the log inside redux
  //basically because when we get an error during mapStateToProps we push an item and
  // somehow we re-run all mounted comps with mapStateToProps which will cause an error -> log infinite loop

  //so we only output a snapshot if an array (logger.ts > myLog) at the time we opened the logger panel
  //readonly log: ReadonlyArray<LogItem>

  /**
   * the selected log level or null (display all)
   */
  readonly selectedLogLevel?: SeverityLevel
}

const initial: State = {
  isOpen: false,
  //log: [],
  selectedLogLevel: null
}


export enum ActionType {
  SET_IS_OPEN = 'LoggerPanel_SET_IS_OPEN',
  //ADD_LOG_ITEM = 'LoggerPanel_ADD_LOG_ITEM',
  SET_SELECTED_LOG_LEVEL = 'LoggerPanel_SET_SELECTED_LOG_LEVEL',
  RESET = 'LoggerPanel_RESET',
}

export interface LoggerPanelActionBase extends Action {
  readonly type: ActionType
}

export interface LoggerPanelActionSetIsOpen extends LoggerPanelActionBase {
  readonly type: ActionType.SET_IS_OPEN
  readonly isOpen: boolean
}

export interface LoggerPanelActionSetLogLevel extends LoggerPanelActionBase {
  readonly type: ActionType.SET_SELECTED_LOG_LEVEL
  readonly logLevel: SeverityLevel
}

// export interface LoggerPanelActionAddLogItem extends LoggerPanelActionBase {
//   readonly type: ActionType.ADD_LOG_ITEM
//   readonly logItem: LogItem
// }


export interface LoggerPanelActionReset extends LoggerPanelActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  LoggerPanelActionSetIsOpen
  | LoggerPanelActionReset
  //| LoggerPanelActionAddLogItem
  | LoggerPanelActionSetLogLevel


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_IS_OPEN:
      return {
        ...state,
        isOpen: action.isOpen
      }

    // case ActionType.ADD_LOG_ITEM:
    //   return {
    //     ...state,
    //     log: [...state.log, action.logItem] //same as state.log.concat(action.logItem) see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/Spread_operator
    //   }

    case ActionType.SET_SELECTED_LOG_LEVEL:
      return {
        ...state,
        selectedLogLevel: action.logLevel
      }
    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

