/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {BaseUserData, IdAndPasswordTuple, SystemRoleFullBase} from "../../../../types/userData";
import {
  getValidationCollection, getValidationMessagesCollection,
  isEmailAndNotEmpty, isNotEmptyOrSpaces
} from "../../../../validation/validations";
import {ActionBase, ActionType} from "./changeActivatedUserDataDialogActionTypes";
import {notExhaustive} from "../../_notExhausiveHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {translate} from "../../../../../i18n/translationHelper";


//TODO make 1 post for backend not two --> then make 1 type (BaseUserData + systemRole)

export type State = BaseUserData & IdAndPasswordTuple & {
  readonly systemRole: SystemRoleFullBase | null
}

export const initial: State = {
  targetUserId: -1,
  email: '',
  lastName: '',
  firstName: '',
  password: '',
  systemRole: null
}

export const validationRules = getValidationCollection<State>({
  targetUserId: [],
  email: [isEmailAndNotEmpty],
  lastName: [isNotEmptyOrSpaces],
  firstName: [isNotEmptyOrSpaces],
  password: [],
  systemRole: [],
})


export const validationMessageKeys = getValidationMessagesCollection<State>({
  targetUserId: '',
  email: translate(TranslateIds.invalidEmail),
  lastName: translate(TranslateIds.fieldShouldNotBeEmptyOrSpaces),
  firstName: translate(TranslateIds.fieldShouldNotBeEmptyOrSpaces),
  password: '',
  systemRole: '',
})


export interface SET_targetUserIdAction extends ActionBase {
  readonly type: ActionType.SET_targetUserId
  readonly targetId: number
}

export interface SET_emailAction extends ActionBase {
  readonly type: ActionType.SET_email
  readonly email: string
}

export interface SET_lastNameAction extends ActionBase {
  readonly type: ActionType.SET_lastName
  readonly lastName: string
}

export interface SET_firstNameAction extends ActionBase {
  readonly type: ActionType.SET_firstName
  readonly firstName: string
}

export interface SET_passwordAction extends ActionBase {
  readonly type: ActionType.SET_password
  readonly password: string
}


export interface SET_systemRoleAction extends ActionBase {
  readonly type: ActionType.SET_systemRole
  readonly systemRole: SystemRoleFullBase | null
}

export type AllActions =
  SET_targetUserIdAction
  | SET_emailAction
  | SET_lastNameAction
  | SET_firstNameAction
  | SET_passwordAction
  | SET_systemRoleAction

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_targetUserId:
      return {
        ...state,
        targetUserId: action.targetId
      }
    case ActionType.SET_email:
      return {
        ...state,
        email: action.email
      }
    case ActionType.SET_lastName:
      return {
        ...state,
        lastName: action.lastName
      }
    case ActionType.SET_firstName:
      return {
        ...state,
        firstName: action.firstName
      }
    case ActionType.SET_password:
      return {
        ...state,
        password: action.password
      }
    case ActionType.SET_systemRole:
      return {
        ...state,
        systemRole: action.systemRole
      }

    default:
      notExhaustive(action)
      return state
  }
}

