/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./groupExercisesSiteActionTypes";
import {GroupPreviewExerciseTupleFromBackend} from "../../../types/exercisePreview";
import {State, initial} from "./groupExercisesSiteReducer";
import Logger from "../../../helpers/logger";

export interface GET_groupExercisesAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<GroupPreviewExerciseTupleFromBackend>, GET_groupExercises_PENDINGAction, GET_groupExercises_FULFILLEDAction, GET_groupExercises_REJECTEDAction> {
  readonly type: ActionType.GET_groupExercises
  readonly payload: Promise<ReadonlyArray<GroupPreviewExerciseTupleFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_groupExercises_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_groupExercises_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_groupExercises_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<GroupPreviewExerciseTupleFromBackend>> {
  readonly type: ActionType.GET_groupExercises_FULFILLED
  readonly payload: ReadonlyArray<GroupPreviewExerciseTupleFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_groupExercises_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<ReadonlyArray<GroupPreviewExerciseTupleFromBackend>> {
  readonly type: ActionType.GET_groupExercises_REJECTED
  readonly payload: any
}


export type AllActions =
   GET_groupExercises_PENDINGAction
  | GET_groupExercises_FULFILLEDAction
  | GET_groupExercises_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_groupExercises_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_groupExercises_FULFILLED:
      return {
        ...state,
        groupExercises: action.payload,
        isLoading: false
      }
    case ActionType.GET_groupExercises_REJECTED:
      return {
        ...state,
        groupExercises: [],
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

