/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {
  EditableExercisePreviewFromBackend,
  GroupPreviewExerciseTupleFromBackend
} from "../../../types/exercisePreview";
import {notExhaustive} from "../_notExhausiveHelper";
import {SiteViewState} from "../../../types/siteView";
import {SortDirection} from "../../../types/sortDirection";
import {ActionType, ActionBase} from "./groupExercisesSiteActionTypes";
import {AllActions as GetGroupExercisesActions, reducer as getGroupExercisesReducer} from './getGroupExercisesReducer'
import {
  AllActions as DeleteGroupExerciseActions,
  reducer as deleteGroupExerciseReducer
} from './deleteGroupExerciseReducer'
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";

export type State = {
  readonly groupExercises: ReadonlyArray<GroupPreviewExerciseTupleFromBackend>
  readonly viewState: SiteViewState
  readonly isLoading: boolean

  readonly sortByKey: keyof EditableExercisePreviewFromBackend
  readonly sortDirection: SortDirection

  readonly isTagsFilterDisplayed: boolean
}


export const initial: State = {
  groupExercises: [],
  viewState: SiteViewState.List,
  isLoading: false,

  sortByKey: 'displayName',
  sortDirection: SortDirection.ascending,

  isTagsFilterDisplayed: false,
}

export interface SET_groupExercisesAction extends ActionBase {
  readonly type: ActionType.SET_groupExercises
  readonly exercises: ReadonlyArray<GroupPreviewExerciseTupleFromBackend>
}

export interface SET_viewStateAction extends ActionBase {
  readonly type: ActionType.SET_viewState
  readonly viewState: SiteViewState
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof EditableExercisePreviewFromBackend
  readonly sortDirection: SortDirection
}

export interface SET_isTagsFilterDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isTagsFilterDisplayed
  readonly isTagsFilterDisplayed: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_groupExercisesAction
  | SET_viewStateAction
  | SET_sortByKeyAction
  | SET_isTagsFilterDisplayedAction

  | GetGroupExercisesActions
  | DeleteGroupExerciseActions


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_groupExercises:
      return {
        ...state,
        groupExercises: action.exercises
      }

    case ActionType.SET_viewState:

      FrontendSettingsManager.setGroupExercisesSiteViewState(action.viewState)

      return {
        ...state,
        viewState: action.viewState
      }

    case ActionType.SET_isTagsFilterDisplayed:

      FrontendSettingsManager.setGroupExercisesSiteTagsFilterIsDisplayed(action.isTagsFilterDisplayed)

      return {
        ...state,
        isTagsFilterDisplayed: action.isTagsFilterDisplayed
      }

    case ActionType.GET_groupExercises_PENDING:
    case ActionType.GET_groupExercises_FULFILLED:
    case ActionType.GET_groupExercises_REJECTED:
      return getGroupExercisesReducer(state, action)

    case ActionType.DELETE_groupExercise_PENDING:
    case ActionType.DELETE_groupExercise_FULFILLED:
    case ActionType.DELETE_groupExercise_REJECTED:
      return deleteGroupExerciseReducer(state, action)

    case ActionType.SET_sortByKey:
      return {
        ...state,
        sortByKey: action.sortByKey,
        sortDirection: state.sortByKey !== action.sortByKey ? SortDirection.ascending : action.sortDirection % 2
      }

    case ActionType.RESET:
      return {
        ...initial,
        viewState: FrontendSettingsManager.getFrontendSettings().groupExercisesSiteViewState,
        isTagsFilterDisplayed: FrontendSettingsManager.getFrontendSettings().groupExercisesSiteTagsFilterIsDisplayed
      }

    default:
      notExhaustive(action)
      return state
  }
}

