/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {ActionBase, ActionType} from "./manageNewUsersSiteActionTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {State, initial} from './manageNewUsersSiteReducer'
import Logger from "../../../helpers/logger";
import {notExhaustive} from "../_notExhausiveHelper";

export type Meta = {
  readonly shouldHideLoading: boolean
}

export interface ACTIVATE_newUserAction
  extends ActionBase, ReduxPromiseAction<void, ACTIVATE_newUser_PENDINGAction, ACTIVATE_newUser_FULFILLEDAction, ACTIVATE_newUser_REJECTEDAction, Meta> {
  readonly type: ActionType.ACTIVATE_newUser
  readonly payload: Promise<void>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface ACTIVATE_newUser_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.ACTIVATE_newUser_PENDING
}

//automatically dispatched by redux promise middleware
export interface ACTIVATE_newUser_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void, Meta> {
  readonly type: ActionType.ACTIVATE_newUser_FULFILLED
  readonly payload: void
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface ACTIVATE_newUser_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.ACTIVATE_newUser_REJECTED
  readonly payload: any
  readonly meta: Meta
}

export type AllActions =
  ACTIVATE_newUser_PENDINGAction
  | ACTIVATE_newUser_FULFILLEDAction
  | ACTIVATE_newUser_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.ACTIVATE_newUser_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.ACTIVATE_newUser_FULFILLED:
      return {
        ...state,
        isLoading: action.meta.shouldHideLoading ? false : state.isLoading
      }
    case ActionType.ACTIVATE_newUser_REJECTED:
      Logger.error('rejected delete data, prev state', state)
      return {
        ...state,
        isLoading: action.meta.shouldHideLoading ? false : state.isLoading
      }

    default:
      notExhaustive(action)
      return state
  }
}

