/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {EditableExercisePreviewFromBackend} from "../../../types/exercisePreview";
import {ActionBase, ActionType} from "./ownExercisesSiteActionTypes";
import {initial, State} from "./ownExercisesSiteReducer";
import Logger from "../../../helpers/logger";

export interface GET_ownExercisesAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<EditableExercisePreviewFromBackend>, GET_ownExercises_PENDINGAction, GET_ownExercises_FULFILLEDAction, GET_ownExercises_REJECTEDAction> {
  readonly type: ActionType.GET_ownExercises
  readonly payload: Promise<ReadonlyArray<EditableExercisePreviewFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_ownExercises_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_ownExercises_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_ownExercises_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<EditableExercisePreviewFromBackend>> {
  readonly type: ActionType.GET_ownExercises_FULFILLED
  readonly payload: ReadonlyArray<EditableExercisePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_ownExercises_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<ReadonlyArray<EditableExercisePreviewFromBackend>> {
  readonly type: ActionType.GET_ownExercises_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_ownExercises_PENDINGAction
  | GET_ownExercises_FULFILLEDAction
  | GET_ownExercises_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_ownExercises_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_ownExercises_FULFILLED:
      return {
        ...state,
        isLoading: false,
        ownExercises: action.payload,
      }
    case ActionType.GET_ownExercises_REJECTED:
      return {
        ...state,
        isLoading: false,
        ownExercises: [],
      }

    default:
      notExhaustive(action)
      return state
  }
}

