/**
 * Created by janis dähne.
 */
import {EditableExercisePreviewFromBackend} from "../../../types/exercisePreview";
import {notExhaustive} from "../_notExhausiveHelper";
import {SiteViewState} from "../../../types/siteView";
import {ActionType, ActionBase} from "./ownExercisesSiteActionTypes";
import {AllActions as GetOwnExercisesActions, reducer as getOwnExercisesReducer} from './getOwnExercisesReducer'
import {SortDirection} from "../../../types/sortDirection";
import {AllActions as DeleteOwnExerciseActions, reducer as deleteOwnExerciseReducer} from './deleteOwnExerciseReducer'
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";

export type State = {
  readonly ownExercises: ReadonlyArray<EditableExercisePreviewFromBackend>
  readonly viewState: SiteViewState
  readonly isLoading: boolean

  readonly sortByKey: keyof EditableExercisePreviewFromBackend
  readonly sortDirection: SortDirection

  readonly isTagsFilterDisplayed: boolean
}


export const initial: State = {
  ownExercises: [],
  viewState: SiteViewState.List,
  isLoading: false,

  sortByKey: 'displayName',
  sortDirection: SortDirection.ascending,

  isTagsFilterDisplayed: false,
}


export interface SET_viewStateAction extends ActionBase {
  readonly type: ActionType.SET_viewState
  readonly viewState: SiteViewState
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof EditableExercisePreviewFromBackend
  readonly sortDirection: SortDirection
}

export interface SET_isTagsFilterDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isTagsFilterDisplayed
  readonly isTagsFilterDisplayed: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_viewStateAction
  | SET_sortByKeyAction
  | SET_isTagsFilterDisplayedAction

  | DeleteOwnExerciseActions
  | GetOwnExercisesActions


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_ownExercises_PENDING:
    case ActionType.GET_ownExercises_FULFILLED:
    case ActionType.GET_ownExercises_REJECTED:
      return getOwnExercisesReducer(state, action)

    case ActionType.DELETE_ownExercise_PENDING:
    case ActionType.DELETE_ownExercise_FULFILLED:
    case ActionType.DELETE_ownExercise_REJECTED:
      return deleteOwnExerciseReducer(state, action)

    case ActionType.SET_viewState:

      FrontendSettingsManager.setOwnExercisesSiteViewState(action.viewState)

      return {
        ...state,
        viewState: action.viewState
      }

    case ActionType.SET_isTagsFilterDisplayed:

      FrontendSettingsManager.setOwnExercisesSiteTagsFilterIsDisplayed(action.isTagsFilterDisplayed)

      return {
        ...state,
        isTagsFilterDisplayed: action.isTagsFilterDisplayed
      }

    case ActionType.SET_sortByKey:
      return {
        ...state,
        sortByKey: action.sortByKey,
        sortDirection: state.sortByKey !== action.sortByKey ? SortDirection.ascending : action.sortDirection % 2
      }

    case ActionType.RESET:
      return {
        ...initial,
        viewState: FrontendSettingsManager.getFrontendSettings().ownExercisesSiteViewState,
        isTagsFilterDisplayed: FrontendSettingsManager.getFrontendSettings().ownExercisesSiteTagsFilterIsDisplayed
      }

    default:
      notExhaustive(action)
      return state
  }
}

