/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./manageTagsSiteActionTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {TagFullBase} from "../../../types/tag";
import {initial, State} from "./manageTagsSiteReducer";
import Logger from "../../../helpers/logger";


export interface DELETE_tagAction
  extends ActionBase, ReduxPromiseAction<void, DELETE_tag_PENDINGAction, DELETE_tag_FULFILLEDAction, DELETE_tag_REJECTEDAction> {
  readonly type: ActionType.DELETE_tag
  readonly payload: Promise<void>
}

//automatically dispatched by redux promise middleware
export interface DELETE_tag_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DELETE_tag_PENDING
}

//automatically dispatched by redux promise middleware
export interface DELETE_tag_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void> {
  readonly type: ActionType.DELETE_tag_FULFILLED
  readonly payload: void
}

//automatically dispatched by redux promise middleware
export interface DELETE_tag_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.DELETE_tag_REJECTED
  readonly payload: any
}


export type AllActions =
  DELETE_tag_PENDINGAction
  | DELETE_tag_FULFILLEDAction
  | DELETE_tag_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.DELETE_tag_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.DELETE_tag_FULFILLED:
      return {
        ...state,
        isLoading: false
      }
    case ActionType.DELETE_tag_REJECTED:
      Logger.error('rejected delete data, prev state', state)
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

