/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "../_notExhausiveHelper";
import {TagFullBase} from "../../../types/tag";
import {ActionBase, ActionType} from "./manageTagsSiteActionTypes";
import {AllActions as GetActions, reducer as getReducer} from './getTagsReducer'
import {AllActions as DeleteActions, reducer as deleteReducer} from './deleteTagReducer'

export type State = {
  readonly tags: ReadonlyArray<TagFullBase>
  readonly isLoading: boolean
  readonly isChangeDialogDisplayed: boolean
}


export const initial: State = {
  tags: [],
  isLoading: false,
  isChangeDialogDisplayed: false
}


export interface SET_tagsAction extends ActionBase {
  readonly type: ActionType.SET_tags
  readonly tags: ReadonlyArray<TagFullBase>
}

export interface SET_isChangeDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeDialogDisplayed
  readonly isDisplayed: boolean
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_tagsAction
  | SET_isChangeDialogDisplayedAction
  | GetActions
  | DeleteActions

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_tags:
      return {
        ...state,
        tags: action.tags
      }
    case ActionType.SET_isChangeDialogDisplayed:
      return {
        ...state,
        isChangeDialogDisplayed: action.isDisplayed
      }


    case ActionType.GET_tags_PENDING:
    case ActionType.GET_tags_FULFILLED:
    case ActionType.GET_tags_REJECTED:
      return getReducer(state, action)


    case ActionType.DELETE_tag_PENDING:
    case ActionType.DELETE_tag_FULFILLED:
    case ActionType.DELETE_tag_REJECTED:
      return deleteReducer(state, action)

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

