/**
 * Created by janis dähne.
 */


import {ExerciseReleaseForBackend} from "../../../../../../types/exerciseRelease";
import {ReleaseDurationType} from "../../../../../../types/ReleaseDurationType";
import {ReleaseStartType} from "../../../../../../types/ReleaseStartType";
import {ActionBase, ActionType} from "./changeExerciseReleaseDialogActionTypes";
import {notExhaustive} from "../../../../_notExhausiveHelper";
import {
  getValidationCollection, getValidationMessagesCollection,
  isNotNullOrUndefined, isPositiveNumber, isValidId
} from "../../../../../../validation/validations";
import {Moment} from "moment";
import moment = require("moment");
import {TranslateIds} from "../../../../../../../i18n/ids";
import {translate} from "../../../../../../../i18n/translationHelper";

export type State = ExerciseReleaseForBackend


// noinspection TsLint
export const initial: State = {
  id: -1,
  isReleased: false,
  /**
   * not send to backend only received
   */
  releasedAt: null,
  generatedCode: '',
  pLangId: null,
  automaticStartAt: moment().startOf('hour').toDate(),
  automaticEndAt: moment().startOf('hour').add(30, 'minutes').toDate(),
  hasLimitedWorkingTime: false,
  availableWorkingTimeInMinutes: 30,
  note: '',
  isVisibleToAllAfterRelease: false,
  shouldAutomaticAssessSubmissions: true,
  runAlsoNormalTests: true,
  releaseDurationType: ReleaseDurationType.Infinitely,
  releaseStartType: ReleaseStartType.Manual,

  //TODO not set in frontend maybe other type??
  hasAutomaticAssessmentStarted: false,
  hasAutomaticAssessmentFinished: false,
  hadAutomaticAssessmentErrors: false,

  hideInOverviews: false,
  shouldClearClipboard: false,
  hideSiteHeaderBar: false,
}

export const validationRules = getValidationCollection<ExerciseReleaseForBackend>({
  id: [],
  isReleased: [],
  releasedAt: [],
  generatedCode: [],
  pLangId: [isValidId],
  automaticEndAt: [],
  automaticStartAt: [],
  availableWorkingTimeInMinutes: [isPositiveNumber],
  note: [],
  hasLimitedWorkingTime: [],
  isVisibleToAllAfterRelease: [],
  shouldAutomaticAssessSubmissions: [],
  runAlsoNormalTests: [],
  releaseDurationType: [],
  releaseStartType: [],
  hasAutomaticAssessmentStarted: [],
  hasAutomaticAssessmentFinished: [],
  hadAutomaticAssessmentErrors: [],
  hideInOverviews: [],
  shouldClearClipboard: [],
  hideSiteHeaderBar: [],
})

export const validationMessageKeys = getValidationMessagesCollection<ExerciseReleaseForBackend>({
  id: '',
  isReleased: '',
  releasedAt: '',
  generatedCode: '',
  pLangId: translate(TranslateIds.invalidSelectOption),
  automaticEndAt: '',
  automaticStartAt: '',
  availableWorkingTimeInMinutes: translate(TranslateIds.invalidWorkingTime),
  note: '',
  hasLimitedWorkingTime: '',
  isVisibleToAllAfterRelease: '',
  shouldAutomaticAssessSubmissions: '',
  runAlsoNormalTests: '',
  releaseDurationType: '',
  releaseStartType: '',
  hasAutomaticAssessmentStarted: '',
  hasAutomaticAssessmentFinished: '',
  hadAutomaticAssessmentErrors: '',
  hideInOverviews: '',
  shouldClearClipboard: '',
  hideSiteHeaderBar: '',
})


export interface SET_exerciseReleaseIdAction extends ActionBase {
  readonly type: ActionType.SET_exerciseReleaseId
  readonly id: number
}

export interface SET_isReleasedAction extends ActionBase {
  readonly type: ActionType.SET_isReleased
  readonly isReleased: boolean
}

export interface SET_generatedCodeAction extends ActionBase {
  readonly type: ActionType.SET_generatedCode
  readonly generatedCode: string
}

export interface SET_pLangIdAction extends ActionBase {
  readonly type: ActionType.SET_pLangId
  readonly pLangId: number
}

export interface SET_automaticStartAtAction extends ActionBase {
  readonly type: ActionType.SET_automaticStartAt
  readonly automaticStartAt: Moment | null
}

export interface SET_automaticEndAtAction extends ActionBase {
  readonly type: ActionType.SET_automaticEndAt
  readonly automaticEndAt: Moment | null
}

export interface SET_hasLimitedWorkingTimeAction extends ActionBase {
  readonly type: ActionType.SET_hasLimitedWorkingTime
  readonly hasLimitedWorkingTime: boolean
}

export interface SET_availableWorkingTimeInMinutesAction extends ActionBase {
  readonly type: ActionType.SET_availableWorkingTimeInMinutes
  readonly availableWorkingTimeInMinutes: number | null //null if invalid
}

export interface SET_noteAction extends ActionBase {
  readonly type: ActionType.SET_note
  readonly note: string
}

export interface SET_isVisibleToAllAfterReleaseAction extends ActionBase {
  readonly type: ActionType.SET_isVisibleToAllAfterRelease
  readonly isVisibleToAllAfterRelease: boolean
}

export interface SET_shouldAutomaticAssessSubmissionsAction extends ActionBase {
  readonly type: ActionType.SET_shouldAutomaticAssessSubmissions
  readonly shouldAutomaticAssessSubmissions: boolean
}

export interface SET_runAlsoNormalTestsAction extends ActionBase {
  readonly type: ActionType.SET_runAlsoNormalTests
  readonly runAlsoNormalTests: boolean
}

export interface SET_releaseDurationTypeAction extends ActionBase {
  readonly type: ActionType.SET_releaseDurationType
  readonly releaseDurationType: ReleaseDurationType
}

export interface SET_releaseStartTypeAction extends ActionBase {
  readonly type: ActionType.SET_releaseStartType
  readonly releaseStartType: ReleaseStartType
}

export interface SET_hideInOverviewsAction extends ActionBase {
  readonly type: ActionType.SET_hideInOverviews
  readonly hideInOverviews: boolean
}

export interface SET_shouldClearClipboardAction extends ActionBase {
  readonly type: ActionType.SET_shouldClearClipboard
  readonly shouldClearClipboard: boolean
}

export interface SET_hideSiteHeaderBarAction extends ActionBase {
  readonly type: ActionType.SET_hideSiteHeaderBar
  readonly hideSiteHeaderBar: boolean
}

//not used yet
// export interface ResetAction extends ActionBase {
//   readonly type: ActionType.RESET
// }

export type AllActions =
  //ResetAction
  //|
  SET_exerciseReleaseIdAction
  | SET_isReleasedAction
  | SET_generatedCodeAction
  | SET_pLangIdAction
  | SET_automaticStartAtAction
  | SET_automaticEndAtAction
  | SET_hasLimitedWorkingTimeAction
  | SET_availableWorkingTimeInMinutesAction
  | SET_noteAction
  | SET_isVisibleToAllAfterReleaseAction
  | SET_shouldAutomaticAssessSubmissionsAction
  | SET_runAlsoNormalTestsAction
  | SET_releaseDurationTypeAction
  | SET_releaseStartTypeAction
  | SET_hideInOverviewsAction
  | SET_shouldClearClipboardAction
  | SET_hideSiteHeaderBarAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_exerciseReleaseId:
      return {
        ...state,
        id: action.id
      }

    case ActionType.SET_isReleased:
      return {
        ...state,
        isReleased: action.isReleased
      }

    case ActionType.SET_generatedCode:
      return {
        ...state,
        generatedCode: action.generatedCode
      }

    case ActionType.SET_pLangId:
      return {
        ...state,
        pLangId: action.pLangId
      }

    case ActionType.SET_automaticStartAt:
      return {
        ...state,
        automaticStartAt: action.automaticStartAt ? action.automaticStartAt.toDate() : null
      }

    case ActionType.SET_automaticEndAt:

      const _automaticEndAt = action.automaticEndAt ? action.automaticEndAt.toDate() : null

      return {
        ...state,
        automaticEndAt: _automaticEndAt,
      }

    case ActionType.SET_hasLimitedWorkingTime:
      return {
        ...state,
        hasLimitedWorkingTime: action.hasLimitedWorkingTime
      }

    case ActionType.SET_availableWorkingTimeInMinutes:
      return {
        ...state,
        availableWorkingTimeInMinutes: action.availableWorkingTimeInMinutes
      }
    case ActionType.SET_note:
      return {
        ...state,
        note: action.note
      }

    case ActionType.SET_isVisibleToAllAfterRelease:
      return {
        ...state,
        isVisibleToAllAfterRelease: action.isVisibleToAllAfterRelease
      }

    case ActionType.SET_shouldAutomaticAssessSubmissions:
      return {
        ...state,
        shouldAutomaticAssessSubmissions: action.shouldAutomaticAssessSubmissions
      }

    case ActionType.SET_runAlsoNormalTests:
      return {
        ...state,
        runAlsoNormalTests: action.runAlsoNormalTests
      }

    case ActionType.SET_releaseDurationType:
      return {
        ...state,
        releaseDurationType: action.releaseDurationType
      }

    case ActionType.SET_releaseStartType:
      return {
        ...state,
        releaseStartType: action.releaseStartType
      }

    case ActionType.SET_hideInOverviews:
      return {
        ...state,
        hideInOverviews: action.hideInOverviews
      }
    case ActionType.SET_shouldClearClipboard:
      return {
        ...state,
        shouldClearClipboard: action.shouldClearClipboard
      }

    case ActionType.SET_hideSiteHeaderBar:
      return {
        ...state,
        hideSiteHeaderBar: action.hideSiteHeaderBar
      }


    // case ActionType.RESET:
    //   return initial

    default:
      notExhaustive(action)
      return state
  }
}

