/**
 * Created by janis dähne.
 */

import {notExhaustive} from "../../_notExhausiveHelper";
import {EditableExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {AllActions as GetReleasesAction, reducer as getReleasesReducer} from './getReleasesReducer'
import {ActionBase, ActionType} from "./releasesSiteActionTypes";
import {ExerciseReleaseFromBackend} from "../../../../types/exerciseRelease";
import {AllActions as GetOwnExercisesActions, reducer as getOwnExercisesReducer} from './getOwnExercisesReducer'
import {AllActions as GetGroupExercisesActions, reducer as getGroupExercisesReducer} from './getGroupExercisesReducer'
import {ActionType as OwnExercisesActionType} from '../../manageOwnExercisesSite/ownExercisesSiteActionTypes'
import {ActionType as GroupExercisesActionType} from '../../manageGroupExercisesSite/groupExercisesSiteActionTypes'
import {AllActions as DeleteExerciseReleaseActions, reducer as deleteReleaseReducer} from './deleteReleaseReducer'
import {SortDirection} from "../../../../types/sortDirection";
import {GroupRoleWithPermissionFullBase} from "../../../../types/groupRoles";

export type State = {
  readonly selectedExerciseId: number
  readonly exercises: ReadonlyArray<EditableExercisePreviewFromBackend>
  readonly releases: ReadonlyArray<ExerciseReleaseFromBackend>

  readonly isLoading: boolean
  /**
   * true: we use the own exercises (maybe we need to load them)
   * false: we use the group exercises (maybe we need to load them)
   */
  readonly comesFromOwnExercisesSite: boolean

  readonly isZoomedExerciseReleaseDisplayed: boolean

  readonly isChangeDialogDisplayed: boolean

  readonly sortByKey: keyof ExerciseReleaseFromBackend
  readonly sortDirection: SortDirection

}

export const initial: State = {
  exercises: [],
  selectedExerciseId: -1,
  releases: [],
  isLoading: false,
  comesFromOwnExercisesSite: false,
  isZoomedExerciseReleaseDisplayed: false,
  isChangeDialogDisplayed: false,

  sortByKey: 'id',
  sortDirection: SortDirection.ascending
}


export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof ExerciseReleaseFromBackend
  readonly sortDirection: SortDirection
}

export interface SET_comesFromOwnExercisesSiteAction extends ActionBase {
  readonly type: ActionType.SET_comesFromOwnExercisesSite
  readonly comesFromOwnExercisesSite: boolean
}

export interface SET_selectedExerciseIdAction extends ActionBase {
  readonly type: ActionType.SET_selectedExerciseId
  readonly id: number
}

export interface SET_isZoomedExerciseReleaseDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isZoomedExerciseReleaseDisplayed
  readonly isDisplayed: boolean
}

export interface SET_isChangeDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeDialogDisplayed
  readonly isDisplayed: boolean
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
  readonly resetSiteLocalExercises: boolean
}

export type AllActions =
  ResetAction
  | SET_selectedExerciseIdAction

  | SET_comesFromOwnExercisesSiteAction
  | SET_isZoomedExerciseReleaseDisplayedAction
  | SET_isChangeDialogDisplayedAction
  | SET_sortByKeyAction

  | DeleteExerciseReleaseActions
  | GetReleasesAction
  | GetOwnExercisesActions
  | GetGroupExercisesActions


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {


    case ActionType.DELETE_exerciseRelease_PENDING:
    case ActionType.DELETE_exerciseRelease_FULFILLED:
    case ActionType.DELETE_exerciseRelease_REJECTED:
      return deleteReleaseReducer(state, action)

    case ActionType.GET_exerciseReleases_PENDING:
    case ActionType.GET_exerciseReleases_FULFILLED:
    case ActionType.GET_exerciseReleases_REJECTED:
      return getReleasesReducer(state, action)


    case OwnExercisesActionType.GET_ownExercises_PENDING:
    case OwnExercisesActionType.GET_ownExercises_FULFILLED:
    case OwnExercisesActionType.GET_ownExercises_REJECTED:
      return getOwnExercisesReducer(state, action)


    case GroupExercisesActionType.GET_groupExercises_PENDING:
    case GroupExercisesActionType.GET_groupExercises_FULFILLED:
    case GroupExercisesActionType.GET_groupExercises_REJECTED:
      return getGroupExercisesReducer(state, action)

    case ActionType.SET_sortByKey:
      return {
        ...state,
        sortByKey: action.sortByKey,
        sortDirection: state.sortByKey !== action.sortByKey ? SortDirection.ascending : action.sortDirection % 2
      }

    case ActionType.SET_isZoomedExerciseReleaseDisplayed:
      return {
        ...state,
        isZoomedExerciseReleaseDisplayed: action.isDisplayed,
        isChangeDialogDisplayed: false //just to be sure
      }
    case ActionType.SET_isChangeDialogDisplayed:
      return {
        ...state,
        isZoomedExerciseReleaseDisplayed: false,
        isChangeDialogDisplayed: action.isDisplayed
      }


    case ActionType.SET_comesFromOwnExercisesSite:
      return {
        ...state,
        comesFromOwnExercisesSite: action.comesFromOwnExercisesSite
      }

    case ActionType.SET_selectedExerciseId:
      return {
        ...state,
        selectedExerciseId: action.id
      }

    case ActionType.RESET:
      return {
        ...initial,
        exercises: action.resetSiteLocalExercises ? [] : state.exercises,
      }

    default:
      notExhaustive(action)
      return state
  }
}

