/**
 * Created by janis dähne.
 */

import {notExhaustive} from "../../_notExhausiveHelper";
import {AssessmentFullBase, SubmissionTuple} from "../../../../types/submissions";
import {ActionBase, ActionType} from "./submissionsSiteActionTypes";
import {AllActions as GetSubmissionsActions, reducer as getSubmissionsReducer} from './getSubmissionsReducer'
import {
  AllActions as RerunAutomaticAssessmentActions,
  reducer as rerunAutomaticAssessmentReducer
} from './rerunAutomaticAssessmentReducer'
import {
  AllActions as RerunSingleAutomaticAssessmentActions,
  reducer as rerunSingleAutomaticAssessmentReducer
} from './rerunSingleAutomaticAssessmentReducer'
import {
  SET_comesFromOwnExercisesSiteAction,
  SET_selectedExerciseIdAction,

} from "../exerciseReleasesSite/releaseSiteReducer";
import {ActionType as ReleaseSiteActionType} from "../exerciseReleasesSite/releasesSiteActionTypes";
import {ExerciseReleaseFromBackend} from "../../../../types/exerciseRelease";
import {AllActions as GetReleasesActions, reducer as getExerciseReleasesReducer} from './_getExerciseReleasesReducer'
import {ActionType as ReleasesSiteActionTypes} from "../exerciseReleasesSite/releasesSiteActionTypes";
import {EditableExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {AllActions as GetOwnExercisesActions, reducer as getOwnExercisesReducer} from './_getOwnExercisesReducer'
import {AllActions as GetGroupExercisesActions, reducer as getGroupExercisesReducer} from './_getGroupExercisesReducer'
import {ActionType as OwnExercisesActionType} from '../../manageOwnExercisesSite/ownExercisesSiteActionTypes'
import {ActionType as GroupExercisesActionType} from '../../manageGroupExercisesSite/groupExercisesSiteActionTypes'
import {SortDirection} from "../../../../types/sortDirection";
import {FrontendSettingsManager} from "../../../../helpers/frontendSettingsManager";


export type State = {
  readonly submissionTuple: SubmissionTuple | null
  readonly comesFromOwnExercisesSite: boolean
  readonly isLoading: boolean
  readonly releases: ReadonlyArray<ExerciseReleaseFromBackend>
  readonly selectedReleaseId: number
  readonly exercises: ReadonlyArray<EditableExercisePreviewFromBackend>
  readonly selectedExerciseId: number

  readonly isAutomaticRefreshViewEnabled: boolean
  readonly automaticRefreshViewIntervalCounter: number

  readonly runningAssessment: AssessmentFullBase | null

  readonly sortByKey: keyof AssessmentFullBase
  readonly sortDirection: SortDirection

}


export const initial: State = {
  submissionTuple: null,
  comesFromOwnExercisesSite: false,
  isLoading: true,
  selectedReleaseId: -1,
  releases: [],
  exercises: [],
  selectedExerciseId: -1,
  isAutomaticRefreshViewEnabled: false,
  automaticRefreshViewIntervalCounter: 1, //could be 0

  runningAssessment: null,

  sortByKey: 'lastName',
  sortDirection: SortDirection.ascending
}


export interface SET_isAutomaticRefreshViewEnabledAction extends ActionBase {
  readonly type: ActionType.SET_isAutomaticRefreshViewEnabled
  readonly isAutomaticRefreshViewEnabled: boolean
}

export interface SET_automaticRefreshViewIntervalCounterAction extends ActionBase {
  readonly type: ActionType.SET_automaticRefreshViewIntervalCounter
  readonly automaticRefreshViewIntervalCounter: number
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof AssessmentFullBase
  readonly sortDirection: SortDirection
}


export interface SET_selectedReleaseIdAction extends ActionBase {
  readonly type: ActionType.SET_selectedReleaseId
  readonly id: number
}

export interface SET_runningAssessmentAction extends ActionBase {
  readonly type: ActionType.SET_runningAssessment
  readonly runningAssessment: AssessmentFullBase | null
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
  readonly resetSiteLocalExercises: boolean
  readonly resetSiteLocalReleases: boolean
}


export type AllActions =
  ResetAction
  | SET_comesFromOwnExercisesSiteAction
  | SET_selectedReleaseIdAction
  | GetSubmissionsActions
  | RerunAutomaticAssessmentActions
  | RerunSingleAutomaticAssessmentActions
  | GetReleasesActions
  | SET_selectedExerciseIdAction
  | GetOwnExercisesActions
  | GetGroupExercisesActions
  | SET_runningAssessmentAction
  | SET_sortByKeyAction
  | SET_isAutomaticRefreshViewEnabledAction
  | SET_automaticRefreshViewIntervalCounterAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RERUN_automaticAssessment_PENDING:
    case ActionType.RERUN_automaticAssessment_FULFILLED:
    case ActionType.RERUN_automaticAssessment_REJECTED:
      return rerunAutomaticAssessmentReducer(state, action)

    case ActionType.RERUN_singleAutomaticAssessment_PENDING:
    case ActionType.RERUN_singleAutomaticAssessment_FULFILLED:
    case ActionType.RERUN_singleAutomaticAssessment_REJECTED:
      return rerunSingleAutomaticAssessmentReducer(state, action)

    case ActionType.GET_submissions_PENDING:
    case ActionType.GET_submissions_FULFILLED:
    case ActionType.GET_submissions_REJECTED:
      return getSubmissionsReducer(state, action)

    case ReleasesSiteActionTypes.GET_exerciseReleases_PENDING:
    case ReleasesSiteActionTypes.GET_exerciseReleases_FULFILLED:
    case ReleasesSiteActionTypes.GET_exerciseReleases_REJECTED:
      return getExerciseReleasesReducer(state, action)

    case OwnExercisesActionType.GET_ownExercises_PENDING:
    case OwnExercisesActionType.GET_ownExercises_FULFILLED:
    case OwnExercisesActionType.GET_ownExercises_REJECTED:
      return getOwnExercisesReducer(state, action) //in submissions site

    case GroupExercisesActionType.GET_groupExercises_PENDING:
    case GroupExercisesActionType.GET_groupExercises_FULFILLED:
    case GroupExercisesActionType.GET_groupExercises_REJECTED:
      return getGroupExercisesReducer(state, action) //in submissions site


    case ActionType.SET_sortByKey:

      const newState = {
        ...state,
        sortByKey: action.sortByKey,
        sortDirection: state.sortByKey !== action.sortByKey ? SortDirection.ascending : action.sortDirection % 2
      }

      FrontendSettingsManager.setSubmissionsSiteSorting({
        sortByKey: newState.sortByKey,
        sortDirection: newState.sortDirection
      })

      return newState

    case ReleaseSiteActionType.SET_selectedExerciseId:
      return {
        ...state,
        selectedExerciseId: action.id
      }

    case ActionType.SET_selectedReleaseId:
      return {
        ...state,
        selectedReleaseId: action.id
      }

    case ActionType.SET_runningAssessment:

      //reset the points of the assessment that is running
      return {
        ...state,
        runningAssessment: action.runningAssessment,
        submissionTuple: !state.submissionTuple
          ? null
          : !action.runningAssessment
            ? state.submissionTuple
            : {
              ...state.submissionTuple,
              submissions: state.submissionTuple.submissions.map<AssessmentFullBase>(p => {
                return p.pLangId === action.runningAssessment.pLangId && p.userId === action.runningAssessment.userId ?
                  {
                    ...p,
                    submitTestPoints: null,
                    maxSubmitTestPoints: null,
                    normalTestPoints: null,
                    maxNormalTestPoints: null
                  }
                  : p
              })
            }
      }

    case ReleasesSiteActionTypes.SET_comesFromOwnExercisesSite:
      return {
        ...state,
        comesFromOwnExercisesSite: action.comesFromOwnExercisesSite
      }

    case ActionType.SET_isAutomaticRefreshViewEnabled:
      return {
        ...state,
        isAutomaticRefreshViewEnabled: action.isAutomaticRefreshViewEnabled
      }
    case ActionType.SET_automaticRefreshViewIntervalCounter:
      return {
        ...state,
        automaticRefreshViewIntervalCounter: action.automaticRefreshViewIntervalCounter
      }

    case ActionType.RESET:
      return {
        ...initial,
        exercises: action.resetSiteLocalExercises ? [] : state.exercises,
        releases: action.resetSiteLocalReleases ? [] : state.releases,

        sortByKey: FrontendSettingsManager.getFrontendSettings().settingsSiteSorting === null
          ? initial.sortByKey
          : FrontendSettingsManager.getFrontendSettings().settingsSiteSorting.sortByKey,
        sortDirection: FrontendSettingsManager.getFrontendSettings().settingsSiteSorting === null
          ? initial.sortDirection
          : FrontendSettingsManager.getFrontendSettings().settingsSiteSorting.sortDirection
      }

    default:
      notExhaustive(action)
      return state
  }
}

