/**
 * Created by janis dähne.
 */
import {Action} from "redux";

import {PLangFullBase} from "../../types/pLangs";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "./index";
import {notExhaustive} from "./_notExhausiveHelper";

export type State = {
  readonly pLangs: ReadonlyArray<PLangFullBase>
  readonly isLoading: boolean
}

export const initial: State = {
  pLangs: [],
  isLoading: false
}

export enum ActionType {

  GET_pLangs = 'pLangssChangeDialogReducer_GET_pLangs',
  GET_pLangs_PENDING = 'pLangssChangeDialogReducer_GET_pLangs_PENDING',
  GET_pLangs_FULFILLED = 'pLangssChangeDialogReducer_GET_pLangs_FULFILLED',
  GET_pLangs_REJECTED = 'pLangssChangeDialogReducer_GET_pLangs_REJECTED',

  RESET = 'pLangsReducer_RESET',
}

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface GET_pLangsAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<PLangFullBase>, GET_pLangs_PENDINGAction, GET_pLangs_FULFILLEDAction, GET_pLangs_REJECTEDAction> {
  readonly type: ActionType.GET_pLangs
  readonly payload: Promise<ReadonlyArray<PLangFullBase>>
}

//automatically dispatched by redux promise middleware
export interface GET_pLangs_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_pLangs_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_pLangs_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<PLangFullBase>> {
  readonly type: ActionType.GET_pLangs_FULFILLED
  readonly payload: ReadonlyArray<PLangFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_pLangs_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_pLangs_REJECTED
  readonly payload: any
}

export type GetPLangActions = GET_pLangs_PENDINGAction | GET_pLangs_FULFILLEDAction | GET_pLangs_REJECTEDAction

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | GET_pLangs_PENDINGAction
  | GET_pLangs_FULFILLEDAction
  | GET_pLangs_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_pLangs_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_pLangs_FULFILLED:
      return {
        ...state,
        isLoading: false,
        pLangs: action.payload
      }
    case ActionType.GET_pLangs_REJECTED:
      return {
        ...state,
        isLoading: false,
        pLangs: []
      }

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

