/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "./settingsSiteActionTypes";
import {notExhaustive} from "../_notExhausiveHelper";
import {UserSettingsFullBase} from "../../../types/userData";


export type State = UserSettingsFullBase

export interface SET_langIdAction extends ActionBase {
  readonly type: ActionType.SET_langId
  readonly langId: number
}

export interface SET_themeAction extends ActionBase {
  readonly type: ActionType.SET_theme
  readonly theme: string
}

export interface RESET_userSettingsAction extends ActionBase {
  readonly type: ActionType.RESET_userSettings
}

export const defaultUserSettings: UserSettingsFullBase = {
  theme: '',
  langId: -1,
  id: -1,
  codeEditorSettings: null
}

export type AllActions =
  | SET_langIdAction
  | SET_themeAction
  | RESET_userSettingsAction


export function reducer(state: State = defaultUserSettings, action: AllActions): State {

  switch (action.type) {
    case ActionType.SET_langId:
      return {
        ...state,
        langId: action.langId
      }
    case ActionType.SET_theme:
      return {
        ...state,
        theme: action.theme
      }

    case ActionType.RESET_userSettings:
      return defaultUserSettings

    default:
      notExhaustive(action)
      return state
  }
}