/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./singleGroupSiteActionTypes";
import {initial, State} from "./singleGroupSiteReducer";
import {GroupUserFromBackend} from "../../../types/group";
import Logger from "../../../helpers/logger";
import {assignGroupRolesToMembers} from "../../../communicationLayer/groupsLayer";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";


export type Meta = {
  readonly currentUserId: number
  readonly groupRoles: ReadonlyArray<GroupRoleWithPermissionFullBase>
}

export interface GET_membersAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<GroupUserFromBackend>, GET_members_PENDINGAction, GET_members_FULFILLEDAction, GET_members_REJECTEDAction, Meta> {
  readonly type: ActionType.GET_members
  readonly payload: Promise<ReadonlyArray<GroupUserFromBackend>>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface GET_members_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_members_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_members_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<GroupUserFromBackend>, Meta> {
  readonly type: ActionType.GET_members_FULFILLED
  readonly payload: ReadonlyArray<GroupUserFromBackend>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface GET_members_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<ReadonlyArray<GroupUserFromBackend>, Meta> {
  readonly type: ActionType.GET_members_REJECTED
  readonly payload: any
  readonly meta: Meta
}


export type AllActions =
  GET_members_PENDINGAction
  | GET_members_FULFILLEDAction
  | GET_members_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_members_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_members_FULFILLED:
      return {
        ...state,
        isLoading: false,
        members: assignGroupRolesToMembers(action.payload, action.meta.groupRoles, action.meta.currentUserId),
      }
    case ActionType.GET_members_REJECTED:
      return {
        ...state,
        isLoading: false,
        members: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

