/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "./_notExhausiveHelper";

export type State = {
  readonly isSiteMenuOpen: boolean

  /**
   * currently only supported in the do exercise site for the exam mode...
   * true: displayed
   * false: not displayed and cannot be changed by the user
   */
  readonly isSiteHeaderBarDisplayedAndEnabled: boolean

  /**
   * applies only if isSiteHeaderBarDisplayed is true
   *
   * true: header bar is not displayed but a button to expand the header bar back
   * false: display header bar
   */
  readonly isSiteHeaderBarCollapsed: boolean

  /**
   * true: collapse the header bar of each individual site
   * false: not
   */
  readonly isSubHeaderBarCollapsed: boolean

  readonly isUserMenuOpen: boolean

}

const initial: State = {
  isSiteMenuOpen: false,
  isSiteHeaderBarDisplayedAndEnabled: true,
  isSiteHeaderBarCollapsed: false,
  isSubHeaderBarCollapsed: false,

  isUserMenuOpen: false
}


export enum ActionType {
  SET_isSiteMenuOpen = 'SiteHeaderBarReducer_SET_isSiteMenuOpen',
  SET_isSiteHeaderBarDisplayedAndEnabled = 'siteHeaderBarReducer_SET_isSiteHeaderBarDisplayedAndEnabled',
  SET_isSiteHeaderBarCollapsed = 'siteHeaderBarReducer_SET_isSiteHeaderBarCollapsed',
  SET_isSubHeaderBarCollapsed = 'siteHeaderBarReducer_SET_isSubHeaderBarCollapsed',
  SET_isUserMenuOpen = 'siteHeaderBarReducer_SET_isUserMenuOpen',
  RESET = 'siteHeaderBarReducer_RESET'
}

export interface ActionBase extends Action {
  readonly type: ActionType
}


export interface SET_isSiteMenuOpenAction extends ActionBase {
  readonly type: ActionType.SET_isSiteMenuOpen
  readonly isSiteMenuOpen: boolean
}

export interface SET_isSiteHeaderBarDisplayedAndEnabledAction extends ActionBase {
  readonly type: ActionType.SET_isSiteHeaderBarDisplayedAndEnabled
  readonly isDisplayed: boolean
}

export interface SET_isSiteHeaderBarCollapsedAction extends ActionBase {
  readonly type: ActionType.SET_isSiteHeaderBarCollapsed
  readonly isSiteHeaderBarCollapsed: boolean
}

export interface SET_isSubHeaderBarCollapsedAction extends ActionBase {
  readonly type: ActionType.SET_isSubHeaderBarCollapsed
  readonly isSubHeaderBarCollapsed: boolean
}

export interface SET_isUserMenuOpenAction extends ActionBase {
  readonly type: ActionType.SET_isUserMenuOpen
  readonly isUserMenuOpen: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}


export type AllActions =
  SET_isSiteMenuOpenAction
  | SET_isSiteHeaderBarDisplayedAndEnabledAction
  | SET_isSiteHeaderBarCollapsedAction
  | SET_isSubHeaderBarCollapsedAction
  | SET_isUserMenuOpenAction
  | ResetAction //all reducers need a reset action (or an action without any extra args just type so that the
                //union (or) type is compatible to (redux) Action because the Action don't allow any extra args


export function reducer(state: State = initial, action: AllActions): State {


  switch (action.type) {

    case ActionType.SET_isSiteMenuOpen:
      return {
        ...state,
        isSiteMenuOpen: action.isSiteMenuOpen
      }

    case ActionType.SET_isSiteHeaderBarDisplayedAndEnabled:
      return {
        ...state,
        isSiteHeaderBarDisplayedAndEnabled: action.isDisplayed
      }

    case ActionType.SET_isSiteHeaderBarCollapsed:
      return {
        ...state,
        isSiteHeaderBarCollapsed: action.isSiteHeaderBarCollapsed,

        //expand the sub header bar too (because this is probably what the user wanted...)
        isSubHeaderBarCollapsed: action.isSiteHeaderBarCollapsed === false ? false : state.isSubHeaderBarCollapsed
      }

    case ActionType.SET_isSubHeaderBarCollapsed:
      return {
        ...state,
        isSubHeaderBarCollapsed: action.isSubHeaderBarCollapsed
      }

    case ActionType.SET_isUserMenuOpen:
      return {
        ...state,
        isUserMenuOpen: action.isUserMenuOpen
      }

    case ActionType.RESET:
      return {
        ...initial,
      }

    default:
      notExhaustive(action)
      return state
  }
}

