/**
 * Created by janis dähne.
 */

import {Action} from "redux";
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {initial, State} from "./systemRolesSiteReducer";
import {ActionType, ActionBase} from "./systemRolesSiteActionTypes";
import Logger from "../../../helpers/logger";



export interface DELETE_systemRoleAction
  extends ActionBase, ReduxPromiseAction<void, DELETE_systemRole_PENDINGAction, DELETE_systemRole_FULFILLEDAction, DELETE_systemRole_REJECTEDAction> {
  readonly type: ActionType.DELETE_systemRole
  readonly payload: Promise<void>
}

//automatically dispatched by redux promise middleware
export interface DELETE_systemRole_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DELETE_systemRole_PENDING
}

//automatically dispatched by redux promise middleware
export interface DELETE_systemRole_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void> {
  readonly type: ActionType.DELETE_systemRole_FULFILLED
  readonly payload: void
}

//automatically dispatched by redux promise middleware
export interface DELETE_systemRole_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.DELETE_systemRole_REJECTED
  readonly payload: any
}


export type AllActions =
  DELETE_systemRole_PENDINGAction
  | DELETE_systemRole_FULFILLEDAction
  | DELETE_systemRole_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.DELETE_systemRole_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.DELETE_systemRole_FULFILLED:
      return {
        ...state,
        isLoading: false
      }
    case ActionType.DELETE_systemRole_REJECTED:
      Logger.error('rejected delete data, prev state', state)
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}


