/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {SystemRoleFullBase} from "../../../types/userData";
import {notExhaustive} from "../_notExhausiveHelper";

import {AllActions as GetActions, reducer as getReducer} from './getSystemRolesReducer'
import {AllActions as DeleteActions, reducer as deleteReducer} from './deleteSystemRoleReducer'
import {ActionType, ActionBase} from "./systemRolesSiteActionTypes";


export type State = {
  readonly systemRoles: ReadonlyArray<SystemRoleFullBase>,
  /**
   * true: display the dialog,
   * false: display main view
   */
  readonly isChangeDialogDisplayed: boolean

  /**
   * true: feting data, display spinner,
   * false: not (finished error/or success)
   */
  readonly isLoading: boolean
}

export const initial: State = {
  systemRoles: [],
  isChangeDialogDisplayed: false,
  isLoading: false
}

export interface SetListAction extends ActionBase {
  readonly type: ActionType.SET_systemRoles
  readonly systemRoles: ReadonlyArray<SystemRoleFullBase>
}

export interface SET_isChangeDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeDialogDisplayed
  readonly isDisplayed: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  SetListAction
  | ResetAction
  | SET_isChangeDialogDisplayedAction
  | GetActions
  | DeleteActions


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_systemRoles:
      return {
        ...state,
        systemRoles: action.systemRoles
      }

    case ActionType.SET_isChangeDialogDisplayed:
      return {
        ...state,
        isChangeDialogDisplayed: action.isDisplayed
      }

    case ActionType.GET_systemRoles_PENDING:
    case ActionType.GET_systemRoles_FULFILLED:
    case ActionType.GET_systemRoles_REJECTED:
      return getReducer(state, action)


    case ActionType.DELETE_systemRole_PENDING:
    case ActionType.DELETE_systemRole_FULFILLED:
    case ActionType.DELETE_systemRole_REJECTED:
      return deleteReducer(state, action)

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

