/**
 * Created by janis dähne.
 */

import {Action} from "redux";
import {SystemSettingsFromBackend} from "../../types/systemSettings";
import {notExhaustive} from "./_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "./index";
import Logger from "../../helpers/logger";

export type State = {
  readonly isLoading: boolean
  readonly systemSettings: SystemSettingsFromBackend
}


export const initial: State = {
  isLoading: false,
  systemSettings: {
    maxSolutionVersionsPerUser: 1,
    defaultGroupCreatorGroupRoleId: -1,
    maxNumberOfTestsWithOneRequest: 1,
    defaultUserGroupId: -1,
    maxCustomProjectsPerUser: -1,
  }
}

export interface ActionBase extends Action {
  readonly type: ActionType
}

export enum ActionType {
  GET_systemSettings = 'systemSettingsReducer_GET_systemSettings',
  GET_systemSettings_PENDING = 'systemSettingsReducer_GET_systemSettings_PENDING',
  GET_systemSettings_FULFILLED = 'systemSettingsReducer_GET_systemSettings_FULFILLED',
  GET_systemSettings_REJECTED = 'systemSettingsReducer_GET_systemSettings_REJECTED',

  RESET = 'systemSettingsReducer_RESET',
}


export interface GET_systemSettingsAction
  extends ActionBase, ReduxPromiseAction<SystemSettingsFromBackend, GET_systemSettings_PENDINGAction, GET_systemSettings_FULFILLEDAction, GET_systemSettings_REJECTEDAction> {
  readonly type: ActionType.GET_systemSettings
  readonly payload: Promise<SystemSettingsFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_systemSettings_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_systemSettings_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_systemSettings_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<SystemSettingsFromBackend> {
  readonly type: ActionType.GET_systemSettings_FULFILLED
  readonly payload: SystemSettingsFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_systemSettings_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_systemSettings_REJECTED
  readonly payload: any
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction

  | GET_systemSettings_PENDINGAction
  | GET_systemSettings_FULFILLEDAction
  | GET_systemSettings_REJECTEDAction

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_systemSettings_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_systemSettings_FULFILLED:
      return {
        ...state,
        isLoading: false,
        systemSettings: action.payload
      }
    case ActionType.GET_systemSettings_REJECTED:
      return {
        ...state,
        isLoading: false,
      }

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

