/**
 * Created by janis dähne.
 */

import {Action} from "redux";
import {TagFullBase} from "../../types/tag";
import {notExhaustive} from "./_notExhausiveHelper";

export type State = {
  /**
   * the tag ids where the exercise needs these tags to match
   */
  readonly selectedFilterTagsIds: ReadonlyArray<number>
  /**
   * the exercises must not contain these tags to match
   */
  readonly selectedNegativeFilterTagsIds: ReadonlyArray<number>
}


export const initial: State = {
  selectedFilterTagsIds: [],
  selectedNegativeFilterTagsIds: []
}

export interface ActionBase extends Action {
  readonly type: ActionType
}


export enum ActionType {
  SET_selectedTagIds = 'tagsFilterReducer_SET_selectedTagIds',
  SET_selectedNegativeFilterTagsIds = 'tagsFilterReducer_SET_selectedNegativeFilterTagsIds',
  RESET = 'tagsFilterReducer_RESET',
}


export interface SET_selectedTagIdsAction extends ActionBase {
  readonly type: ActionType.SET_selectedTagIds
  readonly selectedTagIds: ReadonlyArray<number>
}

export interface SET_selectedNegativeFilterTagsIdsAction extends ActionBase {
  readonly type: ActionType.SET_selectedNegativeFilterTagsIds
  readonly selectedNegativeFilterTagsIds: ReadonlyArray<number>
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_selectedTagIdsAction
  | SET_selectedNegativeFilterTagsIdsAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_selectedTagIds:
      return {
        ...state,
        selectedFilterTagsIds: action.selectedTagIds
      }

    case ActionType.SET_selectedNegativeFilterTagsIds:
      return {
        ...state,
        selectedNegativeFilterTagsIds: action.selectedNegativeFilterTagsIds
      }

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

