/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../tutorViewSiteActionTypes";
import {initial, State} from "../tutorViewSiteReducer";
import {StoredTestResultTupleFromBackend} from "../../../../types/testResults";
import Logger from "../../../../helpers/logger";
import {mergeTestResultsIntoDoExercise} from "../../../../helpers/convertersAndTransformers";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";

//this is actually only the normal tests, custom tests

export interface GET_allUserTestResultsAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultTupleFromBackend, GET_allUserTestResults_PENDINGAction, GET_allUserTestResults_FULFILLEDAction, GET_allUserTestResults_REJECTEDAction> {
  readonly type: ActionType.GET_allUserTestResults
  readonly payload: Promise<StoredTestResultTupleFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_allUserTestResults_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_allUserTestResults_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_allUserTestResults_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultTupleFromBackend> {
  readonly type: ActionType.GET_allUserTestResults_FULFILLED
  readonly payload: StoredTestResultTupleFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_allUserTestResults_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_allUserTestResults_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_allUserTestResults_PENDINGAction
  | GET_allUserTestResults_FULFILLEDAction
  | GET_allUserTestResults_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_allUserTestResults_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_allUserTestResults_FULFILLED:

      const mergedExercise = mergeTestResultsIntoDoExercise(state.exercise, action.payload)
      return {
        ...state,
        isLoading: false,
        exercise: mergedExercise,
        normalTestsForTutor: [...mergedExercise.tests]
      }
    case ActionType.GET_allUserTestResults_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

