/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../tutorViewSiteActionTypes";
import {initial, State} from "../tutorViewSiteReducer";
import Logger from "../../../../helpers/logger";
import {AssessmentFullBase} from "../../../../types/submissions";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";

export interface GET_singleAssessmentAction
  extends ActionBase, ReduxPromiseAction<AssessmentFullBase, GET_singleAssessment_PENDINGAction, GET_singleAssessment_FULFILLEDAction, GET_singleAssessment_REJECTEDAction> {
  readonly type: ActionType.GET_singleAssessment
  readonly payload: Promise<AssessmentFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_singleAssessment_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleAssessment_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleAssessment_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<AssessmentFullBase> {
  readonly type: ActionType.GET_singleAssessment_FULFILLED
  readonly payload: AssessmentFullBase
}

//automatically dispatched by redux promise middleware
export interface GET_singleAssessment_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleAssessment_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleAssessment_PENDINGAction
  | GET_singleAssessment_FULFILLEDAction
  | GET_singleAssessment_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleAssessment_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleAssessment_FULFILLED:
      return {
        ...state,
        isLoading: false,
        assessment: action.payload
      }
    case ActionType.GET_singleAssessment_REJECTED:
      return {
        ...state,
        isLoading: false,
        assessment: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

