/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import Logger from "../../../../helpers/logger";
import {SolutionVersionTupleFromBackend} from "../../../../types/exerciseSolution";
import {ActionBase, ActionType} from "../tutorViewSiteActionTypes";
import {initial, State} from "../tutorViewSiteReducer";

export interface GET_user_solutionVersionAction
  extends ActionBase, ReduxPromiseAction<SolutionVersionTupleFromBackend, GET_user_solutionVersion_PENDINGAction, GET_user_solutionVersion_FULFILLEDAction, GET_user_solutionVersion_REJECTEDAction> {
  readonly type: ActionType.GET_user_solutionVersion
  readonly payload: Promise<SolutionVersionTupleFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_user_solutionVersion_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_user_solutionVersion_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_user_solutionVersion_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<SolutionVersionTupleFromBackend> {
  readonly type: ActionType.GET_user_solutionVersion_FULFILLED
  readonly payload: SolutionVersionTupleFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_user_solutionVersion_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_user_solutionVersion_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_user_solutionVersion_PENDINGAction
  | GET_user_solutionVersion_FULFILLEDAction
  | GET_user_solutionVersion_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_user_solutionVersion_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_user_solutionVersion_FULFILLED:
      return {
        ...state,
        isLoading: false,
        userSolution: action.payload.solutionVersion,
        allUserSolutionVersionPreviews: action.payload.allSolutionVersionPreviews,
        tutorEditorSolution: {...action.payload.solutionVersion}
      }
    case ActionType.GET_user_solutionVersion_REJECTED:
      return {
        ...state,
        isLoading: false,
        userSolution: null,
        allUserSolutionVersionPreviews: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

