/**
 * Created by janis dähne.
 */
import {Moment} from "moment";


export interface AssessmentStatisticsFullBase {
  readonly userId: number
  readonly lastName: string
  readonly firstName: string
  readonly someResultsAreMissingBecauseOfPermissions: boolean
  
  readonly assessmentStatistics: ReadonlyArray<SingleAssessmentStatistic>
}


export interface SingleAssessmentStatistic {
  readonly pLangId: number
  readonly exerciseReleaseId: number
  readonly exerciseReleaseIsReleased: boolean
  readonly shouldNotCount: boolean
  readonly submitTestPoints: number | null
  readonly maxSubmitTestPoints: number | null
  readonly normalTestPoints: number | null
  readonly maxNormalTestPoints: number | null
  readonly manualPoints: number | null
  readonly feedbackForStudent: string
  readonly releaseCode: string
  readonly exerciseId: number

  readonly exerciseDisplayName: string
  readonly exerciseShortDescription: string

  readonly userGroupId: number
  /**
   * true: an assessment was created (but the assessment might not finished yet, or aborted, or null points)
   * false: there is no assessment
   */
  readonly hasAssessment: boolean

  readonly createdAt: Moment

  readonly lastUpdatedAt: Moment

  /**
   * a user can solve an exercise multiple times (connected through release.exercise)
   * so this gives us the times the user participated on the exercise (through different releases)
   *
   * IF a user solved the exercise with different p langs then this does not count separate because
   * all restrictions are stored at the participation (e.g. time) and thus the p lang does not matter!
   *
   * CAN be -1 on error
   */
  readonly participationCount: number

}