/**
 * Created by janis dähne.
 */


import {ExerciseTestFromBackend} from "./exerciseEditor";

/**
 * the test results (ever plang has own test results for the same tests)
 */
export interface StoredTestResultTupleFromBackend {
  /**
   * the normal test results
   */
  readonly testResults: ReadonlyArray<StoredTestResultFromBackend>
  /**
   * the results of the custom tests
   */
  readonly customTestResults: ReadonlyArray<StoredTestResultFromBackend>
}

/**
 * this is not a real answer from the test server just the stored result...
 */
export interface StoredTestResultFromBackend {
  /**
   * the id of the test (custom or normal)
   */
  readonly testId: number
  /**
   * null: not ran yet, true: test was successful, false: not
   */
  readonly passed: boolean | null
  /**
   * null: not ran yet, true: code compiled, false: not
   */
  readonly hasCompiled: boolean | null

  /**
   * the exit code of the user program, not used yet (not event set)
   */
  readonly programExitCode: number
  /**
   * the test result code (see test server codes)
   */
  readonly testResultCode: number

  /**
   * the conversation protocol between the user program and the test runner
   */
  readonly protocol: ReadonlyArray<string>
}

export interface AssessmentForUserWithTestResults {
  readonly assessmentResult: AssessmentResultForUser | null
  readonly submitTestResults: ReadonlyArray<AssessedSubmitTestFromBackend>
}


export interface AssessmentResultForUser {

  readonly pLangId: number

  /**
   * the summed test points from the submit tests
   */
  readonly submitTestPoints: number |null

  /**
   * the summed max reachable submit test points
   */
  readonly maxSubmitTestPoints: number | null

  /**
   * the summed points from the normal test
   */
  readonly normalTestPoints: number | null

  /**
   * the summed max reachable normal test points
   */
  readonly maxNormalTestPoints: number | null

  /**
   * optional the manual given poitns (by the tutor)
   */
  readonly manualPoints: number | null

  /**
   * feedback for the student
   */
  readonly feedbackForStudent: string | null
}

export interface AssessedSubmitTestFromBackend extends ExerciseTestFromBackend {
  /**
   * the test result
   */
  readonly result: StoredTestResultFromBackend
}

export interface AssessedSubmitTestFrontendExclusive extends AssessedSubmitTestFromBackend {
  /**
   * true: test is running, false: not
   * used to display a spinner
   */
  readonly uiIsRunning: boolean
}

/**
 * the known test results from the test server
 */
export enum TestResultCodes {
  /**
   * all ok, test ran
   */
  ok = 0,
  /**
   * the output mismatched from the expected
   * @type {number}
   */
  OutputMismatch = 1,
  /**
   * the users program encountered an error during execution
   * @type {number}
   */
  UserProgramError = 2,
  /**
   * the timeout was hit
   * @type {number}
   */
  TimeoutHit = 3,
  /**
   * the memory limit was hit
   * @type {number}
   */
  MemoryLimitHit = 4,
  /**
   * the disk limit was hit
   * @type {number}
   */
  DiskLimitHit = 5,
  /**
   * a compile error occurred
   * @type {number}
   */
  CompileError = 50,

  /**
   * some other server error occurred
   * @type {number}
   */
  SomeServerError = 100,

  /**
   * a unknown test method was specified
   * @type {number}
   */
  UnknownTestMethod = 101
}


/**
 * the test protocol from test server (e.g. regex blackbox line is missing because this is a normal output)
 */
export enum TestProtocolType {
  /**
   * this item describe the program arguments,
   */
  isProgramArguments,
  /**
   * the content was injected/fed in the users program
   */
  isInput,
  /**
   * content that was read from the user program
   */
  isOutput,
  /**
   * content is an error (e.g. compile error),
   */
  isError,
  /**
   * a line representing the expected output from the program (the previous line should be the real output),
   */
  isExpectedOutput,
  /**
   * this line indicates that the test ended here ... separate from following output
   */
  isEndOfTestLine,

  /**
   * used for plain text e.g. regex output
   */
  plain,
  /**
   * unknown type (some error?)
   */
  unknown
}

/**
 * this is an item created by parsing the test result protocol
 */
export interface TestProtocolItem {

  /**
   * the item type
   */
  readonly type: TestProtocolType
  /**
   * true: the output line was wrong (mismatched), the next lien should be the expected output,
   * false: not
   *
   * should (can) only be set/true for isOutput type
   */
  readonly hasOutputMismatched: boolean
  /**
   * the content
   */
  readonly content: string
}