/**
 * Created by janis dähne.
 */

import {State} from '../state/reducers/systemRolesSite/dialog/changeSystemRolesDialogReducer'
import {ValidationFunc, ValidationRulesCollection} from "./validations";
import Logger from '../helpers/logger'

/**
 * validates all values of the dialog state
 *
 * this can fail because e.g. number values are not longer stored as strings in the state
 * so we call o string ... for numbers this might work but dates & moment.js & co ??
 * @param {T} dialogState
 * @param {ValidationRulesCollection<T>} validators
 * @returns {boolean}
 */
export function isValidDialog<T>(dialogState: T, validators: ValidationRulesCollection<T>): boolean {

  for (const key in validators) {
    if (validators.hasOwnProperty(key)) {
      const validationFuncs: ReadonlyArray<ValidationFunc<T[keyof T]>> = validators[key]
      const value: T[keyof T] = dialogState[key]

      if (typeof value === 'object') {
        console.debug('validating no plain object... this will probably fail, obj: ', value)
        //Logger.debug('validating no plain object... this will probably fail, obj: ', value)
      }

      const valid = validationFuncs.every((func: ValidationFunc<T[keyof T]>) => func(value))

      if (!valid) return false

    }
  }
  return true
}