/**
 * Created by janis dähne
 */

const path = require('path')
const webpack = require('webpack')
const HtmlWebpackPlugin = require('html-webpack-plugin');
const ExtractTextPlugin = require('extract-text-webpack-plugin')
const CopyWebpackPlugin = require('copy-webpack-plugin');
const poststylus = require('poststylus')
//const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;

module.exports = {

  entry: {
    app: './src/index.tsx',
  },

  output: {
    publicPath: "/",
    path: path.join(__dirname, 'dist'),
    filename: 'static/[name]_bundle.js',
  },
  resolve: {
    // Add '.ts' and '.tsx' as resolvable extensions.
    extensions: [".ts", ".tsx", ".js", ".json", ".css"]
  },
  module: {

    loaders: [
      // {
      //   test: /\.html$/,
      //   loader: "html"
      // },
      {test: /\.(jpe?g|gif|png|svg|woff|ttf|wav|mp3)$/, loader: "file-loader"}, //|css
      {test: /\.tsx?$/, loader: "awesome-typescript-loader"},

      {enforce: "pre", test: /\.js$/, loader: "source-map-loader"},

      // { test: /\.gif$/, loader: "file-loader" },
      // {
      //   test: /\.css$/,
      //   loader: ExtractTextPlugin.extract({fallback: 'style-loader', use: 'css-loader'})
      // },
      {
        test: /\.(woff|woff2|eot|ttf)$/,
        loader: 'url-loader?limit=100000',
      },

      {
        test: /\.css$/,
        loaders: [
          'style-loader?sourceMap',
          'css-loader'
        ]
      },
      {
        test: /\.styl$/,
        loader: ExtractTextPlugin.extract({
          fallback: {loader: 'style-loader?sourceMa'},
          use: [
            'css-loader', {
              loader: 'stylus-loader',
              options: {}
            }
          ]
        })
      },
      {
        test: /\.md$/,
        loader: 'raw-loader'
      }
      /*
       {
       test: /\.js$/,
       exclude: /(node_modules|REMOVE|_surveys_|VERSION|libs)/,
       loader: 'babel',
       query: {
       presets: ['es2015'],
       plugins: ['transform-runtime']
       }
       }
       */
    ]
  },
  plugins: [
    //from https://medium.com/@adamrackis/vendor-and-code-splitting-in-webpack-2-6376358f1923
    new webpack.optimize.CommonsChunkPlugin({
      name: 'node-static',
      filename: 'static/node-static.js',
      minChunks(module, count) {
        var context = module.context;
        return context && context.indexOf('node_modules') >= 0;
      },
    }),
    new webpack.optimize.UglifyJsPlugin({
      compress: { warnings: false },
      uglifyOptions: {
        ie8: false,
        mangle: true,
        compress: {
          properties: true,
          dead_code: true,
          conditionals: true,
          comparisons: true,
          evaluate: true,
          booleans: true,
          loops: true,
          unused: true,
          if_return: true,
          join_vars: true,
          collapse_vars: true,
          reduce_vars: true,
          negate_iife: true,
          warnings: true
        }
      }
    }),
    new webpack.DefinePlugin({
      'process.env.NODE_ENV': '"production"'
    }),
    new webpack.ContextReplacementPlugin(
      /moment[\/\\]locale$/,
      /de|en/
    ),
    new webpack.LoaderOptionsPlugin({
      options: {
        stylus: {
          use: [poststylus([ 'autoprefixer'])]
        }
      }
    }),
    new webpack.optimize.ModuleConcatenationPlugin(),
    new CopyWebpackPlugin([
      {
        from: 'node_modules/react/dist/react.min.js',
        to: 'static/react.js',
      },
      {
        from: 'node_modules/react-dom/dist/react-dom.min.js',
        to: 'static/react-dom.js',
      },
      {
        from: 'node_modules/semantic-ui-css/semantic.min.css',
        to: 'static/semantic.min.css',
      },
      {
        from: 'node_modules/semantic-ui-css/themes',
        to: 'static/themes',
      },
      //copy also static pages & assets
      {
        from: 'static_pages/copyright/index.html',
        to: 'copyright.html',
      },
      {
        from: 'static_pages/firstExternalLoginPage/firstLoginExternalLogin.html',
        to: 'firstLoginExternalLogin.html',
      },
      {
        from: 'static_pages/firstExternalLoginPage/firstLoginExternalLogin.js',
        to: 'firstLoginExternalLogin.js',
      },
      {
        from: 'static_pages/passwordMigrationHelper/passwordMigrationHelper.html',
        to: 'passwordMigrationHelper.html',
      },
      {
        from: 'static_pages/passwordMigrationHelper/passwordMigrationHelper.js',
        to: 'passwordMigrationHelper.js',
      },
      {
        from: 'static_pages/static/sha512.js',
        to: 'static/sha512.js',
      },
      {
        from: 'static_pages/static/favicon.png',
        to: 'favicon.png',
      },
      {
        from: 'static_pages/static/about_icon.png',
        to: 'about_icon.png',
      },
      //hack because react-notifications cannot properly bundled ... the icons are not displayed
      //TODO not working because these files are overwritten by the loader!!
      {
        from: 'node_modules/react-notifications/dist/fonts/notification.ttf',
        to: 'a8c8792074cb2979091029707dbf0c46.ttf',
      },
      {
        from: 'node_modules/react-notifications/dist/fonts/notification.woff',
        to: '80abd98351a79a932614527b220f71e8.woff',
      },
    ]),
    new ExtractTextPlugin("static/styles.css"),
    new HtmlWebpackPlugin({
      hash: true,
      filename: 'index_deploy.html',
      template: 'htmlTemplates/index_deploy.ejs',
      inject: false,
      cache: false,
      //this is only for the template file
      minify: {
        caseSensitive: true,
        minifyCSS: true,
        collapseWhitespace: false,
        removeAttributeQuotes: true,
        removeComments: true
      }
    }),
    // new webpack.IgnorePlugin(/highlight.js/),
    // new ExtractTextPlugin("styles.css", {
    //   allChunks: true
    // }),
    //enable to get a nice treemap showing all sizes
    //new BundleAnalyzerPlugin()
  ],
  externals: {
    //better include this via cdn... (too big or no module version)
    MathJax: 'MathJax',
    "react": "React",
    "react-dom": "ReactDOM"

  }

}

/*
css modules
wihtout extract stylues
loaders: [
          'style-loader?sourceMap',
          'css-loader?modules&importLoaders=1&localIdentName=[path]___[name]__[local]___[hash:base64:5],typings-for-css-modules-loader',
          'stylus-loader'

        ]
------------------

stylus loader extract
      {
        test: /\.styl$/,
        loader: ExtractTextPlugin.extract({
          fallback: {loader: 'style-loader?sourceMa'},
          use: [
            'css-loader?modules&importLoaders=1&localIdentName=[path]___[name]__[local]___[hash:base64:5],typings-for-css-modules-loader\'', {
            loader: 'stylus-loader',
              options: {
            }
            }
          ]
        })
      }
 */


