/**
 * Created by janis dähne
 */

const path = require('path')
const webpack = require('webpack')
const HtmlWebpackPlugin = require('html-webpack-plugin');
const ExtractTextPlugin = require('extract-text-webpack-plugin')
const CopyWebpackPlugin = require('copy-webpack-plugin');
const poststylus = require('poststylus')

module.exports = {

  entry: {
    app: './src/index.tsx',
  },

  output: {
    publicPath: "/",
    path: path.join(__dirname, 'dist'),
    filename: '[name]_bundle.js',
  },
  devtool: 'source-map', // if we want a source map!! for debugging saves ~5s for incremental bundle
  resolve: {
    // Add '.ts' and '.tsx' as resolvable extensions.
    extensions: [".ts", ".tsx", ".js", ".json", ".css"]
  },
  module: {

    loaders: [
      // {
      //   test: /\.html$/,
      //   loader: "html"
      // },
      {test: /\.(jpe?g|gif|png|svg|woff|ttf|wav|mp3)$/, loader: "file-loader"}, //|css
      {test: /\.tsx?$/, loader: "awesome-typescript-loader", exclude: /node_modules/},

      {enforce: "pre", test: /\.js$/, loader: "source-map-loader", exclude: /node_modules/},

      // { test: /\.gif$/, loader: "file-loader" },
      // {
      //   test: /\.css$/,
      //   loader: ExtractTextPlugin.extract({fallback: 'style-loader', use: 'css-loader'})
      // },
      {
        test: /\.(woff|woff2|eot|ttf)$/,
        loader: 'url-loader?limit=100000'
      },
      {
        test: /\.css$/,
        loaders: [
          'style-loader?sourceMap',
          'css-loader'
        ]
      },
      {
        test: /\.styl$/,
        loader: ExtractTextPlugin.extract({
          fallback: {loader: 'style-loader?sourceMap'},
          use: [
            'css-loader', {
              loader: 'stylus-loader',
              options: {
                preferPathResolver: 'webpack',
              }
            }
          ]
        })
      },
      {
        test: /\.md$/,
        loader: 'raw-loader'
      }
      /*
       {
       test: /\.js$/,
       exclude: /(node_modules|REMOVE|_surveys_|VERSION|libs)/,
       loader: 'babel',
       query: {
       presets: ['es2015'],
       plugins: ['transform-runtime']
       }
       }
       */
    ]
  },
  plugins: [
    //from https://medium.com/@adamrackis/vendor-and-code-splitting-in-webpack-2-6376358f1923
    new webpack.optimize.CommonsChunkPlugin({
      name: 'node-static',
      filename: 'node-static.js',
      minChunks(module, count) {
        var context = module.context;
        return context && context.indexOf('node_modules') >= 0;
      },
    }),
    new ExtractTextPlugin("styles.css"),
    new HtmlWebpackPlugin({
      hash: true,
      filename: 'index.html',
      template: 'htmlTemplates/index.ejs',
      inject: false,
      cache: false,
      //this is only for the template file
      minify: {
        caseSensitive: true,
        minifyCSS: true,
        collapseWhitespace: true,
        removeAttributeQuotes: true,
        removeComments: true
      }
    }),
    new webpack.DefinePlugin({
      'process.env.NODE_ENV': '"development"'
    }),
    // new webpack.IgnorePlugin(/highlight.js/),
    // new ExtractTextPlugin("styles.css", {
    //   allChunks: true
    // }),
    //this will include jquery whenevery $ or jQuery is used as a var
    //see https://webpack.github.io/docs/shimming-modules.html
    new webpack.ProvidePlugin({
      $: 'jquery',
      jQuery: 'jquery',
      "window.jQuery": 'jquery',
    }),
  ],
  externals: {
    //better include this via cdn... (too big or no module version)
    MathJax: 'MathJax',
    hljs: 'hljs',
    "react": "React",
    "react-dom": "ReactDOM"
  }

}

/*
css modules
wihtout extract stylues
loaders: [
          'style-loader?sourceMap',
          'css-loader?modules&importLoaders=1&localIdentName=[path]___[name]__[local]___[hash:base64:5],typings-for-css-modules-loader',
          'stylus-loader'

        ]
------------------

stylus loader extract
      {
        test: /\.styl$/,
        loader: ExtractTextPlugin.extract({
          fallback: {loader: 'style-loader?sourceMa'},
          use: [
            'css-loader?modules&importLoaders=1&localIdentName=[path]___[name]__[local]___[hash:base64:5],typings-for-css-modules-loader\'', {
            loader: 'stylus-loader',
              options: {
            }
            }
          ]
        })
      }
 */


